@@ -6,26 +6,31 @@ configuration files for Astropy and affiliated packages.
     The configuration system makes use of the 'configobj' package, which stores
     configuration in a text format like that used in the standard library
     `ConfigParser`. More information and documentation for configobj can be
-    found at http://www.voidspace.org.uk/python/configobj.html.
+    found at https://configobj.readthedocs.io .
 """
 
-from contextlib import contextmanager
-import hashlib
 import io
+import pkgutil
+import warnings
+import importlib
+import contextlib
+import os
 from os import path
-import re
+from textwrap import TextWrapper
 from warnings import warn
+from contextlib import contextmanager, nullcontext
 
 from astropy.extern.configobj import configobj, validate
-from astropy.utils.exceptions import AstropyWarning, AstropyDeprecationWarning
-from astropy.utils import find_current_module
+from astropy.utils import find_current_module, silence
+from astropy.utils.decorators import deprecated
+from astropy.utils.exceptions import AstropyDeprecationWarning, AstropyWarning
 from astropy.utils.introspection import resolve_name
+
 from .paths import get_config_dir
 
-
-__all__ = ['InvalidConfigurationItemWarning',
-           'ConfigurationMissingWarning', 'get_config',
-           'reload_config', 'ConfigNamespace', 'ConfigItem']
+__all__ = ('InvalidConfigurationItemWarning', 'ConfigurationMissingWarning',
+           'get_config', 'reload_config', 'ConfigNamespace', 'ConfigItem',
+           'generate_config', 'create_config_file')
 
 
 class InvalidConfigurationItemWarning(AstropyWarning):
@@ -35,6 +40,9 @@ class InvalidConfigurationItemWarning(AstropyWarning):
     """
 
 
+# This was raised with Astropy < 4.3 when the configuration file was not found.
+# It is kept for compatibility and should be removed at some point.
+@deprecated('5.0')
 class ConfigurationMissingWarning(AstropyWarning):
     """ A Warning that is issued when the configuration directory cannot be
     accessed (usually due to a permissions problem). If this warning appears,
@@ -91,6 +99,25 @@ class ConfigNamespace(metaclass=_ConfigNamespaceMeta):
                 aliases=['astropy.utils.console.USE_COLOR'])
         conf = Conf()
     """
+    def __iter__(self):
+        for key, val in self.__class__.__dict__.items():
+            if isinstance(val, ConfigItem):
+                yield key
+
+    keys = __iter__
+    """Iterate over configuration item names."""
+
+    def values(self):
+        """Iterate over configuration item values."""
+        for val in self.__class__.__dict__.values():
+            if isinstance(val, ConfigItem):
+                yield val
+
+    def items(self):
+        """Iterate over configuration item ``(name, value)`` pairs."""
+        for key, val in self.__class__.__dict__.items():
+            if isinstance(val, ConfigItem):
+                yield key, val
 
     def set_temp(self, attr, value):
         """
@@ -131,9 +158,8 @@ class ConfigNamespace(metaclass=_ConfigNamespaceMeta):
                 return self.__class__.__dict__[attr].reload()
             raise AttributeError(f"No configuration parameter '{attr}'")
 
-        for item in self.__class__.__dict__.values():
-            if isinstance(item, ConfigItem):
-                item.reload()
+        for item in self.values():
+            item.reload()
 
     def reset(self, attr=None):
         """
@@ -152,9 +178,8 @@ class ConfigNamespace(metaclass=_ConfigNamespaceMeta):
                 return
             raise AttributeError(f"No configuration parameter '{attr}'")
 
-        for item in self.__class__.__dict__.values():
-            if isinstance(item, ConfigItem):
-                item.set(item.defaultvalue)
+        for item in self.values():
+            item.set(item.defaultvalue)
 
 
 class ConfigItem:
@@ -338,8 +363,9 @@ class ConfigItem:
 
         Returns
         -------
-        val
+        val : object
             The new value loaded from the configuration file.
+
         """
         self.set(self.defaultvalue)
         baseobj = get_config(self.module, True, rootname=self.rootname)
@@ -382,7 +408,7 @@ class ConfigItem:
 
         Returns
         -------
-        val
+        val : object
             This item's value, with a type determined by the ``cfgtype``
             attribute.
 
@@ -390,6 +416,7 @@ class ConfigItem:
         ------
         TypeError
             If the configuration value as stored is not this item's type.
+
         """
         def section_name(section):
             if section == '':
@@ -458,7 +485,7 @@ class ConfigItem:
         return self._validator.check(self.cfgtype, val)
 
 
-# this dictionary stores the master copy of the ConfigObj's for each
+# this dictionary stores the primary copy of the ConfigObj's for each
 # root package
 _cfgobjs = {}
 
@@ -485,7 +512,7 @@ def get_config(packageormod=None, reload=False, rootname=None):
     package or module.
 
     Parameters
-    -----------
+    ----------
     packageormod : str or None
         The package for which to retrieve the configuration object. If a
         string, it must be a valid package name, or if ``None``, the package from
@@ -549,17 +576,12 @@ def get_config(packageormod=None, reload=False, rootname=None):
             else:
                 cfgfn = path.join(get_config_dir(rootname=rootname), pkgname + '.cfg')
             cobj = configobj.ConfigObj(cfgfn, interpolation=False)
-        except OSError as e:
-            msg = ('Configuration defaults will be used due to ')
-            errstr = '' if len(e.args) < 1 else (':' + str(e.args[0]))
-            msg += e.__class__.__name__ + errstr
-            msg += f' on {cfgfn}'
-            warn(ConfigurationMissingWarning(msg))
-
-            # This caches the object, so if the file becomes accessible, this
-            # function won't see it unless the module is reloaded
+        except OSError:
+            # This can happen when HOME is not set
             cobj = configobj.ConfigObj(interpolation=False)
 
+        # This caches the object, so if the file becomes accessible, this
+        # function won't see it unless the module is reloaded
         _cfgobjs[pkgname] = cobj
 
     if secname:  # not the root package
@@ -570,6 +592,98 @@ def get_config(packageormod=None, reload=False, rootname=None):
         return cobj
 
 
+def generate_config(pkgname='astropy', filename=None, verbose=False):
+    """Generates a configuration file, from the list of `ConfigItem`
+    objects for each subpackage.
+
+    .. versionadded:: 4.1
+
+    Parameters
+    ----------
+    pkgname : str or None
+        The package for which to retrieve the configuration object.
+    filename : str or file-like or None
+        If None, the default configuration path is taken from `get_config`.
+
+    """
+    if verbose:
+        verbosity = nullcontext
+        filter_warnings = AstropyDeprecationWarning
+    else:
+        verbosity = silence
+        filter_warnings = Warning
+
+    package = importlib.import_module(pkgname)
+    with verbosity(), warnings.catch_warnings():
+        warnings.simplefilter('ignore', category=filter_warnings)
+        for mod in pkgutil.walk_packages(path=package.__path__,
+                                         prefix=package.__name__ + '.'):
+
+            if (mod.module_finder.path.endswith(('test', 'tests')) or
+                    mod.name.endswith('setup_package')):
+                # Skip test and setup_package modules
+                continue
+            if mod.name.split('.')[-1].startswith('_'):
+                # Skip private modules
+                continue
+
+            with contextlib.suppress(ImportError):
+                importlib.import_module(mod.name)
+
+    wrapper = TextWrapper(initial_indent="## ", subsequent_indent='## ',
+                          width=78)
+
+    if filename is None:
+        filename = get_config_filename(pkgname)
+
+    with contextlib.ExitStack() as stack:
+        if isinstance(filename, (str, os.PathLike)):
+            fp = stack.enter_context(open(filename, 'w'))
+        else:
+            # assume it's a file object, or io.StringIO
+            fp = filename
+
+        # Parse the subclasses, ordered by their module name
+        subclasses = ConfigNamespace.__subclasses__()
+        processed = set()
+
+        for conf in sorted(subclasses, key=lambda x: x.__module__):
+            mod = conf.__module__
+
+            # Skip modules for other packages, e.g. astropy modules that
+            # would be imported when running the function for astroquery.
+            if mod.split('.')[0] != pkgname:
+                continue
+
+            # Check that modules are not processed twice, which can happen
+            # when they are imported in another module.
+            if mod in processed:
+                continue
+            else:
+                processed.add(mod)
+
+            print_module = True
+            for item in conf().values():
+                if print_module:
+                    # If this is the first item of the module, we print the
+                    # module name, but not if this is the root package...
+                    if item.module != pkgname:
+                        modname = item.module.replace(f'{pkgname}.', '')
+                        fp.write(f"[{modname}]\n\n")
+                    print_module = False
+
+                fp.write(wrapper.fill(item.description) + '\n')
+                if isinstance(item.defaultvalue, (tuple, list)):
+                    if len(item.defaultvalue) == 0:
+                        fp.write(f'# {item.name} = ,\n\n')
+                    elif len(item.defaultvalue) == 1:
+                        fp.write(f'# {item.name} = {item.defaultvalue[0]},\n\n')
+                    else:
+                        fp.write(f'# {item.name} = {",".join(map(str, item.defaultvalue))}\n\n')
+                else:
+                    fp.write(f'# {item.name} = {item.defaultvalue}\n\n')
+
+
 def reload_config(packageormod=None, rootname=None):
     """ Reloads configuration settings from a configuration file for the root
     package of the requested package/module.
@@ -598,53 +712,20 @@ def reload_config(packageormod=None, rootname=None):
 def is_unedited_config_file(content, template_content=None):
     """
     Determines if a config file can be safely replaced because it doesn't
-    actually contain any meaningful content.
-
-    To meet this criteria, the config file must be either:
-
-    - All comments or completely empty
-
-    - An exact match to a "legacy" version of the config file prior to
-      Astropy 0.4, when APE3 was implemented and the config file
-      contained commented-out values by default.
+    actually contain any meaningful content, i.e. if it contains only comments
+    or is completely empty.
     """
-    # We want to calculate the md5sum using universal line endings, so
-    # that even if the files had their line endings converted to \r\n
-    # on Windows, this will still work.
-
-    content = content.encode('latin-1')
-
-    # The jquery_url setting, present in 0.3.2 and later only, is
-    # effectively auto-generated by the build system, so we need to
-    # ignore it in the md5sum calculation for 0.3.2.
-    content = re.sub(br'\njquery_url\s*=\s*[^\n]+', b'', content)
-
-    # First determine if the config file has any effective content
-    buffer = io.BytesIO(content)
-    buffer.seek(0)
+    buffer = io.StringIO(content)
     raw_cfg = configobj.ConfigObj(buffer, interpolation=True)
-    for v in raw_cfg.values():
-        if len(v):
-            break
-    else:
-        return True
-
-    # Now determine if it matches the md5sum of a known, unedited
-    # config file.
-    known_configs = set([
-        '7d4b4f1120304b286d71f205975b1286',  # v0.3.2
-        '5df7e409425e5bfe7ed041513fda3288',  # v0.3
-        '8355f99a01b3bdfd8761ef45d5d8b7e5',  # v0.2
-        '4ea5a84de146dc3fcea2a5b93735e634'   # v0.2.1, v0.2.2, v0.2.3, v0.2.4, v0.2.5
-    ])
-
-    md5 = hashlib.md5()
-    md5.update(content)
-    digest = md5.hexdigest()
-    return digest in known_configs
+    # If any of the items is set, return False
+    return not any(len(v) > 0 for v in raw_cfg.values())
 
 
+# This function is no more used by astropy but it is kept for the other
+# packages that may use it (e.g. astroquery). It should be removed at some
+# point.
 # this is not in __all__ because it's not intended that a user uses it
+@deprecated('5.0')
 def update_default_config(pkg, default_cfg_dir_or_fn, version=None, rootname='astropy'):
     """
     Checks if the configuration file for the specified package exists,
@@ -743,3 +824,59 @@ def update_default_config(pkg, default_cfg_dir_or_fn, version=None, rootname='as
             return True
 
     return False
+
+
+def create_config_file(pkg, rootname='astropy', overwrite=False):
+    """
+    Create the default configuration file for the specified package.
+    If the file already exists, it is updated only if it has not been
+    modified.  Otherwise the ``overwrite`` flag is needed to overwrite it.
+
+    Parameters
+    ----------
+    pkg : str
+        The package to be updated.
+    rootname : str
+        Name of the root configuration directory.
+    overwrite : bool
+        Force updating the file if it already exists.
+
+    Returns
+    -------
+    updated : bool
+        If the profile was updated, `True`, otherwise `False`.
+
+    """
+
+    # local import to prevent using the logger before it is configured
+    from astropy.logger import log
+
+    cfgfn = get_config_filename(pkg, rootname=rootname)
+
+    # generate the default config template
+    template_content = io.StringIO()
+    generate_config(pkg, template_content)
+    template_content.seek(0)
+    template_content = template_content.read()
+
+    doupdate = True
+
+    # if the file already exists, check that it has not been modified
+    if cfgfn is not None and path.exists(cfgfn):
+        with open(cfgfn, 'rt', encoding='latin-1') as fd:
+            content = fd.read()
+
+        doupdate = is_unedited_config_file(content, template_content)
+
+    if doupdate or overwrite:
+        with open(cfgfn, 'wt', encoding='latin-1') as fw:
+            fw.write(template_content)
+        log.info('The configuration file has been successfully written '
+                 f'to {cfgfn}')
+        return True
+    elif not doupdate:
+        log.warning('The configuration file already exists and seems to '
+                    'have been customized, so it has not been updated. '
+                    'Use overwrite=True if you really want to update it.')
+
+    return False
