@@ -1,14 +1,15 @@
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-import warnings
+import copy
+from collections.abc import MappingView
+from types import MappingProxyType
 
 import numpy as np
 
 from astropy import units as u
 from astropy.utils.state import ScienceState
-from astropy.utils.decorators import format_doc
-from astropy.utils.exceptions import AstropyDeprecationWarning
+from astropy.utils.decorators import format_doc, classproperty, deprecated
 from astropy.coordinates.angles import Angle
 from astropy.coordinates.matrix_utilities import rotation_matrix, matrix_product, matrix_transpose
 from astropy.coordinates import representation as r
@@ -33,6 +34,25 @@ __all__ = ['Galactocentric']
 _ROLL0 = Angle(58.5986320306*u.degree)
 
 
+class _StateProxy(MappingView):
+    """
+    `~collections.abc.MappingView` with a read-only ``getitem`` through
+    `~types.MappingProxyType`.
+
+    """
+
+    def __init__(self, mapping):
+        super().__init__(mapping)
+        self._mappingproxy = MappingProxyType(self._mapping)  # read-only
+
+    def __getitem__(self, key):
+        """Read-only ``getitem``."""
+        return self._mappingproxy[key]
+
+    def __deepcopy__(self, memo):
+        return copy.deepcopy(self._mapping, memo=memo)
+
+
 class galactocentric_frame_defaults(ScienceState):
     """This class controls the global setting of default values for the frame
     attributes in the `~astropy.coordinates.Galactocentric` frame, which may be
@@ -44,14 +64,32 @@ class galactocentric_frame_defaults(ScienceState):
     ``Galactocentric`` or ``Galactocentric()`` with no explicit arguments.
 
     This class controls the parameter settings by specifying a string name,
-    which can be one of:
+    with the following pre-specified options:
 
     - 'pre-v4.0': The current default value, which sets the default frame
       attribute values to their original (pre-astropy-v4.0) values.
     - 'v4.0': The attribute values as updated in Astropy version 4.0.
     - 'latest': An alias of the most recent parameter set (currently: 'v4.0')
 
-    See :ref:`astropy-coordinates-galactocentric-defaults` for more information.
+    Alternatively, user-defined parameter settings may be registered, with
+    :meth:`~astropy.coordinates.galactocentric_frame_defaults.register`,
+    and used identically as pre-specified parameter sets. At minimum,
+    registrations must have unique names and a dictionary of parameters
+    with keys "galcen_coord", "galcen_distance", "galcen_v_sun", "z_sun",
+    "roll". See examples below.
+
+    This class also tracks the references for all parameter values in the
+    attribute ``references``, as well as any further information the registry.
+    The pre-specified options can be extended to include similar
+    state information as user-defined parameter settings -- for example, to add
+    parameter uncertainties.
+
+    The preferred method for getting a parameter set and metadata, by name, is
+    :meth:`~galactocentric_frame_defaults.get_from_registry` since
+    it ensures the immutability of the registry.
+
+    See :ref:`astropy:astropy-coordinates-galactocentric-defaults` for more
+    information.
 
     Examples
     --------
@@ -59,12 +97,12 @@ class galactocentric_frame_defaults(ScienceState):
     modified globally::
 
         >>> from astropy.coordinates import galactocentric_frame_defaults
-        >>> _ = galactocentric_frame_defaults.set('v4.0')
-        >>> Galactocentric() # doctest: +FLOAT_CMP
+        >>> _ = galactocentric_frame_defaults.set('v4.0') # doctest: +SKIP
+        >>> Galactocentric() # doctest: +SKIP
         <Galactocentric Frame (galcen_coord=<ICRS Coordinate: (ra, dec) in deg
             (266.4051, -28.936175)>, galcen_distance=8.122 kpc, galcen_v_sun=(12.9, 245.6, 7.78) km / s, z_sun=20.8 pc, roll=0.0 deg)>
-        >>> _ = galactocentric_frame_defaults.set('pre-v4.0')
-        >>> Galactocentric() # doctest: +FLOAT_CMP
+        >>> _ = galactocentric_frame_defaults.set('pre-v4.0') # doctest: +SKIP
+        >>> Galactocentric() # doctest: +SKIP
         <Galactocentric Frame (galcen_coord=<ICRS Coordinate: (ra, dec) in deg
             (266.4051, -28.936175)>, galcen_distance=8.3 kpc, galcen_v_sun=(11.1, 232.24, 7.25) km / s, z_sun=27.0 pc, roll=0.0 deg)>
 
@@ -85,98 +123,220 @@ class galactocentric_frame_defaults(ScienceState):
         <Galactocentric Frame (galcen_coord=<ICRS Coordinate: (ra, dec) in deg
             (266.4051, -28.936175)>, galcen_distance=8.0 kpc, galcen_v_sun=(11.1, 232.24, 7.25) km / s, z_sun=27.0 pc, roll=0.0 deg)>
 
+    Additional parameter sets may be registered, for instance to use the
+    Dehnen & Binney (1998) measurements of the solar motion. We can also
+    add metadata, such as the 1-sigma errors. In this example we will modify
+    the required key "parameters", change the recommended key "references" to
+    match "parameters", and add the extra key "error" (any key can be added)::
+
+        >>> state = galactocentric_frame_defaults.get_from_registry("v4.0")
+        >>> state["parameters"]["galcen_v_sun"] = (10.00, 225.25, 7.17) * (u.km / u.s)
+        >>> state["references"]["galcen_v_sun"] = "https://ui.adsabs.harvard.edu/full/1998MNRAS.298..387D"
+        >>> state["error"] = {"galcen_v_sun": (0.36, 0.62, 0.38) * (u.km / u.s)}
+        >>> galactocentric_frame_defaults.register(name="DB1998", **state)
+
+    Just as in the previous examples, the new parameter set can be retrieved with::
+
+        >>> state = galactocentric_frame_defaults.get_from_registry("DB1998")
+        >>> print(state["error"]["galcen_v_sun"])  # doctest: +FLOAT_CMP
+        [0.36 0.62 0.38] km / s
+
     """
 
-    # the default is to use the original definition of this frame
-    # TODO: change this to 'latest' in v4.1?
-    _value = 'pre-v4.0'
+    _latest_value = 'v4.0'
+    _value = None
     _references = None
+    _state = dict()  # all other data
 
-    @staticmethod
-    def get_solar_params_from_string(arg):
-        """Return Galactocentric solar parameters given string names for the
-        parameter sets.
+    # Note: _StateProxy() produces read-only view of enclosed mapping.
+    _registry = {
+        "v4.0": {
+            "parameters": _StateProxy(
+                {
+                    "galcen_coord": ICRS(
+                        ra=266.4051 * u.degree, dec=-28.936175 * u.degree
+                    ),
+                    "galcen_distance": 8.122 * u.kpc,
+                    "galcen_v_sun": r.CartesianDifferential(
+                        [12.9, 245.6, 7.78] * (u.km / u.s)
+                    ),
+                    "z_sun": 20.8 * u.pc,
+                    "roll": 0 * u.deg,
+                }
+            ),
+            "references": _StateProxy(
+                {
+                    "galcen_coord": "https://ui.adsabs.harvard.edu/abs/2004ApJ...616..872R",
+                    "galcen_distance": "https://ui.adsabs.harvard.edu/abs/2018A%26A...615L..15G",
+                    "galcen_v_sun": [
+                        "https://ui.adsabs.harvard.edu/abs/2018RNAAS...2..210D",
+                        "https://ui.adsabs.harvard.edu/abs/2018A%26A...615L..15G",
+                        "https://ui.adsabs.harvard.edu/abs/2004ApJ...616..872R",
+                    ],
+                    "z_sun": "https://ui.adsabs.harvard.edu/abs/2019MNRAS.482.1417B",
+                    "roll": None,
+                }
+            ),
+        },
+        "pre-v4.0": {
+            "parameters": _StateProxy(
+                {
+                    "galcen_coord": ICRS(
+                        ra=266.4051 * u.degree, dec=-28.936175 * u.degree
+                    ),
+                    "galcen_distance": 8.3 * u.kpc,
+                    "galcen_v_sun": r.CartesianDifferential(
+                        [11.1, 220 + 12.24, 7.25] * (u.km / u.s)
+                    ),
+                    "z_sun": 27.0 * u.pc,
+                    "roll": 0 * u.deg,
+                }
+            ),
+            "references": _StateProxy(
+                {
+                    "galcen_coord": "https://ui.adsabs.harvard.edu/abs/2004ApJ...616..872R",
+                    "galcen_distance": "https://ui.adsabs.harvard.edu/#abs/2009ApJ...692.1075G",
+                    "galcen_v_sun": [
+                        "https://ui.adsabs.harvard.edu/#abs/2010MNRAS.403.1829S",
+                        "https://ui.adsabs.harvard.edu/#abs/2015ApJS..216...29B",
+                    ],
+                    "z_sun": "https://ui.adsabs.harvard.edu/#abs/2001ApJ...553..184C",
+                    "roll": None,
+                }
+            ),
+        },
+    }
+
+    @classproperty  # read-only
+    def parameters(cls):
+        return cls._value
+
+    @classproperty  # read-only
+    def references(cls):
+        return cls._references
+
+    @classmethod
+    def get_from_registry(cls, name: str):
         """
-        # Resolve the meaning of 'latest': The latest parameter set is from v4.0
+        Return Galactocentric solar parameters and metadata given string names
+        for the parameter sets. This method ensures the returned state is a
+        mutable copy, so any changes made do not affect the registry state.
+
+        Returns
+        -------
+        state : dict
+            Copy of the registry for the string name.
+            Should contain, at minimum:
+
+            - "parameters": dict
+                Galactocentric solar parameters
+            - "references" : Dict[str, Union[str, Sequence[str]]]
+                References for "parameters".
+                Fields are str or sequence of str.
+
+        Raises
+        ------
+        KeyError
+            If invalid string input to registry
+            to retrieve solar parameters for Galactocentric frame.
+
+        """
+        # Resolve the meaning of 'latest': latest parameter set is from v4.0
         # - update this as newer parameter choices are added
-        if arg == 'latest':
-            arg = 'v4.0'
+        if name == 'latest':
+            name = cls._latest_value
 
-        params = dict()
-        references = dict()
+        # Get the state from the registry.
+        # Copy to ensure registry is immutable to modifications of "_value".
+        # Raises KeyError if `name` is invalid string input to registry
+        # to retrieve solar parameters for Galactocentric frame.
+        state = copy.deepcopy(cls._registry[name])  # ensure mutable
 
-        # Currently, all versions use the same sky position for Sgr A*:
-        params['galcen_coord'] = ICRS(ra=266.4051*u.degree,
-                                      dec=-28.936175*u.degree)
-        references['galcen_coord'] = \
-            'https://ui.adsabs.harvard.edu/abs/2004ApJ...616..872R'
+        return state
 
-        # The roll angle is the same for both frames:
-        params['roll'] = 0 * u.deg
+    @deprecated("v4.2", alternative="`get_from_registry`")
+    @classmethod
+    def get_solar_params_from_string(cls, arg):
+        """
+        Return Galactocentric solar parameters given string names
+        for the parameter sets.
 
-        if arg == 'pre-v4.0':
-            params['galcen_distance'] = 8.3 * u.kpc
-            references['galcen_distance'] = \
-                'https://ui.adsabs.harvard.edu/#abs/2009ApJ...692.1075G'
+        Returns
+        -------
+        parameters : dict
+            Copy of Galactocentric solar parameters from registry
 
-            params['galcen_v_sun'] = r.CartesianDifferential([11.1,
-                                                              220+12.24,
-                                                              7.25]*u.km/u.s)
-            references['galcen_v_sun'] = \
-                ['https://ui.adsabs.harvard.edu/#abs/2010MNRAS.403.1829S',
-                 'https://ui.adsabs.harvard.edu/#abs/2015ApJS..216...29B']
+        Raises
+        ------
+        KeyError
+            If invalid string input to registry
+            to retrieve solar parameters for Galactocentric frame.
 
-            params['z_sun'] = 27.0 * u.pc
-            references['z_sun'] = \
-                'https://ui.adsabs.harvard.edu/#abs/2001ApJ...553..184C'
-
-        elif arg == 'v4.0':
-            params['galcen_distance'] = 8.122 * u.kpc
-            references['galcen_distance'] = \
-                'https://ui.adsabs.harvard.edu/abs/2018A%26A...615L..15G'
-
-            params['galcen_v_sun'] = r.CartesianDifferential([12.9,
-                                                              245.6,
-                                                              7.78]*u.km/u.s)
-            references['galcen_v_sun'] = \
-                ['https://ui.adsabs.harvard.edu/abs/2018RNAAS...2..210D',
-                 'https://ui.adsabs.harvard.edu/abs/2018A%26A...615L..15G',
-                 'https://ui.adsabs.harvard.edu/abs/2004ApJ...616..872R']
-
-            params['z_sun'] = 20.8 * u.pc
-            references['z_sun'] = \
-                'https://ui.adsabs.harvard.edu/abs/2019MNRAS.482.1417B'
-
-        else:
-            raise ValueError('Invalid string input to retrieve solar '
-                             'parameters for Galactocentric frame: "{}"'
-                             .format(arg))
-
-        return params, references
+        """
+        return cls.get_from_registry(arg)["parameters"]
 
     @classmethod
     def validate(cls, value):
+        if value is None:
+            value = cls._latest_value
+
         if isinstance(value, str):
-            params, refs = cls.get_solar_params_from_string(value)
-            cls._references = refs
-            return params
+            state = cls.get_from_registry(value)
+            cls._references = state["references"]
+            cls._state = state
+            parameters = state["parameters"]
 
         elif isinstance(value, dict):
-            return value
+            parameters = value
 
         elif isinstance(value, Galactocentric):
             # turn the frame instance into a dict of frame attributes
-            attrs = dict()
+            parameters = dict()
             for k in value.frame_attributes:
-                attrs[k] = getattr(value, k)
-            cls._references = value.frame_attribute_references()
-            return attrs
+                parameters[k] = getattr(value, k)
+            cls._references = value.frame_attribute_references.copy()
+            cls._state = dict(parameters=parameters,
+                              references=cls._references)
 
         else:
             raise ValueError("Invalid input to retrieve solar parameters for "
                              "Galactocentric frame: input must be a string, "
                              "dict, or Galactocentric instance")
 
+        return parameters
+
+    @classmethod
+    def register(cls, name: str, parameters: dict, references=None,
+                 **meta: dict):
+        """Register a set of parameters.
+
+        Parameters
+        ----------
+        name : str
+            The registration name for the parameter and metadata set.
+        parameters : dict
+            The solar parameters for Galactocentric frame.
+        references : dict or None, optional
+            References for contents of `parameters`.
+            None becomes empty dict.
+        **meta : dict, optional
+            Any other properties to register.
+
+        """
+        # check on contents of `parameters`
+        must_have = {"galcen_coord", "galcen_distance", "galcen_v_sun",
+                     "z_sun", "roll"}
+        missing = must_have.difference(parameters)
+        if missing:
+            raise ValueError(f"Missing parameters: {missing}")
+
+        references = references or {}  # None -> {}
+
+        state = dict(parameters=parameters, references=references)
+        state.update(meta)  # meta never has keys "parameters" or "references"
+
+        cls._registry[name] = state
+
 
 doc_components = """
     x : `~astropy.units.Quantity`, optional
@@ -197,16 +357,16 @@ doc_components = """
 doc_footer = """
     Other parameters
     ----------------
-    galcen_coord : `ICRS`, optional, must be keyword
+    galcen_coord : `ICRS`, optional, keyword-only
         The ICRS coordinates of the Galactic center.
-    galcen_distance : `~astropy.units.Quantity`, optional, must be keyword
+    galcen_distance : `~astropy.units.Quantity`, optional, keyword-only
         The distance from the sun to the Galactic center.
-    galcen_v_sun : `~astropy.coordinates.representation.CartesianDifferential`, optional, must be keyword
+    galcen_v_sun : `~astropy.coordinates.representation.CartesianDifferential`, `~astropy.units.Quantity` ['speed'], optional, keyword-only
         The velocity of the sun *in the Galactocentric frame* as Cartesian
         velocity components.
-    z_sun : `~astropy.units.Quantity`, optional, must be keyword
+    z_sun : `~astropy.units.Quantity` ['length'], optional, keyword-only
         The distance from the sun to the Galactic midplane.
-    roll : `~astropy.coordinates.Angle`, optional, must be keyword
+    roll : `~astropy.coordinates.Angle`, optional, keyword-only
         The angle to rotate about the final x-axis, relative to the
         orientation for Galactic. For example, if this roll angle is 0,
         the final x-z plane will align with the Galactic coordinates x-z
@@ -215,63 +375,55 @@ doc_footer = """
 
     Examples
     --------
-    .. testsetup::
-
-        >>> from astropy.coordinates import galactocentric_frame_defaults
-        >>> _ = galactocentric_frame_defaults.set('pre-v4.0')
 
     To transform to the Galactocentric frame with the default
     frame attributes, pass the uninstantiated class name to the
-    ``transform_to()`` method of a coordinate frame or
-    `~astropy.coordinates.SkyCoord` object::
+    ``transform_to()`` method of a `~astropy.coordinates.SkyCoord` object::
 
         >>> import astropy.units as u
         >>> import astropy.coordinates as coord
-        >>> c = coord.ICRS(ra=[158.3122, 24.5] * u.degree,
-        ...                dec=[-17.3, 81.52] * u.degree,
-        ...                distance=[11.5, 24.12] * u.kpc)
+        >>> c = coord.SkyCoord(ra=[158.3122, 24.5] * u.degree,
+        ...                    dec=[-17.3, 81.52] * u.degree,
+        ...                    distance=[11.5, 24.12] * u.kpc,
+        ...                    frame='icrs')
         >>> c.transform_to(coord.Galactocentric) # doctest: +FLOAT_CMP
-        <Galactocentric Coordinate (galcen_coord=<ICRS Coordinate: (ra, dec) in deg
-            ( 266.4051, -28.936175)>, galcen_distance=8.3 kpc, galcen_v_sun=( 11.1,  232.24,  7.25) km / s, z_sun=27.0 pc, roll=0.0 deg): (x, y, z) in kpc
-            [( -9.6083819 ,  -9.40062188,  6.52056066),
-             (-21.28302307,  18.76334013,  7.84693855)]>
+        <SkyCoord (Galactocentric: galcen_coord=<ICRS Coordinate: (ra, dec) in deg
+            (266.4051, -28.936175)>, galcen_distance=8.122 kpc, galcen_v_sun=(12.9, 245.6, 7.78) km / s, z_sun=20.8 pc, roll=0.0 deg): (x, y, z) in kpc
+            [( -9.43489286, -9.40062188, 6.51345359),
+             (-21.11044918, 18.76334013, 7.83175149)]>
+
 
     To specify a custom set of parameters, you have to include extra keyword
     arguments when initializing the Galactocentric frame object::
 
         >>> c.transform_to(coord.Galactocentric(galcen_distance=8.1*u.kpc)) # doctest: +FLOAT_CMP
-        <Galactocentric Coordinate (galcen_coord=<ICRS Coordinate: (ra, dec) in deg
-            ( 266.4051, -28.936175)>, galcen_distance=8.1 kpc, galcen_v_sun=( 11.1,  232.24,  7.25) km / s, z_sun=27.0 pc, roll=0.0 deg): (x, y, z) in kpc
-            [( -9.40785924,  -9.40062188,  6.52066574),
-             (-21.08239383,  18.76334013,  7.84798135)]>
+        <SkyCoord (Galactocentric: galcen_coord=<ICRS Coordinate: (ra, dec) in deg
+            (266.4051, -28.936175)>, galcen_distance=8.1 kpc, galcen_v_sun=(12.9, 245.6, 7.78) km / s, z_sun=20.8 pc, roll=0.0 deg): (x, y, z) in kpc
+            [( -9.41284763, -9.40062188, 6.51346272),
+             (-21.08839478, 18.76334013, 7.83184184)]>
 
     Similarly, transforming from the Galactocentric frame to another coordinate frame::
 
-        >>> c = coord.Galactocentric(x=[-8.3, 4.5] * u.kpc,
-        ...                          y=[0., 81.52] * u.kpc,
-        ...                          z=[0.027, 24.12] * u.kpc)
+        >>> c = coord.SkyCoord(x=[-8.3, 4.5] * u.kpc,
+        ...                    y=[0., 81.52] * u.kpc,
+        ...                    z=[0.027, 24.12] * u.kpc,
+        ...                    frame=coord.Galactocentric)
         >>> c.transform_to(coord.ICRS) # doctest: +FLOAT_CMP
-        <ICRS Coordinate: (ra, dec, distance) in (deg, deg, kpc)
-            [(  86.22349059, 28.83894138,  4.39157788e-05),
-             ( 289.66802652, 49.88763881,  8.59640735e+01)]>
+        <SkyCoord (ICRS): (ra, dec, distance) in (deg, deg, kpc)
+            [( 88.22423301, 29.88672864,  0.17813456),
+             (289.72864549, 49.9865043 , 85.93949064)]>
 
     Or, with custom specification of the Galactic center::
 
-        >>> c = coord.Galactocentric(x=[-8.0, 4.5] * u.kpc,
-        ...                          y=[0., 81.52] * u.kpc,
-        ...                          z=[21.0, 24120.0] * u.pc,
-        ...                          z_sun=21 * u.pc, galcen_distance=8. * u.kpc)
+        >>> c = coord.SkyCoord(x=[-8.0, 4.5] * u.kpc,
+        ...                    y=[0., 81.52] * u.kpc,
+        ...                    z=[21.0, 24120.0] * u.pc,
+        ...                    frame=coord.Galactocentric,
+        ...                    z_sun=21 * u.pc, galcen_distance=8. * u.kpc)
         >>> c.transform_to(coord.ICRS) # doctest: +FLOAT_CMP
-        <ICRS Coordinate: (ra, dec, distance) in (deg, deg, kpc)
-            [(  86.2585249 ,  28.85773187,  2.75625475e-05),
-             ( 289.77285255,  50.06290457,  8.59216010e+01)]>
-
-
-    # TODO: this needs to be audited and removed if we change the default
-    # defaults (yes) for v4.1. This is a hack to get around the fact that
-    # the doctests here actually change the *class* state:
-
-        >>> galactocentric_frame_defaults._value = 'pre-v4.0'
+        <SkyCoord (ICRS): (ra, dec, distance) in (deg, deg, kpc)
+            [( 86.2585249 , 28.85773187, 2.75625475e-05),
+             (289.77285255, 50.06290457, 8.59216010e+01)]>
 
 """
 
@@ -310,7 +462,7 @@ class Galactocentric(BaseCoordinateFrame):
     roughly towards the North Galactic Pole (:math:`b=90^\circ`).
 
     For a more detailed look at the math behind this transformation, see
-    the document :ref:`coordinates-galactocentric`.
+    the document :ref:`astropy:coordinates-galactocentric`.
 
     The frame attributes are listed under **Other Parameters**.
     """
@@ -333,45 +485,17 @@ class Galactocentric(BaseCoordinateFrame):
         # for the solar parameters defined above
         default_params = galactocentric_frame_defaults.get()
         self.frame_attribute_references = \
-            galactocentric_frame_defaults._references.copy()
+            galactocentric_frame_defaults.references.copy()
 
-        warn = False
         for k in default_params:
             if k in kwargs:
                 # If a frame attribute is set by the user, remove its reference
                 self.frame_attribute_references.pop(k, None)
 
-            else:
-                # If a parameter is read from the defaults, we might want to
-                # warn the user that the defaults will change (see below)
-                warn = True
-
             # Keep the frame attribute if it is set by the user, otherwise use
             # the default value
             kwargs[k] = kwargs.get(k, default_params[k])
 
-        # If the frame defaults have not been updated with the ScienceState
-        # class, and the user uses any default parameter value, raise a
-        # deprecation warning to inform them that the defaults will change in
-        # the future:
-        if galactocentric_frame_defaults._value == 'pre-v4.0' and warn:
-            docs_link = 'http://docs.astropy.org/en/latest/coordinates/galactocentric.html'
-            warnings.warn('In v4.1 and later versions, the Galactocentric '
-                          'frame will adopt default parameters that may update '
-                          'with time. An updated default parameter set is '
-                          'already available through the '
-                          'astropy.coordinates.galactocentric_frame_defaults '
-                          'ScienceState object, as described in but the '
-                          'default is currently still set to the pre-v4.0 '
-                          'parameter defaults. The safest way to guard against '
-                          'changing default parameters in the future is to '
-                          'either (1) specify all Galactocentric frame '
-                          'attributes explicitly when using the frame, '
-                          'or (2) set the galactocentric_frame_defaults '
-                          f'parameter set name explicitly. See {docs_link} for more '
-                          'information.',
-                          AstropyDeprecationWarning)
-
         super().__init__(*args, **kwargs)
 
     @classmethod
@@ -384,7 +508,7 @@ class Galactocentric(BaseCoordinateFrame):
         """
         # note that the actual value is defined at the module level.  We make at
         # a property here because this module isn't actually part of the public
-        # API, so it's better for it to be accessable from Galactocentric
+        # API, so it's better for it to be accessible from Galactocentric
         return _ROLL0
 
 # ICRS to/from Galactocentric ----------------------->
@@ -461,3 +585,7 @@ def icrs_to_galactocentric(icrs_coord, galactocentric_frame):
 def galactocentric_to_icrs(galactocentric_coord, icrs_frame):
     _check_coord_repr_diff_types(galactocentric_coord)
     return get_matrix_vectors(galactocentric_coord, inverse=True)
+
+
+# Create loopback transformation
+frame_transform_graph._add_merged_transform(Galactocentric, ICRS, Galactocentric)
