@@ -4,38 +4,73 @@
 from copy import deepcopy
 import numpy as np
 import pytest
+import re
 
 from astropy import units as u
-from astropy.tests.helper import (catch_warnings,
-                                  assert_quantity_allclose as assert_allclose)
-from astropy.utils import OrderedDescriptorContainer
-from astropy.utils.exceptions import AstropyWarning, AstropyDeprecationWarning
-from astropy.coordinates import representation as r
-from astropy.coordinates.representation import REPRESENTATION_CLASSES
 from astropy.units import allclose
+from astropy.tests.helper import assert_quantity_allclose as assert_allclose
+from astropy.utils.exceptions import AstropyWarning
+from astropy.time import Time
+
+from astropy.coordinates import (
+    EarthLocation,
+    galactocentric_frame_defaults,
+    representation as r,
+    SkyCoord,
+)
+from astropy.coordinates.attributes import (
+    Attribute,
+    CoordinateAttribute,
+    DifferentialAttribute,
+    EarthLocationAttribute,
+    QuantityAttribute,
+    TimeAttribute,
+)
+from astropy.coordinates.baseframe import (
+    BaseCoordinateFrame,
+    RepresentationMapping
+)
+from astropy.coordinates.builtin_frames import (
+    AltAz,
+    HADec,
+    FK4,
+    FK5,
+    Galactic,
+    Galactocentric,
+    GCRS,
+    HCRS,
+    ICRS,
+    ITRS
+)
+from astropy.coordinates.representation import (
+    CartesianDifferential,
+    REPRESENTATION_CLASSES,
+)
 
 from .test_representation import unitphysics  # this fixture is used below # noqa
 
 
 def setup_function(func):
+    """Copy original 'REPRESENTATIONCLASSES' as attribute in function."""
     func.REPRESENTATION_CLASSES_ORIG = deepcopy(REPRESENTATION_CLASSES)
 
 
 def teardown_function(func):
+    """Reset REPRESENTATION_CLASSES to original value."""
     REPRESENTATION_CLASSES.clear()
     REPRESENTATION_CLASSES.update(func.REPRESENTATION_CLASSES_ORIG)
 
 
 def test_frame_attribute_descriptor():
-    """ Unit tests of the Attribute descriptor """
-    from astropy.coordinates.attributes import Attribute
-
-    class TestAttributes(metaclass=OrderedDescriptorContainer):
+    """Unit tests of the Attribute descriptor."""
+    class TestAttributes:
         attr_none = Attribute()
         attr_2 = Attribute(default=2)
         attr_3_attr2 = Attribute(default=3, secondary_attribute='attr_2')
         attr_none_attr2 = Attribute(default=None, secondary_attribute='attr_2')
-        attr_none_nonexist = Attribute(default=None, secondary_attribute='nonexist')
+        attr_none_nonexist = Attribute(
+            default=None, secondary_attribute='nonexist'
+        )
 
     t = TestAttributes()
 
@@ -46,7 +81,8 @@ def test_frame_attribute_descriptor():
     assert t.attr_none_attr2 == t.attr_2
     assert t.attr_none_nonexist is None  # No default and non-existent secondary attr
 
-    # Setting values via '_'-prefixed internal vars (as would normally done in __init__)
+    # Setting values via '_'-prefixed internal vars
+    # (as would normally done in __init__)
     t._attr_none = 10
     assert t.attr_none == 10
 
@@ -65,10 +101,7 @@ def test_frame_attribute_descriptor():
 
 
 def test_frame_subclass_attribute_descriptor():
-    from astropy.coordinates.builtin_frames import FK4
-    from astropy.coordinates.attributes import Attribute, TimeAttribute
-    from astropy.time import Time
-
+    """Unit test of the attribute descriptors in subclasses."""
     _EQUINOX_B1980 = Time('B1980', scale='tai')
 
     class MyFK4(FK4):
@@ -89,9 +122,40 @@ def test_frame_subclass_attribute_descriptor():
     assert mfk4.newattr == 'world'
 
 
+def test_frame_multiple_inheritance_attribute_descriptor():
+    """
+    Ensure that all attributes are accumulated in case of inheritance from
+    multiple BaseCoordinateFrames.  See
+    https://github.com/astropy/astropy/pull/11099#issuecomment-735829157
+    """
+
+    class Frame1(BaseCoordinateFrame):
+        attr1 = Attribute()
+
+    class Frame2(BaseCoordinateFrame):
+        attr2 = Attribute()
+
+    class Frame3(Frame1, Frame2):
+        pass
+
+    assert len(Frame3.frame_attributes) == 2
+    assert 'attr1' in Frame3.frame_attributes
+    assert 'attr2' in Frame3.frame_attributes
+
+    # In case the same attribute exists in both frames, the one from the
+    # left-most class in the MRO should take precedence
+    class Frame4(BaseCoordinateFrame):
+        attr1 = Attribute()
+        attr2 = Attribute()
+
+    class Frame5(Frame1, Frame4):
+        pass
+
+    assert Frame5.frame_attributes['attr1'] is Frame1.frame_attributes['attr1']
+    assert Frame5.frame_attributes['attr2'] is Frame4.frame_attributes['attr2']
+
+
 def test_differentialattribute():
-    from astropy.coordinates import BaseCoordinateFrame
-    from astropy.coordinates.attributes import DifferentialAttribute
 
     # Test logic of passing input through to allowed class
     vel = [1, 2, 3]*u.km/u.s
@@ -121,8 +185,6 @@ def test_differentialattribute():
 
 
 def test_create_data_frames():
-    from astropy.coordinates.builtin_frames import ICRS
-
     # from repr
     i1 = ICRS(r.SphericalRepresentation(1*u.deg, 2*u.deg, 3*u.kpc))
     i2 = ICRS(r.UnitSphericalRepresentation(lon=1*u.deg, lat=2*u.deg))
@@ -148,7 +210,6 @@ def test_create_data_frames():
 
 
 def test_create_orderered_data():
-    from astropy.coordinates.builtin_frames import ICRS, Galactic, AltAz
 
     TOL = 1e-10*u.deg
 
@@ -173,7 +234,6 @@ def test_create_orderered_data():
 
 
 def test_create_nodata_frames():
-    from astropy.coordinates.builtin_frames import ICRS, FK4, FK5
 
     i = ICRS()
     assert len(i.get_frame_attr_names()) == 0
@@ -190,8 +250,6 @@ def test_create_nodata_frames():
 
 
 def test_no_data_nonscalar_frames():
-    from astropy.coordinates.builtin_frames import AltAz
-    from astropy.time import Time
     a1 = AltAz(obstime=Time('2012-01-01') + np.arange(10.) * u.day,
                temperature=np.ones((3, 1)) * u.deg_C)
     assert a1.obstime.shape == (3, 10)
@@ -204,8 +262,6 @@ def test_no_data_nonscalar_frames():
 
 
 def test_frame_repr():
-    from astropy.coordinates.builtin_frames import ICRS, FK5
-
     i = ICRS()
     assert repr(i) == '<ICRS Frame>'
 
@@ -232,7 +288,6 @@ def test_frame_repr():
 
 
 def test_frame_repr_vels():
-    from astropy.coordinates.builtin_frames import ICRS
 
     i = ICRS(ra=1*u.deg, dec=2*u.deg,
              pm_ra_cosdec=1*u.marcsec/u.yr, pm_dec=2*u.marcsec/u.yr)
@@ -246,9 +301,6 @@ def test_frame_repr_vels():
 
 
 def test_converting_units():
-    import re
-    from astropy.coordinates.baseframe import RepresentationMapping
-    from astropy.coordinates.builtin_frames import ICRS, FK5
 
     # this is a regular expression that with split (see below) removes what's
     # the decimal point  to fix rounding problems
@@ -261,8 +313,8 @@ def test_converting_units():
     # converting from FK5 to ICRS and back changes the *internal* representation,
     # but it should still come out in the preferred form
 
-    i4 = i2.transform_to(FK5).transform_to(ICRS)
-    i4_many = i2_many.transform_to(FK5).transform_to(ICRS)
+    i4 = i2.transform_to(FK5()).transform_to(ICRS())
+    i4_many = i2_many.transform_to(FK5()).transform_to(ICRS())
 
     ri2 = ''.join(rexrepr.split(repr(i2)))
     ri4 = ''.join(rexrepr.split(repr(i4)))
@@ -298,8 +350,6 @@ def test_converting_units():
 
 
 def test_representation_info():
-    from astropy.coordinates.baseframe import RepresentationMapping
-    from astropy.coordinates.builtin_frames import ICRS
 
     class NewICRS1(ICRS):
         frame_specific_representation_info = {
@@ -366,8 +416,6 @@ def test_representation_info():
 
 
 def test_realizing():
-    from astropy.coordinates.builtin_frames import ICRS, FK5
-    from astropy.time import Time
 
     rep = r.SphericalRepresentation(1*u.deg, 2*u.deg, 3*u.kpc)
 
@@ -395,8 +443,6 @@ def test_realizing():
 
 
 def test_replicating():
-    from astropy.coordinates.builtin_frames import ICRS, AltAz
-    from astropy.time import Time
 
     i = ICRS(ra=[1]*u.deg, dec=[2]*u.deg)
 
@@ -419,7 +465,6 @@ def test_replicating():
 
 
 def test_getitem():
-    from astropy.coordinates.builtin_frames import ICRS
 
     rep = r.SphericalRepresentation(
         [1, 2, 3]*u.deg, [4, 5, 6]*u.deg, [7, 8, 9]*u.kpc)
@@ -437,14 +482,12 @@ def test_getitem():
 def test_transform():
     """
     This test just makes sure the transform architecture works, but does *not*
-    actually test all the builtin transforms themselves are accurate
-    """
-    from astropy.coordinates.builtin_frames import ICRS, FK4, FK5, Galactic
-    from astropy.time import Time
+    actually test all the builtin transforms themselves are accurate.
 
+    """
     i = ICRS(ra=[1, 2]*u.deg, dec=[3, 4]*u.deg)
-    f = i.transform_to(FK5)
-    i2 = f.transform_to(ICRS)
+    f = i.transform_to(FK5())
+    i2 = f.transform_to(ICRS())
 
     assert i2.data.__class__ == r.UnitSphericalRepresentation
 
@@ -452,28 +495,28 @@ def test_transform():
     assert_allclose(i.dec, i2.dec)
 
     i = ICRS(ra=[1, 2]*u.deg, dec=[3, 4]*u.deg, distance=[5, 6]*u.kpc)
-    f = i.transform_to(FK5)
-    i2 = f.transform_to(ICRS)
+    f = i.transform_to(FK5())
+    i2 = f.transform_to(ICRS())
 
     assert i2.data.__class__ != r.UnitSphericalRepresentation
 
     f = FK5(ra=1*u.deg, dec=2*u.deg, equinox=Time('J2001'))
-    f4 = f.transform_to(FK4)
+    f4 = f.transform_to(FK4())
     f4_2 = f.transform_to(FK4(equinox=f.equinox))
 
     # make sure attributes are copied over correctly
-    assert f4.equinox == FK4.get_frame_attr_names()['equinox']
+    assert f4.equinox == FK4().equinox
     assert f4_2.equinox == f.equinox
 
     # make sure self-transforms also work
     i = ICRS(ra=[1, 2]*u.deg, dec=[3, 4]*u.deg)
-    i2 = i.transform_to(ICRS)
+    i2 = i.transform_to(ICRS())
 
     assert_allclose(i.ra, i2.ra)
     assert_allclose(i.dec, i2.dec)
 
     f = FK5(ra=1*u.deg, dec=2*u.deg, equinox=Time('J2001'))
-    f2 = f.transform_to(FK5)  # default equinox, so should be *different*
+    f2 = f.transform_to(FK5())  # default equinox, so should be *different*
     assert f2.equinox == FK5().equinox
     with pytest.raises(AssertionError):
         assert_allclose(f.ra, f2.ra)
@@ -482,7 +525,7 @@ def test_transform():
 
     # finally, check Galactic round-tripping
     i1 = ICRS(ra=[1, 2]*u.deg, dec=[3, 4]*u.deg)
-    i2 = i1.transform_to(Galactic).transform_to(ICRS)
+    i2 = i1.transform_to(Galactic()).transform_to(ICRS())
 
     assert_allclose(i1.ra, i2.ra)
     assert_allclose(i1.dec, i2.dec)
@@ -490,8 +533,6 @@ def test_transform():
 
 def test_transform_to_nonscalar_nodata_frame():
     # https://github.com/astropy/astropy/pull/5254#issuecomment-241592353
-    from astropy.coordinates.builtin_frames import ICRS, FK5
-    from astropy.time import Time
     times = Time('2016-08-23') + np.linspace(0, 10, 12)*u.day
     coo1 = ICRS(ra=[[0.], [10.], [20.]]*u.deg,
                 dec=[[-30.], [30.], [60.]]*u.deg)
@@ -499,14 +540,130 @@ def test_transform_to_nonscalar_nodata_frame():
     assert coo2.shape == (3, 12)
 
 
+def test_setitem_no_velocity():
+    """Test different flavors of item setting for a Frame without a velocity.
+
+    """
+    obstime = 'B1955'
+    sc0 = FK4([1, 2]*u.deg, [3, 4]*u.deg, obstime=obstime)
+    sc2 = FK4([10, 20]*u.deg, [30, 40]*u.deg, obstime=obstime)
+
+    sc1 = sc0.copy()
+    sc1_repr = repr(sc1)
+    assert 'representation' in sc1.cache
+    sc1[1] = sc2[0]
+    assert sc1.cache == {}
+    assert repr(sc2) != sc1_repr
+
+    assert np.allclose(sc1.ra.to_value(u.deg), [1, 10])
+    assert np.allclose(sc1.dec.to_value(u.deg), [3, 30])
+    assert sc1.obstime == sc2.obstime
+    assert sc1.name == 'fk4'
+
+    sc1 = sc0.copy()
+    sc1[:] = sc2[0]
+    assert np.allclose(sc1.ra.to_value(u.deg), [10, 10])
+    assert np.allclose(sc1.dec.to_value(u.deg), [30, 30])
+
+    sc1 = sc0.copy()
+    sc1[:] = sc2[:]
+    assert np.allclose(sc1.ra.to_value(u.deg), [10, 20])
+    assert np.allclose(sc1.dec.to_value(u.deg), [30, 40])
+
+    sc1 = sc0.copy()
+    sc1[[1, 0]] = sc2[:]
+    assert np.allclose(sc1.ra.to_value(u.deg), [20, 10])
+    assert np.allclose(sc1.dec.to_value(u.deg), [40, 30])
+
+    # Works for array-valued obstime so long as they are considered equivalent
+    sc1 = FK4(sc0.ra, sc0.dec, obstime=[obstime, obstime])
+    sc1[0] = sc2[0]
+
+    # Multidimensional coordinates
+    sc1 = FK4([[1, 2], [3, 4]] * u.deg, [[5, 6], [7, 8]] * u.deg)
+    sc2 = FK4([[10, 20], [30, 40]] * u.deg, [[50, 60], [70, 80]] * u.deg)
+    sc1[0] = sc2[0]
+    assert np.allclose(sc1.ra.to_value(u.deg), [[10, 20], [3, 4]])
+    assert np.allclose(sc1.dec.to_value(u.deg), [[50, 60], [7, 8]])
+
+
+def test_setitem_velocities():
+    """Test different flavors of item setting for a Frame with a velocity.
+
+    """
+    sc0 = FK4([1, 2]*u.deg, [3, 4]*u.deg, radial_velocity=[1, 2]*u.km/u.s,
+              obstime='B1950')
+    sc2 = FK4([10, 20]*u.deg, [30, 40]*u.deg, radial_velocity=[10, 20]*u.km/u.s,
+              obstime='B1950')
+
+    sc1 = sc0.copy()
+    sc1[1] = sc2[0]
+    assert np.allclose(sc1.ra.to_value(u.deg), [1, 10])
+    assert np.allclose(sc1.dec.to_value(u.deg), [3, 30])
+    assert np.allclose(sc1.radial_velocity.to_value(u.km / u.s), [1, 10])
+    assert sc1.obstime == sc2.obstime
+    assert sc1.name == 'fk4'
+
+    sc1 = sc0.copy()
+    sc1[:] = sc2[0]
+    assert np.allclose(sc1.ra.to_value(u.deg), [10, 10])
+    assert np.allclose(sc1.dec.to_value(u.deg), [30, 30])
+    assert np.allclose(sc1.radial_velocity.to_value(u.km / u.s), [10, 10])
+
+    sc1 = sc0.copy()
+    sc1[:] = sc2[:]
+    assert np.allclose(sc1.ra.to_value(u.deg), [10, 20])
+    assert np.allclose(sc1.dec.to_value(u.deg), [30, 40])
+    assert np.allclose(sc1.radial_velocity.to_value(u.km / u.s), [10, 20])
+
+    sc1 = sc0.copy()
+    sc1[[1, 0]] = sc2[:]
+    assert np.allclose(sc1.ra.to_value(u.deg), [20, 10])
+    assert np.allclose(sc1.dec.to_value(u.deg), [40, 30])
+    assert np.allclose(sc1.radial_velocity.to_value(u.km / u.s), [20, 10])
+
+
+def test_setitem_exceptions():
+
+    obstime = 'B1950'
+    sc0 = FK4([1, 2]*u.deg, [3, 4]*u.deg)
+    sc2 = FK4([10, 20]*u.deg, [30, 40]*u.deg, obstime=obstime)
+
+    sc1 = Galactic(sc0.ra, sc0.dec)
+    with pytest.raises(TypeError, match='can only set from object of same class: '
+                       'Galactic vs. FK4'):
+        sc1[0] = sc2[0]
+
+    sc1 = FK4(sc0.ra, sc0.dec, obstime='B2001')
+    with pytest.raises(ValueError, match='can only set frame item from an equivalent frame'):
+        sc1[0] = sc2[0]
+
+    sc1 = FK4(sc0.ra[0], sc0.dec[0], obstime=obstime)
+    with pytest.raises(TypeError, match="scalar 'FK4' frame object does not support "
+                       'item assignment'):
+        sc1[0] = sc2[0]
+
+    sc1 = FK4(obstime=obstime)
+    with pytest.raises(ValueError, match='cannot set frame which has no data'):
+        sc1[0] = sc2[0]
+
+    sc1 = FK4(sc0.ra, sc0.dec, obstime=[obstime, 'B1980'])
+    with pytest.raises(ValueError, match='can only set frame item from an equivalent frame'):
+        sc1[0] = sc2[0]
+
+    # Wrong shape
+    sc1 = FK4([sc0.ra], [sc0.dec], obstime=[obstime, 'B1980'])
+    with pytest.raises(ValueError, match='can only set frame item from an equivalent frame'):
+        sc1[0] = sc2[0]
+
+
 def test_sep():
-    from astropy.coordinates.builtin_frames import ICRS
 
     i1 = ICRS(ra=0*u.deg, dec=1*u.deg)
     i2 = ICRS(ra=0*u.deg, dec=2*u.deg)
 
     sep = i1.separation(i2)
-    assert sep.deg == 1
+    assert_allclose(sep.deg, 1.)
 
     i3 = ICRS(ra=[1, 2]*u.deg, dec=[3, 4]*u.deg, distance=[5, 6]*u.kpc)
     i4 = ICRS(ra=[1, 2]*u.deg, dec=[3, 4]*u.deg, distance=[4, 5]*u.kpc)
@@ -539,10 +696,8 @@ def test_sep():
 def test_time_inputs():
     """
     Test validation and conversion of inputs for equinox and obstime attributes.
-    """
-    from astropy.time import Time
-    from astropy.coordinates.builtin_frames import FK4
 
+    """
     c = FK4(1 * u.deg, 2 * u.deg, equinox='J2001.5', obstime='2000-01-01 12:00:00')
     assert c.equinox == Time('J2001.5')
     assert c.obstime == Time('2000-01-01 12:00:00')
@@ -566,10 +721,8 @@ def test_time_inputs():
 def test_is_frame_attr_default():
     """
     Check that the `is_frame_attr_default` machinery works as expected
-    """
-    from astropy.time import Time
-    from astropy.coordinates.builtin_frames import FK5
 
+    """
     c1 = FK5(ra=1*u.deg, dec=1*u.deg)
     c2 = FK5(ra=1*u.deg, dec=1*u.deg, equinox=FK5.get_frame_attr_names()['equinox'])
     c3 = FK5(ra=1*u.deg, dec=1*u.deg, equinox=Time('J2001.5'))
@@ -589,9 +742,6 @@ def test_is_frame_attr_default():
 
 
 def test_altaz_attributes():
-    from astropy.time import Time
-    from astropy.coordinates import EarthLocation, AltAz
-
     aa = AltAz(1*u.deg, 2*u.deg)
     assert aa.obstime is None
     assert aa.location is None
@@ -603,12 +753,29 @@ def test_altaz_attributes():
     assert isinstance(aa3.location, EarthLocation)
 
 
+def test_hadec_attributes():
+    hd = HADec(1*u.hourangle, 2*u.deg)
+    assert hd.ha == 1.*u.hourangle
+    assert hd.dec == 2*u.deg
+    assert hd.obstime is None
+    assert hd.location is None
+
+    hd2 = HADec(23*u.hourangle, -2*u.deg, obstime='J2000',
+                location=EarthLocation(0*u.deg, 0*u.deg, 0*u.m))
+    assert_allclose(hd2.ha, -1*u.hourangle)
+    assert hd2.dec == -2*u.deg
+    assert hd2.obstime == Time('J2000')
+    assert isinstance(hd2.location, EarthLocation)
+
+    sr = hd2.represent_as(r.SphericalRepresentation)
+    assert_allclose(sr.lon, -1*u.hourangle)
+
+
 def test_representation():
     """
     Test the getter and setter properties for `representation`
-    """
-    from astropy.coordinates.builtin_frames import ICRS
 
+    """
     # Create the frame object.
     icrs = ICRS(ra=1*u.deg, dec=1*u.deg)
     data = icrs.data
@@ -679,7 +846,6 @@ def test_representation():
 
 
 def test_represent_as():
-    from astropy.coordinates.builtin_frames import ICRS
 
     icrs = ICRS(ra=1*u.deg, dec=1*u.deg)
 
@@ -701,11 +867,9 @@ def test_represent_as():
     assert isinstance(rep2.differentials['s'], r.CylindricalDifferential)
 
     # single class with positional in_frame_units, verify that warning raised
-    with catch_warnings() as w:
+    with pytest.warns(AstropyWarning, match='argument position') as w:
         icrs.represent_as(r.CylindricalRepresentation, False)
-        assert len(w) == 1
-        assert w[0].category == AstropyWarning
-        assert 'argument position' in str(w[0].message)
+    assert len(w) == 1
 
     # TODO: this should probably fail in the future once we figure out a better
     # workaround for dealing with UnitSphericalRepresentation's with
@@ -721,7 +885,6 @@ def test_represent_as():
 
 
 def test_shorthand_representations():
-    from astropy.coordinates.builtin_frames import ICRS
 
     rep = r.CartesianRepresentation([1, 2, 3]*u.pc)
     dif = r.CartesianDifferential([1, 2, 3]*u.km/u.s)
@@ -742,8 +905,89 @@ def test_shorthand_representations():
     assert isinstance(sph.differentials['s'], r.SphericalCosLatDifferential)
 
 
+def test_equal():
+
+    obstime = 'B1955'
+    sc1 = FK4([1, 2]*u.deg, [3, 4]*u.deg, obstime=obstime)
+    sc2 = FK4([1, 20]*u.deg, [3, 4]*u.deg, obstime=obstime)
+
+    # Compare arrays and scalars
+    eq = sc1 == sc2
+    ne = sc1 != sc2
+    assert np.all(eq == [True, False])
+    assert np.all(ne == [False, True])
+    assert (sc1[0] == sc2[0]) == True  # noqa  (numpy True not Python True)
+    assert (sc1[0] != sc2[0]) == False  # noqa
+
+    # Broadcasting
+    eq = sc1[0] == sc2
+    ne = sc1[0] != sc2
+    assert np.all(eq == [True, False])
+    assert np.all(ne == [False, True])
+
+    # With diff only in velocity
+    sc1 = FK4([1, 2]*u.deg, [3, 4]*u.deg, radial_velocity=[1, 2]*u.km/u.s)
+    sc2 = FK4([1, 2]*u.deg, [3, 4]*u.deg, radial_velocity=[1, 20]*u.km/u.s)
+
+    eq = sc1 == sc2
+    ne = sc1 != sc2
+    assert np.all(eq == [True, False])
+    assert np.all(ne == [False, True])
+    assert (sc1[0] == sc2[0]) == True  # noqa
+    assert (sc1[0] != sc2[0]) == False  # noqa
+
+    assert (FK4() == ICRS()) is False
+    assert (FK4() == FK4(obstime='J1999')) is False
+
+
+def test_equal_exceptions():
+
+    # Shape mismatch
+    sc1 = FK4([1, 2, 3]*u.deg, [3, 4, 5]*u.deg)
+    with pytest.raises(ValueError, match='cannot compare: shape mismatch'):
+        sc1 == sc1[:2]
+
+    # Different representation_type
+    sc1 = FK4(1, 2, 3, representation_type='cartesian')
+    sc2 = FK4(1*u.deg, 2*u.deg, 2, representation_type='spherical')
+    with pytest.raises(TypeError, match='cannot compare: objects must have same '
+                       'class: CartesianRepresentation vs. SphericalRepresentation'):
+        sc1 == sc2
+
+    # Different differential type
+    sc1 = FK4(1*u.deg, 2*u.deg, radial_velocity=1*u.km/u.s)
+    sc2 = FK4(1*u.deg, 2*u.deg, pm_ra_cosdec=1*u.mas/u.yr, pm_dec=1*u.mas/u.yr)
+    with pytest.raises(TypeError, match='cannot compare: objects must have same '
+                       'class: RadialDifferential vs. UnitSphericalCosLatDifferential'):
+        sc1 == sc2
+
+    # Different frame attribute
+    sc1 = FK5(1*u.deg, 2*u.deg)
+    sc2 = FK5(1*u.deg, 2*u.deg, equinox='J1999')
+    with pytest.raises(TypeError, match=r'cannot compare: objects must have equivalent '
+                       r'frames: <FK5 Frame \(equinox=J2000.000\)> '
+                       r'vs. <FK5 Frame \(equinox=J1999.000\)>'):
+        sc1 == sc2
+
+    # Different frame
+    sc1 = FK4(1*u.deg, 2*u.deg)
+    sc2 = FK5(1*u.deg, 2*u.deg, equinox='J2000')
+    with pytest.raises(TypeError, match='cannot compare: objects must have equivalent '
+                       r'frames: <FK4 Frame \(equinox=B1950.000, obstime=B1950.000\)> '
+                       r'vs. <FK5 Frame \(equinox=J2000.000\)>'):
+        sc1 == sc2
+
+    sc1 = FK4(1*u.deg, 2*u.deg)
+    sc2 = FK4()
+    with pytest.raises(ValueError, match='cannot compare: one frame has data and '
+                       'the other does not'):
+        sc1 == sc2
+    with pytest.raises(ValueError, match='cannot compare: one frame has data and '
+                       'the other does not'):
+        sc2 == sc1
+
+
 def test_dynamic_attrs():
-    from astropy.coordinates.builtin_frames import ICRS
     c = ICRS(1*u.deg, 2*u.deg)
     assert 'ra' in dir(c)
     assert 'dec' in dir(c)
@@ -761,7 +1005,6 @@ def test_dynamic_attrs():
 
 
 def test_nodata_error():
-    from astropy.coordinates.builtin_frames import ICRS
 
     i = ICRS()
     with pytest.raises(ValueError) as excinfo:
@@ -771,7 +1014,6 @@ def test_nodata_error():
 
 
 def test_len0_data():
-    from astropy.coordinates.builtin_frames import ICRS
 
     i = ICRS([]*u.deg, []*u.deg)
     assert i.has_data
@@ -779,7 +1021,6 @@ def test_len0_data():
 
 
 def test_quantity_attributes():
-    from astropy.coordinates.builtin_frames import GCRS
 
     # make sure we can create a GCRS frame with valid inputs
     GCRS(obstime='J2002', obsgeoloc=[1, 2, 3]*u.km, obsgeovel=[4, 5, 6]*u.km/u.s)
@@ -794,7 +1035,6 @@ def test_quantity_attributes():
 
 
 def test_quantity_attribute_default():
-    from astropy.coordinates import BaseCoordinateFrame, QuantityAttribute
 
     # The default default (yes) is None:
     class MyCoord(BaseCoordinateFrame):
@@ -852,9 +1092,7 @@ def test_quantity_attribute_default():
             someval = QuantityAttribute()
 
 
-@pytest.mark.remote_data
 def test_eloc_attributes():
-    from astropy.coordinates import AltAz, ITRS, GCRS, EarthLocation
 
     el = EarthLocation(lon=12.3*u.deg, lat=45.6*u.deg, height=1*u.km)
     it = ITRS(r.SphericalRepresentation(lon=12.3*u.deg, lat=45.6*u.deg, distance=1*u.km))
@@ -889,9 +1127,6 @@ def test_eloc_attributes():
 
 
 def test_equivalent_frames():
-    from astropy.coordinates import SkyCoord
-    from astropy.coordinates.builtin_frames import ICRS, FK4, FK5, AltAz
-
     i = ICRS()
     i2 = ICRS(1*u.deg, 2*u.deg)
     assert i.is_equivalent_frame(i)
@@ -922,12 +1157,51 @@ def test_equivalent_frames():
     assert not aa1.is_equivalent_frame(aa2)
 
 
+def test_equivalent_frame_coordinateattribute():
+
+    class FrameWithCoordinateAttribute(BaseCoordinateFrame):
+        coord_attr = CoordinateAttribute(HCRS)
+
+    # These frames should not be considered equivalent
+    f0 = FrameWithCoordinateAttribute()
+    f1 = FrameWithCoordinateAttribute(coord_attr=HCRS(1*u.deg, 2*u.deg, obstime='J2000'))
+    f2 = FrameWithCoordinateAttribute(coord_attr=HCRS(3*u.deg, 4*u.deg, obstime='J2000'))
+    f3 = FrameWithCoordinateAttribute(coord_attr=HCRS(1*u.deg, 2*u.deg, obstime='J2001'))
+
+    assert not f0.is_equivalent_frame(f1)
+    assert not f1.is_equivalent_frame(f0)
+    assert not f1.is_equivalent_frame(f2)
+    assert not f1.is_equivalent_frame(f3)
+    assert not f2.is_equivalent_frame(f3)
+
+    # They each should still be equivalent to a deep copy of themselves
+    assert f0.is_equivalent_frame(deepcopy(f0))
+    assert f1.is_equivalent_frame(deepcopy(f1))
+    assert f2.is_equivalent_frame(deepcopy(f2))
+    assert f3.is_equivalent_frame(deepcopy(f3))
+
+
+def test_equivalent_frame_locationattribute():
+
+    class FrameWithLocationAttribute(BaseCoordinateFrame):
+        loc_attr = EarthLocationAttribute()
+
+    # These frames should not be considered equivalent
+    f0 = FrameWithLocationAttribute()
+    location = EarthLocation(lat=-34, lon=19, height=300)
+    f1 = FrameWithLocationAttribute(loc_attr=location)
+
+    assert not f0.is_equivalent_frame(f1)
+    assert not f1.is_equivalent_frame(f0)
+
+    # They each should still be equivalent to a deep copy of themselves
+    assert f0.is_equivalent_frame(deepcopy(f0))
+    assert f1.is_equivalent_frame(deepcopy(f1))
+
+
 def test_representation_subclass():
-
     # Regression test for #3354
 
-    from astropy.coordinates.builtin_frames import FK5
-
     # Normally when instantiating a frame without a distance the frame will try
     # and use UnitSphericalRepresentation internally instead of
     # SphericalRepresentation.
@@ -970,7 +1244,6 @@ def test_getitem_representation():
     Make sure current representation survives __getitem__ even if different
     from data representation.
     """
-    from astropy.coordinates.builtin_frames import ICRS
     c = ICRS([1, 1] * u.deg, [2, 2] * u.deg)
     c.representation_type = 'cartesian'
     assert c[0].representation_type is r.CartesianRepresentation
@@ -981,7 +1254,6 @@ def test_component_error_useful():
     Check that a data-less frame gives useful error messages about not having
     data when the attributes asked for are possible coordinate components
     """
-    from astropy.coordinates.builtin_frames import ICRS
 
     i = ICRS()
 
@@ -998,7 +1270,6 @@ def test_component_error_useful():
 
 
 def test_cache_clear():
-    from astropy.coordinates.builtin_frames import ICRS
 
     i = ICRS(1*u.deg, 2*u.deg)
 
@@ -1013,7 +1284,6 @@ def test_cache_clear():
 
 
 def test_inplace_array():
-    from astropy.coordinates.builtin_frames import ICRS
 
     i = ICRS([[1, 2], [3, 4]]*u.deg, [[10, 20], [30, 40]]*u.deg)
 
@@ -1035,7 +1305,6 @@ def test_inplace_array():
 
 
 def test_inplace_change():
-    from astropy.coordinates.builtin_frames import ICRS
 
     i = ICRS(1*u.deg, 2*u.deg)
 
@@ -1057,7 +1326,6 @@ def test_inplace_change():
 
 
 def test_representation_with_multiple_differentials():
-    from astropy.coordinates.builtin_frames import ICRS
 
     dif1 = r.CartesianDifferential([1, 2, 3]*u.km/u.s)
     dif2 = r.CartesianDifferential([1, 2, 3]*u.km/u.s**2)
@@ -1069,36 +1337,6 @@ def test_representation_with_multiple_differentials():
         ICRS(rep)
 
 
-def test_representation_arg_backwards_compatibility():
-    # TODO: this test can be removed when the `representation` argument is
-    # removed from the BaseCoordinateFrame initializer.
-    from astropy.coordinates.builtin_frames import ICRS
-
-    c1 = ICRS(x=1*u.pc, y=2*u.pc, z=3*u.pc,
-              representation_type=r.CartesianRepresentation)
-
-    c2 = ICRS(x=1*u.pc, y=2*u.pc, z=3*u.pc,
-              representation_type=r.CartesianRepresentation)
-
-    c3 = ICRS(x=1*u.pc, y=2*u.pc, z=3*u.pc,
-              representation_type='cartesian')
-
-    assert c1.x == c2.x
-    assert c1.y == c2.y
-    assert c1.z == c2.z
-
-    assert c1.x == c3.x
-    assert c1.y == c3.y
-    assert c1.z == c3.z
-
-    assert c1.representation_type == c1.representation_type
-
-    with pytest.raises(ValueError):
-        ICRS(x=1*u.pc, y=2*u.pc, z=3*u.pc,
-             representation_type='cartesian',
-             representation='cartesian')
-
-
 def test_missing_component_error_names():
     """
     This test checks that the component names are frame component names, not
@@ -1110,9 +1348,8 @@ def test_missing_component_error_names():
     should state:
 
     TypeError: __init__() missing 1 required positional argument: 'dec'
-    """
-    from astropy.coordinates.builtin_frames import ICRS
 
+    """
     with pytest.raises(TypeError) as e:
         ICRS(ra=150 * u.deg)
     assert "missing 1 required positional argument: 'dec'" in str(e.value)
@@ -1124,7 +1361,6 @@ def test_missing_component_error_names():
 
 
 def test_non_spherical_representation_unit_creation(unitphysics):
-    from astropy.coordinates.builtin_frames import ICRS
 
     class PhysicsICRS(ICRS):
         default_representation = r.PhysicsSphericalRepresentation
@@ -1137,8 +1373,6 @@ def test_non_spherical_representation_unit_creation(unitphysics):
 
 
 def test_attribute_repr():
-    from astropy.coordinates.attributes import Attribute
-    from astropy.coordinates.baseframe import BaseCoordinateFrame
 
     class Spam:
         def _astropy_repr_in_frame(self):
@@ -1151,8 +1385,6 @@ def test_attribute_repr():
 
 
 def test_component_names_repr():
-    from astropy.coordinates.baseframe import BaseCoordinateFrame, RepresentationMapping
-
     # Frame class with new component names that includes a name swap
     class NameChangeFrame(BaseCoordinateFrame):
         default_representation = r.PhysicsSphericalRepresentation
@@ -1172,23 +1404,7 @@ def test_component_names_repr():
     assert repr(frame).count("JUSTONCE") == 1
 
 
-@pytest.fixture
-def reset_galactocentric_defaults():
-    # TODO: this can be removed, along with the "warning" test below, once we
-    # switch the default to 'latest' in v4.1
-    from astropy.coordinates import galactocentric_frame_defaults
-
-    # Resets before each test, and after (the yield is pytest magic)
-    galactocentric_frame_defaults._value = 'pre-v4.0'
-    yield
-    galactocentric_frame_defaults._value = 'pre-v4.0'
-
-
-def test_galactocentric_defaults(reset_galactocentric_defaults):
-    from astropy.coordinates import (Galactocentric,
-                                     galactocentric_frame_defaults,
-                                     BaseCoordinateFrame,
-                                     CartesianDifferential)
+def test_galactocentric_defaults():
 
     with galactocentric_frame_defaults.set('pre-v4.0'):
         galcen_pre40 = Galactocentric()
@@ -1214,32 +1430,30 @@ def test_galactocentric_defaults(reset_galactocentric_defaults):
         else:
             assert getattr(galcen_40, k) == getattr(galcen_latest, k)
 
-
-def test_galactocentric_default_warning(reset_galactocentric_defaults):
-    from astropy.coordinates import (Galactocentric,
-                                     galactocentric_frame_defaults)
-
-    # Make sure a warning is thrown if the frame is created with no args
-    with pytest.warns(AstropyDeprecationWarning,
-                      match=r"In v4\.1 and later versions"):
-        Galactocentric()
-
-    # Throw a warning even if only a subset of args are specified
-    with pytest.warns(AstropyDeprecationWarning,
-                      match=r"In v4\.1 and later versions"):
-        Galactocentric(galcen_distance=8.2*u.kpc)
-
-    # No warning if using the latest parameter set:
+    # test validate Galactocentric
     with galactocentric_frame_defaults.set('latest'):
-        Galactocentric()
+        params = galactocentric_frame_defaults.validate(galcen_latest)
+        references = galcen_latest.frame_attribute_references
+        state = dict(parameters=params, references=references)
+
+        assert galactocentric_frame_defaults.parameters == params
+        assert galactocentric_frame_defaults.references == references
+        assert galactocentric_frame_defaults._state == state
+
+    # Test not one of accepted parameter types
+    with pytest.raises(ValueError):
+        galactocentric_frame_defaults.validate(ValueError)
+
+    # test parameters property
+    assert (
+        galactocentric_frame_defaults.parameters
+        == galactocentric_frame_defaults.parameters
+    )
 
 
-def test_galactocentric_references(reset_galactocentric_defaults):
+def test_galactocentric_references():
     # references in the "scientific paper"-sense
 
-    from astropy.coordinates import (Galactocentric,
-                                     galactocentric_frame_defaults)
-
     with galactocentric_frame_defaults.set('pre-v4.0'):
         galcen_pre40 = Galactocentric()
 
@@ -1269,3 +1483,53 @@ def test_galactocentric_references(reset_galactocentric_defaults):
                 assert k not in galcen_custom.frame_attribute_references
             else:
                 assert k in galcen_custom.frame_attribute_references
+
+
+def test_coordinateattribute_transformation():
+
+    class FrameWithCoordinateAttribute(BaseCoordinateFrame):
+        coord_attr = CoordinateAttribute(HCRS)
+
+    hcrs = HCRS(1*u.deg, 2*u.deg, 3*u.AU, obstime='2001-02-03')
+    f1_frame = FrameWithCoordinateAttribute(coord_attr=hcrs)
+    f1_skycoord = FrameWithCoordinateAttribute(coord_attr=SkyCoord(hcrs))
+
+    # The input is already HCRS, so the frame attribute should not change it
+    assert f1_frame.coord_attr == hcrs
+    # The output should not be different if a SkyCoord is provided
+    assert f1_skycoord.coord_attr == f1_frame.coord_attr
+
+    gcrs = GCRS(4*u.deg, 5*u.deg, 6*u.AU, obstime='2004-05-06')
+    f2_frame = FrameWithCoordinateAttribute(coord_attr=gcrs)
+    f2_skycoord = FrameWithCoordinateAttribute(coord_attr=SkyCoord(gcrs))
+
+    # The input needs to be converted from GCRS to HCRS
+    assert isinstance(f2_frame.coord_attr, HCRS)
+    # The `obstime` frame attribute should have been "merged" in a SkyCoord-style transformation
+    assert f2_frame.coord_attr.obstime == gcrs.obstime
+    # The output should not be different if a SkyCoord is provided
+    assert f2_skycoord.coord_attr == f2_frame.coord_attr
+
+
+def test_realize_frame_accepts_kwargs():
+    c1 = ICRS(x=1*u.pc, y=2*u.pc, z=3*u.pc,
+              representation_type=r.CartesianRepresentation)
+    new_data = r.CartesianRepresentation(x=11*u.pc, y=12*u.pc, z=13*u.pc)
+
+    c2 = c1.realize_frame(new_data, representation_type="cartesian")
+    c3 = c1.realize_frame(new_data, representation_type="cylindrical")
+
+    assert c2.representation_type == r.CartesianRepresentation
+    assert c3.representation_type == r.CylindricalRepresentation
+
+
+def test_nameless_frame_subclass():
+    """Note: this is a regression test for #11096"""
+
+    class Test:
+        pass
+
+    # Subclass from a frame class and a non-frame class.
+    # This subclassing is the test!
+    class NewFrame(ICRS, Test):
+        pass
