@@ -1,11 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-
 import os
 import copy
+from contextlib import nullcontext
 from io import StringIO
 from itertools import chain
-
+import pathlib
 
 import pytest
 import numpy as np
@@ -13,18 +13,15 @@ import numpy as np
 from astropy.io import ascii
 from astropy import table
 from astropy.table.table_helpers import simple_table
-from astropy.tests.helper import catch_warnings
-from astropy.utils.exceptions import AstropyWarning, AstropyDeprecationWarning
+from astropy.utils.exceptions import AstropyWarning
+from astropy.utils.compat.optional_deps import HAS_BS4
+from astropy.utils.misc import _NOT_OVERWRITING_MSG_MATCH
 from astropy import units as u
 
 from .common import setup_function, teardown_function  # noqa
 
-# Check to see if the BeautifulSoup dependency is present.
-try:
+if HAS_BS4:
     from bs4 import BeautifulSoup, FeatureNotFound  # noqa
-    HAS_BEAUTIFUL_SOUP = True
-except ImportError:
-    HAS_BEAUTIFUL_SOUP = False
 
 test_defs = [
     dict(kwargs=dict(),
@@ -394,26 +391,59 @@ a,b,c
 ]
 
 
-def check_write_table(test_def, table, fast_writer):
-    out = StringIO()
+@pytest.fixture
+def home_is_tmpdir(monkeypatch, tmpdir):
+    """
+    Pytest fixture to run a test case with tilde-prefixed paths.
+
+    In the tilde-path case, environment variables are temporarily
+    modified so that '~' resolves to the temp directory.
+    """
+    # For Unix
+    monkeypatch.setenv('HOME', str(tmpdir))
+    # For Windows
+    monkeypatch.setenv('USERPROFILE', str(tmpdir))
+
+
+def check_write_table(test_def, table, fast_writer, out=None):
+    if out is None:
+        out = StringIO()
+
     try:
         ascii.write(table, out, fast_writer=fast_writer, **test_def['kwargs'])
     except ValueError as e:  # if format doesn't have a fast writer, ignore
         if 'not in the list of formats with fast writers' not in str(e.value):
             raise e
         return
-    print('Expected:\n{}'.format(test_def['out']))
-    print('Actual:\n{}'.format(out.getvalue()))
-    assert [x.strip() for x in out.getvalue().strip().splitlines()] == [
+
+    if isinstance(out, StringIO):
+        # Output went to a buffer
+        actual = out.getvalue()
+    else:
+        # Output went to a file
+        if str(out).startswith('~'):
+            # Ensure a file hasn't been accidentally written to a literal tilde
+            # path
+            assert not os.path.exists(out)
+            out = os.path.expanduser(out)
+        assert os.path.exists(out)
+        with open(out) as f:
+            actual = f.read()
+        os.remove(out)
+
+    print(f"Expected:\n{test_def['out']}")
+    print(f'Actual:\n{actual}')
+    assert [x.strip() for x in actual.strip().splitlines()] == [
         x.strip() for x in test_def['out'].strip().splitlines()]
 
 
-def check_write_table_via_table(test_def, table, fast_writer):
-    out = StringIO()
+def check_write_table_via_table(test_def, table, fast_writer, out=None):
+    if out is None:
+        out = StringIO()
 
     test_def = copy.deepcopy(test_def)
     if 'Writer' in test_def['kwargs']:
-        format = 'ascii.{}'.format(test_def['kwargs']['Writer']._format_name)
+        format = f"ascii.{test_def['kwargs']['Writer']._format_name}"
         del test_def['kwargs']['Writer']
     else:
         format = 'ascii'
@@ -424,20 +454,49 @@ def check_write_table_via_table(test_def, table, fast_writer):
         if 'not in the list of formats with fast writers' not in str(e.value):
             raise e
         return
-    print('Expected:\n{}'.format(test_def['out']))
-    print('Actual:\n{}'.format(out.getvalue()))
-    assert [x.strip() for x in out.getvalue().strip().splitlines()] == [
+
+    if isinstance(out, StringIO):
+        # Output went to a buffer
+        actual = out.getvalue()
+    else:
+        # Output went to a file
+        if str(out).startswith('~'):
+            # Ensure a file hasn't been accidentally written to a literal tilde
+            # path
+            assert not os.path.exists(out)
+            out = os.path.expanduser(out)
+        assert os.path.exists(out)
+        with open(out) as f:
+            actual = f.read()
+        os.remove(out)
+
+    print(f"Expected:\n{test_def['out']}")
+    print(f'Actual:\n{actual}')
+    assert [x.strip() for x in actual.strip().splitlines()] == [
         x.strip() for x in test_def['out'].strip().splitlines()]
 
 
 @pytest.mark.parametrize("fast_writer", [True, False])
-def test_write_table(fast_writer):
+@pytest.mark.parametrize('path_format',
+        ['buffer', 'plain', 'tilde-str', 'tilde-pathlib'])
+def test_write_table(
+        fast_writer, tmpdir, home_is_tmpdir, path_format):
     table = ascii.get_reader(Reader=ascii.Daophot)
     data = table.read('data/daophot.dat')
 
+    if path_format == 'buffer':
+        out_name = None
+    elif path_format == 'plain':
+        out_name = os.path.join(tmpdir, 'table')
+    elif path_format == 'tilde-str':
+        out_name = os.path.join('~', 'table')
+    else:
+        out_name = pathlib.Path('~', 'table')
+
     for test_def in test_defs:
-        check_write_table(test_def, data, fast_writer)
-        check_write_table_via_table(test_def, data, fast_writer)
+        check_write_table(test_def, data, fast_writer, out=out_name)
+        check_write_table_via_table(
+            test_def, data, fast_writer, out=out_name)
 
 
 @pytest.mark.parametrize("fast_writer", [True, False])
@@ -478,12 +537,11 @@ def test_write_invalid_toplevel_meta_ipac():
     table = ascii.get_reader(Reader=ascii.Ipac)
     data = table.read('data/no_data_ipac.dat')
     data.meta['blah'] = 'extra'
+    out = StringIO()
 
-    with catch_warnings(AstropyWarning) as ASwarn:
-        out = StringIO()
+    with pytest.warns(AstropyWarning, match=r'.*were not written.*') as warn:
         data.write(out, format='ascii.ipac')
-    assert len(ASwarn) == 1
-    assert "were not written" in str(ASwarn[0].message)
+    assert len(warn) == 1
 
 
 def test_write_invalid_keyword_meta_ipac():
@@ -494,12 +552,11 @@ def test_write_invalid_keyword_meta_ipac():
     table = ascii.get_reader(Reader=ascii.Ipac)
     data = table.read('data/no_data_ipac.dat')
     data.meta['keywords']['blah'] = 'invalid'
+    out = StringIO()
 
-    with catch_warnings(AstropyWarning) as ASwarn:
-        out = StringIO()
+    with pytest.warns(AstropyWarning, match=r'.*has been skipped.*') as warn:
         data.write(out, format='ascii.ipac')
-    assert len(ASwarn) == 1
-    assert "has been skipped" in str(ASwarn[0].message)
+    assert len(warn) == 1
 
 
 def test_write_valid_meta_ipac():
@@ -508,11 +565,8 @@ def test_write_valid_meta_ipac():
     table = ascii.get_reader(Reader=ascii.Ipac)
     data = table.read('data/no_data_ipac.dat')
     data.meta['keywords']['blah'] = {'value': 'invalid'}
-
-    with catch_warnings(AstropyWarning) as ASwarn:
-        out = StringIO()
-        data.write(out, format='ascii.ipac')
-    assert len(ASwarn) == 0
+    out = StringIO()
+    data.write(out, format='ascii.ipac')
 
 
 @pytest.mark.parametrize("fast_writer", [True, False])
@@ -629,11 +683,19 @@ def test_byte_string_output(fast_writer):
 def test_names_with_formats(names, include_names, exclude_names, formats, issues_warning):
     """Test for #4508."""
     t = table.Table([[1, 2, 3], [4.1, 5.2, 6.3]])
-    with catch_warnings(AstropyWarning) as ASwarn:
-        out = StringIO()
+    out = StringIO()
+
+    if issues_warning:
+        ctx = pytest.warns(AstropyWarning)
+    else:
+        ctx = nullcontext()
+
+    with ctx as warn:
         ascii.write(t, out, names=names, include_names=include_names,
                     exclude_names=exclude_names, formats=formats)
-    assert (issues_warning == (len(ASwarn) == 1))
+
+    if issues_warning:
+        assert len(warn) == 1
 
 
 @pytest.mark.parametrize('formats, issues_warning', [
@@ -645,10 +707,18 @@ def test_names_with_formats(names, include_names, exclude_names, formats, issues
 def test_columns_names_with_formats(formats, issues_warning):
     """Test the fix for #4508."""
     t = table.Table([[1, 2, 3], [4.1, 5.2, 6.3]])
-    with catch_warnings(AstropyWarning) as ASwarn:
-        out = StringIO()
+    out = StringIO()
+
+    if issues_warning:
+        ctx = pytest.warns(AstropyWarning)
+    else:
+        ctx = nullcontext()
+
+    with ctx as warn:
         ascii.write(t, out, formats=formats)
-    assert (issues_warning == (len(ASwarn) == 1))
+
+    if issues_warning:
+        assert len(warn) == 1
 
 
 @pytest.mark.parametrize("fast_writer", [True, False])
@@ -679,38 +749,40 @@ def test_write_empty_table(fast_writer):
 @pytest.mark.parametrize("format", ['ascii', 'csv', 'html', 'latex',
                                     'ascii.fixed_width', 'html'])
 @pytest.mark.parametrize("fast_writer", [True, False])
-def test_write_overwrite_ascii(format, fast_writer, tmpdir):
+@pytest.mark.parametrize('path_format', ['plain', 'tilde-str', 'tilde-pathlib'])
+def test_write_overwrite_ascii(format, fast_writer, tmpdir, home_is_tmpdir,
+        path_format):
     """Test overwrite argument for various ASCII writers"""
-    filename = tmpdir.join("table-tmp.dat").strpath
-    with open(filename, 'w'):
+    true_filename = tmpdir.join("table-tmp.dat").strpath
+    if path_format == 'plain':
+        filename = true_filename
+    elif path_format == 'tilde-str':
+        filename = os.path.join('~', 'table-tmp.dat')
+    else:
+        filename = pathlib.Path('~', 'table-tmp.dat')
+
+    with open(true_filename, 'w'):
         # create empty file
         pass
     t = table.Table([['Hello', ''], ['', '']], dtype=['S10', 'S10'])
 
-    with pytest.raises(OSError) as err:
-        t.write(filename, overwrite=False, format=format,
-                fast_writer=fast_writer)
-    assert str(err.value).endswith('already exists')
-
-    with catch_warnings(AstropyDeprecationWarning) as warning:
+    with pytest.raises(OSError, match=_NOT_OVERWRITING_MSG_MATCH):
         t.write(filename, format=format, fast_writer=fast_writer)
-    assert len(warning) == 1
-    assert str(warning[0].message).endswith(
-        "Automatically overwriting ASCII files is deprecated. "
-        "Use the argument 'overwrite=True' in the future.")
 
     t.write(filename, overwrite=True, format=format,
             fast_writer=fast_writer)
 
     # If the output is a file object, overwrite is ignored
-    with open(filename, 'w') as fp:
-        t.write(fp, format=format,
-                fast_writer=fast_writer)
+    with open(true_filename, 'w') as fp:
         t.write(fp, overwrite=False, format=format,
                 fast_writer=fast_writer)
         t.write(fp, overwrite=True, format=format,
                 fast_writer=fast_writer)
 
+    if 'tilde' in path_format:
+        # Ensure no files have been accidentally written to a literal tilde path
+        assert not os.path.exists(filename)
+
 
 fmt_name_classes = list(chain(ascii.core.FAST_CLASSES.items(),
                               ascii.core.FORMAT_CLASSES.items()))
@@ -728,25 +800,31 @@ def test_roundtrip_masked(fmt_name_class):
         return
 
     # Skip tests for fixed_width or HTML without bs4
-    if ((fmt_name == 'html' and not HAS_BEAUTIFUL_SOUP)
+    if ((fmt_name == 'html' and not HAS_BS4)
             or fmt_name == 'fixed_width'):
         return
 
-    t = simple_table(masked=True)
+    if 'qdp' in fmt_name:
+        # QDP tables are for numeric values only
+        t = simple_table(masked=True, kinds=['f', 'i'])
+    else:
+        t = simple_table(masked=True)
 
     out = StringIO()
     fast = fmt_name in ascii.core.FAST_CLASSES
     try:
         ascii.write(t, out, format=fmt_name, fast_writer=fast)
-    except ImportError:  # Some failed dependency, e.g. PyYAML, skip test
+    except ImportError:  # Some failed dependency, skip test
         return
 
     # No-header formats need to be told the column names
     kwargs = {'names': t.colnames} if 'no_header' in fmt_name else {}
+    if 'qdp' in fmt_name:
+        kwargs.update({'table_id': 0, 'names': t.colnames})
 
     t2 = ascii.read(out.getvalue(), format=fmt_name, fast_reader=fast, guess=False, **kwargs)
-
     assert t.colnames == t2.colnames
+
     for col, col2 in zip(t.itercols(), t2.itercols()):
         assert col.dtype.kind == col2.dtype.kind
         assert np.all(col == col2)
@@ -770,6 +848,24 @@ def test_write_newlines(fast_writer, tmpdir):
     assert content == os.linesep.join(['col', 'a', 'b', 'c']) + os.linesep
 
 
+@pytest.mark.parametrize("fast_writer", [True, False])
+def test_write_csv_with_comments(fast_writer):
+    """
+    Test fix for #7357 where writing a Table with comments to 'csv' fails with
+    a cryptic message. The comments are dropped by default, but when comment='#'
+    is supplied they are still written.
+    """
+    out = StringIO()
+    t = table.Table([[1, 2], [3, 4]], names=['a', 'b'])
+    t.meta['comments'] = ['hello']
+    ascii.write(t, out, format='csv', fast_writer=fast_writer)
+    assert out.getvalue().splitlines() == ['a,b', '1,3', '2,4']
+
+    out = StringIO()
+    ascii.write(t, out, format='csv', fast_writer=fast_writer, comment='#')
+    assert out.getvalue().splitlines() == ['#hello', 'a,b', '1,3', '2,4']
+
+
 @pytest.mark.parametrize("fast_writer", [True, False])
 def test_write_formatted_mixin(fast_writer):
     """
@@ -782,3 +878,56 @@ def test_write_formatted_mixin(fast_writer):
     assert out.getvalue().splitlines() == ['a b',
                                            '01 1.00',
                                            '02 2.00']
+
+
+def test_validate_write_kwargs():
+    out = StringIO()
+    t = table.QTable([[1, 2], [1, 2]], names=['a', 'b'])
+
+    with pytest.raises(TypeError, match=r"write\(\) argument 'fast_writer' must be a "
+                       r"\(<class 'bool'>, <class 'str'>\) object, "
+                       r"got <class 'int'> instead"):
+        ascii.write(t, out, fast_writer=12)
+
+
+@pytest.mark.parametrize("fmt_name_class", fmt_name_classes)
+def test_multidim_column_error(fmt_name_class):
+    """
+    Test that trying to write a multidim column fails in every format except
+    ECSV.
+    """
+    fmt_name, fmt_cls = fmt_name_class
+
+    if not getattr(fmt_cls, '_io_registry_can_write', True):
+        return
+
+    # Skip tests for ecsv or HTML without bs4. See the comment in latex.py
+    # Latex class where max_ndim = None is defined regarding latex and aastex.
+    if ((fmt_name == 'html' and not HAS_BS4)
+            or fmt_name in ('ecsv', 'latex', 'aastex')):
+        return
+
+    out = StringIO()
+    t = table.Table()
+    t['a'] = np.arange(16).reshape(2, 2, 2, 2)
+    t['b'] = [1, 2]
+    fast = fmt_name in ascii.core.FAST_CLASSES
+    with pytest.raises(ValueError, match=r'column\(s\) with dimension'):
+        ascii.write(t, out, format=fmt_name, fast_writer=fast)
+
+
+@pytest.mark.parametrize("fast_writer", [True, False])
+def test_write_as_columns(fast_writer):
+    """
+    Test that writing a set of columns also roundtrips (as long as the
+    table does not have metadata, etc.)
+    """
+    # Use masked in case that makes it more difficult.
+    data = ascii.read(tab_to_fill)
+    data = table.Table(data, masked=True)
+    data['a'].mask = [True, False]
+    data['c'].mask = [False, True]
+    data = list(data.columns.values())
+
+    for test_def in test_def_masked_fill_value:
+        check_write_table(test_def, data, fast_writer)
