@@ -9,8 +9,8 @@ import fnmatch
 import glob
 import io
 import operator
+import os
 import os.path
-import pathlib
 import textwrap
 
 from collections import defaultdict
@@ -23,12 +23,12 @@ from astropy import __version__
 
 from .card import Card, BLANK_CARD
 from .header import Header
-from astropy.utils.decorators import deprecated_renamed_argument
 # HDUList is used in one of the doctests
 from .hdu.hdulist import fitsopen, HDUList  # pylint: disable=W0611
 from .hdu.table import _TableLikeHDU
 from astropy.utils.diff import (report_diff_values, fixed_width_indent,
                                 where_not_allclose, diff_values)
+from astropy.utils.misc import NOT_OVERWRITING_MSG
 
 __all__ = ['FITSDiff', 'HDUDiff', 'HeaderDiff', 'ImageDataDiff', 'RawDataDiff',
            'TableDataDiff']
@@ -120,7 +120,6 @@ class _BaseDiff:
         return not any(getattr(self, attr) for attr in self.__dict__
                        if attr.startswith('diff_'))
 
-    @deprecated_renamed_argument('clobber', 'overwrite', '2.0')
     def report(self, fileobj=None, indent=0, overwrite=False):
         """
         Generates a text report on the differences (if any) between two
@@ -129,7 +128,7 @@ class _BaseDiff:
 
         Parameters
         ----------
-        fileobj : file-like object, string, or None, optional
+        fileobj : file-like, string, or None, optional
             If `None`, this method returns the report as a string. Otherwise it
             returns `None` and writes the report to the given file-like object
             (which must have a ``.write()`` method at a minimum), or to a new
@@ -143,9 +142,6 @@ class _BaseDiff:
             ``OSError`` if ``False`` and the output file exists. Default is
             ``False``.
 
-            .. versionchanged:: 1.3
-               ``overwrite`` replaces the deprecated ``clobber`` argument.
-
         Returns
         -------
         report : str or None
@@ -156,8 +152,7 @@ class _BaseDiff:
 
         if isinstance(fileobj, str):
             if os.path.exists(fileobj) and not overwrite:
-                raise OSError("File {} exists, aborting (pass in "
-                              "overwrite=True to overwrite)".format(fileobj))
+                raise OSError(NOT_OVERWRITING_MSG.format(fileobj))
             else:
                 filepath = fileobj
                 fileobj = open(filepath, 'w')
@@ -269,7 +264,7 @@ class FITSDiff(_BaseDiff):
             whitespace (default: True).
         """
 
-        if isinstance(a, (str, pathlib.Path)):
+        if isinstance(a, (str, os.PathLike)):
             try:
                 a = fitsopen(a)
             except Exception as exc:
@@ -279,7 +274,7 @@ class FITSDiff(_BaseDiff):
         else:
             close_a = False
 
-        if isinstance(b, (str, pathlib.Path)):
+        if isinstance(b, (str, os.PathLike)):
             try:
                 b = fitsopen(b)
             except Exception as exc:
@@ -327,13 +322,11 @@ class FITSDiff(_BaseDiff):
         # Record filenames for use later in _report
         self.filenamea = self.a.filename()
         if not self.filenamea:
-            self.filenamea = '<{} object at {:#x}>'.format(
-                self.a.__class__.__name__, id(self.a))
+            self.filenamea = f'<{self.a.__class__.__name__} object at {id(self.a):#x}>'
 
         self.filenameb = self.b.filename()
         if not self.filenameb:
-            self.filenameb = '<{} object at {:#x}>'.format(
-                self.b.__class__.__name__, id(self.b))
+            self.filenameb = f'<{self.b.__class__.__name__} object at {id(self.b):#x}>'
 
         if self.ignore_hdus:
             self.a = HDUList([h for h in self.a if h.name not in self.ignore_hdus])
@@ -356,7 +349,10 @@ class FITSDiff(_BaseDiff):
             hdu_diff = HDUDiff.fromdiff(self, self.a[idx], self.b[idx])
 
             if not hdu_diff.identical:
-                self.diff_hdus.append((idx, hdu_diff))
+                if self.a[idx].name == self.b[idx].name and self.a[idx].ver == self.b[idx].ver:
+                    self.diff_hdus.append((idx, hdu_diff, self.a[idx].name, self.a[idx].ver))
+                else:
+                    self.diff_hdus.append((idx, hdu_diff, "", self.a[idx].ver))
 
     def _report(self):
         wrapper = textwrap.TextWrapper(initial_indent='  ',
@@ -368,8 +364,7 @@ class FITSDiff(_BaseDiff):
 
         if self.ignore_hdus:
             ignore_hdus = ' '.join(sorted(self.ignore_hdus))
-            self._writeln(' HDU(s) not to be compared:\n{}'
-                          .format(wrapper.fill(ignore_hdus)))
+            self._writeln(f' HDU(s) not to be compared:\n{wrapper.fill(ignore_hdus)}')
 
         if self.ignore_hdu_patterns:
             ignore_hdu_patterns = ' '.join(sorted(self.ignore_hdu_patterns))
@@ -399,8 +394,8 @@ class FITSDiff(_BaseDiff):
         if self.diff_hdu_count:
             self._fileobj.write('\n')
             self._writeln('Files contain different numbers of HDUs:')
-            self._writeln(' a: {}'.format(self.diff_hdu_count[0]))
-            self._writeln(' b: {}'.format(self.diff_hdu_count[1]))
+            self._writeln(f' a: {self.diff_hdu_count[0]}')
+            self._writeln(f' b: {self.diff_hdu_count[1]}')
 
             if not self.diff_hdus:
                 self._writeln('No differences found between common HDUs.')
@@ -410,14 +405,17 @@ class FITSDiff(_BaseDiff):
             self._writeln('No differences found.')
             return
 
-        for idx, hdu_diff in self.diff_hdus:
+        for idx, hdu_diff, extname, extver in self.diff_hdus:
             # print out the extension heading
             if idx == 0:
                 self._fileobj.write('\n')
                 self._writeln('Primary HDU:')
             else:
                 self._fileobj.write('\n')
-                self._writeln(f'Extension HDU {idx}:')
+                if extname:
+                    self._writeln(f'Extension HDU {idx} ({extname}, {extver}):')
+                else:
+                    self._writeln(f'Extension HDU {idx}:')
             hdu_diff.report(self._fileobj, indent=self._indent + 1)
 
 
@@ -456,11 +454,11 @@ class HDUDiff(_BaseDiff):
         """
         Parameters
         ----------
-        a : `HDUList`
-            An `HDUList` object.
+        a : BaseHDU
+            An HDU object.
 
-        b : str or `HDUList`
-            An `HDUList` object to compare to the first `HDUList` object.
+        b : BaseHDU
+            An HDU object to compare to the first HDU object.
 
         ignore_keywords : sequence, optional
             Header keywords to ignore when comparing two headers; the presence
@@ -520,6 +518,7 @@ class HDUDiff(_BaseDiff):
 
         self.numdiffs = numdiffs
         self.ignore_blanks = ignore_blanks
+        self.ignore_blank_cards = ignore_blank_cards
 
         self.diff_extnames = ()
         self.diff_extvers = ()
@@ -652,11 +651,11 @@ class HeaderDiff(_BaseDiff):
         """
         Parameters
         ----------
-        a : `HDUList`
-            An `HDUList` object.
+        a : `~astropy.io.fits.Header` or string or bytes
+            A header.
 
-        b : `HDUList`
-            An `HDUList` object to compare to the first `HDUList` object.
+        b : `~astropy.io.fits.Header` or string or bytes
+            A header to compare to the first header.
 
         ignore_keywords : sequence, optional
             Header keywords to ignore when comparing two headers; the presence
@@ -873,28 +872,25 @@ class HeaderDiff(_BaseDiff):
     def _report(self):
         if self.diff_keyword_count:
             self._writeln(' Headers have different number of cards:')
-            self._writeln('  a: {}'.format(self.diff_keyword_count[0]))
-            self._writeln('  b: {}'.format(self.diff_keyword_count[1]))
+            self._writeln(f'  a: {self.diff_keyword_count[0]}')
+            self._writeln(f'  b: {self.diff_keyword_count[1]}')
         if self.diff_keywords:
             for keyword in self.diff_keywords[0]:
                 if keyword in Card._commentary_keywords:
                     val = self.a[keyword][0]
                 else:
                     val = self.a[keyword]
-                self._writeln(' Extra keyword {!r:8} in a: {!r}'.format(
-                                keyword, val))
+                self._writeln(f' Extra keyword {keyword!r:8} in a: {val!r}')
             for keyword in self.diff_keywords[1]:
                 if keyword in Card._commentary_keywords:
                     val = self.b[keyword][0]
                 else:
                     val = self.b[keyword]
-                self._writeln(' Extra keyword {!r:8} in b: {!r}'.format(
-                                keyword, val))
+                self._writeln(f' Extra keyword {keyword!r:8} in b: {val!r}')
 
         if self.diff_duplicate_keywords:
             for keyword, count in sorted(self.diff_duplicate_keywords.items()):
-                self._writeln(' Inconsistent duplicates of keyword {!r:8}:'
-                              .format(keyword))
+                self._writeln(f' Inconsistent duplicates of keyword {keyword!r:8}:')
                 self._writeln('  Occurs {} time(s) in a, {} times in (b)'
                               .format(*count))
 
@@ -949,11 +945,11 @@ class ImageDataDiff(_BaseDiff):
         """
         Parameters
         ----------
-        a : `HDUList`
-            An `HDUList` object.
+        a : BaseHDU
+            An HDU object.
 
-        b : `HDUList`
-            An `HDUList` object to compare to the first `HDUList` object.
+        b : BaseHDU
+            An HDU object to compare to the first HDU object.
 
         numdiffs : int, optional
             The number of pixel/table values to output when reporting HDU data
@@ -1089,11 +1085,11 @@ class RawDataDiff(ImageDataDiff):
         """
         Parameters
         ----------
-        a : `HDUList`
-            An `HDUList` object.
+        a : BaseHDU
+            An HDU object.
 
-        b : `HDUList`
-            An `HDUList` object to compare to the first `HDUList` object.
+        b : BaseHDU
+            An HDU object to compare to the first HDU object.
 
         numdiffs : int, optional
             The number of pixel/table values to output when reporting HDU data
@@ -1120,8 +1116,8 @@ class RawDataDiff(ImageDataDiff):
     def _report(self):
         if self.diff_dimensions:
             self._writeln(' Data sizes differ:')
-            self._writeln('  a: {} bytes'.format(self.diff_dimensions[0]))
-            self._writeln('  b: {} bytes'.format(self.diff_dimensions[1]))
+            self._writeln(f'  a: {self.diff_dimensions[0]} bytes')
+            self._writeln(f'  b: {self.diff_dimensions[1]} bytes')
             # For now we don't do any further comparison if the dimensions
             # differ; though in the future it might be nice to be able to
             # compare at least where the images intersect
@@ -1193,11 +1189,11 @@ class TableDataDiff(_BaseDiff):
         """
         Parameters
         ----------
-        a : `HDUList`
-            An `HDUList` object.
+        a : BaseHDU
+            An HDU object.
 
-        b : `HDUList`
-            An `HDUList` object to compare to the first `HDUList` object.
+        b : BaseHDU
+            An HDU object to compare to the first HDU object.
 
         ignore_fields : sequence, optional
             The (case-insensitive) names of any table columns to ignore if any
@@ -1402,34 +1398,31 @@ class TableDataDiff(_BaseDiff):
     def _report(self):
         if self.diff_column_count:
             self._writeln(' Tables have different number of columns:')
-            self._writeln('  a: {}'.format(self.diff_column_count[0]))
-            self._writeln('  b: {}'.format(self.diff_column_count[1]))
+            self._writeln(f'  a: {self.diff_column_count[0]}')
+            self._writeln(f'  b: {self.diff_column_count[1]}')
 
         if self.diff_column_names:
             # Show columns with names unique to either table
             for name in self.diff_column_names[0]:
                 format = self.diff_columns[0][name.lower()].format
-                self._writeln(' Extra column {} of format {} in a'.format(
-                                name, format))
+                self._writeln(f' Extra column {name} of format {format} in a')
             for name in self.diff_column_names[1]:
                 format = self.diff_columns[1][name.lower()].format
-                self._writeln(' Extra column {} of format {} in b'.format(
-                                name, format))
+                self._writeln(f' Extra column {name} of format {format} in b')
 
         col_attrs = dict(_COL_ATTRS)
         # Now go through each table again and show columns with common
         # names but other property differences...
         for col_attr, vals in self.diff_column_attributes:
             name, attr = col_attr
-            self._writeln(' Column {} has different {}:'.format(
-                    name, col_attrs[attr]))
+            self._writeln(f' Column {name} has different {col_attrs[attr]}:')
             report_diff_values(vals[0], vals[1], fileobj=self._fileobj,
                                indent_width=self._indent + 1)
 
         if self.diff_rows:
             self._writeln(' Table rows differ:')
-            self._writeln('  a: {}'.format(self.diff_rows[0]))
-            self._writeln('  b: {}'.format(self.diff_rows[1]))
+            self._writeln(f'  a: {self.diff_rows[0]}')
+            self._writeln(f'  b: {self.diff_rows[1]}')
             self._writeln(' No further data comparison performed.')
             return
 
@@ -1468,7 +1461,7 @@ def report_diff_keyword_attr(fileobj, attr, diffs, keyword, ind=0):
             if idx == 0:
                 dup = ''
             else:
-                dup = '[{}]'.format(idx + 1)
+                dup = f'[{idx + 1}]'
             fileobj.write(
                 fixed_width_indent(' Keyword {:8}{} has different {}:\n'
                                    .format(keyword, dup, attr), ind))
