@@ -14,12 +14,13 @@ import numpy as np
 from .base import DELAYED, ExtensionHDU, BITPIX2DTYPE, DTYPE2BITPIX
 from .image import ImageHDU
 from .table import BinTableHDU
+from astropy.io.fits import conf
 from astropy.io.fits.card import Card
 from astropy.io.fits.column import Column, ColDefs, TDEF_RE
 from astropy.io.fits.column import KEYWORD_NAMES as TABLE_KEYWORD_NAMES
 from astropy.io.fits.fitsrec import FITS_rec
 from astropy.io.fits.header import Header
-from astropy.io.fits.util import (_is_pseudo_unsigned, _unsigned_zero, _is_int,
+from astropy.io.fits.util import (_is_pseudo_integer, _pseudo_zero, _is_int,
                                   _get_array_mmap)
 
 from astropy.utils import lazyproperty
@@ -56,21 +57,7 @@ DEFAULT_HCOMP_SMOOTH = 0
 DEFAULT_BLOCK_SIZE = 32
 DEFAULT_BYTE_PIX = 4
 
-CMTYPE_ALIASES = {}
-
-# CFITSIO version-specific features
-if COMPRESSION_SUPPORTED:
-    try:
-        CFITSIO_SUPPORTS_GZIPDATA = compression.CFITSIO_VERSION >= 3.28
-        CFITSIO_SUPPORTS_Q_FORMAT = compression.CFITSIO_VERSION >= 3.35
-        if compression.CFITSIO_VERSION >= 3.35:
-            CMTYPE_ALIASES['RICE_ONE'] = 'RICE_1'
-    except AttributeError:
-        # This generally shouldn't happen unless running setup.py in an
-        # environment where an old build of pyfits exists
-        CFITSIO_SUPPORTS_GZIPDATA = True
-        CFITSIO_SUPPORTS_Q_FORMAT = True
-
+CMTYPE_ALIASES = {'RICE_ONE': 'RICE_1'}
 
 COMPRESSION_KEYWORDS = {'ZIMAGE', 'ZCMPTYPE', 'ZBITPIX', 'ZNAXIS', 'ZMASKCMP',
                         'ZSIMPLE', 'ZTENSION', 'ZEXTEND'}
@@ -204,6 +191,12 @@ class CompImageHeader(Header):
         super().append(card=card, useblanks=useblanks, bottom=bottom, end=end)
 
         remapped_keyword = self._remap_keyword(card.keyword)
+
+        # card.keyword strips the HIERARCH if present so this must be added
+        # back to avoid a warning.
+        if str(card).startswith("HIERARCH ") and not remapped_keyword.startswith("HIERARCH "):
+            remapped_keyword = "HIERARCH " + remapped_keyword
+
         card = Card(remapped_keyword, card.value, card.comment)
 
         # Here we disable the use of blank cards, because the call above to
@@ -381,6 +374,14 @@ class CompImageHeader(Header):
 
         return idx
 
+    def clear(self):
+        """
+        Remove all cards from the header.
+        """
+
+        self._table_header.clear()
+        super().clear()
+
 
 # TODO: Fix this class so that it doesn't actually inherit from BinTableHDU,
 # but instead has an internal BinTableHDU reference
@@ -398,6 +399,8 @@ class CompImageHDU(BinTableHDU):
     responsibility).
     """
 
+    _default_name = "COMPRESSED_IMAGE"
+
     def __init__(self, data=None, header=None, name=None,
                  compression_type=DEFAULT_COMPRESSION_TYPE,
                  tile_size=None,
@@ -666,6 +669,48 @@ class CompImageHDU(BinTableHDU):
         self._orig_bscale = self._bscale
         self._orig_bitpix = self._bitpix
 
+    def _remove_unnecessary_default_extnames(self, header):
+        """Remove default EXTNAME values if they are unnecessary.
+
+        Some data files (eg from CFHT) can have the default EXTNAME and
+        an explicit value.  This method removes the default if a more
+        specific header exists. It also removes any duplicate default
+        values.
+        """
+        if 'EXTNAME' in header:
+            indices = header._keyword_indices['EXTNAME']
+            # Only continue if there is more than one found
+            n_extname = len(indices)
+            if n_extname > 1:
+                extnames_to_remove = [index for index in indices
+                                      if header[index] == self._default_name]
+                if len(extnames_to_remove) == n_extname:
+                    # Keep the first (they are all the same)
+                    extnames_to_remove.pop(0)
+                # Remove them all in reverse order to keep the index unchanged.
+                for index in reversed(sorted(extnames_to_remove)):
+                    del header[index]
+
+    @property
+    def name(self):
+        # Convert the value to a string to be flexible in some pathological
+        # cases (see ticket #96)
+        # Similar to base class but uses .header rather than ._header
+        return str(self.header.get('EXTNAME', self._default_name))
+
+    @name.setter
+    def name(self, value):
+        # This is a copy of the base class but using .header instead
+        # of ._header to ensure that the name stays in sync.
+        if not isinstance(value, str):
+            raise TypeError("'name' attribute must be a string")
+        if not conf.extension_name_case_sensitive:
+            value = value.upper()
+        if 'EXTNAME' in self.header:
+            self.header['EXTNAME'] = value
+        else:
+            self.header['EXTNAME'] = (value, 'extension name')
+
     @classmethod
     def match_header(cls, header):
         card = header.cards[0]
@@ -758,6 +803,9 @@ class CompImageHDU(BinTableHDU):
             DITHER_SEED_CHECKSUM (-1)
         """
 
+        # Clean up EXTNAME duplicates
+        self._remove_unnecessary_default_extnames(self._header)
+
         image_hdu = ImageHDU(data=self.data, header=self._header)
         self._image_header = CompImageHeader(self._header, image_hdu.header)
         self._axes = image_hdu._axes
@@ -774,26 +822,19 @@ class CompImageHDU(BinTableHDU):
         # almost entirely contrived corner cases, so it will do for now
         if self._has_data:
             huge_hdu = self.data.nbytes > 2 ** 32
-
-            if huge_hdu and not CFITSIO_SUPPORTS_Q_FORMAT:
-                raise OSError(
-                    "Astropy cannot compress images greater than 4 GB in size "
-                    "({} is {} bytes) without CFITSIO >= 3.35".format(
-                        (self.name, self.ver), self.data.nbytes))
         else:
             huge_hdu = False
 
         # Update the extension name in the table header
         if not name and 'EXTNAME' not in self._header:
-            name = 'COMPRESSED_IMAGE'
-
-        if name:
-            self._header.set('EXTNAME', name,
+            # Do not sync this with the image header since the default
+            # name is specific to the table header.
+            self._header.set('EXTNAME', self._default_name,
                              'name of this binary table extension',
                              after='TFIELDS')
+        elif name:
+            # Force the name into table and image headers.
             self.name = name
-        else:
-            self.name = self._header['EXTNAME']
 
         # Set the compression type in the table header.
         if compression_type:
@@ -829,10 +870,14 @@ class CompImageHDU(BinTableHDU):
             if bzero != 0.0:
                 self._header.set('BZERO', bzero, after=after_keyword)
 
+        try:
             bitpix_comment = image_header.comments['BITPIX']
-            naxis_comment = image_header.comments['NAXIS']
-        else:
+        except (AttributeError, KeyError):
             bitpix_comment = 'data type of original image'
+
+        try:
+            naxis_comment = image_header.comments['NAXIS']
+        except (AttributeError, KeyError):
             naxis_comment = 'dimension of original image'
 
         # Set the label for the first column in the table
@@ -873,25 +918,11 @@ class CompImageHDU(BinTableHDU):
             # this behavior so the only way to determine which behavior will
             # be employed is via the CFITSIO version
 
-            if CFITSIO_SUPPORTS_GZIPDATA:
-                ttype2 = 'GZIP_COMPRESSED_DATA'
-                # The required format for the GZIP_COMPRESSED_DATA is actually
-                # missing from the standard docs, but CFITSIO suggests it
-                # should be 1PB, which is logical.
-                tform2 = '1QB' if huge_hdu else '1PB'
-            else:
-                # Q format is not supported for UNCOMPRESSED_DATA columns.
-                ttype2 = 'UNCOMPRESSED_DATA'
-                if zbitpix == 8:
-                    tform2 = '1QB' if huge_hdu else '1PB'
-                elif zbitpix == 16:
-                    tform2 = '1QI' if huge_hdu else '1PI'
-                elif zbitpix == 32:
-                    tform2 = '1QJ' if huge_hdu else '1PJ'
-                elif zbitpix == -32:
-                    tform2 = '1QE' if huge_hdu else '1PE'
-                else:
-                    tform2 = '1QD' if huge_hdu else '1PD'
+            ttype2 = 'GZIP_COMPRESSED_DATA'
+            # The required format for the GZIP_COMPRESSED_DATA is actually
+            # missing from the standard docs, but CFITSIO suggests it
+            # should be 1PB, which is logical.
+            tform2 = '1QB' if huge_hdu else '1PB'
 
             # Set up the second column for the table that will hold any
             # uncompressable data.
@@ -1466,6 +1497,11 @@ class CompImageHDU(BinTableHDU):
         if hasattr(self, '_image_header'):
             return self._image_header
 
+        # Clean up any possible doubled EXTNAME keywords that use
+        # the default. Do this on the original header to ensure
+        # duplicates are removed cleanly.
+        self._remove_unnecessary_default_extnames(self._header)
+
         # Start with a copy of the table header.
         image_header = self._header.copy()
 
@@ -1555,8 +1591,8 @@ class CompImageHDU(BinTableHDU):
 
         # Remove the EXTNAME card if the value in the table header
         # is the default value of COMPRESSED_IMAGE.
-        if ('EXTNAME' in self._header and
-                self._header['EXTNAME'] == 'COMPRESSED_IMAGE'):
+        if ('EXTNAME' in image_header and
+                image_header['EXTNAME'] == self._default_name):
             del image_header['EXTNAME']
 
         # Look to see if there are any blank cards in the table
@@ -1624,10 +1660,10 @@ class CompImageHDU(BinTableHDU):
         # write to a buffer for compression or something. See ticket #88
         # deal with unsigned integer 16, 32 and 64 data
         old_data = self.data
-        if _is_pseudo_unsigned(self.data.dtype):
+        if _is_pseudo_integer(self.data.dtype):
             # Convert the unsigned array to signed
             self.data = np.array(
-                self.data - _unsigned_zero(self.data.dtype),
+                self.data - _pseudo_zero(self.data.dtype),
                 dtype=f'=i{self.data.dtype.itemsize}')
             should_swap = False
         else:
@@ -1798,7 +1834,7 @@ class CompImageHDU(BinTableHDU):
 
             # Use methods in the superclass to update the header with
             # scale/checksum keywords based on the data type of the image data
-            self._update_uint_scale_keywords()
+            self._update_pseudo_int_scale_keywords()
 
             # Shove the image header and data into a new ImageHDU and use that
             # to compute the image checksum
@@ -1918,7 +1954,7 @@ class CompImageHDU(BinTableHDU):
         if seed == DITHER_SEED_CHECKSUM:
             # Determine the tile dimensions from the ZTILEn keywords
             naxis = self._header['ZNAXIS']
-            tile_dims = [self._header['ZTILE{}'.format(idx + 1)]
+            tile_dims = [self._header[f'ZTILE{idx + 1}']
                          for idx in range(naxis)]
             tile_dims.reverse()
 
