@@ -3,6 +3,7 @@
 import collections
 import copy
 import itertools
+import numbers
 import re
 import warnings
 
@@ -14,7 +15,6 @@ from ._utils import parse_header
 
 from astropy.utils import isiterable
 from astropy.utils.exceptions import AstropyUserWarning
-from astropy.utils.decorators import deprecated_renamed_argument
 
 
 BLOCK_SIZE = 2880  # the FITS block size
@@ -86,7 +86,7 @@ class Header:
 
         Parameters
         ----------
-        cards : A list of `Card` objects, optional
+        cards : list of `Card`, optional
             The cards to initialize the header with. Also allowed are other
             `Header` (or `dict`-like) objects.
 
@@ -189,7 +189,7 @@ class Header:
             comment = None
 
         card = None
-        if isinstance(key, int):
+        if isinstance(key, numbers.Integral):
             card = self._cards[key]
         elif isinstance(key, tuple):
             card = self._cards[self._cardindex(key)]
@@ -387,7 +387,7 @@ class Header:
 
         Returns
         -------
-        header
+        `Header`
             A new `Header` instance.
         """
 
@@ -486,7 +486,7 @@ class Header:
 
         Returns
         -------
-        header
+        `Header`
             A new `Header` instance.
         """
 
@@ -680,7 +680,7 @@ class Header:
 
         Returns
         -------
-        s : str
+        str
             A string representing a FITS header.
         """
 
@@ -700,7 +700,6 @@ class Header:
             s += ' ' * _pad_length(len(s))
         return s
 
-    @deprecated_renamed_argument('clobber', 'overwrite', '2.0')
     def tofile(self, fileobj, sep='', endcard=True, padding=True,
                overwrite=False):
         r"""
@@ -712,9 +711,9 @@ class Header:
 
         Parameters
         ----------
-        fileobj : str, file, optional
+        fileobj : path-like or file-like, optional
             Either the pathname of a file, or an open file handle or file-like
-            object
+            object.
 
         sep : str, optional
             The character or string with which to separate cards.  By default
@@ -733,9 +732,6 @@ class Header:
             If ``True``, overwrite the output file if it exists. Raises an
             ``OSError`` if ``False`` and the output file exists. Default is
             ``False``.
-
-            .. versionchanged:: 1.3
-               ``overwrite`` replaces the deprecated ``clobber`` argument.
         """
 
         close_file = fileobj_closed(fileobj)
@@ -777,7 +773,6 @@ class Header:
 
         return cls.fromfile(fileobj, sep='\n', endcard=endcard, padding=False)
 
-    @deprecated_renamed_argument('clobber', 'overwrite', '2.0')
     def totextfile(self, fileobj, endcard=False, overwrite=False):
         """
         Write the header as text to a file or a file-like object.
@@ -787,9 +782,6 @@ class Header:
             >>> Header.tofile(fileobj, sep='\\n', endcard=False,
             ...               padding=False, overwrite=overwrite)
 
-        .. versionchanged:: 1.3
-           ``overwrite`` replaces the deprecated ``clobber`` argument.
-
         See Also
         --------
         tofile
@@ -818,19 +810,19 @@ class Header:
         Parameters
         ----------
         strip : bool, optional
-           If `True`, strip any headers that are specific to one of the
-           standard HDU types, so that this header can be used in a different
-           HDU.
+            If `True`, strip any headers that are specific to one of the
+            standard HDU types, so that this header can be used in a different
+            HDU.
 
         Returns
         -------
-        header
+        `Header`
             A new :class:`Header` instance.
         """
 
         tmp = self.__class__((copy.copy(card) for card in self._cards))
         if strip:
-            tmp._strip()
+            tmp.strip()
         return tmp
 
     def __copy__(self):
@@ -859,7 +851,7 @@ class Header:
 
         Returns
         -------
-        header
+        `Header`
             A new `Header` instance.
         """
 
@@ -886,7 +878,7 @@ class Header:
 
         Returns
         -------
-        value
+        value: str, number, complex, bool, or ``astropy.io.fits.card.Undefined``
             The value associated with the given keyword, or the default value
             if the keyword is not in the header.
         """
@@ -988,7 +980,7 @@ class Header:
         """Like :meth:`dict.items`."""
 
         for card in self._cards:
-            yield (card.keyword, card.value)
+            yield card.keyword, None if card.value == UNDEFINED else card.value
 
     def keys(self):
         """
@@ -1003,7 +995,7 @@ class Header:
         """Like :meth:`dict.values`."""
 
         for card in self._cards:
-            yield card.value
+            yield None if card.value == UNDEFINED else card.value
 
     def pop(self, *args):
         """
@@ -1012,8 +1004,7 @@ class Header:
         """
 
         if len(args) > 2:
-            raise TypeError('Header.pop expected at most 2 arguments, got '
-                            '{}'.format(len(args)))
+            raise TypeError(f'Header.pop expected at most 2 arguments, got {len(args)}')
 
         if len(args) == 0:
             key = -1
@@ -1300,7 +1291,7 @@ class Header:
 
         temp = self.__class__(cards)
         if strip:
-            temp._strip()
+            temp.strip()
 
         if len(self):
             first = self._cards[0].keyword
@@ -1409,8 +1400,7 @@ class Header:
             if self._cards[idx].keyword.upper() == norm_keyword:
                 return idx
         else:
-            raise ValueError('The keyword {!r} is not in the '
-                             ' header.'.format(keyword))
+            raise ValueError(f'The keyword {keyword!r} is not in the  header.')
 
     def insert(self, key, card, useblanks=True, after=False):
         """
@@ -1439,7 +1429,7 @@ class Header:
             rather than before it.  Defaults to `False`.
         """
 
-        if not isinstance(key, int):
+        if not isinstance(key, numbers.Integral):
             # Don't pass through ints to _cardindex because it will not take
             # kindly to indices outside the existing number of cards in the
             # header, which insert needs to be able to support (for example
@@ -1565,8 +1555,7 @@ class Header:
                 raise ValueError('Regular and commentary keys can not be '
                                  'renamed to each other.')
         elif not force and newkeyword in self:
-            raise ValueError('Intended keyword {} already exists in header.'
-                             .format(newkeyword))
+            raise ValueError(f'Intended keyword {newkeyword} already exists in header.')
 
         idx = self.index(oldkeyword)
         card = self._cards[idx]
@@ -1627,6 +1616,54 @@ class Header:
 
         self._add_commentary('', value, before=before, after=after)
 
+    def strip(self):
+        """
+        Strip cards specific to a certain kind of header.
+
+        Strip cards like ``SIMPLE``, ``BITPIX``, etc. so the rest of
+        the header can be used to reconstruct another kind of header.
+        """
+
+        # TODO: Previously this only deleted some cards specific to an HDU if
+        # _hdutype matched that type.  But it seemed simple enough to just
+        # delete all desired cards anyways, and just ignore the KeyErrors if
+        # they don't exist.
+        # However, it might be desirable to make this extendable somehow--have
+        # a way for HDU classes to specify some headers that are specific only
+        # to that type, and should be removed otherwise.
+
+        naxis = self.get('NAXIS', 0)
+        tfields = self.get('TFIELDS', 0)
+
+        for idx in range(naxis):
+            self.remove('NAXIS' + str(idx + 1), ignore_missing=True)
+
+        for name in ('TFORM', 'TSCAL', 'TZERO', 'TNULL', 'TTYPE',
+                     'TUNIT', 'TDISP', 'TDIM', 'THEAP', 'TBCOL'):
+            for idx in range(tfields):
+                self.remove(name + str(idx + 1), ignore_missing=True)
+
+        for name in ('SIMPLE', 'XTENSION', 'BITPIX', 'NAXIS', 'EXTEND',
+                     'PCOUNT', 'GCOUNT', 'GROUPS', 'BSCALE', 'BZERO',
+                     'TFIELDS'):
+            self.remove(name, ignore_missing=True)
+
+    @property
+    def data_size(self):
+        """
+        Return the size (in bytes) of the data portion following the `Header`.
+        """
+        return _hdr_data_size(self)
+
+    @property
+    def data_size_padded(self):
+        """
+        Return the size (in bytes) of the data portion following the `Header`
+        including padding.
+        """
+        size = self.data_size
+        return size + _pad_length(size)
+
     def _update(self, card):
         """
         The real update code.  If keyword already exists, its value and/or
@@ -1679,7 +1716,7 @@ class Header:
         if isinstance(key, str):
             keyword = key
             n = 0
-        elif isinstance(key, int):
+        elif isinstance(key, numbers.Integral):
             # If < 0, determine the actual index
             if key < 0:
                 key += len(self._cards)
@@ -1690,7 +1727,7 @@ class Header:
             return key
         elif isinstance(key, tuple):
             if (len(key) != 2 or not isinstance(key[0], str) or
-                    not isinstance(key[1], int)):
+                    not isinstance(key[1], numbers.Integral)):
                 raise ValueError(
                     'Tuple indices must be 2-tuples consisting of a '
                     'keyword string and an integer index.')
@@ -1755,7 +1792,7 @@ class Header:
             insertionkey = before
 
         def get_insertion_idx():
-            if not (isinstance(insertionkey, int) and
+            if not (isinstance(insertionkey, numbers.Integral) and
                     insertionkey >= len(self._cards)):
                 idx = self._cardindex(insertionkey)
             else:
@@ -1902,54 +1939,6 @@ class Header:
                 idx += maxlen
         return cards
 
-    def _strip(self):
-        """
-        Strip cards specific to a certain kind of header.
-
-        Strip cards like ``SIMPLE``, ``BITPIX``, etc. so the rest of
-        the header can be used to reconstruct another kind of header.
-        """
-
-        # TODO: Previously this only deleted some cards specific to an HDU if
-        # _hdutype matched that type.  But it seemed simple enough to just
-        # delete all desired cards anyways, and just ignore the KeyErrors if
-        # they don't exist.
-        # However, it might be desirable to make this extendable somehow--have
-        # a way for HDU classes to specify some headers that are specific only
-        # to that type, and should be removed otherwise.
-
-        if 'NAXIS' in self:
-            naxis = self['NAXIS']
-        else:
-            naxis = 0
-
-        if 'TFIELDS' in self:
-            tfields = self['TFIELDS']
-        else:
-            tfields = 0
-
-        for idx in range(naxis):
-            try:
-                del self['NAXIS' + str(idx + 1)]
-            except KeyError:
-                pass
-
-        for name in ('TFORM', 'TSCAL', 'TZERO', 'TNULL', 'TTYPE',
-                     'TUNIT', 'TDISP', 'TDIM', 'THEAP', 'TBCOL'):
-            for idx in range(tfields):
-                try:
-                    del self[name + str(idx + 1)]
-                except KeyError:
-                    pass
-
-        for name in ('SIMPLE', 'XTENSION', 'BITPIX', 'NAXIS', 'EXTEND',
-                     'PCOUNT', 'GCOUNT', 'GROUPS', 'BSCALE', 'BZERO',
-                     'TFIELDS'):
-            try:
-                del self[name]
-            except KeyError:
-                pass
-
     def _add_commentary(self, key, value, before=None, after=None):
         """
         Add a commentary card.
@@ -2051,7 +2040,7 @@ class _BasicHeader(collections.abc.Mapping):
         self._modified = False
 
     def __getitem__(self, key):
-        if isinstance(key, int):
+        if isinstance(key, numbers.Integral):
             key = self._keys[key]
 
         try:
@@ -2071,6 +2060,22 @@ class _BasicHeader(collections.abc.Mapping):
     def index(self, keyword):
         return self._keys.index(keyword)
 
+    @property
+    def data_size(self):
+        """
+        Return the size (in bytes) of the data portion following the `Header`.
+        """
+        return _hdr_data_size(self)
+
+    @property
+    def data_size_padded(self):
+        """
+        Return the size (in bytes) of the data portion following the `Header`
+        including padding.
+        """
+        size = self.data_size
+        return size + _pad_length(size)
+
     @classmethod
     def fromfile(cls, fileobj):
         """The main method to parse a FITS header from a file. The parsing is
@@ -2158,10 +2163,6 @@ class _CardAccessor:
         return False
 
 
-collections.abc.Mapping.register(_CardAccessor)
-collections.abc.Sequence.register(_CardAccessor)
-
-
 class _HeaderComments(_CardAccessor):
     """
     A class used internally by the Header class for the Header.comments
@@ -2245,7 +2246,7 @@ class _HeaderCommentaryCards(_CardAccessor):
             n = self.__class__(self._header, self._keyword)
             n._indices = idx.indices(self._count)
             return n
-        elif not isinstance(idx, int):
+        elif not isinstance(idx, numbers.Integral):
             raise ValueError(f'{self._keyword} index must be an integer')
 
         idx = list(range(*self._indices))[idx]
@@ -2306,5 +2307,19 @@ def _check_padding(header_str, block_size, is_eof, check_block_size=True):
         # now, but maybe it shouldn't?
         actual_len = len(header_str) - block_size + BLOCK_SIZE
         # TODO: Pass this error to validation framework
-        raise ValueError('Header size is not multiple of {}: {}'
-                         .format(BLOCK_SIZE, actual_len))
+        raise ValueError(f'Header size is not multiple of {BLOCK_SIZE}: {actual_len}')
+
+
+def _hdr_data_size(header):
+    """Calculate the data size (in bytes) following the given `Header`"""
+    size = 0
+    naxis = header.get('NAXIS', 0)
+    if naxis > 0:
+        size = 1
+        for idx in range(naxis):
+            size = size * header['NAXIS' + str(idx + 1)]
+        bitpix = header['BITPIX']
+        gcount = header.get('GCOUNT', 1)
+        pcount = header.get('PCOUNT', 0)
+        size = abs(bitpix) * gcount * (pcount + size) // 8
+    return size
