@@ -10,10 +10,11 @@ import subprocess
 import pytest
 import numpy as np
 
+from astropy.io.fits.hdu.base import _ValidHDU, _NonstandardHDU
 from astropy.io.fits.verify import VerifyError, VerifyWarning
 from astropy.io import fits
-from astropy.tests.helper import raises, catch_warnings, ignore_warnings
-from astropy.utils.exceptions import AstropyUserWarning, AstropyDeprecationWarning
+from astropy.utils.exceptions import AstropyUserWarning
+from astropy.utils.data import get_pkg_data_filenames
 
 from . import FitsTestCase
 
@@ -159,7 +160,6 @@ class TestHDUListFunctions(FitsTestCase):
 
         assert fits.info(self.temp('test-append.fits'), output=False) == info
 
-    @raises(ValueError)
     def test_append_groupshdu_to_non_empty_list(self):
         """Tests appending a Simple GroupsHDU to an empty HDUList."""
 
@@ -167,7 +167,8 @@ class TestHDUListFunctions(FitsTestCase):
         hdu = fits.PrimaryHDU(np.arange(100, dtype=np.int32))
         hdul.append(hdu)
         hdu = fits.GroupsHDU()
-        hdul.append(hdu)
+        with pytest.raises(ValueError):
+            hdul.append(hdu)
 
     def test_insert_primary_to_empty_list(self):
         """Tests inserting a Simple PrimaryHDU to an empty HDUList."""
@@ -285,7 +286,6 @@ class TestHDUListFunctions(FitsTestCase):
 
         assert fits.info(self.temp('test-insert.fits'), output=False) == info
 
-    @raises(ValueError)
     def test_insert_groupshdu_to_begin_of_hdulist_with_groupshdu(self):
         """
         Tests inserting a Simple GroupsHDU to the beginning of an HDUList
@@ -295,7 +295,8 @@ class TestHDUListFunctions(FitsTestCase):
         hdul = fits.HDUList()
         hdu = fits.GroupsHDU()
         hdul.insert(0, hdu)
-        hdul.insert(0, hdu)
+        with pytest.raises(ValueError):
+            hdul.insert(0, hdu)
 
     def test_insert_extension_to_primary_in_non_empty_list(self):
         # Tests inserting a Simple ExtensionHDU to a non-empty HDUList.
@@ -458,10 +459,9 @@ class TestHDUListFunctions(FitsTestCase):
         oldmtime = os.stat(self.data('test0.fits')).st_mtime
         with fits.open(self.data('test0.fits')) as hdul:
             hdul[0].header['FOO'] = 'BAR'
-            with catch_warnings(AstropyUserWarning) as w:
+            with pytest.warns(AstropyUserWarning, match='mode is not supported') as w:
                 hdul.flush()
             assert len(w) == 1
-            assert 'mode is not supported' in str(w[0].message)
             assert oldmtime == os.stat(self.data('test0.fits')).st_mtime
 
     def test_fix_extend_keyword(self):
@@ -569,6 +569,7 @@ class TestHDUListFunctions(FitsTestCase):
                 fobj.info()
         assert len(w) == 1
 
+    @pytest.mark.filterwarnings('ignore:Unexpected extra padding')
     def test_open_file_with_end_padding(self):
         """Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/106
 
@@ -583,9 +584,8 @@ class TestHDUListFunctions(FitsTestCase):
         with open(self.temp('temp.fits'), 'ab') as f:
             f.seek(0, os.SEEK_END)
             f.write(b'\0' * 2880)
-        with ignore_warnings():
-            assert info == fits.info(self.temp('temp.fits'), output=False,
-                                     do_not_scale_image_data=True)
+        assert info == fits.info(self.temp('temp.fits'), output=False,
+                                 do_not_scale_image_data=True)
 
     def test_open_file_with_bad_header_padding(self):
         """
@@ -606,11 +606,9 @@ class TestHDUListFunctions(FitsTestCase):
             f.seek(padding_start)
             f.write('\0'.encode('ascii') * padding_len)
 
-        with catch_warnings(AstropyUserWarning) as w:
+        with pytest.warns(AstropyUserWarning, match='contains null bytes instead of spaces') as w:
             with fits.open(self.temp('temp.fits')) as hdul:
                 assert (hdul[0].data == a).all()
-        assert ('contains null bytes instead of spaces' in
-                str(w[0].message))
         assert len(w) == 1
         assert len(hdul) == 1
         assert str(hdul[0].header) == str(hdu.header)
@@ -708,7 +706,6 @@ class TestHDUListFunctions(FitsTestCase):
             assert (hdul[1].data == data2).all()
             assert (hdul[2].data == data2).all()
 
-    @ignore_warnings()
     def test_hdul_fromstring(self):
         """
         Test creating the HDUList structure in memory from a string containing
@@ -749,13 +746,14 @@ class TestHDUListFunctions(FitsTestCase):
                         np.testing.assert_array_equal(hdul[idx].data,
                                                       hdul2[idx].data)
 
-        for filename in glob.glob(os.path.join(self.data_dir, '*.fits')):
-            if sys.platform == 'win32' and filename == 'zerowidth.fits':
+        for filename in get_pkg_data_filenames('data', pattern='*.fits'):
+            if sys.platform == 'win32' and filename.endswith('zerowidth.fits'):
                 # Running this test on this file causes a crash in some
                 # versions of Numpy on Windows.  See ticket:
                 # https://aeon.stsci.edu/ssb/trac/pyfits/ticket/174
                 continue
-            elif filename.endswith('variable_length_table.fits'):
+            elif filename.endswith(('variable_length_table.fits',
+                                    'theap-gap.fits')):
                 # Comparing variable length arrays is non-trivial and thus
                 # skipped at this point.
                 # TODO: That's probably possible, so one could make it work.
@@ -765,6 +763,7 @@ class TestHDUListFunctions(FitsTestCase):
         # Test that creating an HDUList from something silly raises a TypeError
         pytest.raises(TypeError, fits.HDUList.fromstring, ['a', 'b', 'c'])
 
+    @pytest.mark.filterwarnings('ignore:Saving a backup')
     def test_save_backup(self):
         """Test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/121
 
@@ -773,13 +772,14 @@ class TestHDUListFunctions(FitsTestCase):
 
         self.copy_file('scale.fits')
 
-        with ignore_warnings():
-            with fits.open(self.temp('scale.fits'), mode='update',
-                           save_backup=True) as hdul:
-                # Make some changes to the original file to force its header
-                # and data to be rewritten
-                hdul[0].header['TEST'] = 'TEST'
-                hdul[0].data[0] = 0
+        with fits.open(self.temp('scale.fits'), mode='update',
+                       save_backup=True) as hdul:
+            # Make some changes to the original file to force its header
+            # and data to be rewritten
+            hdul[0].header['TEST'] = 'TEST'
+            # This emits warning that needs to be ignored at the
+            # pytest.mark.filterwarnings level.
+            hdul[0].data[0] = 0
 
         assert os.path.exists(self.temp('scale.fits.bak'))
         with fits.open(self.data('scale.fits'),
@@ -789,12 +789,11 @@ class TestHDUListFunctions(FitsTestCase):
                 assert hdul1[0].header == hdul2[0].header
                 assert (hdul1[0].data == hdul2[0].data).all()
 
-        with ignore_warnings():
-            with fits.open(self.temp('scale.fits'), mode='update',
-                           save_backup=True) as hdul:
-                # One more time to see if multiple backups are made
-                hdul[0].header['TEST2'] = 'TEST'
-                hdul[0].data[0] = 1
+        with fits.open(self.temp('scale.fits'), mode='update',
+                       save_backup=True) as hdul:
+            # One more time to see if multiple backups are made
+            hdul[0].header['TEST2'] = 'TEST'
+            hdul[0].data[0] = 1
 
         assert os.path.exists(self.temp('scale.fits.bak'))
         assert os.path.exists(self.temp('scale.fits.bak.1'))
@@ -825,14 +824,13 @@ class TestHDUListFunctions(FitsTestCase):
             with fits.open(self.temp('test_a.fits')) as hdul_a:
                 assert np.all(hdul_a[0].data == arr_b)
 
-        with ignore_warnings():
-            test(True, True)
+        test(True, True)
 
-            # Repeat the same test but this time don't mmap A
-            test(False, True)
+        # Repeat the same test but this time don't mmap A
+        test(False, True)
 
-            # Finally, without mmaping B
-            test(True, False)
+        # Finally, without mmaping B
+        test(True, False)
 
     def test_replace_mmap_data_2(self):
         """Regression test for
@@ -865,14 +863,13 @@ class TestHDUListFunctions(FitsTestCase):
                 assert 'a' not in hdul_a[1].columns.names
                 assert np.all(hdul_a[1].data['b'] == arr_b)
 
-        with ignore_warnings():
-            test(True, True)
+        test(True, True)
 
-            # Repeat the same test but this time don't mmap A
-            test(False, True)
+        # Repeat the same test but this time don't mmap A
+        test(False, True)
 
-            # Finally, without mmaping B
-            test(True, False)
+        # Finally, without mmaping B
+        test(True, False)
 
     def test_extname_in_hdulist(self):
         """
@@ -894,16 +891,10 @@ class TestHDUListFunctions(FitsTestCase):
             assert hdulist[0] in hdulist
             assert fits.ImageHDU() not in hdulist
 
-    def test_overwrite_vs_clobber(self):
+    def test_overwrite(self):
         hdulist = fits.HDUList([fits.PrimaryHDU()])
         hdulist.writeto(self.temp('test_overwrite.fits'))
         hdulist.writeto(self.temp('test_overwrite.fits'), overwrite=True)
-        with catch_warnings(AstropyDeprecationWarning) as warning_lines:
-            hdulist.writeto(self.temp('test_overwrite.fits'), clobber=True)
-            assert warning_lines[0].category == AstropyDeprecationWarning
-            assert (str(warning_lines[0].message) == '"clobber" was '
-                    'deprecated in version 2.0 and will be removed in a '
-                    'future version. Use argument "overwrite" instead.')
 
     def test_invalid_hdu_key_in_contains(self):
         """
@@ -957,6 +948,111 @@ class TestHDUListFunctions(FitsTestCase):
         assert len(read_exts) == 2
         f.close()
 
+    def test_read_non_standard_hdu(self):
+        filename = self.temp('bad-fits.fits')
+        hdu = fits.PrimaryHDU()
+        hdu.header['FOO'] = 'BAR'
+        buf = io.BytesIO()
+        hdu.writeto(buf)
+        buf.seek(0)
+        hdustr = buf.read()
+        hdustr = hdustr.replace(b'SIMPLE  =                    T',
+                                b'SIMPLE  =                    F')
+        with open(filename, mode='wb') as f:
+            f.write(hdustr)
+
+        with fits.open(filename) as hdul:
+            assert isinstance(hdul[0], _NonstandardHDU)
+            assert hdul[0].header['FOO'] == 'BAR'
+
+    def test_proper_error_raised_on_non_fits_file(self):
+        filename = self.temp('not-fits.fits')
+        with open(filename, mode='w', encoding='utf=8') as f:
+            f.write('Not a FITS file')
+
+        match = ('No SIMPLE card found, this file '
+                 'does not appear to be a valid FITS file')
+
+        # This should raise an OSError because there is no end card.
+        with pytest.raises(OSError, match=match):
+            fits.open(filename)
+
+        with pytest.raises(OSError, match=match):
+            fits.open(filename, mode='append')
+
+        with pytest.raises(OSError, match=match):
+            fits.open(filename, mode='update')
+
+    def test_proper_error_raised_on_invalid_fits_file(self):
+        filename = self.temp('bad-fits.fits')
+        hdu = fits.PrimaryHDU()
+        hdu.header['FOO'] = 'BAR'
+        buf = io.BytesIO()
+        hdu.writeto(buf)
+        # write 80 additional bytes so the block will have the correct size
+        buf.write(b' '*80)
+        buf.seek(0)
+        buf.seek(80)  # now remove the SIMPLE card
+        with open(filename, mode='wb') as f:
+            f.write(buf.read())
+
+        match = ('No SIMPLE card found, this file '
+                 'does not appear to be a valid FITS file')
+
+        # This should raise an OSError because there is no end card.
+        with pytest.raises(OSError, match=match):
+            fits.open(filename)
+
+        with pytest.raises(OSError, match=match):
+            fits.open(filename, mode='append')
+
+        with pytest.raises(OSError, match=match):
+            fits.open(filename, mode='update')
+
+        with fits.open(filename, ignore_missing_simple=True) as hdul:
+            assert isinstance(hdul[0], _ValidHDU)
+            assert hdul[0].header['FOO'] == 'BAR'
+
+    def test_warning_raised_on_non_standard_simple_card(self):
+        filename = self.temp('bad-fits.fits')
+        hdu = fits.PrimaryHDU()
+        hdu.header['FOO'] = 'BAR'
+        buf = io.BytesIO()
+        hdu.writeto(buf)
+
+        # change the simple card format
+        buf.seek(0)
+        buf.write(b'SIMPLE  = T                   ')
+        buf.seek(0)
+        with open(filename, mode='wb') as f:
+            f.write(buf.read())
+
+        match = ("Found a SIMPLE card but its format doesn't"
+                 " respect the FITS Standard")
+
+        with pytest.warns(VerifyWarning, match=match):
+            fits.open(filename)
+
+        with pytest.warns(VerifyWarning, match=match):
+            fits.open(filename, mode='append')
+
+        with pytest.warns(VerifyWarning, match=match):
+            fits.open(filename, mode='update')
+
+        with fits.open(filename, ignore_missing_simple=True) as hdul:
+            assert isinstance(hdul[0], _ValidHDU)
+            assert hdul[0].header['FOO'] == 'BAR'
+
+        # change the simple card format
+        buf.seek(0)
+        buf.write(b'SIMPLE  =                           T / This is a FITS file')
+        buf.seek(0)
+        with open(filename, mode='wb') as f:
+            f.write(buf.read())
+
+        with pytest.warns(VerifyWarning, match=match):
+            fits.open(filename)
+
     def test_proper_error_raised_on_non_fits_file_with_unicode(self):
         """
         Regression test for https://github.com/astropy/astropy/issues/5594
@@ -965,16 +1061,14 @@ class TestHDUListFunctions(FitsTestCase):
         with unicode content that is not actually a FITS file. See:
         https://github.com/astropy/astropy/issues/5594#issuecomment-266583218
         """
-        import codecs
         filename = self.temp('not-fits-with-unicode.fits')
-        with codecs.open(filename, mode='w', encoding='utf=8') as f:
+        with open(filename, mode='w', encoding='utf=8') as f:
             f.write('Ce\xe7i ne marche pas')
 
         # This should raise an OSError because there is no end card.
-        with pytest.raises(OSError):
-            with pytest.warns(AstropyUserWarning, match=r'non-ASCII characters '
-                              r'are present in the FITS file header'):
-                fits.open(filename)
+        with pytest.raises(OSError, match='No SIMPLE card found, this file '
+                           'does not appear to be a valid FITS file'):
+            fits.open(filename)
 
     def test_no_resource_warning_raised_on_non_fits_file(self):
         """
@@ -999,30 +1093,26 @@ class TestHDUListFunctions(FitsTestCase):
         #
         # Explicit tests are added to make sure the file handle is not
         # closed when passed in to fits.open. In this case the ResourceWarning
-        # was not raised, but a check is still included.
-        #
-        with catch_warnings(ResourceWarning) as ws:
-
-            # Make sure that files opened by the user are not closed
-            with open(filename, mode='rb') as f:
-                with pytest.raises(OSError):
-                    fits.open(f, ignore_missing_end=False)
-
-                assert not f.closed
-
-            with open(filename, mode='rb') as f:
-                with pytest.raises(OSError):
-                    fits.open(f, ignore_missing_end=True)
-
-                assert not f.closed
+        # was not raised.
 
+        # Make sure that files opened by the user are not closed
+        with open(filename, mode='rb') as f:
             with pytest.raises(OSError):
-                fits.open(filename, ignore_missing_end=False)
+                fits.open(f, ignore_missing_end=False)
 
-            with pytest.raises(OSError):
-                fits.open(filename, ignore_missing_end=True)
+            assert not f.closed
 
-        assert len(ws) == 0
+        with open(filename, mode='rb') as f:
+            with pytest.raises(OSError), pytest.warns(VerifyWarning):
+                fits.open(f, ignore_missing_end=True)
+
+            assert not f.closed
+
+        with pytest.raises(OSError):
+            fits.open(filename, ignore_missing_end=False)
+
+        with pytest.raises(OSError), pytest.warns(VerifyWarning):
+            fits.open(filename, ignore_missing_end=True)
 
     def test_pop_with_lazy_load(self):
         filename = self.data('checksum.fits')
