@@ -4,26 +4,20 @@ import math
 import os
 import re
 import time
-import warnings
 
 import pytest
 import numpy as np
 from numpy.testing import assert_equal
 
 from astropy.io import fits
-from astropy.tests.helper import catch_warnings, ignore_warnings
 from astropy.io.fits.hdu.compressed import SUBTRACTIVE_DITHER_1, DITHER_SEED_CHECKSUM
+from astropy.utils.exceptions import AstropyUserWarning
+from astropy.utils.data import get_pkg_data_filename
+from astropy.utils.compat.optional_deps import HAS_SCIPY  # noqa
 from .test_table import comparerecords
 
 from . import FitsTestCase
 
-try:
-    import scipy  # noqa
-except ImportError:
-    HAS_SCIPY = False
-else:
-    HAS_SCIPY = True
-
 
 class TestImageFunctions(FitsTestCase):
     def test_constructor_name_arg(self):
@@ -167,18 +161,18 @@ class TestImageFunctions(FitsTestCase):
 
     def test_fortran_array(self):
         # Test that files are being correctly written+read for "C" and "F" order arrays
-        a = np.arange(21).reshape(3,7)
+        a = np.arange(21).reshape(3, 7)
         b = np.asfortranarray(a)
 
         afits = self.temp('a_str.fits')
         bfits = self.temp('b_str.fits')
-        # writting to str specified files
+        # writing to str specified files
         fits.PrimaryHDU(data=a).writeto(afits)
         fits.PrimaryHDU(data=b).writeto(bfits)
         np.testing.assert_array_equal(fits.getdata(afits), a)
         np.testing.assert_array_equal(fits.getdata(bfits), a)
 
-        # writting to fileobjs
+        # writing to fileobjs
         aafits = self.temp('a_fileobj.fits')
         bbfits = self.temp('b_fileobj.fits')
         with open(aafits, mode='wb') as fd:
@@ -190,10 +184,10 @@ class TestImageFunctions(FitsTestCase):
 
     def test_fortran_array_non_contiguous(self):
         # Test that files are being correctly written+read for 'C' and 'F' order arrays
-        a = np.arange(105).reshape(3,5,7)
+        a = np.arange(105).reshape(3, 5, 7)
         b = np.asfortranarray(a)
 
-        # writting to str specified files
+        # writing to str specified files
         afits = self.temp('a_str_slice.fits')
         bfits = self.temp('b_str_slice.fits')
         fits.PrimaryHDU(data=a[::2, ::2]).writeto(afits)
@@ -201,7 +195,7 @@ class TestImageFunctions(FitsTestCase):
         np.testing.assert_array_equal(fits.getdata(afits), a[::2, ::2])
         np.testing.assert_array_equal(fits.getdata(bfits), a[::2, ::2])
 
-        # writting to fileobjs
+        # writing to fileobjs
         aafits = self.temp('a_fileobj_slice.fits')
         bbfits = self.temp('b_fileobj_slice.fits')
         with open(aafits, mode='wb') as fd:
@@ -386,18 +380,14 @@ class TestImageFunctions(FitsTestCase):
         # make a defect HDUList first
         x = fits.ImageHDU()
         hdu = fits.HDUList(x)  # HDUList can take a list or one single HDU
-        with catch_warnings() as w:
+        with pytest.warns(AstropyUserWarning, match=r"HDUList's 0th element is not a primary HDU\.") as w:
             hdu.verify()
-        text = "HDUList's 0th element is not a primary HDU."
         assert len(w) == 3
-        assert text in str(w[1].message)
 
-        with catch_warnings() as w:
+        with pytest.warns(AstropyUserWarning, match=r"HDUList's 0th element is not a primary HDU\.  "
+                          r"Fixed by inserting one as 0th HDU\.") as w:
             hdu.writeto(self.temp('test_new2.fits'), 'fix')
-        text = ("HDUList's 0th element is not a primary HDU.  "
-                "Fixed by inserting one as 0th HDU.")
         assert len(w) == 3
-        assert text in str(w[1].message)
 
     def test_section(self):
         # section testing
@@ -640,7 +630,7 @@ class TestImageFunctions(FitsTestCase):
         Regression test for https://github.com/astropy/astropy/issues/2305
 
         This ensures that an HDU containing unsigned integer data always has
-        the apppriate BZERO value in its header.
+        the appropriate BZERO value in its header.
         """
 
         for int_size in (16, 32, 64):
@@ -757,22 +747,21 @@ class TestImageFunctions(FitsTestCase):
         hdu = fits.PrimaryHDU(data=arr)
         hdu.header['BLANK'] = 2
 
-        with catch_warnings() as w:
+        with pytest.warns(AstropyUserWarning, match="Invalid 'BLANK' keyword in header") as w:
             hdu.writeto(self.temp('test_new.fits'))
-            # Allow the HDU to be written, but there should be a warning
-            # when writing a header with BLANK when then data is not
-            # int
-            assert len(w) == 1
-            assert "Invalid 'BLANK' keyword in header" in str(w[0].message)
+        # Allow the HDU to be written, but there should be a warning
+        # when writing a header with BLANK when then data is not
+        # int
+        assert len(w) == 1
 
         # Should also get a warning when opening the file, and the BLANK
         # value should not be applied
-        with catch_warnings() as w:
+        with pytest.warns(AstropyUserWarning, match="Invalid 'BLANK' keyword in header") as w:
             with fits.open(self.temp('test_new.fits')) as h:
-                assert len(w) == 1
-                assert "Invalid 'BLANK' keyword in header" in str(w[0].message)
                 assert np.all(arr == h[0].data)
+        assert len(w) == 1
 
+    @pytest.mark.filterwarnings("ignore:Invalid 'BLANK' keyword in header")
     def test_scale_back_with_blanks(self):
         """
         Test that when auto-rescaling integer data with "blank" values (where
@@ -810,17 +799,17 @@ class TestImageFunctions(FitsTestCase):
 
         # Now reopen the newly written file.  It should not have a 'BLANK'
         # keyword
-        with catch_warnings() as w:
-            with fits.open(self.temp('test2.fits')) as hdul2:
-                assert len(w) == 0
-                assert 'BLANK' not in hdul2[0].header
-                data = hdul2[0].data
-                assert np.isnan(data[0])
+        with fits.open(self.temp('test2.fits')) as hdul2:
+            assert 'BLANK' not in hdul2[0].header
+            data = hdul2[0].data
+            assert np.isnan(data[0])
 
         # Finally, test that scale_back keeps the BLANKs correctly
         with fits.open(filename, scale_back=True,
                        mode='update') as hdul3:
             data = hdul3[0].data
+            # This emits warning that pytest cannot catch properly, so we
+            # catch it with pytest.mark.filterwarnings above.
             assert np.isnan(data[0])
 
         with fits.open(filename,
@@ -850,8 +839,7 @@ class TestImageFunctions(FitsTestCase):
 
         hdul = fits.open(self.data('fixed-1890.fits'))
         orig_data = hdul[0].data
-        with ignore_warnings():
-            hdul.writeto(self.temp('test_new.fits'), overwrite=True)
+        hdul.writeto(self.temp('test_new.fits'), overwrite=True)
         hdul.close()
         hdul = fits.open(self.temp('test_new.fits'))
         assert (hdul[0].data == orig_data).all()
@@ -861,8 +849,7 @@ class TestImageFunctions(FitsTestCase):
         # back out--this is the case that failed in
         # https://aeon.stsci.edu/ssb/trac/pyfits/ticket/84
         hdul = fits.open(self.data('fixed-1890.fits'))
-        with ignore_warnings():
-            hdul.writeto(self.temp('test_new.fits'), overwrite=True)
+        hdul.writeto(self.temp('test_new.fits'), overwrite=True)
         hdul.close()
         hdul = fits.open(self.temp('test_new.fits'))
         assert (hdul[0].data == orig_data).all()
@@ -881,7 +868,6 @@ class TestImageFunctions(FitsTestCase):
         hdul.close()
         hdul = fits.open(self.temp('test_new.fits'))
         assert (hdul[0].data == orig_data).all()
-        hdul = fits.open(self.temp('test_new.fits'))
         hdul.close()
 
     def test_image_update_header(self):
@@ -943,7 +929,7 @@ class TestImageFunctions(FitsTestCase):
         assert (orig_data == hdul[0].data).all()
 
         # Try reshaping the data, then closing and reopening the file; let's
-        # see if all the changes are preseved properly
+        # see if all the changes are preserved properly
         hdul[0].data.shape = (42, 10)
         hdul.close()
 
@@ -1013,7 +999,7 @@ class TestImageFunctions(FitsTestCase):
                           r"'BLANK' keyword in header: 'nan'"):
             hdu.writeto(self.temp('test.fits'))
 
-        with catch_warnings() as w:
+        with pytest.warns(AstropyUserWarning) as w:
             with fits.open(self.temp('test.fits')) as hdul:
                 assert np.all(hdul[0].data == data)
 
@@ -1225,15 +1211,12 @@ class TestCompressedImage(FitsTestCase):
             assert np.all(hdul[1].data == array)
 
     def test_disable_image_compression(self):
-        with catch_warnings():
-            # No warnings should be displayed in this case
-            warnings.simplefilter('error')
-            with fits.open(self.data('comp.fits'),
-                           disable_image_compression=True) as hdul:
-                # The compressed image HDU should show up as a BinTableHDU, but
-                # *not* a CompImageHDU
-                assert isinstance(hdul[1], fits.BinTableHDU)
-                assert not isinstance(hdul[1], fits.CompImageHDU)
+        with fits.open(self.data('comp.fits'),
+                       disable_image_compression=True) as hdul:
+            # The compressed image HDU should show up as a BinTableHDU, but
+            # *not* a CompImageHDU
+            assert isinstance(hdul[1], fits.BinTableHDU)
+            assert not isinstance(hdul[1], fits.CompImageHDU)
 
         with fits.open(self.data('comp.fits')) as hdul:
             assert isinstance(hdul[1], fits.CompImageHDU)
@@ -1258,6 +1241,7 @@ class TestCompressedImage(FitsTestCase):
         # opening and closing it.
         assert mtime == os.stat(self.temp('comp.fits')).st_mtime
 
+    @pytest.mark.slow
     def test_open_scaled_in_update_mode_compressed(self):
         """
         Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/88 2
@@ -1300,7 +1284,7 @@ class TestCompressedImage(FitsTestCase):
         assert 'BSCALE' not in hdul[1].header
 
         # Try reshaping the data, then closing and reopening the file; let's
-        # see if all the changes are preseved properly
+        # see if all the changes are preserved properly
         hdul[1].data.shape = (42, 10)
         hdul.close()
 
@@ -1338,8 +1322,7 @@ class TestCompressedImage(FitsTestCase):
 
         hdul = fits.open(self.temp('fixed-1890-z.fits'))
         orig_data = hdul[1].data
-        with ignore_warnings():
-            hdul.writeto(self.temp('test_new.fits'), overwrite=True)
+        hdul.writeto(self.temp('test_new.fits'), overwrite=True)
         hdul.close()
         hdul = fits.open(self.temp('test_new.fits'))
         assert (hdul[1].data == orig_data).all()
@@ -1349,8 +1332,7 @@ class TestCompressedImage(FitsTestCase):
         # back out--this is the case that failed in
         # https://aeon.stsci.edu/ssb/trac/pyfits/ticket/84
         hdul = fits.open(self.temp('fixed-1890-z.fits'))
-        with ignore_warnings():
-            hdul.writeto(self.temp('test_new.fits'), overwrite=True)
+        hdul.writeto(self.temp('test_new.fits'), overwrite=True)
         hdul.close()
         hdul = fits.open(self.temp('test_new.fits'))
         assert (hdul[1].data == orig_data).all()
@@ -1369,7 +1351,6 @@ class TestCompressedImage(FitsTestCase):
         hdul.close()
         hdul = fits.open(self.temp('test_new.fits'))
         assert (hdul[1].data == orig_data).all()
-        hdul = fits.open(self.temp('test_new.fits'))
         hdul.close()
 
     def test_scale_back_compressed(self):
@@ -1416,7 +1397,7 @@ class TestCompressedImage(FitsTestCase):
     def test_lossless_gzip_compression(self):
         """Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/198"""
 
-        rng = np.random.RandomState(seed=42)
+        rng = np.random.default_rng(42)
         noise = rng.normal(size=(20, 20))
 
         chdu1 = fits.CompImageHDU(data=noise, compression_type='GZIP_1')
@@ -1432,8 +1413,7 @@ class TestCompressedImage(FitsTestCase):
 
         chdu2 = fits.CompImageHDU(data=noise, compression_type='GZIP_1',
                                   quantize_level=0.0)  # No quantization
-        with ignore_warnings():
-            chdu2.writeto(self.temp('test.fits'), overwrite=True)
+        chdu2.writeto(self.temp('test.fits'), overwrite=True)
 
         with fits.open(self.temp('test.fits')) as h:
             assert (noise == h[1].data).all()
@@ -1527,12 +1507,12 @@ class TestCompressedImage(FitsTestCase):
         """
 
         def test_set_keyword(hdr, keyword, value):
-            with catch_warnings() as w:
+            with pytest.warns(UserWarning) as w:
                 hdr[keyword] = value
-                assert len(w) == 1
-                assert str(w[0].message).startswith(
-                        f"Keyword {keyword!r} is reserved")
-                assert keyword not in hdr
+            assert len(w) == 1
+            assert str(w[0].message).startswith(
+                f"Keyword {keyword!r} is reserved")
+            assert keyword not in hdr
 
         with fits.open(self.data('comp.fits')) as hdul:
             hdr = hdul[1].header
@@ -1545,10 +1525,10 @@ class TestCompressedImage(FitsTestCase):
         with fits.open(self.data('comp.fits')) as hdul:
             imghdr = hdul[1].header
             tblhdr = hdul[1]._header
-            with catch_warnings() as w:
+            with pytest.warns(UserWarning, match="Keyword 'TFIELDS' is reserved") as w:
                 imghdr.append('TFIELDS')
-                assert len(w) == 1
-                assert 'TFIELDS' not in imghdr
+            assert len(w) == 1
+            assert 'TFIELDS' not in imghdr
 
             imghdr.append(('FOO', 'bar', 'qux'), end=True)
             assert 'FOO' in imghdr
@@ -1565,7 +1545,7 @@ class TestCompressedImage(FitsTestCase):
 
     def test_compression_header_append2(self):
         """
-        Regresion test for issue https://github.com/astropy/astropy/issues/5827
+        Regression test for issue https://github.com/astropy/astropy/issues/5827
         """
         with fits.open(self.data('comp.fits')) as hdul:
             header = hdul[1].header
@@ -1582,11 +1562,11 @@ class TestCompressedImage(FitsTestCase):
             imghdr = hdul[1].header
             tblhdr = hdul[1]._header
             # First try inserting a restricted keyword
-            with catch_warnings() as w:
+            with pytest.warns(UserWarning, match="Keyword 'TFIELDS' is reserved") as w:
                 imghdr.insert(1000, 'TFIELDS')
-                assert len(w) == 1
-                assert 'TFIELDS' not in imghdr
-                assert tblhdr.count('TFIELDS') == 1
+            assert len(w) == 1
+            assert 'TFIELDS' not in imghdr
+            assert tblhdr.count('TFIELDS') == 1
 
             # First try keyword-relative insert
             imghdr.insert('TELESCOP', ('OBSERVER', 'Phil Plait'))
@@ -1609,12 +1589,12 @@ class TestCompressedImage(FitsTestCase):
             imghdr = hdul[1].header
             tblhdr = hdul[1]._header
 
-            with catch_warnings() as w:
+            with pytest.warns(UserWarning, match="Keyword 'ZBITPIX' is reserved ") as w:
                 imghdr.set('ZBITPIX', 77, 'asdf', after='XTENSION')
-                assert len(w) == 1
-                assert 'ZBITPIX' not in imghdr
-                assert tblhdr.count('ZBITPIX') == 1
-                assert tblhdr['ZBITPIX'] != 77
+            assert len(w) == 1
+            assert 'ZBITPIX' not in imghdr
+            assert tblhdr.count('ZBITPIX') == 1
+            assert tblhdr['ZBITPIX'] != 77
 
             # Move GCOUNT before PCOUNT (not that there's any reason you'd
             # *want* to do that, but it's just a test...)
@@ -1745,6 +1725,81 @@ class TestCompressedImage(FitsTestCase):
         with pytest.raises(KeyError):
             comp_hdu.compressed_data
 
+    def test_compressed_header_double_extname(self):
+        """Test that a double EXTNAME with one default value does not
+        mask the non-default value."""
+        with fits.open(self.data('double_ext.fits')) as hdul:
+            hdu = hdul[1]
+
+            # Raw header has 2 EXTNAME entries
+            indices = hdu._header._keyword_indices['EXTNAME']
+            assert len(indices) == 2
+
+            # The non-default name should be returned.
+            assert hdu.name == 'ccd00'
+            assert 'EXTNAME' in hdu.header
+            assert hdu.name == hdu.header['EXTNAME']
+
+            # There should be 1 non-default EXTNAME entries.
+            indices = hdu.header._keyword_indices['EXTNAME']
+            assert len(indices) == 1
+
+            # Test header sync from property set.
+            new_name = 'NEW_NAME'
+            hdu.name = new_name
+            assert hdu.name == new_name
+            assert hdu.header['EXTNAME'] == new_name
+            assert hdu._header['EXTNAME'] == new_name
+            assert hdu._image_header['EXTNAME'] == new_name
+
+            # Check that setting the header will change the name property.
+            hdu.header['EXTNAME'] = 'NEW2'
+            assert hdu.name == 'NEW2'
+
+            hdul.writeto(self.temp('tmp.fits'), overwrite=True)
+            with fits.open(self.temp('tmp.fits')) as hdul1:
+                hdu1 = hdul1[1]
+                assert len(hdu1._header._keyword_indices['EXTNAME']) == 1
+                assert hdu1.name == 'NEW2'
+
+            # Check that deleting EXTNAME will and setting the name will
+            # work properly.
+            del hdu.header['EXTNAME']
+            hdu.name = 'RE-ADDED'
+            assert hdu.name == 'RE-ADDED'
+
+            with pytest.raises(TypeError):
+                hdu.name = 42
+
+    def test_compressed_header_extname(self):
+        """Test consistent EXTNAME / hdu name interaction."""
+        name = 'FOO'
+        hdu = fits.CompImageHDU(data=np.arange(10), name=name)
+        assert hdu._header['EXTNAME'] == name
+        assert hdu.header['EXTNAME'] == name
+        assert hdu.name == name
+
+        name = 'BAR'
+        hdu.name = name
+        assert hdu._header['EXTNAME'] == name
+        assert hdu.header['EXTNAME'] == name
+        assert hdu.name == name
+
+        assert len(hdu._header._keyword_indices['EXTNAME']) == 1
+
+    def test_compressed_header_minimal(self):
+        """
+        Regression test for https://github.com/astropy/astropy/issues/11694
+
+        Tests that CompImageHDU can be initialized with a Header that
+        contains few or no cards, and doesn't require specific cards
+        such as 'BITPIX' or 'NAXIS'.
+        """
+        fits.CompImageHDU(data=np.arange(10), header=fits.Header())
+        header = fits.Header({'HELLO': 'world'})
+        hdu = fits.CompImageHDU(data=np.arange(10), header=header)
+        assert hdu.header['HELLO'] == 'world'
+
     @pytest.mark.parametrize(
         ('keyword', 'dtype', 'expected'),
         [('BSCALE', np.uint8, np.float32), ('BSCALE', np.int16, np.float32),
@@ -1869,8 +1924,7 @@ def test_bzero_implicit_casting_compressed():
     # case BZERO should be 32768. But if the keyword is stored as 32768.0, then
     # it was possible to trigger the implicit casting error.
 
-    filename = os.path.join(os.path.dirname(__file__),
-                            'data', 'compressed_float_bzero.fits')
+    filename = get_pkg_data_filename('data/compressed_float_bzero.fits')
 
     with fits.open(filename) as hdul:
         hdu = hdul[1]
@@ -1919,3 +1973,17 @@ def test_image_write_readonly(tmpdir):
 
     with fits.open(filename) as hdulist:
         assert_equal(hdulist[1].data, [1.0, 2.0, 3.0])
+
+
+def test_int8(tmp_path):
+    '''Test for int8 support, https://github.com/astropy/astropy/issues/11995'''
+    img = np.arange(-50, 50, dtype=np.int8).reshape(10, 10)
+    hdu = fits.PrimaryHDU(img)
+    hdu.writeto(tmp_path / "int8.fits")
+
+    with fits.open(tmp_path / "int8.fits") as hdul:
+        assert hdul[0].header['BITPIX'] == 8
+        assert hdul[0].header['BZERO'] == -128
+        assert hdul[0].header['BSCALE'] == 1.0
+        assert_equal(hdul[0].data, img)
+        assert hdul[0].data.dtype == img.dtype
