@@ -1,9 +1,6 @@
-from asdf.yamlutil import custom_tree_to_tagged_tree
-
 import astropy.units as u
 import astropy.coordinates.representation
 from astropy.coordinates.representation import BaseRepresentationOrDifferential
-
 from astropy.io.misc.asdf.types import AstropyType
 
 
@@ -27,7 +24,7 @@ class RepresentationType(AstropyType):
 
         node = {}
         node['type'] = t.__name__
-        node['components'] = custom_tree_to_tagged_tree(components, ctx)
+        node['components'] = components
 
         return node
 
