@@ -1,35 +1,31 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-
+import os
 import pytest
 import numpy as np
 
-from astropy.tests.helper import catch_warnings
-from astropy.table import Table, QTable, NdarrayMixin, Column
+from astropy.table import Table, QTable, Column
 from astropy.table.table_helpers import simple_table
 
-from astropy import units as u
-
-from astropy.coordinates import SkyCoord, Latitude, Longitude, Angle, EarthLocation
-from astropy.time import Time, TimeDelta
+from astropy.units import allclose as quantity_allclose
 from astropy.units.quantity import QuantityInfo
 from astropy.utils.exceptions import AstropyUserWarning
 from astropy.utils.data import get_pkg_data_filename
+from astropy.utils.misc import _NOT_OVERWRITING_MSG_MATCH
 from astropy.io.misc.hdf5 import meta_path
-
-try:
+from astropy.utils.compat import NUMPY_LT_1_22
+from astropy.utils.compat.optional_deps import HAS_H5PY  # noqa
+if HAS_H5PY:
     import h5py
-except ImportError:
-    HAS_H5PY = False
-else:
-    HAS_H5PY = True
 
-try:
-    import yaml
-except ImportError:
-    HAS_YAML = False
-else:
-    HAS_YAML = True
+from astropy.io.tests.mixin_columns import mixin_cols, compare_attrs, serialized_names
+
+
+# HDF5 does not support object dtype (since it stores binary representations).
+unsupported_cols = {name: col for name, col in mixin_cols.items()
+                    if (isinstance(col, np.ndarray) and col.dtype.kind == 'O')}
+mixin_cols = {name: col for name, col in mixin_cols.items()
+              if name not in unsupported_cols}
 
 ALL_DTYPES = [np.uint8, np.uint16, np.uint32, np.uint64, np.int8,
               np.int16, np.int32, np.int64, np.float32, np.float64,
@@ -45,14 +41,18 @@ def _default_values(dtype):
         return [1, 2, 3]
 
 
-@pytest.mark.skipif('not HAS_H5PY')
-def test_write_nopath(tmpdir):
-    test_file = str(tmpdir.join('test.hdf5'))
-    t1 = Table()
-    t1.add_column(Column(name='a', data=[1, 2, 3]))
-    with pytest.raises(ValueError) as exc:
-        t1.write(test_file)
-    assert exc.value.args[0] == "table path should be set via the path= argument"
+@pytest.fixture
+def home_is_tmpdir(monkeypatch, tmpdir):
+    """
+    Pytest fixture to run a test case with tilde-prefixed paths.
+
+    In the tilde-path case, environment variables are temporarily
+    modified so that '~' resolves to the temp directory.
+    """
+    # For Unix
+    monkeypatch.setenv('HOME', str(tmpdir))
+    # For Windows
+    monkeypatch.setenv('USERPROFILE', str(tmpdir))
 
 
 @pytest.mark.skipif('not HAS_H5PY')
@@ -61,12 +61,9 @@ def test_write_nopath(tmpdir):
     t1 = Table()
     t1.add_column(Column(name='a', data=[1, 2, 3]))
 
-    with catch_warnings() as warns:
+    with pytest.warns(UserWarning, match="table path was not set via the path= argument"):
         t1.write(test_file)
 
-    assert np.any([str(w.message).startswith(
-        "table path was not set via the path= argument")
-                   for w in warns])
     t1 = Table.read(test_file, path='__astropy_table__')
 
 
@@ -88,9 +85,8 @@ def test_write_nopath_nonempty(tmpdir):
 def test_read_notable_nopath(tmpdir):
     test_file = str(tmpdir.join('test.hdf5'))
     h5py.File(test_file, 'w').close()  # create empty file
-    with pytest.raises(ValueError) as exc:
-        t1 = Table.read(test_file, path='/', format='hdf5')
-    assert exc.value.args[0] == 'no table found in HDF5 group /'
+    with pytest.raises(ValueError, match='no table found in HDF5 group /'):
+        Table.read(test_file, path='/', format='hdf5')
 
 
 @pytest.mark.skipif('not HAS_H5PY')
@@ -99,10 +95,7 @@ def test_read_nopath(tmpdir):
     t1 = Table()
     t1.add_column(Column(name='a', data=[1, 2, 3]))
     t1.write(test_file, path="the_table")
-    with catch_warnings(AstropyUserWarning) as warning_lines:
-        t2 = Table.read(test_file)
-        assert not np.any(["path= was not sp" in str(wl.message)
-                           for wl in warning_lines])
+    t2 = Table.read(test_file)
 
     assert np.all(t1['a'] == t2['a'])
 
@@ -208,9 +201,8 @@ def test_read_write_existing(tmpdir):
     h5py.File(test_file, 'w').close()  # create empty file
     t1 = Table()
     t1.add_column(Column(name='a', data=[1, 2, 3]))
-    with pytest.raises(OSError) as exc:
+    with pytest.raises(OSError, match=_NOT_OVERWRITING_MSG_MATCH):
         t1.write(test_file, path='the_table')
-    assert exc.value.args[0].startswith("File exists:")
 
 
 @pytest.mark.skipif('not HAS_H5PY')
@@ -326,9 +318,8 @@ def test_read_wrong_fileobj():
 
     f = FakeFile()
 
-    with pytest.raises(TypeError) as exc:
-        t1 = Table.read(f, format='hdf5')
-    assert exc.value.args[0] == 'h5py can only open regular files'
+    with pytest.raises(TypeError, match='h5py can only open regular files'):
+        Table.read(f, format='hdf5')
 
 
 @pytest.mark.skipif('not HAS_H5PY')
@@ -346,6 +337,31 @@ def test_write_fileobj(tmpdir):
     assert np.all(t2['a'] == [1, 2, 3])
 
 
+@pytest.mark.skipif('not HAS_H5PY')
+def test_write_create_dataset_kwargs(tmpdir):
+
+    test_file = str(tmpdir.join('test.hdf5'))
+    the_path = 'the_table'
+
+    import h5py
+    with h5py.File(test_file, 'w') as output_file:
+        t1 = Table()
+        t1.add_column(Column(name='a', data=[1, 2, 3]))
+        t1.write(output_file, path=the_path,
+                 maxshape=(None, ))
+
+    # A roundabout way of checking this, but the table created above should be
+    # resizable if the kwarg was passed through successfully
+    t2 = Table()
+    t2.add_column(Column(name='a', data=[4, 5]))
+    with h5py.File(test_file, 'a') as output_file:
+        output_file[the_path].resize((len(t1) + len(t2), ))
+        output_file[the_path][len(t1):] = t2.as_array()
+
+    t3 = Table.read(test_file, path='the_table')
+    assert np.all(t3['a'] == [1, 2, 3, 4, 5])
+
+
 @pytest.mark.skipif('not HAS_H5PY')
 def test_write_filobj_group(tmpdir):
 
@@ -433,7 +449,7 @@ def test_preserve_meta(tmpdir):
         assert np.all(t1.meta[key] == t2.meta[key])
 
 
-@pytest.mark.skipif('not HAS_H5PY or not HAS_YAML')
+@pytest.mark.skipif('not HAS_H5PY')
 def test_preserve_serialized(tmpdir):
     test_file = str(tmpdir.join('test.hdf5'))
 
@@ -457,12 +473,12 @@ def test_preserve_serialized(tmpdir):
     assert t1.meta == t2.meta
 
     # Check that the meta table is fixed-width bytes (see #11299)
-    h5 = h5py.File(test_file)
+    h5 = h5py.File(test_file, 'r')
     meta_lines = h5[meta_path('the_table')]
     assert meta_lines.dtype.kind == 'S'
 
 
-@pytest.mark.skipif('not HAS_H5PY or not HAS_YAML')
+@pytest.mark.skipif('not HAS_H5PY')
 def test_preserve_serialized_old_meta_format(tmpdir):
     """Test the old meta format
 
@@ -488,7 +504,7 @@ def test_preserve_serialized_old_meta_format(tmpdir):
     assert t1.meta == t2.meta
 
 
-@pytest.mark.skipif('not HAS_H5PY or not HAS_YAML')
+@pytest.mark.skipif('not HAS_H5PY')
 def test_preserve_serialized_in_complicated_path(tmpdir):
     test_file = str(tmpdir.join('test.hdf5'))
 
@@ -513,7 +529,7 @@ def test_preserve_serialized_in_complicated_path(tmpdir):
     assert t1.meta == t2.meta
 
 
-@pytest.mark.skipif('not HAS_H5PY or not HAS_YAML')
+@pytest.mark.skipif('not HAS_H5PY')
 def test_metadata_very_large(tmpdir):
     """Test that very large datasets work, now!"""
     test_file = str(tmpdir.join('test.hdf5'))
@@ -555,14 +571,13 @@ def test_skip_meta(tmpdir):
     t1.meta['e'] = np.array([1, 2, 3])
     t1.meta['f'] = str
 
-    with catch_warnings() as w:
+    wtext = f"Attribute `f` of type {type(t1.meta['f'])} cannot be written to HDF5 files - skipping"
+    with pytest.warns(AstropyUserWarning, match=wtext) as w:
         t1.write(test_file, path='the_table')
     assert len(w) == 1
-    assert str(w[0].message).startswith(
-        "Attribute `f` of type {} cannot be written to HDF5 files - skipping".format(type(t1.meta['f'])))
 
 
-@pytest.mark.skipif('not HAS_H5PY or not HAS_YAML')
+@pytest.mark.skipif('not HAS_H5PY')
 def test_fail_meta_serialize(tmpdir):
 
     test_file = str(tmpdir.join('test.hdf5'))
@@ -637,7 +652,8 @@ def assert_objects_equal(obj1, obj2, attrs, compare_class=True):
     if compare_class:
         assert obj1.__class__ is obj2.__class__
 
-    info_attrs = ['info.name', 'info.format', 'info.unit', 'info.description', 'info.meta']
+    info_attrs = ['info.name', 'info.format', 'info.unit', 'info.description', 'info.meta',
+                  'info.dtype']
     for attr in attrs + info_attrs:
         a1 = obj1
         a2 = obj2
@@ -657,62 +673,22 @@ def assert_objects_equal(obj1, obj2, attrs, compare_class=True):
             if a2 is None:
                 a2 = {}
 
-        assert np.all(a1 == a2)
-
-# Testing HDF5 table read/write with mixins.  This is mostly
-# copied from FITS mixin testing.
+        if isinstance(a1, np.ndarray) and a1.dtype.kind == 'f':
+            assert quantity_allclose(a1, a2, rtol=1e-15)
+        elif isinstance(a1, np.dtype):
+            # HDF5 does not perfectly preserve dtype: byte order can change, and
+            # unicode gets stored as bytes.  So, we just check safe casting, to
+            # ensure we do not, e.g., accidentally change integer to float, etc.
+            if NUMPY_LT_1_22 and a1.names:
+                # For old numpy, can_cast does not deal well with structured dtype.
+                assert a1.names == a2.names
+            else:
+                assert np.can_cast(a2, a1, casting='safe')
+        else:
+            assert np.all(a1 == a2)
 
 
-el = EarthLocation(x=1 * u.km, y=3 * u.km, z=5 * u.km)
-el2 = EarthLocation(x=[1, 2] * u.km, y=[3, 4] * u.km, z=[5, 6] * u.km)
-sc = SkyCoord([1, 2], [3, 4], unit='deg,deg', frame='fk4',
-              obstime='J1990.5')
-scc = sc.copy()
-scc.representation_type = 'cartesian'
-tm = Time([2450814.5, 2450815.5], format='jd', scale='tai', location=el)
-
-# NOTE: in the test below the name of the column "x" for the Quantity is
-# important since it tests the fix for #10215 (namespace clash, where "x"
-# clashes with "el2.x").
-mixin_cols = {
-    'tm': tm,
-    'dt': TimeDelta([1, 2] * u.day),
-    'sc': sc,
-    'scc': scc,
-    'scd': SkyCoord([1, 2], [3, 4], [5, 6], unit='deg,deg,m', frame='fk4',
-                    obstime=['J1990.5', 'J1991.5']),
-    'x': [1, 2] * u.m,
-    'qdb': [10, 20] * u.dB(u.mW),
-    'qdex': [4.5, 5.5] * u.dex(u.cm/u.s**2),
-    'qmag': [21, 22] * u.ABmag,
-    'lat': Latitude([1, 2] * u.deg),
-    'lon': Longitude([1, 2] * u.deg, wrap_angle=180. * u.deg),
-    'ang': Angle([1, 2] * u.deg),
-    'el2': el2,
-}
-
-time_attrs = ['value', 'shape', 'format', 'scale', 'location']
-compare_attrs = {
-    'c1': ['data'],
-    'c2': ['data'],
-    'tm': time_attrs,
-    'dt': ['shape', 'value', 'format', 'scale'],
-    'sc': ['ra', 'dec', 'representation_type', 'frame.name'],
-    'scc': ['x', 'y', 'z', 'representation_type', 'frame.name'],
-    'scd': ['ra', 'dec', 'distance', 'representation_type', 'frame.name'],
-    'x': ['value', 'unit'],
-    'qdb': ['value', 'unit'],
-    'qdex': ['value', 'unit'],
-    'qmag': ['value', 'unit'],
-    'lon': ['value', 'unit', 'wrap_angle'],
-    'lat': ['value', 'unit'],
-    'ang': ['value', 'unit'],
-    'el2': ['x', 'y', 'z', 'ellipsoid'],
-    'nd': ['x', 'y', 'z'],
-}
-
-
-@pytest.mark.skipif('not HAS_H5PY or not HAS_YAML')
+@pytest.mark.skipif('not HAS_H5PY')
 def test_hdf5_mixins_qtable_to_table(tmpdir):
     """Test writing as QTable and reading as Table.  Ensure correct classes
     come out.
@@ -730,11 +706,6 @@ def test_hdf5_mixins_qtable_to_table(tmpdir):
     for name, col in t.columns.items():
         col2 = t2[name]
 
-        # Special-case Time, which does not yet support round-tripping
-        # the format.
-        if isinstance(col2, Time):
-            col2.format = col.format
-
         attrs = compare_attrs[name]
         compare_class = True
 
@@ -750,28 +721,15 @@ def test_hdf5_mixins_qtable_to_table(tmpdir):
         assert_objects_equal(col, col2, attrs, compare_class)
 
 
-@pytest.mark.skipif('not HAS_H5PY or not HAS_YAML')
+@pytest.mark.skipif('not HAS_H5PY')
 @pytest.mark.parametrize('table_cls', (Table, QTable))
 def test_hdf5_mixins_as_one(table_cls, tmpdir):
     """Test write/read all cols at once and validate intermediate column names"""
     filename = str(tmpdir.join('test_simple.hdf5'))
     names = sorted(mixin_cols)
-
-    serialized_names = ['ang',
-                        'dt.jd1', 'dt.jd2',
-                        'el2.x', 'el2.y', 'el2.z',
-                        'lat',
-                        'lon',
-                        'qdb',
-                        'qdex',
-                        'qmag',
-                        'sc.ra', 'sc.dec',
-                        'scc.x', 'scc.y', 'scc.z',
-                        'scd.ra', 'scd.dec', 'scd.distance',
-                        'scd.obstime.jd1', 'scd.obstime.jd2',
-                        'tm.jd1', 'tm.jd2',
-                        'x',
-                        ]
+    all_serialized_names = []
+    for name in names:
+        all_serialized_names.extend(serialized_names[name])
 
     t = table_cls([mixin_cols[name] for name in names], names=names)
     t.meta['C'] = 'spam'
@@ -789,11 +747,12 @@ def test_hdf5_mixins_as_one(table_cls, tmpdir):
 
     # Read directly via hdf5 and confirm column names
     h5 = h5py.File(filename, 'r')
-    assert list(h5['root'].dtype.names) == serialized_names
+    h5_names = list(h5['root'].dtype.names)
+    assert h5_names == all_serialized_names
     h5.close()
 
 
-@pytest.mark.skipif('not HAS_H5PY or not HAS_YAML')
+@pytest.mark.skipif('not HAS_H5PY')
 @pytest.mark.parametrize('name_col', list(mixin_cols.items()))
 @pytest.mark.parametrize('table_cls', (Table, QTable))
 def test_hdf5_mixins_per_column(table_cls, name_col, tmpdir):
@@ -809,16 +768,14 @@ def test_hdf5_mixins_per_column(table_cls, name_col, tmpdir):
     if not t.has_mixin_columns:
         pytest.skip('column is not a mixin (e.g. Quantity subclass in Table)')
 
-    if isinstance(t[name], NdarrayMixin):
-        pytest.xfail('NdarrayMixin not supported')
-
     t.write(filename, format="hdf5", path='root', serialize_meta=True)
     t2 = table_cls.read(filename, format='hdf5', path='root')
 
     assert t.colnames == t2.colnames
 
     for colname in t.colnames:
-        assert_objects_equal(t[colname], t2[colname], compare_attrs[colname])
+        compare = ['data'] if colname in ('c1', 'c2') else compare_attrs[colname]
+        assert_objects_equal(t[colname], t2[colname], compare)
 
     # Special case to make sure Column type doesn't leak into Time class data
     if name.startswith('tm'):
@@ -826,28 +783,18 @@ def test_hdf5_mixins_per_column(table_cls, name_col, tmpdir):
         assert t2[name]._time.jd2.__class__ is np.ndarray
 
 
-@pytest.mark.skipif('HAS_YAML or not HAS_H5PY')
-def test_warn_for_dropped_info_attributes(tmpdir):
-    filename = str(tmpdir.join('test.hdf5'))
-    t = Table([[1, 2]])
-    t['col0'].info.description = 'hello'
-    with catch_warnings() as warns:
-        t.write(filename, path='root', serialize_meta=False)
-    assert len(warns) == 1
-    assert str(warns[0].message).startswith(
-        "table contains column(s) with defined 'unit'")
+@pytest.mark.parametrize('name_col', unsupported_cols.items())
+@pytest.mark.xfail(reason='column type unsupported')
+def test_fits_unsupported_mixin(self, name_col, tmpdir):
+    # Check that we actually fail in writing unsupported columns defined
+    # on top.
+    filename = str(tmpdir.join('test_simple.fits'))
+    name, col = name_col
+    Table([col], names=[name]).write(filename, format='hdf5', path='root',
+                                     serialize_meta=True)
 
 
-@pytest.mark.skipif('HAS_YAML or not HAS_H5PY')
-def test_error_for_mixins_but_no_yaml(tmpdir):
-    filename = str(tmpdir.join('test.hdf5'))
-    t = Table([mixin_cols['sc']])
-    with pytest.raises(TypeError) as err:
-        t.write(filename, path='root', serialize_meta=True)
-    assert "cannot write type SkyCoord column 'col0' to HDF5 without PyYAML" in str(err.value)
-
-
-@pytest.mark.skipif('not HAS_YAML or not HAS_H5PY')
+@pytest.mark.skipif('not HAS_H5PY')
 def test_round_trip_masked_table_default(tmpdir):
     """Test round-trip of MaskedColumn through HDF5 using default serialization
     that writes a separate mask column.  Note:
@@ -881,7 +828,7 @@ def test_round_trip_masked_table_default(tmpdir):
         assert np.all(t2[name] == t[name])
 
 
-@pytest.mark.skipif('not HAS_YAML or not HAS_H5PY')
+@pytest.mark.skipif('not HAS_H5PY')
 def test_overwrite_serialized_meta():
     # This used to cause an error because the meta data table
     # was not removed from the existing file.
@@ -902,3 +849,18 @@ def test_overwrite_serialized_meta():
         t2 = Table.read(out, path='data')
         assert all(t3 == t2)
         assert t3.info(out=None) == t2.info(out=None)
+
+
+@pytest.mark.skipif('not HAS_H5PY')
+def test_read_write_tilde_path(tmpdir, home_is_tmpdir):
+    test_file = os.path.join('~', 'test.hdf5')
+    t1 = Table()
+    t1['a'] = [1, 2, 3]
+
+    t1.write(test_file, path='the_table')
+
+    t1 = Table.read(test_file, path='the_table')
+    t1 = Table.read(test_file, path='the_table', format='hdf5')
+
+    # Ensure the data wasn't written to the literal tilde-prefixed path
+    assert not os.path.exists(test_file)
