@@ -20,7 +20,8 @@ from astropy.utils.xml.writer import xml_escape_cdata
 
 # LOCAL
 from .exceptions import (vo_raise, vo_warn, warn_or_raise, W01,
-    W30, W31, W39, W46, W47, W49, W51, E01, E02, E03, E04, E05, E06)
+    W30, W31, W39, W46, W47, W49, W51, W55, E01, E02, E03, E04,
+    E05, E06, E24)
 
 
 __all__ = ['get_converter', 'Converter', 'table_column_to_votable_datatype']
@@ -136,7 +137,7 @@ def bool_to_bitarray(value):
     if bit_no != 7:
         bytes.append(byte)
 
-    return struct_pack("{}B".format(len(bytes)), *bytes)
+    return struct_pack(f"{len(bytes)}B", *bytes)
 
 
 class Converter:
@@ -188,7 +189,8 @@ class Converter:
 
         Returns
         -------
-        native : tuple (value, mask)
+        native : tuple
+            A two-element tuple of: value, mask.
             The value as a Numpy array or scalar, and *mask* is True
             if the value is missing.
         """
@@ -209,7 +211,8 @@ class Converter:
 
         Returns
         -------
-        native : tuple (value, mask)
+        native : (2,) tuple
+            (value, mask)
             The value as a Numpy array or scalar, and *mask* is True
             if the value is missing.
         """
@@ -223,8 +226,8 @@ class Converter:
 
         Parameters
         ----------
-        value : native type corresponding to this converter
-            The value
+        value
+            The value, the native type corresponding to this converter
 
         mask : bool
             If `True`, will return the string representation of a
@@ -252,9 +255,9 @@ class Converter:
 
         Returns
         -------
-        native : tuple (value, mask)
-            The value as a Numpy array or scalar, and *mask* is True
-            if the value is missing.
+        native : (2,) tuple
+            (value, mask). The value as a Numpy array or scalar, and *mask* is
+            True if the value is missing.
         """
         raise NotImplementedError(
             "This datatype must implement a 'binparse' method.")
@@ -267,8 +270,8 @@ class Converter:
 
         Parameters
         ----------
-        value : native type corresponding to this converter
-            The value
+        value
+            The value, the native type corresponding to this converter
 
         mask : bool
             If `True`, will return the string representation of a
@@ -276,7 +279,7 @@ class Converter:
 
         Returns
         -------
-        bytes : byte string
+        bytes : bytes
             The binary representation of the value, suitable for
             serialization in the BINARY_ format.
         """
@@ -298,6 +301,8 @@ class Char(Converter):
 
         Converter.__init__(self, field, config, pos)
 
+        self.field_name = field.name
+
         if field.arraysize is None:
             vo_warn(W47, (), config, pos)
             field.arraysize = '1'
@@ -314,43 +319,55 @@ class Char(Converter):
                 self.arraysize = int(field.arraysize)
             except ValueError:
                 vo_raise(E01, (field.arraysize, 'char', field.ID), config)
-            self.format = f'S{self.arraysize:d}'
+            self.format = f'U{self.arraysize:d}'
             self.binparse = self._binparse_fixed
             self.binoutput = self._binoutput_fixed
             self._struct_format = f">{self.arraysize:d}s"
 
-        if config.get('verify', 'ignore') == 'exception':
-            self.parse = self._ascii_parse
-        else:
-            self.parse = self._str_parse
-
     def supports_empty_values(self, config):
         return True
 
-    def _ascii_parse(self, value, config=None, pos=None):
+    def parse(self, value, config=None, pos=None):
         if self.arraysize != '*' and len(value) > self.arraysize:
             vo_warn(W46, ('char', self.arraysize), config, pos)
-        return value.encode('ascii'), False
 
-    def _str_parse(self, value, config=None, pos=None):
-        if self.arraysize != '*' and len(value) > self.arraysize:
-            vo_warn(W46, ('char', self.arraysize), config, pos)
-        return value.encode('utf-8'), False
+        # Warn about non-ascii characters if warnings are enabled.
+        try:
+            value.encode('ascii')
+        except UnicodeEncodeError:
+            vo_warn(W55, (self.field_name, value), config, pos)
+        return value, False
 
     def output(self, value, mask):
         if mask:
             return ''
-        if not isinstance(value, str):
-            value = value.decode('ascii')
+
+        # The output methods for Char assume that value is either str or bytes.
+        # This method needs to return a str, but needs to warn if the str contains
+        # non-ASCII characters.
+        try:
+            if isinstance(value, str):
+                value.encode('ascii')
+            else:
+                # Check for non-ASCII chars in the bytes object.
+                value = value.decode('ascii')
+        except (ValueError, UnicodeEncodeError):
+            warn_or_raise(E24, UnicodeEncodeError, (value, self.field_name))
+        finally:
+            if isinstance(value, bytes):
+                # Convert the bytes to str regardless of non-ASCII chars.
+                value = value.decode('utf-8')
+
         return xml_escape_cdata(value)
 
     def _binparse_var(self, read):
         length = self._parse_length(read)
-        return read(length), False
+        return read(length).decode('ascii'), False
 
     def _binparse_fixed(self, read):
         s = struct_unpack(self._struct_format, read(self.arraysize))[0]
         end = s.find(_zero_byte)
+        s = s.decode('ascii')
         if end != -1:
             return s[:end], False
         return s, False
@@ -358,11 +375,21 @@ class Char(Converter):
     def _binoutput_var(self, value, mask):
         if mask or value is None or value == '':
             return _zero_int
+        if isinstance(value, str):
+            try:
+                value = value.encode('ascii')
+            except ValueError:
+                vo_raise(E24, (value, self.field_name))
         return self._write_length(len(value)) + value
 
     def _binoutput_fixed(self, value, mask):
         if mask:
             value = _empty_bytes
+        elif isinstance(value, str):
+            try:
+                value = value.encode('ascii')
+            except ValueError:
+                vo_raise(E24, (value, self.field_name))
         return struct_pack(self._struct_format, value)
 
 
@@ -394,7 +421,7 @@ class UnicodeChar(Converter):
             self.format = f'U{self.arraysize:d}'
             self.binparse = self._binparse_fixed
             self.binoutput = self._binoutput_fixed
-            self._struct_format = ">{:d}s".format(self.arraysize * 2)
+            self._struct_format = f">{self.arraysize*2:d}s"
 
     def parse(self, value, config=None, pos=None):
         if self.arraysize != '*' and len(value) > self.arraysize:
@@ -558,7 +585,7 @@ class NumericArray(Array):
 
         self._base = base
         self._arraysize = arraysize
-        self.format = "{}{}".format(tuple(arraysize), base.format)
+        self.format = f"{tuple(arraysize)}{base.format}"
 
         self._items = 1
         for dim in arraysize:
@@ -681,11 +708,11 @@ class FloatingPoint(Numeric):
 
         if precision is not None:
             if precision.startswith("E"):
-                format_parts.append('.{:d}g'.format(int(precision[1:])))
+                format_parts.append(f'.{int(precision[1:]):d}g')
             elif precision.startswith("F"):
-                format_parts.append('.{:d}f'.format(int(precision[1:])))
+                format_parts.append(f'.{int(precision[1:]):d}f')
             else:
-                format_parts.append('.{:d}f'.format(int(precision)))
+                format_parts.append(f'.{int(precision):d}f')
 
         format_parts.append('}')
 
@@ -1315,20 +1342,6 @@ numpy_dtype_to_field_mapping = {
 numpy_dtype_to_field_mapping[np.bytes_().dtype.num] = 'char'
 
 
-def _all_bytes(column):
-    for x in column:
-        if not isinstance(x, bytes):
-            return False
-    return True
-
-
-def _all_unicode(column):
-    for x in column:
-        if not isinstance(x, str):
-            return False
-    return True
-
-
 def _all_matching_dtype(column):
     first_dtype = False
     first_shape = ()
@@ -1360,8 +1373,8 @@ def numpy_to_votable_dtype(dtype, shape):
     Returns
     -------
     attributes : dict
-       A dict containing 'datatype' and 'arraysize' keys that can be
-       set on a VOTable FIELD element.
+        A dict containing 'datatype' and 'arraysize' keys that can be
+        set on a VOTable FIELD element.
     """
     if dtype.num not in numpy_dtype_to_field_mapping:
         raise TypeError(
@@ -1414,16 +1427,15 @@ def table_column_to_votable_datatype(column):
     Returns
     -------
     attributes : dict
-       A dict containing 'datatype' and 'arraysize' keys that can be
-       set on a VOTable FIELD element.
+        A dict containing 'datatype' and 'arraysize' keys that can be
+        set on a VOTable FIELD element.
     """
+    votable_string_dtype = None
+    if column.info.meta is not None:
+        votable_string_dtype = column.info.meta.get('_votable_string_dtype')
     if column.dtype.char == 'O':
-        if isinstance(column[0], bytes):
-            if _all_bytes(column[1:]):
-                return {'datatype': 'char', 'arraysize': '*'}
-        elif isinstance(column[0], str):
-            if _all_unicode(column[1:]):
-                return {'datatype': 'unicodeChar', 'arraysize': '*'}
+        if votable_string_dtype is not None:
+            return {'datatype': votable_string_dtype, 'arraysize': '*'}
         elif isinstance(column[0], np.ndarray):
             dtype, shape = _all_matching_dtype(column)
             if dtype is not False:
@@ -1437,4 +1449,10 @@ def table_column_to_votable_datatype(column):
         # All bets are off, do the most generic thing
         return {'datatype': 'unicodeChar', 'arraysize': '*'}
 
-    return numpy_to_votable_dtype(column.dtype, column.shape[1:])
+    # For fixed size string columns, datatype here will be unicodeChar,
+    # but honor the original FIELD datatype if present.
+    result = numpy_to_votable_dtype(column.dtype, column.shape[1:])
+    if result['datatype'] == 'unicodeChar' and votable_string_dtype == 'char':
+        result['datatype'] = 'char'
+
+    return result
