@@ -16,10 +16,9 @@ import operator
 import numpy as np
 
 from astropy.units import Quantity
-from astropy.utils import isiterable, OrderedDescriptor
-from .utils import array_repr_oneline
-from .utils import get_inputs_and_params
+from astropy.utils import isiterable
 
+from .utils import array_repr_oneline, get_inputs_and_params
 
 __all__ = ['Parameter', 'InputParameterError', 'ParameterError']
 
@@ -46,23 +45,21 @@ def _tofloat(value):
             # catch arrays with strings or user errors like different
             # types of parameters in a parameter set
             raise InputParameterError(
-                "Parameter of {} could not be converted to "
-                "float".format(type(value)))
+                f"Parameter of {type(value)} could not be converted to float")
     elif isinstance(value, Quantity):
         # Quantities are fine as is
         pass
     elif isinstance(value, np.ndarray):
         # A scalar/dimensionless array
         value = float(value.item())
-    elif isinstance(value, (numbers.Number, np.number)):
+    elif isinstance(value, (numbers.Number, np.number)) and not isinstance(value, bool):
         value = float(value)
     elif isinstance(value, bool):
         raise InputParameterError(
             "Expected parameter to be of numerical type, not boolean")
     else:
         raise InputParameterError(
-            "Don't know how to convert parameter of {} to "
-            "float".format(type(value)))
+            f"Don't know how to convert parameter of {type(value)} to float")
     return value
 
 
@@ -113,14 +110,13 @@ def _unary_arithmetic_operation(op):
     return wrapper
 
 
-class Parameter(OrderedDescriptor):
+class Parameter:
     """
     Wraps individual parameters.
 
-    Since 4.0 Parameters are no longer descriptors (despite the fact that
-    it inherits from OrderedDescriptor) and are based on a new implementation
-    of the Parameter class. Parameters now  (as of 4.0) store values locally
-    (as instead previously in the associated model)
+    Since 4.0 Parameters are no longer descriptors and are based on a new
+    implementation of the Parameter class. Parameters now  (as of 4.0) store
+    values locally (as instead previously in the associated model)
 
     This class represents a model's parameter (in a somewhat broad sense). It
     serves a number of purposes:
@@ -131,7 +127,7 @@ class Parameter(OrderedDescriptor):
 
     2) Managing the handling of allowable parameter values and once defined,
     ensuring updates are consistent with the Parameter definition. This
-    includes the optional use of units and quantities as well as tranforming
+    includes the optional use of units and quantities as well as transforming
     values to an internally consistent representation (e.g., from degrees to
     radians through the use of getters and setters).
 
@@ -141,7 +137,7 @@ class Parameter(OrderedDescriptor):
 
 
 
-    See :ref:`modeling-parameters` for more details.
+    See :ref:`astropy:modeling-parameters` for more details.
 
     Parameters
     ----------
@@ -193,13 +189,9 @@ class Parameter(OrderedDescriptor):
     fitters as of this writing.
     """
 
-    # Settings for OrderedDescriptor
-    _class_attribute_ = '_parameters_'
-    _name_attribute_ = '_name'
-
     def __init__(self, name='', description='', default=None, unit=None,
                  getter=None, setter=None, fixed=False, tied=False, min=None,
-                 max=None, bounds=None):
+                 max=None, bounds=None, prior=None, posterior=None):
         super().__init__()
 
         self._model = None
@@ -213,8 +205,8 @@ class Parameter(OrderedDescriptor):
         if isinstance(default, Quantity):
             if unit is not None and not unit.is_equivalent(default.unit):
                 raise ParameterDefinitionError(
-                    "parameter default {0} does not have units equivalent to "
-                    "the required unit {1}".format(default, unit))
+                    f"parameter default {default} does not have units equivalent to "
+                    f"the required unit {unit}")
             unit = default.unit
             default = default.value
 
@@ -237,8 +229,8 @@ class Parameter(OrderedDescriptor):
         if bounds is not None:
             if min is not None or max is not None:
                 raise ValueError(
-                    'bounds may not be specified simultaneously with min or '
-                    'or max when instantiating Parameter {}'.format(name))
+                    "bounds may not be specified simultaneously with min or "
+                    f"max when instantiating Parameter {name}")
         else:
             bounds = (min, max)
 
@@ -248,8 +240,13 @@ class Parameter(OrderedDescriptor):
         self._order = None
 
         self._validator = None
-        self._prior = None
-        self._posterior = None
+        self._prior = prior
+        self._posterior = posterior
+
+        self._std = None
+
+    def __set_name__(self, owner, name):
+        self._name = name
 
     def __len__(self):
         val = self.value
@@ -274,7 +271,7 @@ class Parameter(OrderedDescriptor):
             if len(oldvalue[key]) == 0:
                 raise InputParameterError(
                     "Slice assignment outside the parameter dimensions for "
-                    "'{}'".format(self.name))
+                    f"'{self.name}'")
             for idx, val in zip(range(*key.indices(len(self))), value):
                 self.__setitem__(idx, val)
         else:
@@ -282,8 +279,8 @@ class Parameter(OrderedDescriptor):
                 oldvalue[key] = value
             except IndexError:
                 raise InputParameterError(
-                    "Input dimension {} invalid for {!r} parameter with "
-                    "dimension {}".format(key, self.name, value.shape[0]))  # likely wrong
+                    f"Input dimension {key} invalid for {self.name!r} parameter with "
+                    f"dimension {value.shape[0]}")  # likely wrong
 
     def __repr__(self):
         args = f"'{self._name}'"
@@ -370,12 +367,7 @@ class Parameter(OrderedDescriptor):
         if force:
             self._unit = unit
         else:
-            if unit is None:
-                raise ValueError('Cannot attach units to parameters that were '
-                                 'not initially specified with units')
-            else:
-                raise ValueError('Cannot change the unit attribute directly, '
-                                 'instead change the parameter to a new quantity')
+            self.unit = unit
 
     @property
     def internal_unit(self):
@@ -430,6 +422,17 @@ class Parameter(OrderedDescriptor):
 
         return np.size(self.value)
 
+    @property
+    def std(self):
+        """Standard deviation, if available from fit."""
+
+        return self._std
+
+    @std.setter
+    def std(self, value):
+
+        self._std = value
+
     @property
     def prior(self):
         return self._prior
@@ -499,7 +502,7 @@ class Parameter(OrderedDescriptor):
 
         if _max is not None:
             if not isinstance(_max, (numbers.Number, Quantity)):
-                raise TypeError("Max value must be a number")
+                raise TypeError("Max value must be a number or a Quantity")
             if isinstance(_max, Quantity):
                 _max = float(_max.value)
             else:
@@ -672,7 +675,7 @@ class Parameter(OrderedDescriptor):
         return arr
 
     def __bool__(self):
-        return bool(self.value)
+        return bool(np.all(self.value))
 
     __add__ = _binary_arithmetic_operation(operator.add)
     __radd__ = _binary_arithmetic_operation(operator.add, reflected=True)
@@ -682,8 +685,6 @@ class Parameter(OrderedDescriptor):
     __rmul__ = _binary_arithmetic_operation(operator.mul, reflected=True)
     __pow__ = _binary_arithmetic_operation(operator.pow)
     __rpow__ = _binary_arithmetic_operation(operator.pow, reflected=True)
-    __div__ = _binary_arithmetic_operation(operator.truediv)
-    __rdiv__ = _binary_arithmetic_operation(operator.truediv, reflected=True)
     __truediv__ = _binary_arithmetic_operation(operator.truediv)
     __rtruediv__ = _binary_arithmetic_operation(operator.truediv,
                                                 reflected=True)
