@@ -3,21 +3,14 @@
 This module tests fitting and model evaluation with various inputs
 """
 
-import pytest
 import numpy as np
+import pytest
 from numpy.testing import assert_allclose
 
-from astropy.modeling import models
-from astropy.modeling import fitting
-from astropy.modeling.core import Model, FittableModel, Fittable1DModel
+from astropy.modeling import fitting, models
+from astropy.modeling.core import Fittable1DModel, FittableModel, Model
 from astropy.modeling.parameters import Parameter
-
-try:
-    from scipy import optimize  # pylint: disable=W0611 # noqa
-    HAS_SCIPY = True
-except ImportError:
-    HAS_SCIPY = False
-
+from astropy.utils.compat.optional_deps import HAS_SCIPY  # noqa: F401
 
 model1d_params = [
     (models.Polynomial1D, [2]),
@@ -33,6 +26,13 @@ model2d_params = [
     (models.Chebyshev2D, [1, 2])
 ]
 
+fitters = [
+    fitting.LevMarLSQFitter,
+    fitting.TRFLSQFitter,
+    fitting.LMLSQFitter,
+    fitting.DogBoxLSQFitter
+]
+
 
 class TestInputType:
     """
@@ -149,7 +149,7 @@ class TestFitting:
             y1 = p1(self.x1)
             p1 = models.Polynomial1D(5, n_models=2)
             pfit = fitting.LinearLSQFitter()
-            model = pfit(p1, self.x1, y1)
+            pfit(p1, self.x1, y1)
 
     def test_wrong_pset(self):
         """A case of 1 set of x and multiple sets of y and parameters."""
@@ -171,47 +171,51 @@ class TestFitting:
         assert_allclose(model.param_sets, expected, atol=10 ** (-7))
 
     @pytest.mark.skipif('not HAS_SCIPY')
-    def test_nonlinear_lsqt_1set_1d(self):
+    @pytest.mark.parametrize('fitter', fitters)
+    def test_nonlinear_lsqt_1set_1d(self, fitter):
         """1 set 1D x, 1 set 1D y, 1 pset NonLinearFitter"""
+        fitter = fitter()
 
         g1 = models.Gaussian1D(10, mean=3, stddev=.2)
         y1 = g1(self.x1)
-        gfit = fitting.LevMarLSQFitter()
-        model = gfit(g1, self.x1, y1)
+        model = fitter(g1, self.x1, y1)
         assert_allclose(model.parameters, [10, 3, .2])
 
     @pytest.mark.skipif('not HAS_SCIPY')
-    def test_nonlinear_lsqt_Nset_1d(self):
+    @pytest.mark.parametrize('fitter', fitters)
+    def test_nonlinear_lsqt_Nset_1d(self, fitter):
         """1 set 1D x, 1 set 1D y, 2 param_sets, NonLinearFitter"""
+        fitter = fitter()
 
         with pytest.raises(ValueError):
             g1 = models.Gaussian1D([10.2, 10], mean=[3, 3.2], stddev=[.23, .2],
                                    n_models=2)
             y1 = g1(self.x1, model_set_axis=False)
-            gfit = fitting.LevMarLSQFitter()
-            model = gfit(g1, self.x1, y1)
+            _ = fitter(g1, self.x1, y1)
 
     @pytest.mark.skipif('not HAS_SCIPY')
-    def test_nonlinear_lsqt_1set_2d(self):
+    @pytest.mark.parametrize('fitter', fitters)
+    def test_nonlinear_lsqt_1set_2d(self, fitter):
         """1 set 2d x, 1set 2D y, 1 pset, NonLinearFitter"""
+        fitter = fitter()
 
         g2 = models.Gaussian2D(10, x_mean=3, y_mean=4, x_stddev=.3,
                                y_stddev=.2, theta=0)
         z = g2(self.x, self.y)
-        gfit = fitting.LevMarLSQFitter()
-        model = gfit(g2, self.x, self.y, z)
+        model = fitter(g2, self.x, self.y, z)
         assert_allclose(model.parameters, [10, 3, 4, .3, .2, 0])
 
     @pytest.mark.skipif('not HAS_SCIPY')
-    def test_nonlinear_lsqt_Nset_2d(self):
+    @pytest.mark.parametrize('fitter', fitters)
+    def test_nonlinear_lsqt_Nset_2d(self, fitter):
         """1 set 2d x, 1set 2D y, 2 param_sets, NonLinearFitter"""
+        fitter = fitter()
 
         with pytest.raises(ValueError):
             g2 = models.Gaussian2D([10, 10], [3, 3], [4, 4], x_stddev=[.3, .3],
                                    y_stddev=[.2, .2], theta=[0, 0], n_models=2)
             z = g2(self.x.flatten(), self.y.flatten())
-            gfit = fitting.LevMarLSQFitter()
-            model = gfit(g2, self.x, self.y, z)
+            _ = fitter(g2, self.x, self.y, z)
 
 
 class TestEvaluation:
@@ -292,12 +296,6 @@ class TestEvaluation:
         assert_allclose(xx.shape, yy.shape)
         yy1 = p1(xx, model_set_axis=1)
         assert_allclose(yy, yy1)
-        #x1 = xx[:, 0]
-        #x2 = xx[:, 1]
-        #p1 = models.Polynomial1D(5)
-        #assert_allclose(p1(x1), yy[0, :], atol=10 ** (-12))
-        #p1 = models.Polynomial1D(5)
-        #assert_allclose(p1(x2), yy[1, :], atol=10 ** (-12))
 
     def test_evaluate_gauss2d(self):
         cov = np.array([[1., 0.8], [0.8, 3]])
@@ -405,7 +403,7 @@ class TestSingleInputSingleOutputSingleModel:
 
         with pytest.raises(ValueError):
             # Doesn't broadcast
-            y3 = t([100, 200, 300])
+            t([100, 200, 300])
 
     def test_2d_array_parameters_2d_array_input(self):
         """
@@ -428,7 +426,7 @@ class TestSingleInputSingleOutputSingleModel:
 
         with pytest.raises(ValueError):
             # Doesn't broadcast
-            y3 = t([[100, 200, 300], [400, 500, 600]])
+            t([[100, 200, 300], [400, 500, 600]])
 
     def test_mixed_array_parameters_1d_array_input(self):
         """
@@ -437,8 +435,8 @@ class TestSingleInputSingleOutputSingleModel:
         """
 
         t = TModel_1_1([[[0.01, 0.02, 0.03], [0.04, 0.05, 0.06]],
-                           [[0.07, 0.08, 0.09], [0.10, 0.11, 0.12]]],
-                          [1, 2, 3])
+                        [[0.07, 0.08, 0.09], [0.10, 0.11, 0.12]]],
+                       [1, 2, 3])
 
         y1 = t([10, 20, 30])
         assert np.shape(y1) == (2, 2, 3)
@@ -497,7 +495,7 @@ class TestSingleInputSingleOutputTwoModel:
         t = TModel_1_1([1, 2], [10, 20], n_models=2)
 
         with pytest.raises(ValueError):
-            y = t(np.arange(5) * 100)
+            t(np.arange(5) * 100)
 
         y1 = t([100, 200])
         assert np.shape(y1) == (2,)
@@ -526,7 +524,7 @@ class TestSingleInputSingleOutputTwoModel:
         y1 = t(np.arange(6).reshape(2, 3) * 100)
         assert np.shape(y1) == (2, 3)
         assert np.all(y1 == [[11, 111, 211],
-                            [322, 422, 522]])
+                             [322, 422, 522]])
 
         y2 = t(np.arange(6).reshape(2, 3) * 100, model_set_axis=False)
         assert np.shape(y2) == (2, 2, 3)
@@ -560,7 +558,7 @@ class TestSingleInputSingleOutputTwoModel:
         """
 
         t = TModel_1_1([[1, 2, 3], [4, 5, 6]],
-                          [[10, 20, 30], [40, 50, 60]], n_models=2)
+                       [[10, 20, 30], [40, 50, 60]], n_models=2)
 
         y = t(100)
         assert np.shape(y) == (2, 3)
@@ -576,7 +574,7 @@ class TestSingleInputSingleOutputTwoModel:
         """
 
         t = TModel_1_1([[1, 2, 3], [4, 5, 6]],
-                          [[10, 20, 30], [40, 50, 60]], n_models=2)
+                       [[10, 20, 30], [40, 50, 60]], n_models=2)
 
         with pytest.raises(ValueError):
             y1 = t([100, 200, 300])
@@ -596,8 +594,8 @@ class TestSingleInputSingleOutputTwoModel:
 
     def test_2d_array_parameters_2d_array_input(self):
         t = TModel_1_1([[[1, 2], [3, 4]], [[5, 6], [7, 8]]],
-                          [[[10, 20], [30, 40]], [[50, 60], [70, 80]]],
-                          n_models=2)
+                       [[[10, 20], [30, 40]], [[50, 60], [70, 80]]],
+                       n_models=2)
         y1 = t([[100, 200], [300, 400]])
         assert np.shape(y1) == (2, 2, 2)
         assert np.all(y1 == [[[111, 222], [133, 244]],
@@ -613,8 +611,8 @@ class TestSingleInputSingleOutputTwoModel:
 
     def test_mixed_array_parameters_1d_array_input(self):
         t = TModel_1_1([[[0.01, 0.02, 0.03], [0.04, 0.05, 0.06]],
-                           [[0.07, 0.08, 0.09], [0.10, 0.11, 0.12]]],
-                          [[1, 2, 3], [4, 5, 6]], n_models=2)
+                        [[0.07, 0.08, 0.09], [0.10, 0.11, 0.12]]],
+                       [[1, 2, 3], [4, 5, 6]], n_models=2)
 
         with pytest.raises(ValueError):
             y = t([10, 20, 30])
@@ -627,7 +625,7 @@ class TestSingleInputSingleOutputTwoModel:
 
 class TModel_1_2(FittableModel):
 
-    n_inputs =1
+    n_inputs = 1
     n_outputs = 2
 
     p1 = Parameter()
@@ -751,7 +749,7 @@ class TestSingleInputDoubleOutputSingleModel:
         """
 
         t = TModel_1_2([[1, 2], [3, 4]], [[10, 20], [30, 40]],
-                          [[1000, 2000], [3000, 4000]])
+                       [[1000, 2000], [3000, 4000]])
 
         y1, z1 = t([[100, 200], [300, 400]])
         assert np.shape(y1) == np.shape(z1) == (2, 2)
@@ -780,8 +778,8 @@ class TestSingleInputDoubleOutputSingleModel:
         """
 
         t = TModel_1_2([[[0.01, 0.02, 0.03], [0.04, 0.05, 0.06]],
-                           [[0.07, 0.08, 0.09], [0.10, 0.11, 0.12]]],
-                          [1, 2, 3], [100, 200, 300])
+                        [[0.07, 0.08, 0.09], [0.10, 0.11, 0.12]]],
+                       [1, 2, 3], [100, 200, 300])
 
         y1, z1 = t([10, 20, 30])
         assert np.shape(y1) == np.shape(z1) == (2, 2, 3)
@@ -821,6 +819,128 @@ class TestSingleInputDoubleOutputSingleModel:
                                [131.10, 232.11, 333.12]]]])
 
 
+# test broadcasting rules
+broadcast_models = [
+    {
+        'model': models.Identity(2),
+        'inputs': [0, [1, 1]],
+        'outputs': [0, [1, 1]]
+    },
+    {
+        'model': models.Identity(2),
+        'inputs': [[1, 1], 0],
+        'outputs': [[1, 1], 0]
+    },
+    {
+        'model': models.Mapping((0, 1)),
+        'inputs': [0, [1, 1]],
+        'outputs': [0, [1, 1]]
+    },
+    {
+        'model': models.Mapping((1, 0)),
+        'inputs': [0, [1, 1]],
+        'outputs': [[1, 1], 0]
+    },
+    {
+        'model': models.Mapping((1, 0), n_inputs=3),
+        'inputs': [0, [1, 1], 2],
+        'outputs': [[1, 1], 0]
+    },
+    {
+        'model': models.Mapping((0, 1, 0)),
+        'inputs': [0, [1, 1]],
+        'outputs': [0, [1, 1], 0]
+    },
+    {
+        'model': models.Mapping((0, 1, 1)),
+        'inputs': [0, [1, 1]],
+        'outputs': [0, [1, 1], [1, 1]]
+    },
+    {
+        'model': models.Polynomial2D(1, c0_0=1),
+        'inputs': [0, [1, 1]],
+        'outputs': [1, 1]
+    },
+    {
+        'model': models.Polynomial2D(1, c0_0=1),
+        'inputs': [0, 1],
+        'outputs': 1
+    },
+    {
+        'model': models.Gaussian2D(1, 1, 2, 1, 1.2),
+        'inputs': [0, [1, 1]],
+        'outputs': [0.42860385, 0.42860385]
+    },
+    {
+        'model': models.Gaussian2D(1, 1, 2, 1, 1.2),
+        'inputs': [0, 1],
+        'outputs': 0.428603846153
+    },
+    {
+        'model': models.Polynomial2D(1, c0_0=1) & models.Polynomial2D(1, c0_0=2),
+        'inputs': [1, 1, 1, 1],
+        'outputs': (1, 2)
+    },
+    {
+        'model': models.Polynomial2D(1, c0_0=1) & models.Polynomial2D(1, c0_0=2),
+        'inputs': [1, 1, [1, 1], [1, 1]],
+        'outputs': (1, [2, 2])
+    },
+    {
+        'model': models.math.MultiplyUfunc(),
+        'inputs': [np.array([np.linspace(0, 1, 5)]).T, np.arange(2)],
+        'outputs': np.array([[0., 0.],
+                             [0., 0.25],
+                             [0., 0.5],
+                             [0., 0.75],
+                             [0., 1.]])
+    }
+]
+
+
+@pytest.mark.parametrize('model', broadcast_models)
+def test_mixed_input(model):
+    result = model['model'](*model['inputs'])
+    if np.isscalar(result):
+        assert_allclose(result, model['outputs'])
+    else:
+        for i in range(len(result)):
+            assert_allclose(result[i], model['outputs'][i])
+
+
+def test_more_outputs():
+
+    class M(FittableModel):
+        standard_broadcasting = False
+        n_inputs = 2
+        n_outputs = 3
+
+        a = Parameter()
+
+        def evaluate(self, x, y, a):
+            return a*x, a-x, a+y
+
+        def __call__(self, *args, **kwargs):
+            inputs, _ = super().prepare_inputs(*args, **kwargs)
+
+            outputs = self.evaluate(*inputs, *self.parameters)
+            output_shapes = [out.shape for out in outputs]
+            output_shapes = [() if shape == (1,) else shape for shape in output_shapes]
+            return self.prepare_outputs((tuple(output_shapes),), *outputs, **kwargs)
+
+    c = M(1)
+    result = c([1, 1], 1)
+    expected = [[1., 1.], [0., 0.], 2.]
+    for r, e in zip(result, expected):
+        assert_allclose(r, e)
+
+    c = M(1)
+    result = c(1, [1, 1])
+    expected = [1., 0., [2., 2.]]
+    for r, e in zip(result, expected):
+        assert_allclose(r, e)
+
+
 class TInputFormatter(Model):
     """
     A toy model to test input/output formatting.
@@ -855,9 +975,10 @@ def test_format_input_arrays_transposed():
 
 @pytest.mark.parametrize('model',
                          [models.Gaussian2D(), models.Polynomial2D(1,),
-                          models.Pix2Sky_TAN(), models.Tabular2D(lookup_table=np.ones((4,5)))])
+                          models.Rotation2D(), models.Pix2Sky_TAN(),
+                          models.Tabular2D(lookup_table=np.ones((4, 5)))])
 @pytest.mark.skipif('not HAS_SCIPY')
-def test_call_keyword_args_2(model):
+def test_call_keyword_args_1(model):
     """
     Test calling a model with positional, keywrd and a mixture of both arguments.
     """
@@ -882,10 +1003,9 @@ def test_call_keyword_args_2(model):
 
 @pytest.mark.parametrize('model',
                          [models.Gaussian1D(), models.Polynomial1D(1,),
-                          models.Tabular1D(lookup_table=np.ones((5,))),
-                          models.Rotation2D(), models.Pix2Sky_TAN()])
+                          models.Tabular1D(lookup_table=np.ones((5,)))])
 @pytest.mark.skipif('not HAS_SCIPY')
-def test_call_keyword_args_1(model):
+def test_call_keyword_args_2(model):
     """
     Test calling a model with positional, keywrd and a mixture of both arguments.
     """
@@ -913,13 +1033,13 @@ def test_call_keyword_args_1(model):
                           models.Gaussian2D() * models.Polynomial2D(1,),
                           models.Identity(2) | models.Polynomial2D(1),
                           models.Mapping((1,)) | models.Polynomial1D(1)])
-def test_call_keyword_args_1(model):
+def test_call_keyword_args_3(model):
     """
     Test calling a model with positional, keywrd and a mixture of both arguments.
     """
     positional = model(1, 2)
     model.inputs = ('r', 't')
-    assert_allclose(positional, model(r=1, t = 2))
+    assert_allclose(positional, model(r=1, t=2))
 
     assert_allclose(positional, model(1, t=2))
 
