@@ -2,20 +2,18 @@
 """
 This module tests model set evaluation and fitting for some common use cases.
 """
+import numpy as np
 # pylint: disable=invalid-name
 import pytest
-import numpy as np
 from numpy.testing import assert_allclose
 
-from astropy.modeling.models import (Polynomial1D, Polynomial2D, Legendre1D, Legendre2D,
-                                     Chebyshev2D, Chebyshev1D, Hermite1D, Hermite2D,
-                                     Linear1D, Planar2D)
-from astropy.modeling.fitting import LinearLSQFitter, FittingWithOutlierRemoval
 from astropy.modeling.core import Model
+from astropy.modeling.fitting import LinearLSQFitter
+from astropy.modeling.models import (
+    Chebyshev1D, Chebyshev2D, Hermite1D, Hermite2D, Legendre1D, Legendre2D, Linear1D, Planar2D,
+    Polynomial1D, Polynomial2D)
 from astropy.modeling.parameters import Parameter
 from astropy.utils import NumpyRNGContext
-from astropy.stats import sigma_clip
-
 
 x = np.arange(4)
 xx = np.array([x, x + 10])
@@ -144,7 +142,7 @@ def test_model2d_axis_2(model_class):
     can be evaluated with model_set_axis=False.
     """
     p2 = model_class(1, 1, c0_0=[[[0, 1, 2]]], c0_1=[[[3, 4, 5]]],
-                     c1_0=[[[5, 6, 7]]], c1_1=[[[1,1,1]]], n_models=3, model_set_axis=2)
+                     c1_0=[[[5, 6, 7]]], c1_1=[[[1, 1, 1]]], n_models=3, model_set_axis=2)
     t1 = model_class(1, 1, c0_0=0, c0_1=3, c1_0=5, c1_1=1)
     t2 = model_class(1, 1, c0_0=1, c0_1=4, c1_0=6, c1_1=1)
     t3 = model_class(1, 1, c0_0=2, c0_1=5, c1_0=7, c1_1=1)
@@ -302,19 +300,20 @@ def test_fitting_shapes():
 
     model = Linear1D(slope=[1, 2], intercept=[3, 4], n_models=2)
     y = model(xx)
-    fit_model = fitter(model, x, y)
+    fitter(model, x, y)
 
     model = Linear1D(slope=[[1, 2]], intercept=[[3, 4]], n_models=2, model_set_axis=1)
-    fit_model = fitter(model, x, y.T)
+    fitter(model, x, y.T)
 
     model = Planar2D(slope_x=[1, 2], slope_y=[1, 2], intercept=[3, 4], n_models=2)
     y = model(xx, xx)
-    fit_model = fitter(model, x, x, y)
+    fitter(model, x, x, y)
 
 
 def test_compound_model_sets():
     with pytest.raises(ValueError):
-        Polynomial1D(1, n_models=2, model_set_axis=1) | Polynomial1D(1, n_models=2, model_set_axis=0)
+        (Polynomial1D(1, n_models=2, model_set_axis=1) |
+         Polynomial1D(1, n_models=2, model_set_axis=0))
 
 
 def test_linear_fit_model_set_errors():
@@ -353,7 +352,7 @@ def test_linear_fit_model_set_common_weight():
     with pytest.raises(ValueError,
                        match='Found NaNs in the coefficient matrix'):
         with pytest.warns(RuntimeWarning,
-                          match=r'invalid value encountered in true_divide'):
+                          match=r'invalid value encountered in.*divide'):
             fitted_model = fitter(init_model, x, y, weights=np.zeros(10))
 
 
@@ -383,12 +382,12 @@ def test_linear_fit_model_set_weights():
     with pytest.raises(ValueError,
                        match='Found NaNs in the coefficient matrix'):
         with pytest.warns(RuntimeWarning,
-                          match=r'invalid value encountered in true_divide'):
+                          match=r'invalid value encountered in.*divide'):
             fitted_model = fitter(init_model, x, y, weights=weights)
 
     # Now we mask the values where weight is 0
     with pytest.warns(RuntimeWarning,
-                      match=r'invalid value encountered in true_divide'):
+                      match=r'invalid value encountered in.*divide'):
         fitted_model = fitter(init_model, x,
                               np.ma.array(y, mask=np.isclose(weights, 0)),
                               weights=weights)
