@@ -9,7 +9,8 @@ from astropy.stats.biweight import (biweight_location, biweight_scale,
                                     biweight_midvariance,
                                     biweight_midcovariance,
                                     biweight_midcorrelation)
-from astropy.tests.helper import catch_warnings
+from astropy.tests.helper import assert_quantity_allclose
+import astropy.units as u
 from astropy.utils.misc import NumpyRNGContext
 
 
@@ -236,10 +237,10 @@ def test_biweight_midvariance_small():
 
 def test_biweight_midvariance_5127():
     # test a regression introduced in #5127
-    rand = np.random.RandomState(12345)
+    rand = np.random.default_rng(12345)
     data = rand.normal(loc=0., scale=20., size=(100, 100))
     var = biweight_midvariance(data)
-    assert_allclose(var, 406.86938710817344)    # verified with R
+    assert_allclose(var, 409.87135608846205)
 
 
 def test_biweight_midvariance_axis():
@@ -451,13 +452,13 @@ def test_biweight_midcovariance_constant():
     cov = biweight_midcovariance(data)
     assert_allclose(cov, np.zeros((3, 3)))
 
-    with NumpyRNGContext(123):
-        data = np.random.random((5, 5))
-        val3 = 5.0
-        data[1] = [val3, 0.8, val3, -0.8, val3]
-        cov = biweight_midcovariance(data)
-        assert_allclose(cov[1, :], 0.)
-        assert_allclose(cov[:, 1], 0.)
+    rng = np.random.default_rng(123)
+    data = rng.random((5, 5))
+    val3 = 5.0
+    data[1] = [val3, 0.8, val3, -0.8, val3]
+    cov = biweight_midcovariance(data)
+    assert_allclose(cov[1, :], 0.)
+    assert_allclose(cov[:, 1], 0.)
 
 
 def test_biweight_midcovariance_midvariance():
@@ -466,7 +467,7 @@ def test_biweight_midcovariance_midvariance():
     biweight_midvariance.
     """
 
-    rng = np.random.RandomState(1)
+    rng = np.random.default_rng(1)
     d = rng.normal(0, 2, size=(100, 3))
     cov = biweight_midcovariance(d)
     var = [biweight_midvariance(a) for a in d]
@@ -508,7 +509,7 @@ def test_biweight_midcovariance_symmetric():
     when ``modify_sample_size=True`` (see #5972).
     """
 
-    rng = np.random.RandomState(1)
+    rng = np.random.default_rng(1)
     d = rng.gamma(2, 2, size=(3, 500))
     cov = biweight_midcovariance(d)
     assert_array_almost_equal_nulp(cov, cov.T, nulp=5)
@@ -552,11 +553,25 @@ def test_biweight_32bit_runtime_warnings():
     with NumpyRNGContext(12345):
         data = np.random.random(100).astype(np.float32)
         data[50] = 30000.
+        biweight_scale(data)
+        biweight_midvariance(data)
 
-        with catch_warnings(RuntimeWarning) as warning_lines:
-            biweight_scale(data)
-            assert len(warning_lines) == 0
 
-        with catch_warnings(RuntimeWarning) as warning_lines:
-            biweight_midvariance(data)
-            assert len(warning_lines) == 0
+def test_biweight_scl_var_constant_units():
+    unit = u.km
+    data = np.ones(10) << unit
+    biwscl = biweight_scale(data)
+    biwvar = biweight_midvariance(data)
+    assert isinstance(biwscl, u.Quantity)
+    assert isinstance(biwvar, u.Quantity)
+    assert_quantity_allclose(biwscl, 0. << unit)
+    assert_quantity_allclose(biwvar, 0. << unit ** 2)
+
+    data = np.ones(10) << unit
+    data[0] = np.nan
+    biwscl = biweight_scale(data)
+    biwvar = biweight_midvariance(data)
+    assert isinstance(biwscl, u.Quantity)
+    assert isinstance(biwvar, u.Quantity)
+    assert_quantity_allclose(biwscl, np.nan << unit)
+    assert_quantity_allclose(biwvar, np.nan << unit ** 2)
