@@ -3,21 +3,19 @@
 import itertools
 import warnings
 import weakref
-import re
 
 from copy import deepcopy
 
 import numpy as np
 from numpy import ma
 
-from astropy.units import Unit, Quantity
+from astropy.units import Unit, Quantity, StructuredUnit
 from astropy.utils.console import color_print
 from astropy.utils.metadata import MetaData
 from astropy.utils.data_info import BaseColumnInfo, dtype_info_name
 from astropy.utils.misc import dtype_bytes_or_chars
 from . import groups
 from . import pprint
-from .np_utils import fix_column_name
 
 # These "shims" provide __getitem__ implementations for Column and MaskedColumn
 from ._column_mixins import _ColumnGetitemShim, _MaskedColumnGetitemShim
@@ -75,26 +73,16 @@ def col_copy(col, copy_indices=True):
     if isinstance(col, BaseColumn):
         return col.copy()
 
-    # The new column should have None for the parent_table ref.  If the
-    # original parent_table weakref there at the point of copying then it
-    # generates an infinite recursion.  Instead temporarily remove the weakref
-    # on the original column and restore after the copy in an exception-safe
-    # manner.
-
-    parent_table = col.info.parent_table
-    indices = col.info.indices
-    col.info.parent_table = None
-    col.info.indices = []
-
-    try:
-        newcol = col.copy() if hasattr(col, 'copy') else deepcopy(col)
+    newcol = col.copy() if hasattr(col, 'copy') else deepcopy(col)
+    # If the column has info defined, we copy it and adjust any indices
+    # to point to the copied column.  By guarding with the if statement,
+    # we avoid side effects (of creating the default info instance).
+    if 'info' in col.__dict__:
         newcol.info = col.info
-        newcol.info.indices = deepcopy(indices or []) if copy_indices else []
-        for index in newcol.info.indices:
-            index.replace_col(col, newcol)
-    finally:
-        col.info.parent_table = parent_table
-        col.info.indices = indices
+        if copy_indices and col.info.indices:
+            newcol.info.indices = deepcopy(col.info.indices)
+            for index in newcol.info.indices:
+                index.replace_col(col, newcol)
 
     return newcol
 
@@ -135,7 +123,7 @@ def _expand_string_array_for_values(arr, values):
     ----------
     arr : np.ndarray
         Input array
-    values : scalar or array_like
+    values : scalar or array-like
         Values for width comparison for string arrays
 
     Returns
@@ -181,7 +169,7 @@ def _convert_sequence_data_to_array(data, dtype=None):
     ----------
     data : N-d sequence
         Input data, typically list or list of lists
-    dtype : None or dtype-compatible
+    dtype : None or dtype-like
         Output datatype (None lets np.array choose)
 
     Returns
@@ -299,6 +287,52 @@ def _convert_sequence_data_to_array(data, dtype=None):
     return np_data
 
 
+def _make_compare(oper):
+    """
+    Make Column comparison methods which encode the ``other`` object to utf-8
+    in the case of a bytestring dtype for Py3+.
+
+    Parameters
+    ----------
+    oper : str
+        Operator name
+    """
+    swapped_oper = {'__eq__': '__eq__',
+                    '__ne__': '__ne__',
+                    '__gt__': '__lt__',
+                    '__lt__': '__gt__',
+                    '__ge__': '__le__',
+                    '__le__': '__ge__'}[oper]
+
+    def _compare(self, other):
+        op = oper  # copy enclosed ref to allow swap below
+
+        # Special case to work around #6838.  Other combinations work OK,
+        # see tests.test_column.test_unicode_sandwich_compare().  In this
+        # case just swap self and other.
+        #
+        # This is related to an issue in numpy that was addressed in np 1.13.
+        # However that fix does not make this problem go away, but maybe
+        # future numpy versions will do so.  NUMPY_LT_1_13 to get the
+        # attention of future maintainers to check (by deleting or versioning
+        # the if block below).  See #6899 discussion.
+        # 2019-06-21: still needed with numpy 1.16.
+        if (isinstance(self, MaskedColumn) and self.dtype.kind == 'U'
+                and isinstance(other, MaskedColumn) and other.dtype.kind == 'S'):
+            self, other = other, self
+            op = swapped_oper
+
+        if self.dtype.char == 'S':
+            other = self._encode_str(other)
+
+        # Now just let the regular ndarray.__eq__, etc., take over.
+        result = getattr(super(Column, self), op)(other)
+        # But we should not return Column instances for this case.
+        return result.data if isinstance(result, Column) else result
+
+    return _compare
+
+
 class ColumnInfo(BaseColumnInfo):
     """
     Container for meta information like name, description, format.
@@ -306,8 +340,98 @@ class ColumnInfo(BaseColumnInfo):
     This is required when the object is used as a mixin column within a table,
     but can be used as a general way to store meta information.
     """
-    attrs_from_parent = BaseColumnInfo.attr_names
+    attr_names = BaseColumnInfo.attr_names | {'groups'}
+    _attrs_no_copy = BaseColumnInfo._attrs_no_copy | {'groups'}
+    attrs_from_parent = attr_names
     _supports_indexing = True
+    # For structured columns, data is used to store a dict of columns.
+    # Store entries in that dict as name.key instead of name.data.key.
+    _represent_as_dict_primary_data = 'data'
+
+    def _represent_as_dict(self):
+        result = super()._represent_as_dict()
+        names = self._parent.dtype.names
+        # For a regular column, we are done, but for a structured
+        # column, we use a SerializedColumns to store the pieces.
+        if names is None:
+            return result
+
+        from .serialize import SerializedColumn
+
+        data = SerializedColumn()
+        # If this column has a StructuredUnit, we split it and store
+        # it on the corresponding part. Otherwise, we just store it
+        # as an attribute below.  All other attributes we remove from
+        # the parts, so that we do not store them multiple times.
+        # (Note that attributes are not linked to the parent, so it
+        # is safe to reset them.)
+        # TODO: deal with (some of) this in Column.__getitem__?
+        # Alternatively: should we store info on the first part?
+        # TODO: special-case format somehow? Can we have good formats
+        # for structured columns?
+        unit = self.unit
+        if isinstance(unit, StructuredUnit) and len(unit) == len(names):
+            units = unit.values()
+            unit = None  # No need to store as an attribute as well.
+        else:
+            units = [None] * len(names)
+        for name, part_unit in zip(names, units):
+            part = self._parent[name]
+            part.unit = part_unit
+            part.description = None
+            part.meta = {}
+            part.format = None
+            data[name] = part
+
+        # Create the attributes required to reconstruct the column.
+        result['data'] = data
+        # Store the shape if needed. Just like scalar data, a structured data
+        # column (e.g. with dtype `f8,i8`) can be multidimensional within each
+        # row and have a shape, and that needs to be distinguished from the
+        # case that each entry in the structure has the same shape (e.g.,
+        # distinguist a column with dtype='f8,i8' and 2 elements per row from
+        # one with dtype '2f8,2i8' and just one element per row).
+        if shape := self._parent.shape[1:]:
+            result['shape'] = list(shape)
+        # Also store the standard info attributes since these are
+        # stored on the parent and can thus just be passed on as
+        # arguments.  TODO: factor out with essentially the same
+        # code in serialize._represent_mixin_as_column.
+        if unit is not None and unit != '':
+            result['unit'] = unit
+        if self.format is not None:
+            result['format'] = self.format
+        if self.description is not None:
+            result['description'] = self.description
+        if self.meta:
+            result['meta'] = self.meta
+
+        return result
+
+    def _construct_from_dict(self, map):
+        if not isinstance(map.get('data'), dict):
+            return super()._construct_from_dict(map)
+
+        # Reconstruct a structured Column, by first making an empty column
+        # and then filling it with the structured data.
+        data = map.pop('data')
+        shape = tuple(map.pop('shape', ()))
+        # There are three elements in the shape of `part`:
+        # (table length, shape of structured column, shape of part like '3f8')
+        # The column `shape` only includes the second, so by adding one to its
+        # length to include the table length, we pick off a possible last bit.
+        dtype = np.dtype([(name, part.dtype, part.shape[len(shape)+1:])
+                          for name, part in data.items()])
+        units = tuple(col.info.unit for col in data.values())
+        if all(unit is not None for unit in units):
+            map['unit'] = StructuredUnit(units, dtype)
+        map.update(dtype=dtype, shape=shape, length=len(data[dtype.names[0]]))
+        # Construct the empty column from `map` (note: 'data' removed above).
+        result = super()._construct_from_dict(map)
+        # Fill it with the structured data.
+        for name in dtype.names:
+            result[name] = data[name]
+        return result
 
     def new_like(self, cols, length, metadata_conflicts='warn', name=None):
         """
@@ -384,13 +508,15 @@ class BaseColumn(_ColumnGetitemShim, np.ndarray):
                 self_data = np.array(data, dtype=dtype, copy=copy)
                 unit = data.unit
             else:
-                self_data = np.array(data.to(unit), dtype=dtype, copy=copy)
-            if description is None:
-                description = data.info.description
-            if format is None:
-                format = data.info.format
-            if meta is None:
-                meta = data.info.meta
+                self_data = Quantity(data, unit, dtype=dtype, copy=copy).value
+            # If 'info' has been defined, copy basic properties (if needed).
+            if 'info' in data.__dict__:
+                if description is None:
+                    description = data.info.description
+                if format is None:
+                    format = data.info.format
+                if meta is None:
+                    meta = data.info.meta
 
         else:
             if np.dtype(dtype).char == 'S':
@@ -398,7 +524,7 @@ class BaseColumn(_ColumnGetitemShim, np.ndarray):
             self_data = np.array(data, dtype=dtype, copy=copy)
 
         self = self_data.view(cls)
-        self._name = fix_column_name(name)
+        self._name = None if name is None else str(name)
         self._parent_table = None
         self.unit = unit
         self._format = format
@@ -414,6 +540,13 @@ class BaseColumn(_ColumnGetitemShim, np.ndarray):
     def data(self):
         return self.view(np.ndarray)
 
+    @property
+    def value(self):
+        """
+        An alias for the existing ``data`` attribute.
+        """
+        return self.data
+
     @property
     def parent_table(self):
         # Note: It seems there are some cases where _parent_table is not set,
@@ -541,6 +674,8 @@ class BaseColumn(_ColumnGetitemShim, np.ndarray):
         if not hasattr(self, 'indices'):  # may have been copied in __new__
             self.indices = []
         self._copy_attrs(obj)
+        if 'info' in getattr(obj, '__dict__', {}):
+            self.info = obj.info
 
     def __array_wrap__(self, out_arr, context=None):
         """
@@ -563,9 +698,10 @@ class BaseColumn(_ColumnGetitemShim, np.ndarray):
            (see #1446 and #1685)
         """
         out_arr = super().__array_wrap__(out_arr, context)
-        if (self.shape != out_arr.shape or
-            (isinstance(out_arr, BaseColumn) and
-             (context is not None and context[0] in _comparison_functions))):
+        if (self.shape != out_arr.shape
+            or (isinstance(out_arr, BaseColumn)
+                and (context is not None
+                     and context[0] in _comparison_functions))):
             return out_arr.data[()]
         else:
             return out_arr
@@ -579,7 +715,8 @@ class BaseColumn(_ColumnGetitemShim, np.ndarray):
 
     @name.setter
     def name(self, val):
-        val = fix_column_name(val)
+        if val is not None:
+            val = str(val)
 
         if self.parent_table is not None:
             table = self.parent_table
@@ -610,8 +747,8 @@ class BaseColumn(_ColumnGetitemShim, np.ndarray):
             self._format = prev_format
             raise ValueError(
                 "Invalid format for column '{}': could not display "
-                "values in this column using this format ({})".format(
-                    self.name, err.args[0]))
+                "values in this column using this format".format(
+                    self.name)) from err
 
     @property
     def descr(self):
@@ -796,6 +933,18 @@ class BaseColumn(_ColumnGetitemShim, np.ndarray):
     def unit(self):
         self._unit = None
 
+    def searchsorted(self, v, side='left', sorter=None):
+        # For bytes type data, encode the `v` value as UTF-8 (if necessary) before
+        # calling searchsorted. This prevents a factor of 1000 slowdown in
+        # searchsorted in this case.
+        a = self.data
+        if a.dtype.kind == 'S' and not isinstance(v, bytes):
+            v = np.asarray(v)
+            if v.dtype.kind == 'U':
+                v = np.char.encode(v, 'utf-8')
+        return np.searchsorted(a, v, side=side, sorter=sorter)
+    searchsorted.__doc__ = np.ndarray.searchsorted.__doc__
+
     def convert_unit_to(self, new_unit, equivalencies=[]):
         """
         Converts the values of the column in-place from the current
@@ -810,9 +959,9 @@ class BaseColumn(_ColumnGetitemShim, np.ndarray):
         new_unit : str or `astropy.units.UnitBase` instance
             The unit to convert to.
 
-        equivalencies : list of equivalence pairs, optional
+        equivalencies : list of tuple
            A list of equivalence pairs to try if the unit are not
-           directly convertible.  See :ref:`unit_equivalencies`.
+           directly convertible.  See :ref:`astropy:unit_equivalencies`.
 
         Raises
         ------
@@ -889,10 +1038,10 @@ class BaseColumn(_ColumnGetitemShim, np.ndarray):
 
         Parameters
         ----------
-        unit : `~astropy.units.Unit` or str
+        unit : unit-like
             The unit to convert to (i.e., a valid argument to the
             :meth:`astropy.units.Quantity.to` method).
-        equivalencies : list of equivalence pairs, optional
+        equivalencies : list of tuple
             Equivalencies to use for this conversion.  See
             :meth:`astropy.units.Quantity.to` for more details.
 
@@ -949,11 +1098,11 @@ class Column(BaseColumn):
 
     Parameters
     ----------
-    data : list, ndarray or None
+    data : list, ndarray, or None
         Column data values
     name : str
         Column name and key for reference within Table
-    dtype : numpy.dtype compatible value
+    dtype : `~numpy.dtype`-like
         Data type for column
     shape : tuple or ()
         Dimensions of a single row element in the column data
@@ -963,7 +1112,7 @@ class Column(BaseColumn):
         Full description of column
     unit : str or None
         Physical unit
-    format : str or None or function or callable
+    format : str, None, or callable
         Format string for outputting column values.  This can be an
         "old-style" (``format % value``) or "new-style" (`str.format`)
         format specification string or a function or any callable object that
@@ -1007,6 +1156,12 @@ class Column(BaseColumn):
 
       The default ``dtype`` is ``np.float64``.  The ``shape`` argument is the
       array shape of a single cell in the column.
+
+    To access the ``Column`` data as a raw `numpy.ndarray` object, you can use
+    one of the ``data`` or ``value`` attributes (which are equivalent)::
+
+        col.data
+        col.value
     """
 
     def __new__(cls, data=None, name=None,
@@ -1124,46 +1279,6 @@ class Column(BaseColumn):
         # order-of-magnitude speed-up. [#2994]
         self.data[index] = value
 
-    def _make_compare(oper):
-        """
-        Make comparison methods which encode the ``other`` object to utf-8
-        in the case of a bytestring dtype for Py3+.
-        """
-        swapped_oper = {'__eq__': '__eq__',
-                        '__ne__': '__ne__',
-                        '__gt__': '__lt__',
-                        '__lt__': '__gt__',
-                        '__ge__': '__le__',
-                        '__le__': '__ge__'}[oper]
-
-        def _compare(self, other):
-            op = oper  # copy enclosed ref to allow swap below
-
-            # Special case to work around #6838.  Other combinations work OK,
-            # see tests.test_column.test_unicode_sandwich_compare().  In this
-            # case just swap self and other.
-            #
-            # This is related to an issue in numpy that was addressed in np 1.13.
-            # However that fix does not make this problem go away, but maybe
-            # future numpy versions will do so.  NUMPY_LT_1_13 to get the
-            # attention of future maintainers to check (by deleting or versioning
-            # the if block below).  See #6899 discussion.
-            # 2019-06-21: still needed with numpy 1.16.
-            if (isinstance(self, MaskedColumn) and self.dtype.kind == 'U' and
-                    isinstance(other, MaskedColumn) and other.dtype.kind == 'S'):
-                self, other = other, self
-                op = swapped_oper
-
-            if self.dtype.char == 'S':
-                other = self._encode_str(other)
-
-            # Now just let the regular ndarray.__eq__, etc., take over.
-            result = getattr(super(), op)(other)
-            # But we should not return Column instances for this case.
-            return result.data if isinstance(result, Column) else result
-
-        return _compare
-
     __eq__ = _make_compare('__eq__')
     __ne__ = _make_compare('__ne__')
     __gt__ = _make_compare('__gt__')
@@ -1178,10 +1293,10 @@ class Column(BaseColumn):
 
         Parameters
         ----------
-        obj : int, slice or sequence of ints
+        obj : int, slice or sequence of int
             Object that defines the index or indices before which ``values`` is
             inserted.
-        values : array_like
+        values : array-like
             Value(s) to insert.  If the type of ``values`` is different from
             that of the column, ``values`` is converted to the matching type.
             ``values`` should be shaped so that it can be broadcast appropriately.
@@ -1256,10 +1371,16 @@ class MaskedColumnInfo(ColumnInfo):
             self.serialize_method = {'fits': 'null_value',
                                      'ecsv': 'null_value',
                                      'hdf5': 'data_mask',
+                                     'parquet': 'data_mask',
                                      None: 'null_value'}
 
     def _represent_as_dict(self):
         out = super()._represent_as_dict()
+        # If we are a structured masked column, then our parent class,
+        # ColumnInfo, will already have set up a dict with masked parts,
+        # which will be serialized later, so no further work needed here.
+        if self._parent.dtype.names is not None:
+            return out
 
         col = self._parent
 
@@ -1293,15 +1414,15 @@ class MaskedColumn(Column, _MaskedColumnGetitemShim, ma.MaskedArray):
 
     Parameters
     ----------
-    data : list, ndarray or None
+    data : list, ndarray, or None
         Column data values
     name : str
         Column name and key for reference within Table
     mask : list, ndarray or None
         Boolean mask for which True indicates missing or invalid data
-    fill_value : float, int, str or None
+    fill_value : float, int, str, or None
         Value used when filling masked column elements
-    dtype : numpy.dtype compatible value
+    dtype : `~numpy.dtype`-like
         Data type for column
     shape : tuple or ()
         Dimensions of a single row element in the column data
@@ -1311,7 +1432,7 @@ class MaskedColumn(Column, _MaskedColumnGetitemShim, ma.MaskedArray):
         Full description of column
     unit : str or None
         Physical unit
-    format : str or None or function or callable
+    format : str, None, or callable
         Format string for outputting column values.  This can be an
         "old-style" (``format % value``) or "new-style" (`str.format`)
         format specification string or a function or any callable object that
@@ -1358,6 +1479,12 @@ class MaskedColumn(Column, _MaskedColumnGetitemShim, ma.MaskedArray):
 
       The default ``dtype`` is ``np.float64``.  The ``shape`` argument is the
       array shape of a single cell in the column.
+
+    To access the ``Column`` data as a raw `numpy.ma.MaskedArray` object, you can
+    use one of the ``data`` or ``value`` attributes (which are equivalent)::
+
+        col.data
+        col.value
     """
     info = MaskedColumnInfo()
 
@@ -1397,6 +1524,11 @@ class MaskedColumn(Column, _MaskedColumnGetitemShim, ma.MaskedArray):
                                unit=unit, format=format, description=description,
                                meta=meta, copy=copy, copy_indices=copy_indices)
         self = ma.MaskedArray.__new__(cls, data=self_data, mask=mask)
+        # The above process preserves info relevant for Column, but this does
+        # not include serialize_method (and possibly other future attributes)
+        # relevant for MaskedColumn, so we set info explicitly.
+        if 'info' in getattr(data, '__dict__', {}):
+            self.info = data.info
 
         # Note: do not set fill_value in the MaskedArray constructor because this does not
         # go through the fill_value workarounds.
@@ -1486,14 +1618,14 @@ class MaskedColumn(Column, _MaskedColumnGetitemShim, ma.MaskedArray):
 
         Parameters
         ----------
-        obj : int, slice or sequence of ints
+        obj : int, slice or sequence of int
             Object that defines the index or indices before which ``values`` is
             inserted.
-        values : array_like
+        values : array-like
             Value(s) to insert.  If the type of ``values`` is different from
             that of the column, ``values`` is converted to the matching type.
             ``values`` should be shaped so that it can be broadcast appropriately.
-        mask : bool or array_like
+        mask : bool or array-like
             Mask value(s) to insert.  If not supplied, and values does not have
             a mask either, then False is used.
         axis : int, optional
@@ -1542,6 +1674,11 @@ class MaskedColumn(Column, _MaskedColumnGetitemShim, ma.MaskedArray):
         # Fixes issue #3023: when calling getitem with a MaskedArray subclass
         # the original object attributes are not copied.
         if out.__class__ is self.__class__:
+            # TODO: this part is essentially the same as what is done in
+            # __array_finalize__ and could probably be called directly in our
+            # override of __getitem__ in _columns_mixins.pyx). Refactor?
+            if 'info' in self.__dict__:
+                out.info = self.info
             out.parent_table = None
             # we need this because __getitem__ does a shallow copy of indices
             if out.indices is self.indices:
