@@ -18,14 +18,17 @@ from collections.abc import Mapping, Sequence
 import numpy as np
 
 from astropy.utils import metadata
+from astropy.utils.masked import Masked
 from .table import Table, QTable, Row, Column, MaskedColumn
 from astropy.units import Quantity
-from astropy.utils.compat import NUMPY_LT_1_17
 
 from . import _np_utils
-from .np_utils import fix_column_name, TableMergeError
+from .np_utils import TableMergeError
 
-__all__ = ['join', 'setdiff', 'hstack', 'vstack', 'unique']
+__all__ = ['join', 'setdiff', 'hstack', 'vstack', 'unique',
+           'join_skycoord', 'join_distance']
+
+__doctest_requires__ = {'join_skycoord': ['scipy'], 'join_distance': ['scipy']}
 
 
 def _merge_table_meta(out, tables, metadata_conflicts='warn'):
@@ -61,9 +64,8 @@ def _get_list_of_tables(tables):
         else:
             try:
                 tables[ii] = Table([val])
-            except (ValueError, TypeError):
-                raise TypeError('cannot convert {} to table column.'
-                                .format(val))
+            except (ValueError, TypeError) as err:
+                raise TypeError(f'Cannot convert {val} to table column.') from err
 
     return tables
 
@@ -73,37 +75,292 @@ def _get_out_class(objs):
     From a list of input objects ``objs`` get merged output object class.
 
     This is just taken as the deepest subclass. This doesn't handle complicated
-    inheritance schemes.
+    inheritance schemes, but as a special case, classes which share ``info``
+    are taken to be compatible.
     """
     out_class = objs[0].__class__
     for obj in objs[1:]:
         if issubclass(obj.__class__, out_class):
             out_class = obj.__class__
 
-    if any(not issubclass(out_class, obj.__class__) for obj in objs):
+    if any(not (issubclass(out_class, obj.__class__)
+                or out_class.info is obj.__class__.info) for obj in objs):
         raise ValueError('unmergeable object classes {}'
                          .format([obj.__class__.__name__ for obj in objs]))
 
     return out_class
 
 
-def join(left, right, keys=None, join_type='inner',
+def join_skycoord(distance, distance_func='search_around_sky'):
+    """Helper function to join on SkyCoord columns using distance matching.
+
+    This function is intended for use in ``table.join()`` to allow performing a
+    table join where the key columns are both ``SkyCoord`` objects, matched by
+    computing the distance between points and accepting values below
+    ``distance``.
+
+    The distance cross-matching is done using either
+    `~astropy.coordinates.search_around_sky` or
+    `~astropy.coordinates.search_around_3d`, depending on the value of
+    ``distance_func``.  The default is ``'search_around_sky'``.
+
+    One can also provide a function object for ``distance_func``, in which case
+    it must be a function that follows the same input and output API as
+    `~astropy.coordinates.search_around_sky`. In this case the function will
+    be called with ``(skycoord1, skycoord2, distance)`` as arguments.
+
+    Parameters
+    ----------
+    distance : `~astropy.units.Quantity` ['angle', 'length']
+        Maximum distance between points to be considered a join match.
+        Must have angular or distance units.
+    distance_func : str or function
+        Specifies the function for performing the cross-match based on
+        ``distance``. If supplied as a string this specifies the name of a
+        function in `astropy.coordinates`. If supplied as a function then that
+        function is called directly.
+
+    Returns
+    -------
+    join_func : function
+        Function that accepts two ``SkyCoord`` columns (col1, col2) and returns
+        the tuple (ids1, ids2) of pair-matched unique identifiers.
+
+    Examples
+    --------
+    This example shows an inner join of two ``SkyCoord`` columns, taking any
+    sources within 0.2 deg to be a match.  Note the new ``sc_id`` column which
+    is added and provides a unique source identifier for the matches.
+
+      >>> from astropy.coordinates import SkyCoord
+      >>> import astropy.units as u
+      >>> from astropy.table import Table, join_skycoord
+      >>> from astropy import table
+
+      >>> sc1 = SkyCoord([0, 1, 1.1, 2], [0, 0, 0, 0], unit='deg')
+      >>> sc2 = SkyCoord([0.5, 1.05, 2.1], [0, 0, 0], unit='deg')
+
+      >>> join_func = join_skycoord(0.2 * u.deg)
+      >>> join_func(sc1, sc2)  # Associate each coordinate with unique source ID
+      (array([3, 1, 1, 2]), array([4, 1, 2]))
+
+      >>> t1 = Table([sc1], names=['sc'])
+      >>> t2 = Table([sc2], names=['sc'])
+      >>> t12 = table.join(t1, t2, join_funcs={'sc': join_skycoord(0.2 * u.deg)})
+      >>> print(t12)  # Note new `sc_id` column with the IDs from join_func()
+      sc_id   sc_1    sc_2
+            deg,deg deg,deg
+      ----- ------- --------
+          1 1.0,0.0 1.05,0.0
+          1 1.1,0.0 1.05,0.0
+          2 2.0,0.0  2.1,0.0
+
+    """
+    if isinstance(distance_func, str):
+        import astropy.coordinates as coords
+        try:
+            distance_func = getattr(coords, distance_func)
+        except AttributeError as err:
+            raise ValueError('distance_func must be a function in astropy.coordinates') from err
+    else:
+        from inspect import isfunction
+        if not isfunction(distance_func):
+            raise ValueError('distance_func must be a str or function')
+
+    def join_func(sc1, sc2):
+
+        # Call the appropriate SkyCoord method to find pairs within distance
+        idxs1, idxs2, d2d, d3d = distance_func(sc1, sc2, distance)
+
+        # Now convert that into unique identifiers for each near-pair. This is
+        # taken to be transitive, so that if points 1 and 2 are "near" and points
+        # 1 and 3 are "near", then 1, 2, and 3 are all given the same identifier.
+        # This identifier will then be used in the table join matching.
+
+        # Identifiers for each column, initialized to all zero.
+        ids1 = np.zeros(len(sc1), dtype=int)
+        ids2 = np.zeros(len(sc2), dtype=int)
+
+        # Start the identifier count at 1
+        id_ = 1
+        for idx1, idx2 in zip(idxs1, idxs2):
+            # If this col1 point is previously identified then set corresponding
+            # col2 point to same identifier.  Likewise for col2 and col1.
+            if ids1[idx1] > 0:
+                ids2[idx2] = ids1[idx1]
+            elif ids2[idx2] > 0:
+                ids1[idx1] = ids2[idx2]
+            else:
+                # Not yet seen so set identifier for col1 and col2
+                ids1[idx1] = id_
+                ids2[idx2] = id_
+                id_ += 1
+
+        # Fill in unique identifiers for points with no near neighbor
+        for ids in (ids1, ids2):
+            for idx in np.flatnonzero(ids == 0):
+                ids[idx] = id_
+                id_ += 1
+
+        # End of enclosure join_func()
+        return ids1, ids2
+
+    return join_func
+
+
+def join_distance(distance, kdtree_args=None, query_args=None):
+    """Helper function to join table columns using distance matching.
+
+    This function is intended for use in ``table.join()`` to allow performing
+    a table join where the key columns are matched by computing the distance
+    between points and accepting values below ``distance``. This numerical
+    "fuzzy" match can apply to 1-D or 2-D columns, where in the latter case
+    the distance is a vector distance.
+
+    The distance cross-matching is done using `scipy.spatial.cKDTree`. If
+    necessary you can tweak the default behavior by providing ``dict`` values
+    for the ``kdtree_args`` or ``query_args``.
+
+    Parameters
+    ----------
+    distance : float or `~astropy.units.Quantity` ['length']
+        Maximum distance between points to be considered a join match
+    kdtree_args : dict, None
+        Optional extra args for `~scipy.spatial.cKDTree`
+    query_args : dict, None
+        Optional extra args for `~scipy.spatial.cKDTree.query_ball_tree`
+
+    Returns
+    -------
+    join_func : function
+        Function that accepts (skycoord1, skycoord2) and returns the tuple
+        (ids1, ids2) of pair-matched unique identifiers.
+
+    Examples
+    --------
+
+      >>> from astropy.table import Table, join_distance
+      >>> from astropy import table
+
+      >>> c1 = [0, 1, 1.1, 2]
+      >>> c2 = [0.5, 1.05, 2.1]
+
+      >>> t1 = Table([c1], names=['col'])
+      >>> t2 = Table([c2], names=['col'])
+      >>> t12 = table.join(t1, t2, join_type='outer', join_funcs={'col': join_distance(0.2)})
+      >>> print(t12)
+      col_id col_1 col_2
+      ------ ----- -----
+           1   1.0  1.05
+           1   1.1  1.05
+           2   2.0   2.1
+           3   0.0    --
+           4    --   0.5
+
+    """
+    try:
+        from scipy.spatial import cKDTree
+    except ImportError as exc:
+        raise ImportError('scipy is required to use join_distance()') from exc
+
+    if kdtree_args is None:
+        kdtree_args = {}
+    if query_args is None:
+        query_args = {}
+
+    def join_func(col1, col2):
+        if col1.ndim > 2 or col2.ndim > 2:
+            raise ValueError('columns for isclose_join must be 1- or 2-dimensional')
+
+        if isinstance(distance, Quantity):
+            # Convert to np.array with common unit
+            col1 = col1.to_value(distance.unit)
+            col2 = col2.to_value(distance.unit)
+            dist = distance.value
+        else:
+            # Convert to np.array to allow later in-place shape changing
+            col1 = np.asarray(col1)
+            col2 = np.asarray(col2)
+            dist = distance
+
+        # Ensure columns are pure np.array and are 2-D for use with KDTree
+        if col1.ndim == 1:
+            col1.shape = col1.shape + (1,)
+        if col2.ndim == 1:
+            col2.shape = col2.shape + (1,)
+
+        # Cross-match col1 and col2 within dist using KDTree
+        kd1 = cKDTree(col1, **kdtree_args)
+        kd2 = cKDTree(col2, **kdtree_args)
+        nears = kd1.query_ball_tree(kd2, r=dist, **query_args)
+
+        # Output of above is nears which is a list of lists, where the outer
+        # list corresponds to each item in col1, and where the inner lists are
+        # indexes into col2 of elements within the distance tolerance.  This
+        # identifies col1 / col2 near pairs.
+
+        # Now convert that into unique identifiers for each near-pair. This is
+        # taken to be transitive, so that if points 1 and 2 are "near" and points
+        # 1 and 3 are "near", then 1, 2, and 3 are all given the same identifier.
+        # This identifier will then be used in the table join matching.
+
+        # Identifiers for each column, initialized to all zero.
+        ids1 = np.zeros(len(col1), dtype=int)
+        ids2 = np.zeros(len(col2), dtype=int)
+
+        # Start the identifier count at 1
+        id_ = 1
+        for idx1, idxs2 in enumerate(nears):
+            for idx2 in idxs2:
+                # If this col1 point is previously identified then set corresponding
+                # col2 point to same identifier.  Likewise for col2 and col1.
+                if ids1[idx1] > 0:
+                    ids2[idx2] = ids1[idx1]
+                elif ids2[idx2] > 0:
+                    ids1[idx1] = ids2[idx2]
+                else:
+                    # Not yet seen so set identifier for col1 and col2
+                    ids1[idx1] = id_
+                    ids2[idx2] = id_
+                    id_ += 1
+
+        # Fill in unique identifiers for points with no near neighbor
+        for ids in (ids1, ids2):
+            for idx in np.flatnonzero(ids == 0):
+                ids[idx] = id_
+                id_ += 1
+
+        # End of enclosure join_func()
+        return ids1, ids2
+
+    return join_func
+
+
+def join(left, right, keys=None, join_type='inner', *,
+         keys_left=None, keys_right=None,
          uniq_col_name='{col_name}_{table_name}',
-         table_names=['1', '2'], metadata_conflicts='warn'):
+         table_names=['1', '2'], metadata_conflicts='warn',
+         join_funcs=None):
     """
     Perform a join of the left table with the right table on specified keys.
 
     Parameters
     ----------
-    left : Table object or a value that will initialize a Table object
-        Left side table in the join
-    right : Table object or a value that will initialize a Table object
-        Right side table in the join
+    left : `~astropy.table.Table`-like object
+        Left side table in the join. If not a Table, will call ``Table(left)``
+    right : `~astropy.table.Table`-like object
+        Right side table in the join. If not a Table, will call ``Table(right)``
     keys : str or list of str
         Name(s) of column(s) used to match rows of left and right tables.
         Default is to use all columns which are common to both tables.
     join_type : str
         Join type ('inner' | 'outer' | 'left' | 'right' | 'cartesian'), default is 'inner'
+    keys_left : str or list of str or list of column-like, optional
+        Left column(s) used to match rows instead of ``keys`` arg. This can be
+        be a single left table column name or list of column names, or a list of
+        column-like values with the same lengths as the left table.
+    keys_right : str or list of str or list of column-like, optional
+        Same as ``keys_left``, but for the right side of the join.
     uniq_col_name : str or None
         String generate a unique output column name in case of a conflict.
         The default is '{col_name}_{table_name}'.
@@ -115,6 +372,9 @@ def join(left, right, keys=None, join_type='inner',
             * ``'silent'``: silently pick the last conflicting meta-data value
             * ``'warn'``: pick the last conflicting meta-data value, but emit a warning (default)
             * ``'error'``: raise an exception.
+    join_funcs : dict, None
+        Dict of functions to use for matching the corresponding key column(s).
+        See `~astropy.table.join_skycoord` for an example and details.
 
     Returns
     -------
@@ -130,7 +390,9 @@ def join(left, right, keys=None, join_type='inner',
 
     col_name_map = OrderedDict()
     out = _join(left, right, keys, join_type,
-                uniq_col_name, table_names, col_name_map, metadata_conflicts)
+                uniq_col_name, table_names, col_name_map, metadata_conflicts,
+                join_funcs,
+                keys_left=keys_left, keys_right=keys_right)
 
     # Merge the column and table meta data. Table subclasses might override
     # these methods for custom merge behavior.
@@ -195,8 +457,8 @@ def setdiff(table1, table2, keys=None):
     if keys is None:
         keys = table1.colnames
 
-    #Check that all keys are in table1 and table2
-    for tbl, tbl_str in ((table1,'table1'), (table2,'table2')):
+    # Check that all keys are in table1 and table2
+    for tbl, tbl_str in ((table1, 'table1'), (table2, 'table2')):
         diff_keys = np.setdiff1d(keys, tbl.colnames)
         if len(diff_keys) != 0:
             raise ValueError("The {} columns are missing from {}, cannot take "
@@ -246,7 +508,7 @@ def dstack(tables, join_type='outer', metadata_conflicts='warn'):
 
     Parameters
     ----------
-    tables : Table or list of Table or Row objects
+    tables : `~astropy.table.Table` or `~astropy.table.Row` or list thereof
         Table(s) to stack along depth-wise with the current table
         Table columns should have same shape and name for depth-wise stacking
     join_type : str
@@ -266,24 +528,24 @@ def dstack(tables, join_type='outer', metadata_conflicts='warn'):
     --------
     To stack two tables along rows do::
 
-      >>> from astropy.table import vstack, Table
-      >>> t1 = Table({'a': [1, 2], 'b': [3, 4]}, names=('a', 'b'))
-      >>> t2 = Table({'a': [5, 6], 'b': [7, 8]}, names=('a', 'b'))
+      >>> from astropy.table import dstack, Table
+      >>> t1 = Table({'a': [1., 2.], 'b': [3., 4.]}, names=('a', 'b'))
+      >>> t2 = Table({'a': [5., 6.], 'b': [7., 8.]}, names=('a', 'b'))
       >>> print(t1)
        a   b
       --- ---
-        1   3
-        2   4
+      1.0 3.0
+      2.0 4.0
       >>> print(t2)
        a   b
       --- ---
-        5   7
-        6   8
+      5.0 7.0
+      6.0 8.0
       >>> print(dstack([t1, t2]))
-      a [2]  b [2]
-      ------ ------
-      1 .. 5 3 .. 7
-      2 .. 6 4 .. 8
+          a          b
+      ---------- ----------
+      1.0 .. 5.0 3.0 .. 7.0
+      2.0 .. 6.0 4.0 .. 8.0
     """
     _check_join_type(join_type, 'dstack')
 
@@ -306,7 +568,11 @@ def dstack(tables, join_type='outer', metadata_conflicts='warn'):
         # are just carried along.
         # [x x x y y y] => [[x x x],
         #                   [y y y]]
-        col.shape = (len(tables), n_row) + col.shape[1:]
+        new_shape = (len(tables), n_row) + col.shape[1:]
+        try:
+            col.shape = (len(tables), n_row) + col.shape[1:]
+        except AttributeError:
+            col = col.reshape(new_shape)
 
         # Transpose the table and row axes to get to
         # [[x, y],
@@ -335,8 +601,8 @@ def vstack(tables, join_type='outer', metadata_conflicts='warn'):
 
     Parameters
     ----------
-    tables : Table or list of Table or Row objects
-        Table(s) to stack along rows (vertically)
+    tables : `~astropy.table.Table` or `~astropy.table.Row` or list thereof
+        Table(s) to stack along rows (vertically) with the current table
     join_type : str
         Join type ('inner' | 'exact' | 'outer'), default is 'outer'
     metadata_conflicts : str
@@ -404,8 +670,8 @@ def hstack(tables, join_type='outer',
 
     Parameters
     ----------
-    tables : Table or list of Table or Row objects
-        Tables to stack along columns (horizontally)
+    tables : `~astropy.table.Table` or `~astropy.table.Row` or list thereof
+        Tables to stack along columns (horizontally) with the current table
     join_type : str
         Join type ('inner' | 'exact' | 'outer'), default is 'outer'
     uniq_col_name : str or None
@@ -417,7 +683,8 @@ def hstack(tables, join_type='outer',
     metadata_conflicts : str
         How to proceed with metadata conflicts. This should be one of:
             * ``'silent'``: silently pick the last conflicting meta-data value
-            * ``'warn'``: pick the last conflicting meta-data value, but emit a warning (default)
+            * ``'warn'``: pick the last conflicting meta-data value,
+              but emit a warning (default)
             * ``'error'``: raise an exception.
 
     Returns
@@ -425,6 +692,10 @@ def hstack(tables, join_type='outer',
     stacked_table : `~astropy.table.Table` object
         New table containing the stacked data from the input tables.
 
+    See Also
+    --------
+    Table.add_columns, Table.replace_column, Table.update
+
     Examples
     --------
     To stack two tables horizontally (along columns) do::
@@ -469,13 +740,11 @@ def unique(input_table, keys=None, silent=False, keep='first'):
 
     Parameters
     ----------
-
-    input_table : `~astropy.table.Table` object or a value that
-        will initialize a `~astropy.table.Table` object
+    input_table : table-like
     keys : str or list of str
         Name(s) of column(s) used to create unique rows.
         Default is to use all columns.
-    keep : one of 'first', 'last' or 'none'
+    keep : {'first', 'last', 'none'}
         Whether to keep the first or last row for each set of
         duplicates. If 'none', all rows that are duplicate are
         removed, leaving only rows that are already unique in
@@ -568,7 +837,6 @@ def unique(input_table, keys=None, silent=False, keep='first'):
             raise ValueError("duplicate key names")
 
     # Check for columns with masked values
-    nkeys = 0
     for key in keys[:]:
         col = input_table[key]
         if hasattr(col, 'mask') and np.any(col.mask):
@@ -675,7 +943,7 @@ def get_descrs(arrays, col_name_map):
             # Beautify the error message when we are trying to merge columns with incompatible
             # types by including the name of the columns that originated the error.
             raise TableMergeError("The '{}' columns have incompatible types: {}"
-                                  .format(names[0], tme._incompat_types))
+                                  .format(names[0], tme._incompat_types)) from tme
 
         # Make sure all input shapes are the same
         uniq_shapes = set(col.shape[1:] for col in in_cols)
@@ -683,7 +951,9 @@ def get_descrs(arrays, col_name_map):
             raise TableMergeError(f'Key columns {names!r} have different shape')
         shape = uniq_shapes.pop()
 
-        out_descrs.append((fix_column_name(out_name), dtype, shape))
+        if out_name is not None:
+            out_name = str(out_name)
+        out_descrs.append((out_name, dtype, shape))
 
     return out_descrs
 
@@ -698,10 +968,9 @@ def common_dtype(cols):
     try:
         return metadata.common_dtype(cols)
     except metadata.MergeConflictError as err:
-        tme = TableMergeError('Columns have incompatible types {}'
-                              .format(err._incompat_types))
+        tme = TableMergeError(f'Columns have incompatible types {err._incompat_types}')
         tme._incompat_types = err._incompat_types
-        raise tme
+        raise tme from err
 
 
 def _get_join_sort_idxs(keys, left, right):
@@ -765,10 +1034,33 @@ def _get_join_sort_idxs(keys, left, right):
     return idxs, idx_sort
 
 
+def _apply_join_funcs(left, right, keys, join_funcs):
+    """Apply join_funcs
+    """
+    # Make light copies of left and right, then add new index columns.
+    left = left.copy(copy_data=False)
+    right = right.copy(copy_data=False)
+    for key, join_func in join_funcs.items():
+        ids1, ids2 = join_func(left[key], right[key])
+        # Define a unique id_key name, and keep adding underscores until we have
+        # a name not yet present.
+        id_key = key + '_id'
+        while id_key in left.columns or id_key in right.columns:
+            id_key = id_key[:-2] + '_id'
+
+        keys = tuple(id_key if orig_key == key else orig_key for orig_key in keys)
+        left.add_column(ids1, index=0, name=id_key)  # [id_key] = ids1
+        right.add_column(ids2, index=0, name=id_key)  # [id_key] = ids2
+
+    return left, right, keys
+
+
 def _join(left, right, keys=None, join_type='inner',
           uniq_col_name='{col_name}_{table_name}',
           table_names=['1', '2'],
-          col_name_map=None, metadata_conflicts='warn'):
+          col_name_map=None, metadata_conflicts='warn',
+          join_funcs=None,
+          keys_left=None, keys_right=None):
     """
     Perform a join of the left and right Tables on specified keys.
 
@@ -797,13 +1089,15 @@ def _join(left, right, keys=None, join_type='inner',
             * ``'silent'``: silently pick the last conflicting meta-data value
             * ``'warn'``: pick the last conflicting meta-data value, but emit a warning (default)
             * ``'error'``: raise an exception.
+    join_funcs : dict, None
+        Dict of functions to use for matching the corresponding key column(s).
+        See `~astropy.table.join_skycoord` for an example and details.
 
     Returns
     -------
     joined_table : `~astropy.table.Table` object
         New table containing the result of the join operation.
     """
-    from astropy.time import Time
     # Store user-provided col_name_map until the end
     _col_name_map = col_name_map
 
@@ -820,6 +1114,9 @@ def _join(left, right, keys=None, join_type='inner',
         if keys:
             raise ValueError('cannot supply keys for a cartesian join')
 
+        if join_funcs:
+            raise ValueError('cannot supply join_funcs for a cartesian join')
+
         # Make light copies of left and right, then add temporary index columns
         # with all the same value so later an outer join turns into a cartesian join.
         left = left.copy(copy_data=False)
@@ -828,12 +1125,23 @@ def _join(left, right, keys=None, join_type='inner',
         right[cartesian_index_name] = np.uint8(0)
         keys = (cartesian_index_name, )
 
-    # If we have a single key, put it in a tuple
+    # Handle the case of join key columns that are different between left and
+    # right via keys_left/keys_right args. This is done by saving the original
+    # input tables and making new left and right tables that contain only the
+    # key cols but with common column names ['0', '1', etc]. This sets `keys` to
+    # those fake key names in the left and right tables
+    if keys_left is not None or keys_right is not None:
+        left_orig = left
+        right_orig = right
+        left, right, keys = _join_keys_left_right(
+            left, right, keys, keys_left, keys_right, join_funcs)
+
     if keys is None:
         keys = tuple(name for name in left.colnames if name in right.colnames)
         if len(keys) == 0:
             raise TableMergeError('No keys in common between left and right tables')
     elif isinstance(keys, str):
+        # If we have a single key, put it in a tuple
         keys = (keys,)
 
     # Check the key columns
@@ -846,20 +1154,35 @@ def _join(left, right, keys=None, join_type='inner',
                 raise TableMergeError('{} key column {!r} has missing values'
                                       .format(arr_label, name))
 
+    if join_funcs is not None:
+        if not all(key in keys for key in join_funcs):
+            raise ValueError(f'join_funcs keys {join_funcs.keys()} must be a '
+                             f'subset of join keys {keys}')
+        left, right, keys = _apply_join_funcs(left, right, keys, join_funcs)
+
     len_left, len_right = len(left), len(right)
 
     if len_left == 0 or len_right == 0:
         raise ValueError('input tables for join must both have at least one row')
 
-    # Joined array dtype as a list of descr (name, type_str, shape) tuples
-    col_name_map = get_col_name_map([left, right], keys, uniq_col_name, table_names)
-    out_descrs = get_descrs([left, right], col_name_map)
-
     try:
         idxs, idx_sort = _get_join_sort_idxs(keys, left, right)
     except NotImplementedError:
         raise TypeError('one or more key columns are not sortable')
 
+    # Now that we have idxs and idx_sort, revert to the original table args to
+    # carry on with making the output joined table. `keys` is set to to an empty
+    # list so that all original left and right columns are included in the
+    # output table.
+    if keys_left is not None or keys_right is not None:
+        keys = []
+        left = left_orig
+        right = right_orig
+
+    # Joined array dtype as a list of descr (name, type_str, shape) tuples
+    col_name_map = get_col_name_map([left, right], keys, uniq_col_name, table_names)
+    out_descrs = get_descrs([left, right], col_name_map)
+
     # Main inner loop in Cython to compute the cartesian product
     # indices for the given join type
     int_join_type = {'inner': 0, 'outer': 1, 'left': 2, 'right': 3,
@@ -883,19 +1206,9 @@ def _join(left, right, keys=None, join_type='inner',
                                           .format(col_cls.__name__))
 
             out[out_name] = col_cls.info.new_like(cols, n_out, metadata_conflicts, out_name)
-
-            if not NUMPY_LT_1_17 or issubclass(col_cls, (Column, Time)):
-                out[out_name][:] = np.where(right_mask,
-                                            left[left_name].take(left_out),
-                                            right[right_name].take(right_out))
-            else:
-                # np.where does not work for mixin columns (e.g. Quantity) so
-                # use a slower workaround.
-                non_right_mask = ~right_mask
-                if np.any(right_mask):
-                    out[out_name][:] = left[left_name].take(left_out)
-                if np.any(non_right_mask):
-                    out[out_name][non_right_mask] = right[right_name].take(right_out)[non_right_mask]
+            out[out_name][:] = np.where(right_mask,
+                                        left[left_name].take(left_out),
+                                        right[right_name].take(right_out))
             continue
         elif left_name:  # out_name came from the left table
             name, array, array_out, array_mask = left_name, left, left_out, left_mask
@@ -915,6 +1228,9 @@ def _join(left, right, keys=None, join_type='inner',
             if isinstance(col, Column) and not isinstance(col, MaskedColumn):
                 col = out.MaskedColumn(col, copy=False)
 
+            if isinstance(col, Quantity) and not isinstance(col, Masked):
+                col = Masked(col, copy=False)
+
             # array_mask is 1-d corresponding to length of output column.  We need
             # make it have the correct shape for broadcasting, i.e. (length, 1, 1, ..).
             # Mixin columns might not have ndim attribute so use len(col.shape).
@@ -925,11 +1241,11 @@ def _join(left, right, keys=None, join_type='inner',
 
             try:
                 col[array_mask] = col.info.mask_val
-            except Exception:  # Not clear how different classes will fail here
+            except Exception as err:  # Not clear how different classes will fail here
                 raise NotImplementedError(
                     "join requires masking column '{}' but column"
                     " type {} does not support masking"
-                    .format(out_name, col.__class__.__name__))
+                    .format(out_name, col.__class__.__name__)) from err
 
         # Set the output table column to the new joined column
         out[out_name] = col
@@ -941,6 +1257,58 @@ def _join(left, right, keys=None, join_type='inner',
     return out
 
 
+def _join_keys_left_right(left, right, keys, keys_left, keys_right, join_funcs):
+    """Do processing to handle keys_left / keys_right args for join.
+
+    This takes the keys_left/right inputs and turns them into a list of left/right
+    columns corresponding to those inputs (which can be column names or column
+    data values). It also generates the list of fake key column names (strings
+    of "1", "2", etc.) that correspond to the input keys.
+    """
+    def _keys_to_cols(keys, table, label):
+        # Process input `keys`, which is a str or list of str column names in
+        # `table` or a list of column-like objects. The `label` is just for
+        # error reporting.
+        if isinstance(keys, str):
+            keys = [keys]
+        cols = []
+        for key in keys:
+            if isinstance(key, str):
+                try:
+                    cols.append(table[key])
+                except KeyError:
+                    raise ValueError(f'{label} table does not have key column {key!r}')
+            else:
+                if len(key) != len(table):
+                    raise ValueError(f'{label} table has different length from key {key}')
+                cols.append(key)
+        return cols
+
+    if join_funcs is not None:
+        raise ValueError('cannot supply join_funcs arg and keys_left / keys_right')
+
+    if keys_left is None or keys_right is None:
+        raise ValueError('keys_left and keys_right must both be provided')
+
+    if keys is not None:
+        raise ValueError('keys arg must be None if keys_left and keys_right are supplied')
+
+    cols_left = _keys_to_cols(keys_left, left, 'left')
+    cols_right = _keys_to_cols(keys_right, right, 'right')
+
+    if len(cols_left) != len(cols_right):
+        raise ValueError('keys_left and keys_right args must have same length')
+
+    # Make two new temp tables for the join with only the join columns and
+    # key columns in common.
+    keys = [f'{ii}' for ii in range(len(cols_left))]
+
+    left = left.__class__(cols_left, names=keys, copy=False)
+    right = right.__class__(cols_right, names=keys, copy=False)
+
+    return left, right, keys
+
+
 def _check_join_type(join_type, func_name):
     """Check join_type arg in hstack and vstack.
 
@@ -1032,7 +1400,7 @@ def _vstack(arrays, join_type='outer', col_name_map=None, metadata_conflicts='wa
             # Beautify the error message when we are trying to merge columns with incompatible
             # types by including the name of the columns that originated the error.
             raise TableMergeError("The '{}' columns have incompatible types: {}"
-                                  .format(out_name, err._incompat_types))
+                                  .format(out_name, err._incompat_types)) from err
 
         idx0 = 0
         for name, array in zip(in_names, arrays):
@@ -1045,13 +1413,16 @@ def _vstack(arrays, join_type='outer', col_name_map=None, metadata_conflicts='wa
                 if isinstance(col, Column) and not isinstance(col, MaskedColumn):
                     col = out.MaskedColumn(col, copy=False)
 
+                if isinstance(col, Quantity) and not isinstance(col, Masked):
+                    col = Masked(col, copy=False)
+
                 try:
                     col[idx0:idx1] = col.info.mask_val
-                except Exception:
+                except Exception as err:
                     raise NotImplementedError(
                         "vstack requires masking column '{}' but column"
                         " type {} does not support masking"
-                        .format(out_name, col.__class__.__name__))
+                        .format(out_name, col.__class__.__name__)) from err
             idx0 = idx1
 
         out[out_name] = col
@@ -1064,7 +1435,7 @@ def _vstack(arrays, join_type='outer', col_name_map=None, metadata_conflicts='wa
 
 
 def _hstack(arrays, join_type='outer', uniq_col_name='{col_name}_{table_name}',
-           table_names=None, col_name_map=None):
+            table_names=None, col_name_map=None):
     """
     Stack tables horizontally (by columns)
 
@@ -1097,7 +1468,7 @@ def _hstack(arrays, join_type='outer', uniq_col_name='{col_name}_{table_name}',
     _col_name_map = col_name_map
 
     if table_names is None:
-        table_names = ['{}'.format(ii + 1) for ii in range(len(arrays))]
+        table_names = [f'{ii + 1}' for ii in range(len(arrays))]
     if len(arrays) != len(table_names):
         raise ValueError('Number of arrays must match number of table_names')
 
@@ -1145,13 +1516,16 @@ def _hstack(arrays, join_type='outer', uniq_col_name='{col_name}_{table_name}',
                 if isinstance(col, Column) and not isinstance(col, MaskedColumn):
                     col = out.MaskedColumn(col, copy=False)
 
+                if isinstance(col, Quantity) and not isinstance(col, Masked):
+                    col = Masked(col, copy=False)
+
                 try:
                     col[arr_len:] = col.info.mask_val
-                except Exception:
+                except Exception as err:
                     raise NotImplementedError(
                         "hstack requires masking column '{}' but column"
                         " type {} does not support masking"
-                        .format(out_name, col.__class__.__name__))
+                        .format(out_name, col.__class__.__name__)) from err
             else:
                 col = array[name][:n_rows]
 
