@@ -3,6 +3,7 @@
 import os
 import sys
 import re
+import fnmatch
 
 import numpy as np
 
@@ -41,6 +42,7 @@ def _possible_string_format_functions(format_):
     yield lambda format_, val: format(val, format_)
     yield lambda format_, val: format_.format(val)
     yield lambda format_, val: format_ % val
+    yield lambda format_, val: format_.format(**{k: val[k] for k in val.dtype.names})
 
 
 def get_auto_format_func(
@@ -81,7 +83,7 @@ def get_auto_format_func(
             return col.info._format_funcs[format_](format_, val)
 
         if callable(format_):
-            format_func = lambda format_, val: format_(val)
+            format_func = lambda format_, val: format_(val)  # noqa
             try:
                 out = format_func(format_, val)
                 if not isinstance(out, str):
@@ -95,8 +97,7 @@ def get_auto_format_func(
                 if val is np.ma.masked:
                     return str(val)
 
-                raise ValueError('Format function for value {} failed: {}'
-                                 .format(val, err))
+                raise ValueError(f'Format function for value {val} failed.') from err
             # If the user-supplied function handles formatting masked elements, use
             # it directly.  Otherwise, wrap it in a function that traps them.
             try:
@@ -136,6 +137,30 @@ def get_auto_format_func(
     return _auto_format_func
 
 
+def _get_pprint_include_names(table):
+    """Get the set of names to show in pprint from the table pprint_include_names
+    and pprint_exclude_names attributes.
+
+    These may be fnmatch unix-style globs.
+    """
+    def get_matches(name_globs, default):
+        match_names = set()
+        if name_globs:  # For None or () use the default
+            for name in table.colnames:
+                for name_glob in name_globs:
+                    if fnmatch.fnmatch(name, name_glob):
+                        match_names.add(name)
+                        break
+        else:
+            match_names.update(default)
+        return match_names
+
+    include_names = get_matches(table.pprint_include_names(), table.colnames)
+    exclude_names = get_matches(table.pprint_exclude_names(), [])
+
+    return include_names - exclude_names
+
+
 class TableFormatter:
     @staticmethod
     def _get_pprint_size(max_lines=None, max_width=None):
@@ -165,6 +190,10 @@ class TableFormatter:
         max_lines, max_width : int
 
         """
+        # Declare to keep static type checker happy.
+        lines = None
+        width = None
+
         if max_lines is None:
             max_lines = conf.max_lines
 
@@ -242,7 +271,9 @@ class TableFormatter:
                                                show_length=show_length,
                                                outs=outs)
 
-        col_strs = list(col_strs_iter)
+        # Replace tab and newline with text representations so they display nicely.
+        # Newline in particular is a problem in a multicolumn table.
+        col_strs = [val.replace('\t', '\\t').replace('\n', '\\n') for val in col_strs_iter]
         if len(col_strs) > 0:
             col_width = max(len(x) for x in col_strs)
 
@@ -254,7 +285,7 @@ class TableFormatter:
                 if i == n_header - 1:
                     continue
                 td = 'th' if i < n_header else 'td'
-                val = '<{}>{}</{}>'.format(td, xml_escape(col_str.strip()), td)
+                val = f'<{td}>{xml_escape(col_str.strip())}</{td}>'
                 row = ('<tr>' + val + '</tr>')
                 if i < n_header:
                     row = ('<thead>' + row + '</thead>')
@@ -309,10 +340,24 @@ class TableFormatter:
                 col_strs[i] = getattr(col_str, justify_method)(*justify_args)
 
         if outs['show_length']:
-            col_strs.append('Length = {} rows'.format(len(col)))
+            col_strs.append(f'Length = {len(col)} rows')
 
         return col_strs, outs
 
+    def _name_and_structure(self, name, dtype, sep=" "):
+        """Format a column name, including a possible structure.
+
+        Normally, just returns the name, but if it has a structured dtype,
+        will add the parts in between square brackets.  E.g.,
+        "name [f0, f1]" or "name [f0[sf0, sf1], f1]".
+        """
+        if dtype is None or dtype.names is None:
+            return name
+
+        structure = ', '.join([self._name_and_structure(name, dt, sep="")
+                               for name, (dt, _) in dtype.fields.items()])
+        return f"{name}{sep}[{structure}]"
+
     def _pformat_col_iter(self, col, max_lines, show_name, show_unit, outs,
                           show_dtype=False, show_length=None):
         """Iterator which yields formatted string representation of column values.
@@ -342,7 +387,7 @@ class TableFormatter:
             if the column is not shown completely.
         """
         max_lines, _ = self._get_pprint_size(max_lines, -1)
-
+        dtype = getattr(col, 'dtype', None)
         multidims = getattr(col, 'shape', [0])[1:]
         if multidims:
             multidim0 = tuple(0 for n in multidims)
@@ -356,11 +401,8 @@ class TableFormatter:
             i_centers.append(n_header)
             # Get column name (or 'None' if not set)
             col_name = str(col.info.name)
-            if multidims:
-                col_name += ' [{}]'.format(
-                    ','.join(str(n) for n in multidims))
             n_header += 1
-            yield col_name
+            yield self._name_and_structure(col_name, dtype)
         if show_unit:
             i_centers.append(n_header)
             n_header += 1
@@ -368,11 +410,11 @@ class TableFormatter:
         if show_dtype:
             i_centers.append(n_header)
             n_header += 1
-            try:
-                dtype = dtype_info_name(col.dtype)
-            except AttributeError:
-                dtype = 'object'
-            yield str(dtype)
+            if dtype is not None:
+                col_dtype = dtype_info_name((dtype, multidims))
+            else:
+                col_dtype = col.__class__.__qualname__ or 'object'
+            yield col_dtype
         if show_unit or show_name or show_dtype:
             i_dashes = n_header
             n_header += 1
@@ -412,8 +454,8 @@ class TableFormatter:
         #    enclosed variables.
         col_format = col.info.format or getattr(col.info, 'default_format',
                                                 None)
-        pssf = (getattr(col.info, 'possible_string_format_functions', None) or
-                _possible_string_format_functions)
+        pssf = (getattr(col.info, 'possible_string_format_functions', None)
+                or _possible_string_format_functions)
         auto_format_func = get_auto_format_func(col, pssf)
         format_func = col.info._format_funcs.get(col_format, auto_format_func)
 
@@ -453,7 +495,7 @@ class TableFormatter:
                     raise ValueError(
                         'Unable to parse format string "{}" for entry "{}" '
                         'in column "{}"'.format(col_format, col[idx],
-                                                 col.info.name))
+                                                col.info.name))
 
         outs['show_length'] = show_length
         outs['n_header'] = n_header
@@ -483,7 +525,7 @@ class TableFormatter:
             for the unit.
 
         show_dtype : bool
-            Include a header row for column dtypes. Default is False.
+            Include a header row for column dtypes. Default is to False.
 
         html : bool
             Format the output as an HTML table. Default is False.
@@ -493,7 +535,7 @@ class TableFormatter:
             "table{id}", where id is the unique integer id of the table object,
             id(table)
 
-        tableclass : str or list of str or `None`
+        tableclass : str or list of str or None
             CSS classes for the table; only used if html is set.  Default is
             none
 
@@ -516,7 +558,6 @@ class TableFormatter:
         # "Print" all the values into temporary lists by column for subsequent
         # use and to determine the width
         max_lines, max_width = self._get_pprint_size(max_lines, max_width)
-        cols = []
 
         if show_unit is None:
             show_unit = any(col.info.unit for col in table.columns.values())
@@ -535,7 +576,16 @@ class TableFormatter:
             raise TypeError('align keyword must be str or list or tuple (got {})'
                             .format(type(align)))
 
+        # Process column visibility from table pprint_include_names and
+        # pprint_exclude_names attributes and get the set of columns to show.
+        pprint_include_names = _get_pprint_include_names(table)
+
+        cols = []
+        outs = None  # Initialize so static type checker is happy
         for align_, col in zip(align, table.columns.values()):
+            if col.info.name not in pprint_include_names:
+                continue
+
             lines, outs = self._pformat_col(col, max_lines, show_name=show_name,
                                             show_unit=show_unit, show_dtype=show_dtype,
                                             align=align_)
@@ -550,7 +600,10 @@ class TableFormatter:
         n_header = outs['n_header']
 
         n_rows = len(cols[0])
-        outwidth = lambda cols: sum(len(c[0]) for c in cols) + len(cols) - 1
+
+        def outwidth(cols):
+            return sum(len(c[0]) for c in cols) + len(cols) - 1
+
         dots_col = ['...'] * n_rows
         middle = len(cols) // 2
         while outwidth(cols) > max_width:
@@ -571,13 +624,12 @@ class TableFormatter:
             from astropy.utils.xml.writer import xml_escape
 
             if tableid is None:
-                tableid = 'table{id}'.format(id=id(table))
+                tableid = f'table{id(table)}'
 
             if tableclass is not None:
                 if isinstance(tableclass, list):
                     tableclass = ' '.join(tableclass)
-                rows.append('<table id="{tid}" class="{tcls}">'.format(
-                    tid=tableid, tcls=tableclass))
+                rows.append(f'<table id="{tableid}" class="{tableclass}">')
             else:
                 rows.append(f'<table id="{tableid}">')
 
@@ -586,7 +638,7 @@ class TableFormatter:
                 if i == n_header - 1:
                     continue
                 td = 'th' if i < n_header else 'td'
-                vals = ('<{}>{}</{}>'.format(td, xml_escape(col[i].strip()), td)
+                vals = (f'<{td}>{xml_escape(col[i].strip())}</{td}>'
                         for col in cols)
                 row = ('<tr>' + ''.join(vals) + '</tr>')
                 if i < n_header:
