@@ -1,6 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-
-
 """Test behavior related to masked tables"""
 
 import pytest
@@ -9,8 +7,8 @@ import numpy.ma as ma
 
 from astropy.table import Column, MaskedColumn, Table, QTable
 from astropy.table.column import BaseColumn
-from astropy.tests.helper import catch_warnings
 from astropy.time import Time
+from astropy.utils.masked import Masked
 import astropy.units as u
 
 
@@ -38,9 +36,9 @@ class TestFilled:
     def setup_method(self, method):
         mask = [True, False, False]
         self.meta = {'a': 1, 'b': [2, 3]}
-        a = self.a = MaskedColumn(name='a', data=[1, 2, 3], fill_value=10, mask=mask, meta={'a': 1})
-        b = self.b = MaskedColumn(name='b', data=[4.0, 5.0, 6.0], fill_value=10.0, mask=mask)
-        c = self.c = MaskedColumn(name='c', data=['7', '8', '9'], fill_value='1', mask=mask)
+        self.a = MaskedColumn(name='a', data=[1, 2, 3], fill_value=10, mask=mask, meta={'a': 1})
+        self.b = MaskedColumn(name='b', data=[4.0, 5.0, 6.0], fill_value=10.0, mask=mask)
+        self.c = MaskedColumn(name='c', data=['7', '8', '9'], fill_value='1', mask=mask)
 
     def test_filled_column(self):
         f = self.a.filled()
@@ -185,31 +183,15 @@ class TestMaskedColumnInit(SetupData):
 class TestTableInit(SetupData):
     """Initializing a table"""
 
-    def test_initialization_with_all_columns(self):
-        t1 = Table([self.a, self.b, self.c, self.d, self.ca, self.sc])
-        assert t1.colnames == ['a', 'b', 'c', 'd', 'ca', 'sc']
-        # Check we get the same result by passing in as list of dict.
-        # (Regression test for error uncovered by scintillometry package.)
-        lofd = [{k: row[k] for k in t1.colnames} for row in t1]
-        t2 = Table(lofd)
-        for k in t1.colnames:
-            assert np.all(t1[k] == t2[k]) in (True, np.ma.masked)
-            assert np.all(getattr(t1[k], 'mask', False) ==
-                          getattr(t2[k], 'mask', False))
-
-    # Filter warnings since these are set to lead to exceptions,
-    # which changes behaviour in Table._convert_data_to_col
-    # (causing conversion of columns with masked elements to object dtype).
-    @pytest.mark.filterwarnings('ignore:.*converting a masked element.*')
     @pytest.mark.parametrize('type_str', ('?', 'b', 'i2', 'f4', 'c8', 'S', 'U', 'O'))
-    @pytest.mark.parametrize('shape', ((4,), (2, 2)))
+    @pytest.mark.parametrize('shape', ((8,), (4, 2), (2, 2, 2)))
     def test_init_from_sequence_data_numeric_typed(self, type_str, shape):
         """Test init from list or list of lists with dtype specified, optionally
         including an np.ma.masked element.
         """
         # Make data of correct dtype and shape, then turn into a list,
         # then use that to init Table with spec'd type_str.
-        data = [0, 1, 2, 3]
+        data = list(range(8))
         np_data = np.array(data, dtype=type_str).reshape(shape)
         np_data_list = np_data.tolist()
         t = Table([np_data_list], dtype=[type_str])
@@ -221,8 +203,10 @@ class TestTableInit(SetupData):
         # Introduce np.ma.masked in the list input and confirm dtype still OK.
         if len(shape) == 1:
             np_data_list[-1] = np.ma.masked
-        else:
+        elif len(shape) == 2:
             np_data_list[-1][-1] = np.ma.masked
+        else:
+            np_data_list[-1][-1][-1] = np.ma.masked
         last_idx = tuple(-1 for _ in shape)
         t = Table([np_data_list], dtype=[type_str])
         col = t['col0']
@@ -232,12 +216,12 @@ class TestTableInit(SetupData):
         assert type(col) is MaskedColumn
 
     @pytest.mark.parametrize('type_str', ('?', 'b', 'i2', 'f4', 'c8', 'S', 'U', 'O'))
-    @pytest.mark.parametrize('shape', ((4,), (2, 2)))
+    @pytest.mark.parametrize('shape', ((8,), (4, 2), (2, 2, 2)))
     def test_init_from_sequence_data_numeric_untyped(self, type_str, shape):
         """Test init from list or list of lists with dtype NOT specified,
         optionally including an np.ma.masked element.
         """
-        data = [0, 1, 2, 3]
+        data = list(range(8))
         np_data = np.array(data, dtype=type_str).reshape(shape)
         np_data_list = np_data.tolist()
         t = Table([np_data_list])
@@ -247,8 +231,10 @@ class TestTableInit(SetupData):
         # Introduce np.ma.masked in the list input and confirm dtype still OK.
         if len(shape) == 1:
             np_data_list[-1] = np.ma.masked
-        else:
+        elif len(shape) == 2:
             np_data_list[-1][-1] = np.ma.masked
+        else:
+            np_data_list[-1][-1][-1] = np.ma.masked
         last_idx = tuple(-1 for _ in shape)
         t = Table([np_data_list])
         col = t['col0']
@@ -269,8 +255,8 @@ class TestTableInit(SetupData):
         for k in t1.colnames:
             assert t1[k].dtype == t2[k].dtype
             assert np.all(t1[k] == t2[k]) in (True, np.ma.masked)
-            assert np.all(getattr(t1[k], 'mask', False) ==
-                          getattr(t2[k], 'mask', False))
+            assert np.all(getattr(t1[k], 'mask', False)
+                          == getattr(t2[k], 'mask', False))
 
     def test_mask_false_if_input_mask_not_true(self):
         """Masking is always False if initial masked arg is not True"""
@@ -499,9 +485,9 @@ class TestAddRow:
         t.add_row((3 * u.m,))  # No problem
         with pytest.raises(ValueError) as exc:
             t.add_row((3 * u.m,), mask=(True,))
-        assert (exc.value.args[0].splitlines() ==
-                ["Unable to insert row because of exception in column 'a':",
-                 "mask was supplied for column 'a' but it does not support masked values"])
+        assert (exc.value.args[0].splitlines()
+                == ["Unable to insert row because of exception in column 'a':",
+                    "mask was supplied for column 'a' but it does not support masked values"])
 
 
 def test_setting_from_masked_column():
@@ -570,34 +556,43 @@ def test_masked_as_array_with_mixin():
     assert isinstance(ta, np.ma.MaskedArray)
     assert np.all(ta['a'].mask == [False, True])
     assert np.isclose(ta['a'][0].cxcsec, 1.0)
-    assert np.all(ta['b'].mask == False)
-    assert np.all(ta['c'].mask == False)
+    assert np.all(ta['b'].mask == False)  # noqa
+    assert np.all(ta['c'].mask == False)  # noqa
 
     # Check table ``mask`` property
     tm = t.mask
     assert np.all(tm['a'] == [False, True])
-    assert np.all(tm['b'] == False)
-    assert np.all(tm['c'] == False)
+    assert np.all(tm['b'] == False)  # noqa
+    assert np.all(tm['c'] == False)  # noqa
 
 
 def test_masked_column_with_unit_in_qtable():
-    """Test that adding a MaskedColumn with a unit to QTable issues warning"""
+    """Test that adding a MaskedColumn with a unit to QTable creates a MaskedQuantity."""
+    MaskedQuantity = Masked(u.Quantity)
+
     t = QTable()
-    with catch_warnings() as w:
-        t['a'] = MaskedColumn([1, 2])
-    assert len(w) == 0
+    t['a'] = MaskedColumn([1, 2])
     assert isinstance(t['a'], MaskedColumn)
 
-    with catch_warnings() as w:
-        t['b'] = MaskedColumn([1, 2], unit=u.m)
-    assert len(w) == 0
-    assert isinstance(t['b'], u.Quantity)
+    t['b'] = MaskedColumn([1, 2], unit=u.m)
+    assert isinstance(t['b'], MaskedQuantity)
+    assert np.all(t['b'].mask == False)  # noqa
 
-    with catch_warnings() as w:
-        t['c'] = MaskedColumn([1, 2], unit=u.m, mask=[True, False])
-    assert len(w) == 1
-    assert "dropping mask in Quantity column 'c'"
-    assert isinstance(t['b'], u.Quantity)
+    t['c'] = MaskedColumn([1, 2], unit=u.m, mask=[True, False])
+    assert isinstance(t['c'], MaskedQuantity)
+    assert np.all(t['c'].mask == [True, False])
+
+
+def test_masked_quantity_in_table():
+    MaskedQuantity = Masked(u.Quantity)
+    t = Table()
+    t['b'] = MaskedQuantity([1, 2], unit=u.m)
+    assert isinstance(t['b'], MaskedColumn)
+    assert np.all(t['b'].mask == False)  # noqa
+
+    t['c'] = MaskedQuantity([1, 2], unit=u.m, mask=[True, False])
+    assert isinstance(t['c'], MaskedColumn)
+    assert np.all(t['c'].mask == [True, False])
 
 
 def test_masked_column_data_attribute_is_plain_masked_array():
