@@ -9,18 +9,19 @@ from decimal import Decimal
 from collections import OrderedDict, defaultdict
 
 import numpy as np
+import erfa
 
 from astropy.utils.decorators import lazyproperty, classproperty
-from astropy.utils.exceptions import AstropyDeprecationWarning, ErfaWarning
-from astropy import units as u
-from astropy import _erfa as erfa
+from astropy.utils.exceptions import AstropyDeprecationWarning
+import astropy.units as u
 
+from . import _parse_times
 from . import utils
 from .utils import day_frac, quantity_day_frac, two_sum, two_product
-
+from . import conf
 
 __all__ = ['TimeFormat', 'TimeJD', 'TimeMJD', 'TimeFromEpoch', 'TimeUnix',
-           'TimeCxcSec', 'TimeGPS', 'TimeDecimalYear',
+           'TimeUnixTai', 'TimeCxcSec', 'TimeGPS', 'TimeDecimalYear',
            'TimePlotDate', 'TimeUnique', 'TimeDatetime', 'TimeString',
            'TimeISO', 'TimeISOT', 'TimeFITS', 'TimeYearDayTime',
            'TimeEpochDate', 'TimeBesselianEpoch', 'TimeJulianEpoch',
@@ -75,37 +76,7 @@ def _regexify_subfmts(subfmts):
     return tuple(new_subfmts)
 
 
-class TimeFormatMeta(type):
-    """
-    Metaclass that adds `TimeFormat` and `TimeDeltaFormat` to the
-    `TIME_FORMATS` and `TIME_DELTA_FORMATS` registries, respectively.
-    """
-
-    _registry = TIME_FORMATS
-
-    def __new__(mcls, name, bases, members):
-        cls = super().__new__(mcls, name, bases, members)
-
-        # Register time formats that have a name, but leave out astropy_time since
-        # it is not a user-accessible format and is only used for initialization into
-        # a different format.
-        if 'name' in members and cls.name != 'astropy_time':
-            # FIXME: check here that we're not introducing a collision with
-            # an existing method or attribute; problem is it could be either
-            # astropy.time.Time or astropy.time.TimeDelta, and at the point
-            # where this is run neither of those classes have necessarily been
-            # constructed yet.
-            if 'value' in members and not hasattr(members['value'], "fget"):
-                raise ValueError("If defined, 'value' must be a property")
-            mcls._registry[cls.name] = cls
-
-        if 'subfmts' in members:
-            cls.subfmts = _regexify_subfmts(members['subfmts'])
-
-        return cls
-
-
-class TimeFormat(metaclass=TimeFormatMeta):
+class TimeFormat:
     """
     Base class for time representations.
 
@@ -130,6 +101,7 @@ class TimeFormat(metaclass=TimeFormatMeta):
 
     _default_scale = 'utc'  # As of astropy 0.4
     subfmts = ()
+    _registry = TIME_FORMATS
 
     def __init__(self, val1, val2, scale, precision,
                  in_subfmt, out_subfmt, from_jd=False):
@@ -147,6 +119,61 @@ class TimeFormat(metaclass=TimeFormatMeta):
             val1, val2 = self._check_val_type(val1, val2)
             self.set_jds(val1, val2)
 
+    def __init_subclass__(cls, **kwargs):
+        # Register time formats that define a name, but leave out astropy_time since
+        # it is not a user-accessible format and is only used for initialization into
+        # a different format.
+        if 'name' in cls.__dict__ and cls.name != 'astropy_time':
+            # FIXME: check here that we're not introducing a collision with
+            # an existing method or attribute; problem is it could be either
+            # astropy.time.Time or astropy.time.TimeDelta, and at the point
+            # where this is run neither of those classes have necessarily been
+            # constructed yet.
+            if 'value' in cls.__dict__ and not hasattr(cls.value, "fget"):
+                raise ValueError("If defined, 'value' must be a property")
+
+            cls._registry[cls.name] = cls
+
+        # If this class defines its own subfmts, preprocess the definitions.
+        if 'subfmts' in cls.__dict__:
+            cls.subfmts = _regexify_subfmts(cls.subfmts)
+
+        return super().__init_subclass__(**kwargs)
+
+    @classmethod
+    def _get_allowed_subfmt(cls, subfmt):
+        """Get an allowed subfmt for this class, either the input ``subfmt``
+        if this is valid or '*' as a default.  This method gets used in situations
+        where the format of an existing Time object is changing and so the
+        out_ or in_subfmt may need to be coerced to the default '*' if that
+        ``subfmt`` is no longer valid.
+        """
+        try:
+            cls._select_subfmts(subfmt)
+        except ValueError:
+            subfmt = '*'
+        return subfmt
+
+    @property
+    def in_subfmt(self):
+        return self._in_subfmt
+
+    @in_subfmt.setter
+    def in_subfmt(self, subfmt):
+        # Validate subfmt value for this class, raises ValueError if not.
+        self._select_subfmts(subfmt)
+        self._in_subfmt = subfmt
+
+    @property
+    def out_subfmt(self):
+        return self._out_subfmt
+
+    @out_subfmt.setter
+    def out_subfmt(self, subfmt):
+        # Validate subfmt value for this class, raises ValueError if not.
+        self._select_subfmts(subfmt)
+        self._out_subfmt = subfmt
+
     @property
     def jd1(self):
         return self._jd1
@@ -230,7 +257,7 @@ class TimeFormat(metaclass=TimeFormatMeta):
         if getattr(val1, 'unit', None) is not None:
             # Convert any quantity-likes to days first, attempting to be
             # careful with the conversion, so that, e.g., large numbers of
-            # seconds get converted without loosing precision because
+            # seconds get converted without losing precision because
             # 1/86400 is not exactly representable as a float.
             val1 = u.Quantity(val1, copy=False)
             if val2 is not None:
@@ -312,10 +339,10 @@ class TimeFormat(metaclass=TimeFormatMeta):
 
         Parameters
         ----------
-        parent : obj
+        parent : object
             Parent `~astropy.time.Time` object associated with this
             `~astropy.time.TimeFormat` object
-        out_subfmt : str or `None`
+        out_subfmt : str or None
             Output subformt (use existing self.out_subfmt if `None`)
 
         Returns
@@ -340,16 +367,31 @@ class TimeFormat(metaclass=TimeFormatMeta):
     def value(self):
         raise NotImplementedError
 
-    def _select_subfmts(self, pattern):
+    @classmethod
+    def _select_subfmts(cls, pattern):
         """
         Return a list of subformats where name matches ``pattern`` using
         fnmatch.
-        """
 
-        fnmatchcase = fnmatch.fnmatchcase
-        subfmts = [x for x in self.subfmts if fnmatchcase(x[0], pattern)]
+        If no subformat matches pattern then a ValueError is raised.  A special
+        case is a format with no allowed subformats, i.e. subfmts=(), and
+        pattern='*'.  This is OK and happens when this method is used for
+        validation of an out_subfmt.
+        """
+        if not isinstance(pattern, str):
+            raise ValueError('subfmt attribute must be a string')
+        elif pattern == '*':
+            return cls.subfmts
+
+        subfmts = [x for x in cls.subfmts if fnmatch.fnmatchcase(x[0], pattern)]
         if len(subfmts) == 0:
-            raise ValueError(f'No subformats match {pattern}')
+            if len(cls.subfmts) == 0:
+                raise ValueError(f'subformat not allowed for format {cls.name}')
+            else:
+                subfmt_names = [x[0] for x in cls.subfmts]
+                raise ValueError(f'subformat {pattern!r} must match one of '
+                                 f'{subfmt_names} for format {cls.name}')
+
         return subfmts
 
 
@@ -409,7 +451,7 @@ class TimeNumeric(TimeFormat):
         Subclasses that require ``parent`` or to adjust the jds should
         override this method.
         """
-        # TODO: do this in metaclass.
+        # TODO: do this in __init_subclass__?
         if self.__class__.value.fget is not self.__class__.to_value:
             return self.value
 
@@ -641,7 +683,8 @@ class TimeFromEpoch(TimeNumeric):
 
 class TimeUnix(TimeFromEpoch):
     """
-    Unix time: seconds from 1970-01-01 00:00:00 UTC.
+    Unix time (UTC): seconds from 1970-01-01 00:00:00 UTC, ignoring leap seconds.
+
     For example, 946684800.0 in Unix time is midnight on January 1, 2000.
 
     NOTE: this quantity is not exactly unix time and differs from the strict
@@ -658,6 +701,58 @@ class TimeUnix(TimeFromEpoch):
     epoch_format = 'iso'
 
 
+class TimeUnixTai(TimeUnix):
+    """
+    Unix time (TAI): SI seconds elapsed since 1970-01-01 00:00:00 TAI (see caveats).
+
+    This will generally differ from standard (UTC) Unix time by the cumulative
+    integral number of leap seconds introduced into UTC since 1972-01-01 UTC
+    plus the initial offset of 10 seconds at that date.
+
+    This convention matches the definition of linux CLOCK_TAI
+    (https://www.cl.cam.ac.uk/~mgk25/posix-clocks.html),
+    and the Precision Time Protocol
+    (https://en.wikipedia.org/wiki/Precision_Time_Protocol), which
+    is also used by the White Rabbit protocol in High Energy Physics:
+    https://white-rabbit.web.cern.ch.
+
+    Caveats:
+
+    - Before 1972, fractional adjustments to UTC were made, so the difference
+      between ``unix`` and ``unix_tai`` time is no longer an integer.
+    - Because of the fractional adjustments, to be very precise, ``unix_tai``
+      is the number of seconds since ``1970-01-01 00:00:00 TAI`` or equivalently
+      ``1969-12-31 23:59:51.999918 UTC``.  The difference between TAI and UTC
+      at that epoch was 8.000082 sec.
+    - On the day of a positive leap second the difference between ``unix`` and
+      ``unix_tai`` times increases linearly through the day by 1.0. See also the
+      documentation for the `~astropy.time.TimeUnix` class.
+    - Negative leap seconds are possible, though none have been needed to date.
+
+    Examples
+    --------
+
+      >>> # get the current offset between TAI and UTC
+      >>> from astropy.time import Time
+      >>> t = Time('2020-01-01', scale='utc')
+      >>> t.unix_tai - t.unix
+      37.0
+
+      >>> # Before 1972, the offset between TAI and UTC was not integer
+      >>> t = Time('1970-01-01', scale='utc')
+      >>> t.unix_tai - t.unix  # doctest: +FLOAT_CMP
+      8.000082
+
+      >>> # Initial offset of 10 seconds in 1972
+      >>> t = Time('1972-01-01', scale='utc')
+      >>> t.unix_tai - t.unix
+      10.0
+    """
+    name = 'unix_tai'
+    epoch_val = '1970-01-01 00:00:00'
+    epoch_scale = 'tai'
+
+
 class TimeCxcSec(TimeFromEpoch):
     """
     Chandra X-ray Center seconds from 1998-01-01 00:00:00 TT.
@@ -731,6 +826,7 @@ class TimePlotDate(TimeFromEpoch):
         else:
             # Get the matplotlib date epoch as an ISOT string in UTC
             epoch_utc = get_epoch()
+            from erfa import ErfaWarning
             with warnings.catch_warnings():
                 # Catch possible dubious year warnings from erfa
                 warnings.filterwarnings('ignore', category=ErfaWarning)
@@ -802,14 +898,7 @@ class TimeAstropyTime(TimeUnique):
                                                subok=True)
                     locations.append(np.atleast_1d(location))
 
-                # Once we no longer have to support NUMPY_LT_1_17, we can
-                # just write np.concatenate(locations).  Right now, we
-                # effectively do what __array_function__ does directly.
-                location0 = locations[0]
-                unit = location0.unit
-                location = np.concatenate([location.to_value(unit)
-                                           for location in locations])
-                location = location0._new_view(location)
+                location = np.concatenate(locations)
 
             else:
                 location = None
@@ -1090,7 +1179,7 @@ class TimezoneInfo(datetime.tzinfo):
         dst : `~astropy.units.Quantity`, optional
             Daylight Savings Time offset in days. Defaults to zero
             (no daylight savings).
-        tzname : str or `None`, optional
+        tzname : str or None, optional
             Name of timezone
 
         Examples
@@ -1129,13 +1218,57 @@ class TimeString(TimeUnique):
     This class assumes that anything following the last decimal point to the
     right is a fraction of a second.
 
-    This is a reference implementation can be made much faster with effort.
+    **Fast C-based parser**
+
+    Time format classes can take advantage of a fast C-based parser if the times
+    are represented as fixed-format strings with year, month, day-of-month,
+    hour, minute, second, OR year, day-of-year, hour, minute, second. This can
+    be a factor of 20 or more faster than the pure Python parser.
+
+    Fixed format means that the components always have the same number of
+    characters. The Python parser will accept ``2001-9-2`` as a date, but the C
+    parser would require ``2001-09-02``.
+
+    A subclass in this case must define a class attribute ``fast_parser_pars``
+    which is a `dict` with all of the keys below. An inherited attribute is not
+    checked, only an attribute in the class ``__dict__``.
+
+    - ``delims`` (tuple of int): ASCII code for character at corresponding
+      ``starts`` position (0 => no character)
+
+    - ``starts`` (tuple of int): position where component starts (including
+      delimiter if present). Use -1 for the month component for format that use
+      day of year.
+
+    - ``stops`` (tuple of int): position where component ends. Use -1 to
+      continue to end of string, or for the month component for formats that use
+      day of year.
+
+    - ``break_allowed`` (tuple of int): if true (1) then the time string can
+          legally end just before the corresponding component (e.g. "2000-01-01"
+          is a valid time but "2000-01-01 12" is not).
+
+    - ``has_day_of_year`` (int): 0 if dates have year, month, day; 1 if year,
+      day-of-year
     """
 
+    def __init_subclass__(cls, **kwargs):
+        if 'fast_parser_pars' in cls.__dict__:
+            fpp = cls.fast_parser_pars
+            fpp = np.array(list(zip(map(chr, fpp['delims']),
+                                    fpp['starts'],
+                                    fpp['stops'],
+                                    fpp['break_allowed'])),
+                           _parse_times.dt_pars)
+            if cls.fast_parser_pars['has_day_of_year']:
+                fpp['start'][1] = fpp['stop'][1] = -1
+            cls._fast_parser = _parse_times.create_parser(fpp)
+
+        super().__init_subclass__(**kwargs)
+
     def _check_val_type(self, val1, val2):
         if val1.dtype.kind not in ('S', 'U') and val1.size:
-            raise TypeError('Input values for {} class must be strings'
-                            .format(self.name))
+            raise TypeError(f'Input values for {self.name} class must be strings')
         if val2 is not None:
             raise ValueError(
                 f'{self.name} objects do not accept a val2 but you provided {val2}')
@@ -1179,11 +1312,33 @@ class TimeString(TimeUnique):
             vals[-1] = vals[-1] + fracsec
             return vals
         else:
-            raise ValueError('Time {} does not match {} format'
-                             .format(timestr, self.name))
+            raise ValueError(f'Time {timestr} does not match {self.name} format')
 
     def set_jds(self, val1, val2):
         """Parse the time strings contained in val1 and set jd1, jd2"""
+        # If specific input subformat is required then use the Python parser.
+        # Also do this if Time format class does not define `use_fast_parser` or
+        # if the fast parser is entirely disabled. Note that `use_fast_parser`
+        # is ignored for format classes that don't have a fast parser.
+        if (self.in_subfmt != '*'
+                or '_fast_parser' not in self.__class__.__dict__
+                or conf.use_fast_parser == 'False'):
+            jd1, jd2 = self.get_jds_python(val1, val2)
+        else:
+            try:
+                jd1, jd2 = self.get_jds_fast(val1, val2)
+            except Exception:
+                # Fall through to the Python parser unless fast is forced.
+                if conf.use_fast_parser == 'force':
+                    raise
+                else:
+                    jd1, jd2 = self.get_jds_python(val1, val2)
+
+        self.jd1 = jd1
+        self.jd2 = jd2
+
+    def get_jds_python(self, val1, val2):
+        """Parse the time strings contained in val1 and get jd1, jd2"""
         # Select subformats based on current self.in_subfmt
         subfmts = self._select_subfmts(self.in_subfmt)
         # Be liberal in what we accept: convert bytes to ascii.
@@ -1201,7 +1356,40 @@ class TimeString(TimeUnique):
 
         jd1, jd2 = erfa.dtf2d(self.scale.upper().encode('ascii'),
                               *iterator.operands[1:])
-        self.jd1, self.jd2 = day_frac(jd1, jd2)
+        jd1, jd2 = day_frac(jd1, jd2)
+
+        return jd1, jd2
+
+    def get_jds_fast(self, val1, val2):
+        """Use fast C parser to parse time strings in val1 and get jd1, jd2"""
+        # Handle bytes or str input and convert to uint8.  We need to the
+        # dtype _parse_times.dt_u1 instead of uint8, since otherwise it is
+        # not possible to create a gufunc with structured dtype output.
+        # See note about ufunc type resolver in pyerfa/erfa/ufunc.c.templ.
+        if val1.dtype.kind == 'U':
+            # Note: val1.astype('S') is *very* slow, so we check ourselves
+            # that the input is pure ASCII.
+            val1_uint32 = val1.view((np.uint32, val1.dtype.itemsize // 4))
+            if np.any(val1_uint32 > 127):
+                raise ValueError('input is not pure ASCII')
+
+            # It might be possible to avoid making a copy via astype with
+            # cleverness in parse_times.c but leave that for another day.
+            chars = val1_uint32.astype(_parse_times.dt_u1)
+
+        else:
+            chars = val1.view((_parse_times.dt_u1, val1.dtype.itemsize))
+
+        # Call the fast parsing ufunc.
+        time_struct = self._fast_parser(chars)
+        jd1, jd2 = erfa.dtf2d(self.scale.upper().encode('ascii'),
+                              time_struct['year'],
+                              time_struct['month'],
+                              time_struct['day'],
+                              time_struct['hour'],
+                              time_struct['minute'],
+                              time_struct['second'])
+        return day_frac(jd1, jd2)
 
     def str_kwargs(self):
         """
@@ -1215,11 +1403,8 @@ class TimeString(TimeUnique):
         # Get the str_fmt element of the first allowed output subformat
         _, _, str_fmt = self._select_subfmts(self.out_subfmt)[0]
 
-        if '{yday:' in str_fmt:
-            has_yday = True
-        else:
-            has_yday = False
-            yday = None
+        yday = None
+        has_yday = '{yday:' in str_fmt
 
         ihrs = ihmsfs['h']
         imins = ihmsfs['m']
@@ -1287,6 +1472,23 @@ class TimeISO(TimeString):
                 '%Y-%m-%d',
                 '{year:d}-{mon:02d}-{day:02d}'))
 
+    # Define positions and starting delimiter for year, month, day, hour,
+    # minute, seconds components of an ISO time. This is used by the fast
+    # C-parser parse_ymdhms_times()
+    #
+    #  "2000-01-12 13:14:15.678"
+    #   01234567890123456789012
+    #   yyyy-mm-dd hh:mm:ss.fff
+    # Parsed as ('yyyy', '-mm', '-dd', ' hh', ':mm', ':ss', '.fff')
+    fast_parser_pars = dict(
+        delims=(0, ord('-'), ord('-'), ord(' '), ord(':'), ord(':'), ord('.')),
+        starts=(0, 4, 7, 10, 13, 16, 19),
+        stops=(3, 6, 9, 12, 15, 18, -1),
+        # Break allowed *before*
+        #              y  m  d  h  m  s  f
+        break_allowed=(0, 0, 0, 1, 0, 1, 1),
+        has_day_of_year=0)
+
     def parse_string(self, timestr, subfmts):
         # Handle trailing 'Z' for UTC time
         if timestr.endswith('Z'):
@@ -1322,6 +1524,16 @@ class TimeISOT(TimeISO):
                 '%Y-%m-%d',
                 '{year:d}-{mon:02d}-{day:02d}'))
 
+    # See TimeISO for explanation
+    fast_parser_pars = dict(
+        delims=(0, ord('-'), ord('-'), ord('T'), ord(':'), ord(':'), ord('.')),
+        starts=(0, 4, 7, 10, 13, 16, 19),
+        stops=(3, 6, 9, 12, 15, 18, -1),
+        # Break allowed *before*
+        #              y  m  d  h  m  s  f
+        break_allowed=(0, 0, 0, 1, 0, 1, 1),
+        has_day_of_year=0)
+
 
 class TimeYearDayTime(TimeISO):
     """
@@ -1347,6 +1559,28 @@ class TimeYearDayTime(TimeISO):
                 '%Y:%j',
                 '{year:d}:{yday:03d}'))
 
+    # Define positions and starting delimiter for year, month, day, hour,
+    # minute, seconds components of an ISO time. This is used by the fast
+    # C-parser parse_ymdhms_times()
+    #
+    #  "2000:123:13:14:15.678"
+    #   012345678901234567890
+    #   yyyy:ddd:hh:mm:ss.fff
+    # Parsed as ('yyyy', ':ddd', ':hh', ':mm', ':ss', '.fff')
+    #
+    # delims: character at corresponding `starts` position (0 => no character)
+    # starts: position where component starts (including delimiter if present)
+    # stops: position where component ends (-1 => continue to end of string)
+
+    fast_parser_pars = dict(
+        delims=(0, 0, ord(':'), ord(':'), ord(':'), ord(':'), ord('.')),
+        starts=(0, -1, 4, 8, 11, 14, 17),
+        stops=(3, -1, 7, 10, 13, 16, -1),
+        # Break allowed before:
+        #              y  m  d  h  m  s  f
+        break_allowed=(0, 0, 0, 1, 0, 1, 1),
+        has_day_of_year=1)
+
 
 class TimeDatetime64(TimeISOT):
     name = 'datetime64'
@@ -1443,8 +1677,7 @@ class TimeFITS(TimeString):
             if tm:
                 break
         else:
-            raise ValueError('Time {} does not match {} format'
-                             .format(timestr, self.name))
+            raise ValueError(f'Time {timestr} does not match {self.name} format')
         tm = tm.groupdict()
         # Scale and realization are deprecated and strings in this form
         # are no longer created.  We issue a warning but still use the value.
@@ -1512,7 +1745,7 @@ class TimeBesselianEpoch(TimeEpochDate):
 
     def _check_val_type(self, val1, val2):
         """Input value validation, typically overridden by derived classes"""
-        if hasattr(val1, 'to') and hasattr(val1, 'unit'):
+        if hasattr(val1, 'to') and hasattr(val1, 'unit') and val1.unit is not None:
             raise ValueError("Cannot use Quantities for 'byear' format, "
                              "as the interpretation would be ambiguous. "
                              "Use float with Besselian year instead. ")
@@ -1550,8 +1783,7 @@ class TimeEpochDateString(TimeString):
                 if epoch_type.upper() != epoch_prefix:
                     raise ValueError
             except (IndexError, ValueError, UnicodeEncodeError):
-                raise ValueError('Time {} does not match {} format'
-                                 .format(time_str, self.name))
+                raise ValueError(f'Time {val} does not match {self.name} format')
             else:
                 years[...] = year
 
@@ -1586,13 +1818,11 @@ class TimeJulianEpochString(TimeEpochDateString):
     epoch_prefix = 'J'
 
 
-class TimeDeltaFormatMeta(TimeFormatMeta):
-    _registry = TIME_DELTA_FORMATS
-
-
-class TimeDeltaFormat(TimeFormat, metaclass=TimeDeltaFormatMeta):
+class TimeDeltaFormat(TimeFormat):
     """Base class for time delta representations"""
 
+    _registry = TIME_DELTA_FORMATS
+
     def _check_scale(self, scale):
         """
         Check that the scale is in the allowed list of scales, or is `None`
