@@ -12,14 +12,14 @@ from hypothesis.strategies import (composite, datetimes, floats, integers,
                                    one_of, sampled_from, timedeltas, tuples)
 
 import numpy as np
+import erfa
+from erfa import ErfaError, ErfaWarning
 
-import astropy._erfa as erfa
 import astropy.units as u
 from astropy.tests.helper import assert_quantity_allclose
 from astropy.time import STANDARD_TIME_SCALES, Time, TimeDelta
 from astropy.time.utils import day_frac, two_sum
 from astropy.utils import iers
-from astropy.utils.exceptions import ErfaError, ErfaWarning
 
 
 allclose_jd = functools.partial(np.allclose, rtol=np.finfo(float).eps, atol=0)
@@ -32,6 +32,12 @@ tiny = np.finfo(float).eps
 dt_tiny = TimeDelta(tiny, format='jd')
 
 
+def setup_module():
+    # Pre-load leap seconds table to avoid flakiness in hypothesis runs.
+    # See https://github.com/astropy/astropy/issues/11030
+    Time('2020-01-01').ut1
+
+
 @pytest.fixture(scope='module')
 def iers_b():
     """This is an expensive operation, so we share it between tests using a
@@ -492,6 +498,28 @@ def test_jd_add_subtract_round_trip(scale, jds, delta):
         raise
 
 
+@given(scale=sampled_from(STANDARD_TIME_SCALES),
+       jds=reasonable_jd(),
+       delta=floats(-3*tiny, 3*tiny))
+@example(scale='tai', jds=(0.0, 3.5762786865234384), delta=2.220446049250313e-16)
+def test_time_argminmaxsort(scale, jds, delta):
+    jd1, jd2 = jds
+    t = Time(jd1, jd2+np.array([0, delta]), scale=scale, format="jd")
+    imin = t.argmin()
+    imax = t.argmax()
+    isort = t.argsort()
+    diff = (t.jd1[1]-t.jd1[0]) + (t.jd2[1]-t.jd2[0])
+    if diff < 0:  # item 1 smaller
+        assert delta < 0
+        assert imin == 1 and imax == 0 and np.all(isort == [1, 0])
+    elif diff == 0:  # identical within precision
+        assert abs(delta) <= tiny
+        assert imin == 0 and imax == 0 and np.all(isort == [0, 1])
+    else:
+        assert delta > 0
+        assert imin == 0 and imax == 1 and np.all(isort == [0, 1])
+
+
 @given(sampled_from(STANDARD_TIME_SCALES), unreasonable_jd(), unreasonable_jd())
 @example(scale='utc',
          jds_a=(2455000.0, 0.0),
@@ -594,7 +622,7 @@ def test_datetime_difference_agrees_with_timedelta(scale, dt1, dt2):
 def test_datetime_to_timedelta(scale, days, microseconds):
     td = timedelta(days=days, microseconds=microseconds)
     assert (TimeDelta(td, scale=scale)
-            == TimeDelta(days, microseconds/(86400*1e6), scale=scale))
+            == TimeDelta(days, microseconds/(86400*1e6), scale=scale, format="jd"))
 
 
 @given(days=integers(-3000*365, 3000*365),
@@ -619,8 +647,9 @@ def test_timedelta_datetime_roundtrip(scale, days, day_frac):
 @example(days=262144, day_frac=2.314815006343452e-11)
 @pytest.mark.parametrize("scale", _utc_bad)
 def test_timedelta_from_parts(scale, days, day_frac):
-    whole = TimeDelta(days, day_frac, format="jd", scale=scale)
-    from_parts = TimeDelta(days, scale=scale) + TimeDelta(day_frac, scale=scale)
+    kwargs = dict(format="jd", scale=scale)
+    whole = TimeDelta(days, day_frac, **kwargs)
+    from_parts = TimeDelta(days, **kwargs) + TimeDelta(day_frac, **kwargs)
     assert whole == from_parts
 
 
