@@ -1,30 +1,27 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """A set of standard astronomical equivalencies."""
 
+import warnings
 from collections import UserList
 
 # THIRD-PARTY
 import numpy as np
-import warnings
 
 # LOCAL
 from astropy.constants import si as _si
 from astropy.utils.exceptions import AstropyDeprecationWarning
 from astropy.utils.misc import isiterable
-from . import si
-from . import cgs
-from . import astrophys
-from .function import units as function_units
-from . import dimensionless_unscaled
-from .core import UnitsError, Unit
 
+from . import astrophys, cgs, dimensionless_unscaled, misc, si
+from .core import Unit, UnitsError
+from .function import units as function_units
 
 __all__ = ['parallax', 'spectral', 'spectral_density', 'doppler_radio',
-           'doppler_optical', 'doppler_relativistic', 'mass_energy',
+           'doppler_optical', 'doppler_relativistic', 'doppler_redshift', 'mass_energy',
            'brightness_temperature', 'thermodynamic_temperature',
            'beam_angular_area', 'dimensionless_angles', 'logarithmic',
            'temperature', 'temperature_energy', 'molar_mass_amu',
-           'pixel_scale', 'plate_scale', 'with_H0']
+           'pixel_scale', 'plate_scale', "Equivalency"]
 
 
 class Equivalency(UserList):
@@ -105,7 +102,7 @@ def parallax():
 def spectral():
     """
     Returns a list of equivalence pairs that handle spectral
-    wavelength, wave number, frequency, and energy equivalences.
+    wavelength, wave number, frequency, and energy equivalencies.
 
     Allows conversions between wavelength units, wave number units,
     frequency units, and energy units as they relate to light.
@@ -173,6 +170,7 @@ def spectral_density(wav, factor=None):
     la_f_la = nu_f_nu
     phot_f_la = astrophys.photon / (si.cm ** 2 * si.s * si.AA)
     phot_f_nu = astrophys.photon / (si.cm ** 2 * si.s * si.Hz)
+    la_phot_f_la = astrophys.photon / (si.cm ** 2 * si.s)
 
     # luminosity density
     L_nu = cgs.erg / si.s / si.Hz
@@ -182,7 +180,7 @@ def spectral_density(wav, factor=None):
     phot_L_la = astrophys.photon / (si.s * si.AA)
     phot_L_nu = astrophys.photon / (si.s * si.Hz)
 
-    # surface brigthness (flux equiv)
+    # surface brightness (flux equiv)
     S_la = cgs.erg / si.angstrom / si.cm ** 2 / si.s / si.sr
     S_nu = cgs.erg / si.Hz / si.cm ** 2 / si.s / si.sr
     nu_S_nu = cgs.erg / si.cm ** 2 / si.s / si.sr
@@ -270,6 +268,8 @@ def spectral_density(wav, factor=None):
         (phot_f_la, phot_f_nu, converter_phot_f_la_phot_f_nu, iconverter_phot_f_la_phot_f_nu),
         (phot_f_nu, f_nu, converter_phot_f_nu_to_f_nu, iconverter_phot_f_nu_to_f_nu),
         (phot_f_nu, f_la, converter_phot_f_nu_to_f_la, iconverter_phot_f_nu_to_f_la),
+        # integrated flux
+        (la_phot_f_la, la_f_la, converter_phot_f_la_to_f_la, iconverter_phot_f_la_to_f_la),
         # luminosity
         (L_la, L_nu, converter, iconverter),
         (L_nu, nu_L_nu, converter_L_nu_to_nu_L_nu, iconverter_L_nu_to_nu_L_nu),
@@ -360,9 +360,9 @@ def doppler_radio(rest):
         return resten * (1-voverc)
 
     return Equivalency([(si.Hz, si.km/si.s, to_vel_freq, from_vel_freq),
-            (si.AA, si.km/si.s, to_vel_wav, from_vel_wav),
-            (si.eV, si.km/si.s, to_vel_en, from_vel_en),
-            ], "doppler_radio", {'rest': rest})
+                        (si.AA, si.km/si.s, to_vel_wav, from_vel_wav),
+                        (si.eV, si.km/si.s, to_vel_en, from_vel_en),
+                        ], "doppler_radio", {'rest': rest})
 
 
 def doppler_optical(rest):
@@ -426,9 +426,9 @@ def doppler_optical(rest):
         return resten / (1+voverc)
 
     return Equivalency([(si.Hz, si.km/si.s, to_vel_freq, from_vel_freq),
-            (si.AA, si.km/si.s, to_vel_wav, from_vel_wav),
-            (si.eV, si.km/si.s, to_vel_en, from_vel_en),
-            ], "doppler_optical", {'rest': rest})
+                        (si.AA, si.km/si.s, to_vel_wav, from_vel_wav),
+                        (si.eV, si.km/si.s, to_vel_en, from_vel_en),
+                        ], "doppler_optical", {'rest': rest})
 
 
 def doppler_relativistic(rest):
@@ -465,7 +465,7 @@ def doppler_relativistic(rest):
     >>> relativistic_wavelength = measured_velocity.to(u.mm, equivalencies=relativistic_CO_equiv)
     >>> relativistic_wavelength  # doctest: +FLOAT_CMP
     <Quantity 2.6116243681798923 mm>
-    """
+    """  # noqa: E501
 
     assert_is_spectral_unit(rest)
 
@@ -499,9 +499,34 @@ def doppler_relativistic(rest):
         return resten * ((1-voverc) / (1+(voverc)))**0.5
 
     return Equivalency([(si.Hz, si.km/si.s, to_vel_freq, from_vel_freq),
-            (si.AA, si.km/si.s, to_vel_wav, from_vel_wav),
-            (si.eV, si.km/si.s, to_vel_en, from_vel_en),
-            ], "doppler_relativistic", {'rest': rest})
+                        (si.AA, si.km/si.s, to_vel_wav, from_vel_wav),
+                        (si.eV, si.km/si.s, to_vel_en, from_vel_en),
+                        ], "doppler_relativistic", {'rest': rest})
+
+
+def doppler_redshift():
+    """
+    Returns the equivalence between Doppler redshift (unitless) and radial velocity.
+
+    .. note::
+
+        This equivalency is not compatible with cosmological
+        redshift in `astropy.cosmology.units`.
+
+    """
+    rv_unit = si.km / si.s
+    C_KMS = _si.c.to_value(rv_unit)
+
+    def convert_z_to_rv(z):
+        zponesq = (1 + z) ** 2
+        return C_KMS * (zponesq - 1) / (zponesq + 1)
+
+    def convert_rv_to_z(rv):
+        beta = rv / C_KMS
+        return np.sqrt((1 + beta) / (1 - beta)) - 1
+
+    return Equivalency([(dimensionless_unscaled, rv_unit, convert_z_to_rv, convert_rv_to_z)],
+                       "doppler_redshift")
 
 
 def molar_mass_amu():
@@ -509,7 +534,7 @@ def molar_mass_amu():
     Returns the equivalence between amu and molar mass.
     """
     return Equivalency([
-        (si.g/si.mol, astrophys.u)
+        (si.g/si.mol, misc.u)
     ], "molar_mass_amu")
 
 
@@ -520,16 +545,16 @@ def mass_energy():
     """
 
     return Equivalency([(si.kg, si.J, lambda x: x * _si.c.value ** 2,
-             lambda x: x / _si.c.value ** 2),
-            (si.kg / si.m ** 2, si.J / si.m ** 2,
-             lambda x: x * _si.c.value ** 2,
-             lambda x: x / _si.c.value ** 2),
-            (si.kg / si.m ** 3, si.J / si.m ** 3,
-             lambda x: x * _si.c.value ** 2,
-             lambda x: x / _si.c.value ** 2),
-            (si.kg / si.s, si.J / si.s, lambda x: x * _si.c.value ** 2,
-             lambda x: x / _si.c.value ** 2),
-    ], "mass_energy")
+                         lambda x: x / _si.c.value ** 2),
+                        (si.kg / si.m ** 2, si.J / si.m ** 2,
+                         lambda x: x * _si.c.value ** 2,
+                         lambda x: x / _si.c.value ** 2),
+                        (si.kg / si.m ** 3, si.J / si.m ** 3,
+                         lambda x: x * _si.c.value ** 2,
+                         lambda x: x / _si.c.value ** 2),
+                        (si.kg / si.s, si.J / si.s, lambda x: x * _si.c.value ** 2,
+                         lambda x: x / _si.c.value ** 2),
+                        ], "mass_energy")
 
 
 def brightness_temperature(frequency, beam_area=None):
@@ -549,14 +574,14 @@ def brightness_temperature(frequency, beam_area=None):
 
     Parameters
     ----------
-    frequency : `~astropy.units.Quantity` with spectral units
+    frequency : `~astropy.units.Quantity`
         The observed ``spectral`` equivalent `~astropy.units.Unit` (e.g.,
         frequency or wavelength).  The variable is named 'frequency' because it
         is more commonly used in radio astronomy.
         BACKWARD COMPATIBILITY NOTE: previous versions of the brightness
         temperature equivalency used the keyword ``disp``, which is no longer
         supported.
-    beam_area : angular area equivalent
+    beam_area : `~astropy.units.Quantity` ['solid angle']
         Beam area in angular units, i.e. steradian equivalent
 
     Examples
@@ -588,7 +613,7 @@ def brightness_temperature(frequency, beam_area=None):
         >>> surf_brightness = 1e6*u.MJy/u.sr
         >>> surf_brightness.to(u.K, equivalencies=u.brightness_temperature(500*u.GHz)) # doctest: +FLOAT_CMP
         <Quantity 130.1931904778803 K>
-    """
+    """  # noqa: E501
     if frequency.unit.is_equivalent(si.sr):
         if not beam_area.unit.is_equivalent(si.Hz):
             raise ValueError("The inputs to `brightness_temperature` are "
@@ -613,8 +638,8 @@ def brightness_temperature(frequency, beam_area=None):
             return (x_K * beam / factor)
 
         return Equivalency([(astrophys.Jy, si.K, convert_Jy_to_K, convert_K_to_Jy),
-                            (astrophys.Jy/astrophys.beam, si.K, convert_Jy_to_K, convert_K_to_Jy),],
-                           "brightness_temperature", {'frequency': frequency, 'beam_area': beam_area})
+                            (astrophys.Jy/astrophys.beam, si.K, convert_Jy_to_K, convert_K_to_Jy)],
+                           "brightness_temperature", {'frequency': frequency, 'beam_area': beam_area})  # noqa: E501
     else:
         def convert_JySr_to_K(x_jysr):
             factor = (2 * _si.k_B * si.K * nu**2 / _si.c**2).to(astrophys.Jy).value
@@ -622,10 +647,10 @@ def brightness_temperature(frequency, beam_area=None):
 
         def convert_K_to_JySr(x_K):
             factor = (astrophys.Jy / (2 * _si.k_B * nu**2 / _si.c**2)).to(si.K).value
-            return (x_K / factor) # multiplied by 1x for 1 steradian
+            return (x_K / factor)  # multiplied by 1x for 1 steradian
 
         return Equivalency([(astrophys.Jy/si.sr, si.K, convert_JySr_to_K, convert_K_to_JySr)],
-                           "brightness_temperature", {'frequency': frequency, 'beam_area': beam_area})
+                           "brightness_temperature", {'frequency': frequency, 'beam_area': beam_area})  # noqa: E501
 
 
 def beam_angular_area(beam_area):
@@ -637,12 +662,13 @@ def beam_angular_area(beam_area):
 
     Parameters
     ----------
-    beam_area : angular area equivalent
+    beam_area : unit-like
         The area of the beam in angular area units (e.g., steradians)
+        Must have angular area equivalent units.
     """
     return Equivalency([(astrophys.beam, Unit(beam_area)),
                         (astrophys.beam**-1, Unit(beam_area)**-1),
-                        (astrophys.Jy/astrophys.beam, astrophys.Jy/Unit(beam_area)),],
+                        (astrophys.Jy/astrophys.beam, astrophys.Jy/Unit(beam_area))],
                        "beam_angular_area", {'beam_area': beam_area})
 
 
@@ -658,12 +684,12 @@ def thermodynamic_temperature(frequency, T_cmb=None):
 
     Parameters
     ----------
-    frequency : `~astropy.units.Quantity` with spectral units
+    frequency : `~astropy.units.Quantity`
         The observed `spectral` equivalent `~astropy.units.Unit` (e.g.,
-        frequency or wavelength)
-    T_cmb :  `~astropy.units.Quantity` with temperature units or None
+        frequency or wavelength). Must have spectral units.
+    T_cmb :  `~astropy.units.Quantity` ['temperature'] or None
         The CMB temperature at z=0.  If `None`, the default cosmology will be
-        used to get this temperature.
+        used to get this temperature. Must have units of temperature.
 
     Notes
     -----
@@ -710,15 +736,18 @@ def thermodynamic_temperature(frequency, T_cmb=None):
 
 
 def temperature():
-    """Convert between Kelvin, Celsius, and Fahrenheit here because
+    """Convert between Kelvin, Celsius, Rankine and Fahrenheit here because
     Unit and CompositeUnit cannot do addition or subtraction properly.
     """
-    from .imperial import deg_F
+    from .imperial import deg_F, deg_R
     return Equivalency([
         (si.K, si.deg_C, lambda x: x - 273.15, lambda x: x + 273.15),
         (si.deg_C, deg_F, lambda x: x * 1.8 + 32.0, lambda x: (x - 32.0) / 1.8),
         (si.K, deg_F, lambda x: (x - 273.15) * 1.8 + 32.0,
-         lambda x: ((x - 32.0) / 1.8) + 273.15)], "temperature")
+         lambda x: ((x - 32.0) / 1.8) + 273.15),
+        (deg_R, deg_F, lambda x: x - 459.67, lambda x: x + 459.67),
+        (deg_R, si.deg_C, lambda x: (x - 491.67) * (5/9), lambda x: x * 1.8 + 491.67),
+        (deg_R, si.K, lambda x: x * (5/9), lambda x: x * 1.8)], "temperature")
 
 
 def temperature_energy():
@@ -738,24 +767,29 @@ def assert_is_spectral_unit(value):
 
 def pixel_scale(pixscale):
     """
-    Convert between pixel distances (in units of ``pix``) and angular units,
+    Convert between pixel distances (in units of ``pix``) and other units,
     given a particular ``pixscale``.
 
     Parameters
     ----------
     pixscale : `~astropy.units.Quantity`
-        The pixel scale either in units of angle/pixel or pixel/angle.
+        The pixel scale either in units of <unit>/pixel or pixel/<unit>.
     """
-    if pixscale.unit.is_equivalent(si.arcsec/astrophys.pix):
-        pixscale_val = pixscale.to_value(si.radian/astrophys.pix)
-    elif pixscale.unit.is_equivalent(astrophys.pix/si.arcsec):
-        pixscale_val = (1/pixscale).to_value(si.radian/astrophys.pix)
-    else:
-        raise UnitsError("The pixel scale must be in angle/pixel or "
-                         "pixel/angle")
 
-    return Equivalency([(astrophys.pix, si.radian,
-                         lambda px: px*pixscale_val, lambda rad: rad/pixscale_val)],
+    decomposed = pixscale.unit.decompose()
+    dimensions = dict(zip(decomposed.bases, decomposed.powers))
+    pix_power = dimensions.get(misc.pix, 0)
+
+    if pix_power == -1:
+        physical_unit = Unit(pixscale * misc.pix)
+    elif pix_power == 1:
+        physical_unit = Unit(misc.pix / pixscale)
+    else:
+        raise UnitsError(
+                "The pixel scale unit must have"
+                " pixel dimensionality of 1 or -1.")
+
+    return Equivalency([(misc.pix, physical_unit)],
                        "pixel_scale", {'pixscale': pixscale})
 
 
@@ -777,31 +811,26 @@ def plate_scale(platescale):
         raise UnitsError("The pixel scale must be in angle/distance or "
                          "distance/angle")
 
-    return Equivalency([(si.m, si.radian, lambda d: d*platescale_val, lambda rad: rad/platescale_val)],
+    return Equivalency([(si.m, si.radian, lambda d: d*platescale_val,
+                         lambda rad: rad/platescale_val)],
                        "plate_scale", {'platescale': platescale})
 
 
-def with_H0(H0=None):
-    """
-    Convert between quantities with little-h and the equivalent physical units.
+# -------------------------------------------------------------------------
 
-    Parameters
-    ----------
-    H0 : `None` or `~astropy.units.Quantity`
-        The value of the Hubble constant to assume. If a `~astropy.units.Quantity`,
-        will assume the quantity *is* ``H0``.  If `None` (default), use the
-        ``H0`` attribute from the default `astropy.cosmology` cosmology.
+def __getattr__(attr):
+    if attr == "with_H0":
+        import warnings
 
-    References
-    ----------
-    For an illuminating discussion on why you may or may not want to use
-    little-h at all, see https://arxiv.org/pdf/1308.4150.pdf
-    """
+        from astropy.cosmology.units import with_H0
+        from astropy.utils.exceptions import AstropyDeprecationWarning
 
-    if H0 is None:
-        from astropy import cosmology
-        H0 = cosmology.default_cosmology.get().H0
+        warnings.warn(
+            ("`with_H0` is deprecated from `astropy.units.equivalencies` "
+             "since astropy 5.0 and may be removed in a future version. "
+             "Use `astropy.cosmology.units.with_H0` instead."),
+            AstropyDeprecationWarning)
 
-    h100_val_unit = Unit(100/(H0.to_value((si.km/si.s)/astrophys.Mpc)) * astrophys.littleh)
+        return with_H0
 
-    return Equivalency([(h100_val_unit, None)], "with_H0", kwargs={"H0": H0})
+    raise AttributeError(f"module {__name__!r} has no attribute {attr!r}.")
