@@ -39,25 +39,17 @@ import functools
 import operator
 
 import numpy as np
+from numpy.lib import recfunctions as rfn
 
-from astropy.units.core import (
-    UnitsError, UnitTypeError, dimensionless_unscaled)
-from astropy.utils.compat import (NUMPY_LT_1_17, NUMPY_LT_1_18,
-                                  NUMPY_LT_1_20)
+from astropy.units.core import UnitsError, UnitTypeError, dimensionless_unscaled
 from astropy.utils import isiterable
+from astropy.utils.compat import NUMPY_LT_1_20, NUMPY_LT_1_23
 
-
-if NUMPY_LT_1_17:  # pragma: no cover
-    # pre 1.16, overrides.py did not exist; in 1.16, it existed, but
-    # __array_function__ overrides were turned off by default.
-    ARRAY_FUNCTION_ENABLED = (hasattr(np.core, 'overrides') and
-                              np.core.overrides.ENABLE_ARRAY_FUNCTION)
-else:
-    # In 1.17, overrides are enabled by default, but it is still possible to
-    # turn them off using an environment variable.  We use getattr since it
-    # is planned to remove that possibility in later numpy versions.
-    ARRAY_FUNCTION_ENABLED = getattr(np.core.overrides,
-                                     'ENABLE_ARRAY_FUNCTION', True)
+# In 1.17, overrides are enabled by default, but it is still possible to
+# turn them off using an environment variable.  We use getattr since it
+# is planned to remove that possibility in later numpy versions.
+ARRAY_FUNCTION_ENABLED = getattr(np.core.overrides,
+                                 'ENABLE_ARRAY_FUNCTION', True)
 SUBCLASS_SAFE_FUNCTIONS = set()
 """Functions with implementations supporting subclasses like Quantity."""
 FUNCTION_HELPERS = {}
@@ -77,7 +69,7 @@ SUBCLASS_SAFE_FUNCTIONS |= {
     np.nonzero, np.argwhere, np.flatnonzero,
     np.diag_indices_from, np.triu_indices_from, np.tril_indices_from,
     np.real, np.imag, np.diagonal, np.diagflat,
-    np.empty_like, np.zeros_like,
+    np.empty_like,
     np.compress, np.extract, np.delete, np.trim_zeros, np.roll, np.take,
     np.put, np.fill_diagonal, np.tile, np.repeat,
     np.split, np.array_split, np.hsplit, np.vsplit, np.dsplit,
@@ -99,9 +91,6 @@ SUBCLASS_SAFE_FUNCTIONS |= {
     np.apply_along_axis, np.take_along_axis, np.put_along_axis,
     np.linalg.cond, np.linalg.multi_dot}
 
-if NUMPY_LT_1_18:
-    SUBCLASS_SAFE_FUNCTIONS |= {np.alen}
-
 # Implemented as methods on Quantity:
 # np.ediff1d is from setops, but we support it anyway; the others
 # currently return NotImplementedError.
@@ -118,27 +107,37 @@ UNSUPPORTED_FUNCTIONS |= {
 # Could be supported if we had a natural logarithm unit.
 UNSUPPORTED_FUNCTIONS |= {np.linalg.slogdet}
 
+# TODO! support whichever of these functions it makes sense to support
+TBD_FUNCTIONS = {
+    rfn.drop_fields, rfn.rename_fields, rfn.append_fields, rfn.join_by,
+    rfn.apply_along_fields, rfn.assign_fields_by_name, rfn.merge_arrays,
+    rfn.find_duplicates, rfn.recursive_fill_fields, rfn.require_fields,
+    rfn.repack_fields, rfn.stack_arrays
+}
+UNSUPPORTED_FUNCTIONS |= TBD_FUNCTIONS
+
 # The following are not just unsupported, but so unlikely to be thought
 # to be supported that we ignore them in testing.  (Kept in a separate
 # variable so that we can check consistency in the test routine -
 # test_quantity_non_ufuncs.py)
 IGNORED_FUNCTIONS = {
-    # Deprecated
-    np.asscalar,
     # I/O - useless for Quantity, since no way to store the unit.
     np.save, np.savez, np.savetxt, np.savez_compressed,
     # Polynomials
     np.poly, np.polyadd, np.polyder, np.polydiv, np.polyfit, np.polyint,
-    np.polymul, np.polysub, np.polyval, np.roots, np.vander}
+    np.polymul, np.polysub, np.polyval, np.roots, np.vander,
+    # functions taking record arrays (which are deprecated)
+    rfn.rec_append_fields, rfn.rec_drop_fields, rfn.rec_join,
+}
 if NUMPY_LT_1_20:
     # financial
     IGNORED_FUNCTIONS |= {np.fv, np.ipmt, np.irr, np.mirr, np.nper,
                           np.npv, np.pmt, np.ppmt, np.pv, np.rate}
-
-if NUMPY_LT_1_18:
-    IGNORED_FUNCTIONS |= {np.rank}
-else:
-    IGNORED_FUNCTIONS |= {np.alen}
+if NUMPY_LT_1_23:
+    IGNORED_FUNCTIONS |= {
+        # Deprecated, removed in numpy 1.23
+        np.asscalar, np.alen,
+    }
 UNSUPPORTED_FUNCTIONS |= IGNORED_FUNCTIONS
 
 
@@ -197,13 +196,17 @@ def invariant_x_helper(x, *args, **kwargs):
     return (x.view(np.ndarray),) + args, kwargs, x.unit, None
 
 
-# Note that ones_like does *not* work by default (unlike zeros_like) since if
-# one creates an empty array with a unit, one cannot just fill it with unity.
-# Indeed, in this respect, it is a bit of an odd function for Quantity. On the
-# other hand, it matches the idea that a unit is the same as the quantity with
-# that unit and value of 1. Also, it used to work without __array_function__.
-@function_helper
-def ones_like(a, *args, **kwargs):
+# Note that ones_like does *not* work by default since if one creates an empty
+# array with a unit, one cannot just fill it with unity.  Indeed, in this
+# respect, it is a bit of an odd function for Quantity. On the other hand, it
+# matches the idea that a unit is the same as the quantity with that unit and
+# value of 1. Also, it used to work without __array_function__.
+# zeros_like does work by default for regular quantities, because numpy first
+# creates an empty array with the unit and then fills it with 0 (which can have
+# any unit), but for structured dtype this fails (0 cannot have an arbitrary
+# structured unit), so we include it here too.
+@function_helper(helps={np.ones_like, np.zeros_like})
+def like_helper(a, *args, **kwargs):
     subok = args[2] if len(args) > 2 else kwargs.pop('subok', True)
     unit = a.unit if subok else None
     return (a.view(np.ndarray),) + args, kwargs, unit, None
@@ -445,8 +448,7 @@ def piecewise(x, condlist, funclist, *args, **kw):
         n += 1
     elif n != n2:
         raise ValueError(
-            "with {} condition(s), either {} or {} functions are expected"
-            .format(n, n, n+1)
+            f"with {n} condition(s), either {n} or {n + 1} functions are expected"
         )
 
     y = np.zeros(x.shape, x.dtype)
@@ -1054,3 +1056,51 @@ def eig(a, *args, **kwargs):
     from astropy.units import dimensionless_unscaled
 
     return (a.value,)+args, kwargs, (a.unit, dimensionless_unscaled), None
+
+
+@function_helper(module=np.lib.recfunctions)
+def structured_to_unstructured(arr, *args, **kwargs):
+    """
+    Convert a structured quantity to an unstructured one.
+    This only works if all the units are compatible.
+
+    """
+    from astropy.units import StructuredUnit
+
+    target_unit = arr.unit.values()[0]
+
+    def replace_unit(x):
+        if isinstance(x, StructuredUnit):
+            return x._recursively_apply(replace_unit)
+        else:
+            return target_unit
+
+    to_unit = arr.unit._recursively_apply(replace_unit)
+    return (arr.to_value(to_unit), ) + args, kwargs, target_unit, None
+
+
+def _build_structured_unit(dtype, unit):
+    """Build structured unit from dtype
+
+    Parameters
+    ----------
+    dtype : `numpy.dtype`
+    unit : `astropy.units.Unit`
+
+    Returns
+    -------
+    `astropy.units.Unit` or tuple
+    """
+    if dtype.fields is None:
+        return unit
+
+    return tuple(_build_structured_unit(v[0], unit) for v in dtype.fields.values())
+
+
+@function_helper(module=np.lib.recfunctions)
+def unstructured_to_structured(arr, dtype, *args, **kwargs):
+    from astropy.units import StructuredUnit
+
+    target_unit = StructuredUnit(_build_structured_unit(dtype, arr.unit))
+
+    return (arr.to_value(arr.unit), dtype) + args, kwargs, target_unit, None
