@@ -2,16 +2,16 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Separate tests specifically for equivalencies."""
 
+import numpy as np
 # THIRD-PARTY
 import pytest
-import numpy as np
 from numpy.testing import assert_allclose
 
 # LOCAL
+from astropy import constants
 from astropy import units as u
+from astropy.tests.helper import assert_quantity_allclose
 from astropy.units.equivalencies import Equivalency
-from astropy import constants, cosmology
-from astropy.tests.helper import assert_quantity_allclose, catch_warnings
 from astropy.utils.exceptions import AstropyDeprecationWarning
 
 
@@ -36,7 +36,7 @@ def test_dimensionless_angles():
     # and check that value is correct
     assert_allclose(Erot_in_erg1.value, (Erot/u.radian**2).to_value(u.erg))
 
-    # test build-in equivalency in subclass
+    # test built-in equivalency in subclass
     class MyRad1(u.Quantity):
         _equivalencies = rad1
 
@@ -144,6 +144,24 @@ def test_bad_restfreqs(function, value):
         function(value)
 
 
+@pytest.mark.parametrize(('z', 'rv_ans'),
+                         [(0, 0 * (u.km / u.s)),
+                          (0.001, 299642.56184583 * (u.m / u.s)),
+                          (-1, -2.99792458e8 * (u.m / u.s))])
+def test_doppler_redshift(z, rv_ans):
+    z_in = z * u.dimensionless_unscaled
+    rv_out = z_in.to(u.km / u.s, u.doppler_redshift())
+    z_out = rv_out.to(u.dimensionless_unscaled, u.doppler_redshift())
+    assert_quantity_allclose(rv_out, rv_ans)
+    assert_quantity_allclose(z_out, z_in)  # Check roundtrip
+
+
+def test_doppler_redshift_no_cosmology():
+    from astropy.cosmology.units import redshift
+    with pytest.raises(u.UnitConversionError, match='not convertible'):
+        (0 * (u.km / u.s)).to(redshift, u.doppler_redshift())
+
+
 def test_massenergy():
     # The relative tolerance of these tests is set by the uncertainties
     # in the charge of the electron, which is known to about
@@ -299,22 +317,36 @@ def test_spectral4(in_val, in_unit):
         assert_allclose(b, in_val)
 
 
-def test_spectraldensity2():
+@pytest.mark.parametrize('wav', (3500 * u.AA,
+                                 8.5654988e+14 * u.Hz,
+                                 1 / (3500 * u.AA),
+                                 5.67555959e-19 * u.J))
+def test_spectraldensity2(wav):
     # flux density
     flambda = u.erg / u.angstrom / u.cm ** 2 / u.s
     fnu = u.erg / u.Hz / u.cm ** 2 / u.s
 
-    a = flambda.to(fnu, 1, u.spectral_density(u.Quantity(3500, u.AA)))
+    a = flambda.to(fnu, 1, u.spectral_density(wav))
     assert_allclose(a, 4.086160166177361e-12)
 
+    # integrated flux
+    f_int = u.erg / u.cm ** 2 / u.s
+    phot_int = u.ph / u.cm ** 2 / u.s
+
+    a = f_int.to(phot_int, 1, u.spectral_density(wav))
+    assert_allclose(a, 1.7619408e+11)
+
+    a = phot_int.to(f_int, 1, u.spectral_density(wav))
+    assert_allclose(a, 5.67555959e-12)
+
     # luminosity density
     llambda = u.erg / u.angstrom / u.s
     lnu = u.erg / u.Hz / u.s
 
-    a = llambda.to(lnu, 1, u.spectral_density(u.Quantity(3500, u.AA)))
+    a = llambda.to(lnu, 1, u.spectral_density(wav))
     assert_allclose(a, 4.086160166177361e-12)
 
-    a = lnu.to(llambda, 1, u.spectral_density(u.Quantity(3500, u.AA)))
+    a = lnu.to(llambda, 1, u.spectral_density(wav))
     assert_allclose(a, 2.44728537142857e11)
 
 
@@ -477,6 +509,24 @@ def test_spectraldensity6():
         snu, sb_flam, u.spectral_density(wave)), sb_fnu, rtol=1e-6)
 
 
+@pytest.mark.parametrize(
+    ('from_unit', 'to_unit'),
+    [(u.ph / u.cm ** 2 / u.s, (u.cm * u.cm * u.s) ** -1),
+     (u.ph / u.cm ** 2 / u.s, u.erg / (u.cm * u.cm * u.s * u.keV)),
+     (u.erg / u.cm ** 2 / u.s, (u.cm * u.cm * u.s) ** -1),
+     (u.erg / u.cm ** 2 / u.s, u.erg / (u.cm * u.cm * u.s * u.keV))])
+def test_spectraldensity_not_allowed(from_unit, to_unit):
+    """Not allowed to succeed as
+    per https://github.com/astropy/astropy/pull/10015
+    """
+    with pytest.raises(u.UnitConversionError, match='not convertible'):
+        from_unit.to(to_unit, 1, u.spectral_density(1 * u.AA))
+
+    # The other way
+    with pytest.raises(u.UnitConversionError, match='not convertible'):
+        to_unit.to(from_unit, 1, u.spectral_density(1 * u.AA))
+
+
 def test_equivalent_units():
     from astropy.units import imperial
     with u.add_enabled_units(imperial):
@@ -495,7 +545,7 @@ def test_equivalent_units():
 def test_equivalent_units2():
     units = set(u.Hz.find_equivalent_units(u.spectral()))
     match = set(
-        [u.AU, u.Angstrom, u.Hz, u.J, u.Ry, u.cm, u.eV, u.erg, u.lyr,
+        [u.AU, u.Angstrom, u.Hz, u.J, u.Ry, u.cm, u.eV, u.erg, u.lyr, u.lsec,
          u.m, u.micron, u.pc, u.solRad, u.Bq, u.Ci, u.k, u.earthRad,
          u.jupiterRad])
     assert units == match
@@ -506,14 +556,14 @@ def test_equivalent_units2():
         match = set(
             [u.AU, u.Angstrom, imperial.BTU, u.Hz, u.J, u.Ry,
              imperial.cal, u.cm, u.eV, u.erg, imperial.ft, imperial.fur,
-             imperial.inch, imperial.kcal, u.lyr, u.m, imperial.mi,
+             imperial.inch, imperial.kcal, u.lyr, u.m, imperial.mi, u.lsec,
              imperial.mil, u.micron, u.pc, u.solRad, imperial.yd, u.Bq, u.Ci,
              imperial.nmi, u.k, u.earthRad, u.jupiterRad])
         assert units == match
 
     units = set(u.Hz.find_equivalent_units(u.spectral()))
     match = set(
-        [u.AU, u.Angstrom, u.Hz, u.J, u.Ry, u.cm, u.eV, u.erg, u.lyr,
+        [u.AU, u.Angstrom, u.Hz, u.J, u.Ry, u.cm, u.eV, u.erg, u.lyr, u.lsec,
          u.m, u.micron, u.pc, u.solRad, u.Bq, u.Ci, u.k, u.earthRad,
          u.jupiterRad])
     assert units == match
@@ -556,12 +606,12 @@ def test_swapped_args_brightness_temperature():
     nu = u.GHz * 5
     tb = 7.052587837212582 * u.K
 
-    with catch_warnings(AstropyDeprecationWarning) as w:
+    with pytest.warns(AstropyDeprecationWarning) as w:
         result = (1*u.Jy).to(
             u.K, equivalencies=u.brightness_temperature(omega_B, nu))
         roundtrip = result.to(
             u.Jy, equivalencies=u.brightness_temperature(omega_B, nu))
-        assert len(w) == 2
+    assert len(w) == 2
     np.testing.assert_almost_equal(tb.value, result.value)
     np.testing.assert_almost_equal(roundtrip.value, 1)
 
@@ -659,7 +709,10 @@ def test_equivalency_context_manager():
 
     tf_dimensionless_angles = just_to_from_units(u.dimensionless_angles())
     tf_spectral = just_to_from_units(u.spectral())
-    assert base_registry.equivalencies == []
+
+    # <=1 b/c might have the dimensionless_redshift equivalency enabled.
+    assert len(base_registry.equivalencies) <= 1
+
     with u.set_enabled_equivalencies(u.dimensionless_angles()):
         new_registry = u.get_current_unit_registry()
         assert (set(just_to_from_units(new_registry.equivalencies)) ==
@@ -686,10 +739,22 @@ def test_equivalency_context_manager():
 
 
 def test_temperature():
-    from astropy.units.imperial import deg_F
+    from astropy.units.imperial import deg_F, deg_R
     t_k = 0 * u.K
     assert_allclose(t_k.to_value(u.deg_C, u.temperature()), -273.15)
     assert_allclose(t_k.to_value(deg_F, u.temperature()), -459.67)
+    t_k = 20 * u.K
+    assert_allclose(t_k.to_value(deg_R, u.temperature()), 36.0)
+    t_k = 20 * deg_R
+    assert_allclose(t_k.to_value(u.K, u.temperature()), 11.11, atol=0.01)
+    t_k = 20 * deg_F
+    assert_allclose(t_k.to_value(deg_R, u.temperature()), 479.67)
+    t_k = 20 * deg_R
+    assert_allclose(t_k.to_value(deg_F, u.temperature()), -439.67)
+    t_k = 20 * u.deg_C
+    assert_allclose(t_k.to_value(deg_R, u.temperature()), 527.67)
+    t_k = 20 * deg_R
+    assert_allclose(t_k.to_value(u.deg_C, u.temperature()), -262.039, atol=0.01)
 
 
 def test_temperature_energy():
@@ -747,6 +812,35 @@ def test_pixel_scale():
     assert_quantity_allclose(asec.to(u.pix, u.pixel_scale(pixscale2)), pix)
 
 
+def test_pixel_scale_invalid_scale_unit():
+
+    pixscale = 0.4 * u.arcsec
+    pixscale2 = 0.4 * u.arcsec / u.pix ** 2
+
+    with pytest.raises(u.UnitsError, match="pixel dimension"):
+        u.pixel_scale(pixscale)
+    with pytest.raises(u.UnitsError, match="pixel dimension"):
+        u.pixel_scale(pixscale2)
+
+
+def test_pixel_scale_acceptable_scale_unit():
+
+    pix = 75 * u.pix
+    v = 3000 * (u.cm / u.s)
+
+    pixscale = 0.4 * (u.m / u.s / u.pix)
+    pixscale2 = 2.5 * (u.pix / (u.m / u.s))
+
+    assert_quantity_allclose(pix.to(u.m / u.s, u.pixel_scale(pixscale)), v)
+    assert_quantity_allclose(pix.to(u.km / u.s, u.pixel_scale(pixscale)), v)
+
+    assert_quantity_allclose(pix.to(u.m / u.s, u.pixel_scale(pixscale2)), v)
+    assert_quantity_allclose(pix.to(u.km / u.s, u.pixel_scale(pixscale2)), v)
+
+    assert_quantity_allclose(v.to(u.pix, u.pixel_scale(pixscale)), pix)
+    assert_quantity_allclose(v.to(u.pix, u.pixel_scale(pixscale2)), pix)
+
+
 def test_plate_scale():
     mm = 1.5*u.mm
     asec = 30*u.arcsec
@@ -764,28 +858,6 @@ def test_plate_scale():
     assert_quantity_allclose(asec.to(u.mm, u.plate_scale(platescale2)), mm)
 
 
-def test_littleh():
-    H0_70 = 70*u.km/u.s/u.Mpc
-    h70dist = 70 * u.Mpc/u.littleh
-
-    assert_quantity_allclose(h70dist.to(u.Mpc, u.with_H0(H0_70)), 100*u.Mpc)
-
-    # make sure using the default cosmology works
-    cosmodist = cosmology.default_cosmology.get().H0.value * u.Mpc/u.littleh
-    assert_quantity_allclose(cosmodist.to(u.Mpc, u.with_H0()), 100*u.Mpc)
-
-    # Now try a luminosity scaling
-    h1lum = .49 * u.Lsun * u.littleh**-2
-    assert_quantity_allclose(h1lum.to(u.Lsun, u.with_H0(H0_70)), 1*u.Lsun)
-
-    # And the trickiest one: magnitudes.  Using H0=10 here for the round numbers
-    H0_10 = 10*u.km/u.s/u.Mpc
-    # assume the "true" magnitude M = 12.
-    # Then M - 5*log_10(h)  = M + 5 = 17
-    withlittlehmag = 17 * (u.mag - u.MagUnit(u.littleh**2))
-    assert_quantity_allclose(withlittlehmag.to(u.mag, u.with_H0(H0_10)), 12*u.mag)
-
-
 def test_equivelency():
     ps = u.pixel_scale(10*u.arcsec/u.pix)
     assert isinstance(ps, Equivalency)
