@@ -1,19 +1,20 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
 import os
-import urllib.request
 import warnings
+from pathlib import Path
 
 import pytest
 import numpy as np
 
-from astropy.tests.helper import assert_quantity_allclose, catch_warnings
+from astropy.tests.helper import assert_quantity_allclose
+from astropy.utils.data import get_pkg_data_filename
+from astropy.config import set_temp_cache
 from astropy.utils.iers import iers
 from astropy import units as u
 from astropy.table import QTable
 from astropy.time import Time, TimeDelta
 
-
 CI = os.environ.get('CI', False)
 
 FILE_NOT_FOUND_ERROR = getattr(__builtins__, 'FileNotFoundError', OSError)
@@ -25,7 +26,18 @@ except OSError:
 else:
     HAS_IERS_A = True
 
-IERS_A_EXCERPT = os.path.join(os.path.dirname(__file__), 'data', 'iers_a_excerpt')
+IERS_A_EXCERPT = get_pkg_data_filename(os.path.join('data', 'iers_a_excerpt'))
+
+
+def setup_module():
+    # Need auto_download so that IERS_B won't be loaded and cause tests to
+    # fail. Files to be downloaded are handled appropriately in the tests.
+    iers.conf.auto_download = True
+
+
+def teardown_module():
+    # This setting is to be consistent with astropy/conftest.py
+    iers.conf.auto_download = False
 
 
 class TestBasic():
@@ -86,7 +98,7 @@ class TestBasic():
 
     def test_open_network_url(self):
         iers.IERS_A.close()
-        iers.IERS_A.open("file:" + urllib.request.pathname2url(IERS_A_EXCERPT))
+        iers.IERS_A.open(Path(IERS_A_EXCERPT).as_uri())
         assert iers.IERS_A.iers_table is not None
         assert isinstance(iers.IERS_A.iers_table, QTable)
         iers.IERS_A.close()
@@ -194,10 +206,12 @@ class TestIERS_Auto():
         """
         self.N = 40
         self.ame = 30.0
-        self.iers_a_file_1 = os.path.join(os.path.dirname(__file__), 'data', 'finals2000A-2016-02-30-test')
-        self.iers_a_file_2 = os.path.join(os.path.dirname(__file__), 'data', 'finals2000A-2016-04-30-test')
-        self.iers_a_url_1 = os.path.normpath('file://' + os.path.abspath(self.iers_a_file_1))
-        self.iers_a_url_2 = os.path.normpath('file://' + os.path.abspath(self.iers_a_file_2))
+        self.iers_a_file_1 = get_pkg_data_filename(
+            os.path.join('data', 'finals2000A-2016-02-30-test'))
+        self.iers_a_file_2 = get_pkg_data_filename(
+            os.path.join('data', 'finals2000A-2016-04-30-test'))
+        self.iers_a_url_1 = Path(self.iers_a_file_1).as_uri()
+        self.iers_a_url_2 = Path(self.iers_a_file_2).as_uri()
         self.t = Time.now() + TimeDelta(10, format='jd') * np.arange(self.N)
 
     def teardown_method(self, method):
@@ -241,10 +255,9 @@ class TestIERS_Auto():
             with iers.conf.set_temp('auto_max_age', 5.0):
                 with pytest.raises(ValueError) as err:
                     iers_table = iers.IERS_Auto.open()
-                    delta = iers_table.ut1_utc(self.t.jd1, self.t.jd2)
+                    _ = iers_table.ut1_utc(self.t.jd1, self.t.jd2)
         assert str(err.value) == 'IERS auto_max_age configuration value must be larger than 10 days'
 
-    @pytest.mark.remote_data
     def test_no_auto_download(self):
         with iers.conf.set_temp('auto_download', False):
             t = iers.IERS_Auto.open()
@@ -274,25 +287,23 @@ class TestIERS_Auto():
             # and an exception when extrapolating into the future with insufficient data.
             dat._time_now = Time(predictive_mjd, format='mjd') + 60 * u.d
             assert np.allclose(dat.ut1_utc(Time(50000, format='mjd').jd).value, 0.1293286)
-            with catch_warnings(iers.IERSStaleWarning) as warns:
-                with pytest.raises(ValueError) as err:
-                    dat.ut1_utc(Time(60000, format='mjd').jd)
-            assert 'interpolating from IERS_Auto using predictive values' in str(err.value)
+            with pytest.warns(iers.IERSStaleWarning, match='IERS_Auto predictive '
+                              'values are older') as warns, \
+                 pytest.raises(ValueError, match='interpolating from IERS_Auto '
+                               'using predictive values'):
+                dat.ut1_utc(Time(60000, format='mjd').jd)
             assert len(warns) == 1
-            assert 'IERS_Auto predictive values are older' in str(warns[0].message)
 
             # Warning only if we are getting return status
-            with catch_warnings(iers.IERSStaleWarning) as warns:
+            with pytest.warns(iers.IERSStaleWarning, match='IERS_Auto '
+                              'predictive values are older') as warns:
                 dat.ut1_utc(Time(60000, format='mjd').jd, return_status=True)
             assert len(warns) == 1
-            assert 'IERS_Auto predictive values are older' in str(warns[0].message)
 
             # Now set auto_max_age = None which says that we don't care how old the
             # available IERS-A file is.  There should be no warnings or exceptions.
             with iers.conf.set_temp('auto_max_age', None):
-                with catch_warnings(iers.IERSStaleWarning) as warns:
-                    dat.ut1_utc(Time(60000, format='mjd').jd)
-                assert not warns
+                dat.ut1_utc(Time(60000, format='mjd').jd)
 
         # Now point to a later file with same values but MJD increased by
         # 60 days and see that things work.  dat._time_now is still the same value
@@ -367,3 +378,54 @@ def test_iers_b_dl():
         assert 'UT1_UTC' in iersb_tab.colnames
     finally:
         iers.IERS_B.close()
+
+
+@pytest.mark.remote_data
+def test_iers_out_of_range_handling(tmpdir):
+    # Make sure we don't have IERS-A data available anywhere
+    with set_temp_cache(tmpdir):
+        iers.IERS_A.close()
+        iers.IERS_Auto.close()
+        iers.IERS.close()
+        now = Time.now()
+        with iers.conf.set_temp('auto_download', False):
+            # Should be fine with built-in IERS_B
+            (now - 300 * u.day).ut1
+
+            # Default is to raise an error
+            match = r'\(some\) times are outside of range covered by IERS table'
+            with pytest.raises(iers.IERSRangeError, match=match):
+                (now + 100 * u.day).ut1
+
+            with iers.conf.set_temp('iers_degraded_accuracy', 'warn'):
+                with pytest.warns(iers.IERSDegradedAccuracyWarning, match=match):
+                    (now + 100 * u.day).ut1
+
+            with iers.conf.set_temp('iers_degraded_accuracy', 'ignore'):
+                (now + 100 * u.day).ut1
+
+
+@pytest.mark.remote_data
+def test_iers_download_error_handling(tmpdir):
+    # Make sure we don't have IERS-A data available anywhere
+    with set_temp_cache(tmpdir):
+        iers.IERS_A.close()
+        iers.IERS_Auto.close()
+        iers.IERS.close()
+        now = Time.now()
+
+        # bad site name
+        with iers.conf.set_temp('iers_auto_url', 'FAIL FAIL'):
+            # site that exists but doesn't have IERS data
+            with iers.conf.set_temp('iers_auto_url_mirror', 'https://google.com'):
+                with pytest.warns(iers.IERSWarning) as record:
+                    with iers.conf.set_temp('iers_degraded_accuracy', 'ignore'):
+                        (now + 100 * u.day).ut1
+
+                assert len(record) == 3
+                assert str(record[0].message).startswith(
+                    'failed to download FAIL FAIL: Malformed URL')
+                assert str(record[1].message).startswith(
+                    'malformed IERS table from https://google.com')
+                assert str(record[2].message).startswith(
+                    'unable to download valid IERS file, using local IERS-B')
