@@ -7,12 +7,31 @@ import platform
 import pytest
 import numpy as np
 from numpy.testing import assert_array_equal
+import erfa
 
-from astropy import _erfa as erfa
 from astropy.time import Time, TimeDelta
 from astropy.utils.iers import iers
 from astropy.utils.data import get_pkg_data_filename
+from astropy.tests.tests.test_imports import test_imports
 
+# Import every top-level astropy module as a test that the ERFA leap second
+# table is not updated for normal imports.
+test_imports()
+
+# Now test that the erfa leap_seconds table has not been updated. This must be
+# done at the module level, which unfortunately will abort the entire test run
+# if if fails. Running within a normal pytest test will not work because the
+# other tests will end up updating this attribute by virtue of doing Time UTC
+# transformations.
+assert erfa.leap_seconds._expires is None
+
+# Tests in this module assume that the erfa.leap_seconds attribute has been
+# updated from the `erfa` package built-in table to the astropy built-in
+# leap-second table. That has the effect of ensuring that the
+# `erfa.leap_seconds.expires` property is sufficiently in the future.
+iers_table = iers.LeapSeconds.auto_open()
+erfa.leap_seconds.update(iers_table)
+assert erfa.leap_seconds._expires is not None
 
 SYSTEM_FILE = '/usr/share/zoneinfo/leap-seconds.list'
 
@@ -120,18 +139,20 @@ def test_fake_file(tmpdir):
     assert fake.expires == Time('2345-06-28', scale='tai')
 
 
-# For this set of tests, leap-seconds are allowed to be expired
-# except as explicitly tested.
-@pytest.mark.filterwarnings(iers.IERSStaleWarning)
 class TestAutoOpenExplicitLists:
+    # For this set of tests, leap-seconds are allowed to be expired
+    # except as explicitly tested.
+    @pytest.mark.filterwarnings(iers.IERSStaleWarning)
     def test_auto_open_simple(self):
         ls = iers.LeapSeconds.auto_open([iers.IERS_LEAP_SECOND_FILE])
         assert ls.meta['data_url'] == iers.IERS_LEAP_SECOND_FILE
 
+    @pytest.mark.filterwarnings(iers.IERSStaleWarning)
     def test_auto_open_erfa(self):
         ls = iers.LeapSeconds.auto_open(['erfa', iers.IERS_LEAP_SECOND_FILE])
         assert ls.meta['data_url'] in ['erfa', iers.IERS_LEAP_SECOND_FILE]
 
+    @pytest.mark.filterwarnings(iers.IERSStaleWarning)
     def test_fake_future_file(self, tmpdir):
         fake_file = make_fake_file('28 June 2345', tmpdir)
         # Try as system file for auto_open, setting auto_max_age such
@@ -165,14 +186,27 @@ class TestAutoOpenExplicitLists:
         assert ls2.expires == Time('2012-06-27', scale='tai')
 
         # Use the fake files to make sure auto_max_age is safe.
+        # Should have no warning in either example.
         with iers.conf.set_temp('auto_max_age', None):
             ls3 = iers.LeapSeconds.auto_open([fake_file1,
                                               iers.IERS_LEAP_SECOND_FILE])
-        assert ls3.meta['data_url'] == fake_file1
+        assert ls3.meta['data_url'] == iers.IERS_LEAP_SECOND_FILE
+        with iers.conf.set_temp('auto_max_age', None):
+            ls4 = iers.LeapSeconds.auto_open([fake_file1, fake_file2])
+        assert ls4.meta['data_url'] == fake_file2
 
 
 @pytest.mark.remote_data
 class TestRemoteURLs:
+    def setup_class(cls):
+        # Need auto_download so that IERS_B won't be loaded and cause tests to
+        # fail.
+        iers.conf.auto_download = True
+
+    def teardown_class(cls):
+        # This setting is to be consistent with astropy/conftest.py
+        iers.conf.auto_download = False
+
     # In these tests, the results may be cached.
     # This is fine - no need to download again.
     def test_iers_url(self):
@@ -226,8 +260,10 @@ class TestDefaultAutoOpen:
     def test_builtin_not_expired(self):
         # TODO: would be nice to have automatic PRs for this!
         ls = iers.LeapSeconds.open(iers.IERS_LEAP_SECOND_FILE)
-        assert ls.expires > self.good_enough, \
-            "The leap second file built in to astropy is expired. PR needed!"
+        assert ls.expires > self.good_enough, (
+            "The leap second file built in to astropy is expired. Fix with:\n"
+            "cd astropy/utils/iers/data/; . update_builtin_iers.sh\n"
+            "and commit as a PR (for details, see release procedure).")
 
     def test_fake_future_file(self, tmpdir):
         fake_file = make_fake_file('28 June 2345', tmpdir)
@@ -290,11 +326,19 @@ class TestDefaultAutoOpen:
     def test_auto_open_urls_always_good_enough(self):
         # Avoid using the erfa, built-in and system files, as they might
         # be good enough already.
-        self.remove_auto_open_files('erfa', iers.IERS_LEAP_SECOND_FILE,
-                                    'system_leap_second_file')
-        ls = iers.LeapSeconds.open()
-        assert ls.expires > self.good_enough
-        assert ls.meta['data_url'].startswith('http')
+        try:
+            # Need auto_download so that IERS_B won't be loaded and
+            # cause tests to fail.
+            iers.conf.auto_download = True
+
+            self.remove_auto_open_files('erfa', iers.IERS_LEAP_SECOND_FILE,
+                                        'system_leap_second_file')
+            ls = iers.LeapSeconds.open()
+            assert ls.expires > self.good_enough
+            assert ls.meta['data_url'].startswith('http')
+        finally:
+            # This setting is to be consistent with astropy/conftest.py
+            iers.conf.auto_download = False
 
 
 class ERFALeapSecondsSafe:
