@@ -26,8 +26,10 @@ from tempfile import NamedTemporaryFile, TemporaryDirectory
 import py.path
 import pytest
 
+from astropy import units as _u  # u is taken
 from astropy.config import paths
 import astropy.utils.data
+from astropy.utils.exceptions import AstropyWarning
 from astropy.utils.data import (
     CacheMissingWarning,
     CacheDamaged,
@@ -54,26 +56,18 @@ from astropy.utils.data import (
     check_free_space_in_dir,
     _get_download_cache_loc,
     download_files_in_parallel,
+    is_url,
+    get_pkg_data_path
 )
 
 CI = os.environ.get('CI', False) == "true"
 TESTURL = "http://www.astropy.org"
 TESTURL2 = "http://www.astropy.org/about.html"
+TESTURL_SSL = "https://www.astropy.org"
 TESTLOCAL = get_pkg_data_filename(os.path.join("data", "local.dat"))
 
-try:
-    import bz2  # noqa
-except ImportError:
-    HAS_BZ2 = False
-else:
-    HAS_BZ2 = True
-
-try:
-    import lzma  # noqa
-except ImportError:
-    HAS_XZ = False
-else:
-    HAS_XZ = True
+# NOTE: Python can be built without bz2 or lzma.
+from astropy.utils.compat.optional_deps import HAS_BZ2, HAS_LZMA
 
 # For when we need "some" test URLs
 FEW = 5
@@ -385,10 +379,8 @@ def test_download_with_sources_and_bogus_original(
         assert is_url_in_cache(u)
 
 
-@pytest.mark.skipif((3, 7) <= sys.version_info < (3, 8) or
-                    (sys.platform.startswith('win') and CI),
-                    reason="mystery segfault that is possibly bug #10008 "
-                           "for python < 3.8, flaky cache error on Windows CI")
+@pytest.mark.skipif((sys.platform.startswith('win') and CI),
+                    reason="flaky cache error on Windows CI")
 def test_download_file_threaded_many(temp_cache, valid_urls):
     """Hammer download_file with multiple threaded requests.
 
@@ -406,10 +398,8 @@ def test_download_file_threaded_many(temp_cache, valid_urls):
         assert get_file_contents(r) == c
 
 
-@pytest.mark.skipif((3, 7) <= sys.version_info < (3, 8) or
-                    (sys.platform.startswith('win') and CI),
-                    reason="mystery segfault that is possibly bug #10008 "
-                           "for python < 3.8, flaky cache error on Windows CI")
+@pytest.mark.skipif((sys.platform.startswith('win') and CI),
+                    reason="flaky cache error on Windows CI")
 def test_threaded_segfault(valid_urls):
     """Demonstrate urllib's segfault."""
     def slurp_url(u):
@@ -424,10 +414,8 @@ def test_threaded_segfault(valid_urls):
                    [u for (u, c) in urls]))
 
 
-@pytest.mark.skipif((3, 7) <= sys.version_info < (3, 8) or
-                    (sys.platform.startswith('win') and CI),
-                    reason="mystery segfault that is possibly bug #10008 "
-                           "for python < 3.8, flaky cache error on Windows CI")
+@pytest.mark.skipif((sys.platform.startswith('win') and CI),
+                    reason="flaky cache error on Windows CI")
 def test_download_file_threaded_many_partial_success(
         temp_cache, valid_urls, invalid_urls):
     """Hammer download_file with multiple threaded requests.
@@ -648,6 +636,26 @@ def test_download_cache():
     assert not os.path.isdir(lockdir), "Cache dir lock was not released!"
 
 
+@pytest.mark.remote_data(source="astropy")
+def test_download_certificate_verification_failed():
+    """Tests for https://github.com/astropy/astropy/pull/10434"""
+
+    # First test the expected exception when download fails due to a
+    # certificate verification error; we simulate this by passing a bogus
+    # CA directory to the ssl_context argument
+    ssl_context = {'cafile': None, 'capath': '/does/not/exist'}
+    msg = f'Verification of TLS/SSL certificate at {TESTURL_SSL} failed'
+    with pytest.raises(urllib.error.URLError, match=msg):
+        download_file(TESTURL_SSL, cache=False, ssl_context=ssl_context)
+
+    with pytest.warns(AstropyWarning, match=msg) as warning_lines:
+        fnout = download_file(TESTURL_SSL, cache=False,
+                ssl_context=ssl_context, allow_insecure=True)
+
+    assert len(warning_lines) == 1
+    assert os.path.isfile(fnout)
+
+
 def test_download_cache_after_clear(tmpdir, temp_cache, valid_urls):
     testurl, contents = next(valid_urls)
     # Test issues raised in #4427 with clear_download_cache() without a URL,
@@ -767,6 +775,7 @@ def test_download_parallel_partial_success(temp_cache, valid_urls, invalid_urls)
     # assert not any([is_url_in_cache(u) for (u, c) in td])
 
 
+@pytest.mark.slow
 def test_download_parallel_partial_success_lock_safe(temp_cache, valid_urls, invalid_urls):
     """Check that a partially successful parallel download leaves the cache unlocked.
 
@@ -827,10 +836,8 @@ def test_download_parallel_update(temp_cache, tmpdir):
         assert get_file_contents(r_3) == c_plus
 
 
-@pytest.mark.skipif(sys.version_info < (3, 8) or
-                    (sys.platform.startswith('win') and CI),
-                    reason="mystery segfault that is possibly bug #10008 "
-                           "for python < 3.8, flaky cache error on Windows CI")
+@pytest.mark.skipif((sys.platform.startswith('win') and CI),
+                    reason="flaky cache error on Windows CI")
 def test_update_parallel(temp_cache, valid_urls):
     u, c = next(valid_urls)
     u2, c2 = next(valid_urls)
@@ -849,10 +856,8 @@ def test_update_parallel(temp_cache, valid_urls):
         assert get_file_contents(f) == c2
 
 
-@pytest.mark.skipif(sys.version_info < (3, 8) or
-                    (sys.platform.startswith('win') and CI),
-                    reason="mystery segfault that is possibly bug #10008 "
-                           "for python < 3.8, flaky cache error on Windows CI")
+@pytest.mark.skipif((sys.platform.startswith('win') and CI),
+                    reason="flaky cache error on Windows CI")
 def test_update_parallel_multi(temp_cache, valid_urls):
     u, c = next(valid_urls)
     iucs = list(islice(valid_urls, N_THREAD_HAMMER))
@@ -900,12 +905,21 @@ def test_find_invalid():
         )
 
 
+@pytest.mark.parametrize("package", [None, "astropy", "numpy"])
+def test_get_invalid(package):
+    """Test can create a file path to an invalid file."""
+    path = get_pkg_data_path("kjfrhgjkla", "hgiulrhgiu", package=package)
+    assert not os.path.isfile(path)
+    assert not os.path.isdir(path)
+
+
 # Package data functions
 @pytest.mark.parametrize(
     ("filename"), ["local.dat", "local.dat.gz", "local.dat.bz2", "local.dat.xz"]
 )
 def test_local_data_obj(filename):
-    if (not HAS_BZ2 and "bz2" in filename) or (not HAS_XZ and "xz" in filename):
+    if ((not HAS_BZ2 and "bz2" in filename) or
+            (not HAS_LZMA and "xz" in filename)):
         with pytest.raises(ValueError) as e:
             with get_pkg_data_fileobj(
                 os.path.join("data", filename), encoding="binary"
@@ -954,11 +968,11 @@ def test_local_data_obj_invalid(bad_compressed):
     # they're not local anymore: they just live in a temporary directory
     # created by pytest. However, we can still use get_readable_fileobj for the
     # test.
-    if (not HAS_BZ2 and is_bz2) or (not HAS_XZ and is_xz):
-        with pytest.raises(ValueError) as e:
+    if (not HAS_BZ2 and is_bz2) or (not HAS_LZMA and is_xz):
+        with pytest.raises(ModuleNotFoundError,
+                           match=r'does not provide the [lb]z[2m]a? module\.'):
             with get_readable_fileobj(bad_compressed, encoding="binary") as f:
                 f.read()
-        assert " format files are not supported" in str(e.value)
     else:
         with get_readable_fileobj(bad_compressed, encoding="binary") as f:
             assert f.read().rstrip().endswith(b"invalid")
@@ -993,7 +1007,9 @@ def test_data_name_third_party_package():
         import test_package
 
         filename = test_package.get_data_filename()
-        assert filename == os.path.join(data_dir, "test_package", "data", "foo.txt")
+        assert os.path.normcase(filename) == (
+            os.path.normcase(os.path.join(data_dir, "test_package", "data", "foo.txt"))
+        )
     finally:
         sys.path.pop(0)
 
@@ -1063,8 +1079,6 @@ def test_data_noastropy_fallback(monkeypatch):
         fnout = download_file(TESTURL, cache=True)
     n_warns = len(warning_lines)
 
-    assert n_warns in (2, 4), f'Expected 2 or 4 warnings, got {n_warns}'
-
     partial_warn_msgs = ['remote data cache could not be accessed', 'temporary file']
     if n_warns == 4:
         partial_warn_msgs.extend(['socket', 'socket'])
@@ -1077,6 +1091,8 @@ def test_data_noastropy_fallback(monkeypatch):
                 break
     assert len(partial_warn_msgs) == 0, f'Got some unexpected warnings: {partial_warn_msgs}'
 
+    assert n_warns in (2, 4), f'Expected 2 or 4 warnings, got {n_warns}'
+
     assert os.path.isfile(fnout)
 
     # clearing the cache should be a no-up that doesn't affect fnout
@@ -1110,7 +1126,7 @@ def test_data_noastropy_fallback(monkeypatch):
         ),
         pytest.param(
             "unicode.txt.xz",
-            marks=pytest.mark.xfail(not HAS_XZ, reason="no lzma support"),
+            marks=pytest.mark.xfail(not HAS_LZMA, reason="no lzma support"),
         ),
     ],
 )
@@ -1368,9 +1384,11 @@ def test_cache_contents_agrees_with_get_urls(temp_cache, valid_urls):
         assert cache_contents()[u] == h
 
 
-def test_free_space_checker_huge(tmpdir):
+@pytest.mark.parametrize('desired_size',
+                         [1_000_000_000_000_000_000, 1 * _u.Ebyte])
+def test_free_space_checker_huge(tmpdir, desired_size):
     with pytest.raises(OSError):
-        check_free_space_in_dir(str(tmpdir), 1_000_000_000_000_000_000)
+        check_free_space_in_dir(str(tmpdir), desired_size)
 
 
 def test_get_free_space_file_directory(tmpdir):
@@ -1379,7 +1397,16 @@ def test_get_free_space_file_directory(tmpdir):
         pass
     with pytest.raises(OSError):
         get_free_space_in_dir(str(fn))
-    assert get_free_space_in_dir(str(tmpdir)) > 0
+
+    free_space = get_free_space_in_dir(str(tmpdir))
+    assert free_space > 0 and not hasattr(free_space, 'unit')
+
+    # TODO: If unit=True starts to auto-guess prefix, this needs updating.
+    free_space = get_free_space_in_dir(str(tmpdir), unit=True)
+    assert free_space > 0 and free_space.unit == _u.byte
+
+    free_space = get_free_space_in_dir(str(tmpdir), unit=_u.Mbit)
+    assert free_space > 0 and free_space.unit == _u.Mbit
 
 
 def test_download_file_bogus_settings(invalid_urls, temp_cache):
@@ -1694,7 +1721,7 @@ def test_download_file_wrong_size(monkeypatch):
     def mockurl(remote_url, timeout=None):
         yield MockURL()
 
-    def mockurl_builder(tlscontext=None):
+    def mockurl_builder(*args, tlscontext=None, **kwargs):
         mock_opener = type('MockOpener', (object,), {})()
         mock_opener.open = mockurl
         return mock_opener
@@ -1709,7 +1736,7 @@ def test_download_file_wrong_size(monkeypatch):
         def read(self, length=None):
             return self.reader.read(length)
 
-    monkeypatch.setattr(urllib.request, "build_opener", mockurl_builder)
+    monkeypatch.setattr(astropy.utils.data, "_build_urlopener", mockurl_builder)
 
     with pytest.raises(urllib.error.ContentTooShortError):
         report_length = 1024
@@ -2046,14 +2073,15 @@ def test_update_of_open_files_windows(temp_cache, valid_urls, monkeypatch):
     assert get_file_contents(download_file(u, cache=True, sources=[])) == c
 
 
-def test_zero_remote_timeout(temp_cache, valid_urls):
+def test_no_allow_internet(temp_cache, valid_urls):
     u, c = next(valid_urls)
-    with pytest.raises(urllib.error.URLError):
-        download_file(u, timeout=0)
-    assert not is_url_in_cache(u)
-    with pytest.raises(urllib.error.URLError):
-        # This will trigger the remote data error if it's allowed to touch the internet
-        download_file(TESTURL, timeout=0)
+    with conf.set_temp('allow_internet', False):
+        with pytest.raises(urllib.error.URLError):
+            download_file(u)
+        assert not is_url_in_cache(u)
+        with pytest.raises(urllib.error.URLError):
+            # This will trigger the remote data error if it's allowed to touch the internet
+            download_file(TESTURL)
 
 
 def test_clear_download_cache_not_too_aggressive(temp_cache, valid_urls):
@@ -2103,7 +2131,7 @@ def test_clear_download_cache_variants(temp_cache, valid_urls):
 @pytest.mark.skipif("CI", reason="Flaky on CI")
 @pytest.mark.remote_data
 def test_ftp_tls_auto(temp_cache):
-    url = "ftp://anonymous:mail%40astropy.org@gdc.cddis.eosdis.nasa.gov/pub/products/iers/finals2000A.all"
+    url = "ftp://anonymous:mail%40astropy.org@gdc.cddis.eosdis.nasa.gov/pub/products/iers/finals2000A.all"  # noqa
     download_file(url)
 
 
@@ -2138,4 +2166,29 @@ def test_download_ftp_file_properly_handles_socket_error():
     with pytest.raises(urllib.error.URLError) as excinfo:
         download_file(faulty_url)
     errmsg = excinfo.exconly()
-    assert "Name or service not known" in errmsg or 'getaddrinfo failed' in errmsg or 'Temporary failure in name resolution' in errmsg
+    found_msg = False
+    possible_msgs = ['Name or service not known',
+                     'nodename nor servname provided, or not known',
+                     'getaddrinfo failed',
+                     'Temporary failure in name resolution',
+                     'No address associated with hostname']
+    for cur_msg in possible_msgs:
+        if cur_msg in errmsg:
+            found_msg = True
+            break
+    assert found_msg, f'Got {errmsg}, expected one of these: {",".join(possible_msgs)}'
+
+
+@pytest.mark.parametrize(
+    ('s', 'ans'),
+    [('http://googlecom', True),
+     ('https://google.com', True),
+     ('ftp://google.com', True),
+     ('sftp://google.com', True),
+     ('ssh://google.com', True),
+     ('file:///c:/path/to/the%20file.txt', True),
+     ('google.com', False),
+     ('C:\\\\path\\\\file.docx', False),
+     ('data://file', False)])
+def test_string_is_url_check(s, ans):
+    assert is_url(s) is ans
