@@ -1,15 +1,17 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
+import concurrent.futures
 import inspect
 import pickle
 
 import pytest
 
-from astropy.utils.decorators import (deprecated_attribute, deprecated, wraps,
-                                      sharedmethod, classproperty,
+from astropy.utils.decorators import (deprecated_attribute, deprecated,
+                                      sharedmethod, classproperty, lazyproperty,
                                       format_doc, deprecated_renamed_argument)
-from astropy.utils.exceptions import AstropyDeprecationWarning, AstropyUserWarning
-from astropy.tests.helper import catch_warnings
+from astropy.utils.exceptions import (AstropyDeprecationWarning,
+                                      AstropyPendingDeprecationWarning,
+                                      AstropyUserWarning)
 
 
 class NewDeprecationWarning(AstropyDeprecationWarning):
@@ -19,76 +21,56 @@ class NewDeprecationWarning(AstropyDeprecationWarning):
     """
 
 
-def test_wraps():
-    """
-    Tests the compatibility replacement for functools.wraps which supports
-    argument preservation across all supported Python versions.
-    """
-
-    def foo(a, b, c=1, d=2, e=3, **kwargs):
-        """A test function."""
-
-        return a, b, c, d, e, kwargs
-
-    @wraps(foo)
-    def bar(*args, **kwargs):
-        return ('test',) + foo(*args, **kwargs)
-
-    expected = ('test', 1, 2, 3, 4, 5, {'f': 6, 'g': 7})
-    assert bar(1, 2, 3, 4, 5, f=6, g=7) == expected
-    assert bar.__name__ == 'foo'
-
-    if foo.__doc__ is not None:
-        # May happen if using optimized opcode
-        assert bar.__doc__ == "A test function."
-
-    if hasattr(foo, '__qualname__'):
-        assert bar.__qualname__ == foo.__qualname__
-
-    sig = inspect.signature(bar)
-    assert list(sig.parameters) == ['a', 'b', 'c', 'd', 'e', 'kwargs']
-
-    defaults = [inspect._empty, inspect._empty, 1, 2, 3, inspect._empty]
-    assert [p.default for p in sig.parameters.values()] == defaults
-
-
 def test_deprecated_attribute():
     class DummyClass:
         def __init__(self):
             self._foo = 42
             self._bar = 4242
+            self._message = '42'
+            self._alternative = [42]
+            self._pending = {42}
 
         def set_private(self):
             self._foo = 100
             self._bar = 1000
+            self._message = '100'
+            self._alternative = [100]
+            self._pending = {100}
 
         foo = deprecated_attribute('foo', '0.2')
 
         bar = deprecated_attribute('bar', '0.2',
                                    warning_type=NewDeprecationWarning)
 
+        alternative = deprecated_attribute('alternative', '0.2',
+                                           alternative='other')
+
+        message = deprecated_attribute('message', '0.2', message='MSG')
+
+        pending = deprecated_attribute('pending', '0.2', pending=True)
+
     dummy = DummyClass()
 
-    with catch_warnings(AstropyDeprecationWarning) as wfoo:
+    with pytest.warns(AstropyDeprecationWarning, match="The foo attribute is "
+                      "deprecated and may be removed in a future version.") as w:
         dummy.foo
+    assert len(w) == 1
 
-    with catch_warnings(AstropyDeprecationWarning) as wbar:
+    with pytest.warns(NewDeprecationWarning, match="The bar attribute is "
+                      "deprecated and may be removed in a future version.") as w:
         dummy.bar
+    assert len(w) == 1
 
-    assert len(wfoo) == 1
-    assert str(wfoo[0].message) == ("The foo attribute is deprecated and may "
-                                    "be removed in a future version.")
-    assert wfoo[0].category == AstropyDeprecationWarning
+    with pytest.warns(AstropyDeprecationWarning, match="MSG"):
+        dummy.message
 
-    assert len(wbar) == 1
-    assert str(wbar[0].message) == ("The bar attribute is deprecated and may "
-                                    "be removed in a future version.")
-    assert wbar[0].category == NewDeprecationWarning
+    with pytest.warns(AstropyDeprecationWarning, match=r"Use other instead\."):
+        dummy.alternative
 
-    with catch_warnings() as w:
-        dummy.set_private()
+    with pytest.warns(AstropyPendingDeprecationWarning):
+        dummy.pending
 
-    assert len(w) == 0
+    dummy.set_private()
 
 
 # This needs to be defined outside of the test function, because we
@@ -134,9 +116,8 @@ def test_deprecated_class():
                      '__subclasshook__', '__init_subclass__'):
             assert getattr(TA, x) == getattr(orig_A, x)
 
-    with catch_warnings(AstropyDeprecationWarning) as w:
+    with pytest.warns(AstropyDeprecationWarning) as w:
         TA()
-
     assert len(w) == 1
     if TA.__doc__ is not None:
         assert 'function' not in TA.__doc__
@@ -147,11 +128,9 @@ def test_deprecated_class():
     # Make sure the object is picklable
     pickle.dumps(TA)
 
-    with catch_warnings(NewDeprecationWarning) as w:
+    with pytest.warns(NewDeprecationWarning) as w:
         TC()
-
     assert len(w) == 1
-    assert w[0].category == NewDeprecationWarning
 
 
 def test_deprecated_class_with_new_method():
@@ -167,7 +146,7 @@ def test_deprecated_class_with_new_method():
             return super().__new__(cls)
 
     # Creating an instance should work but raise a DeprecationWarning
-    with catch_warnings(AstropyDeprecationWarning) as w:
+    with pytest.warns(AstropyDeprecationWarning) as w:
         A(1)
     assert len(w) == 1
 
@@ -180,7 +159,7 @@ def test_deprecated_class_with_new_method():
             pass
 
     # Creating an instance should work but raise a DeprecationWarning
-    with catch_warnings(AstropyDeprecationWarning) as w:
+    with pytest.warns(AstropyDeprecationWarning) as w:
         B(1)
     assert len(w) == 1
 
@@ -197,9 +176,8 @@ def test_deprecated_class_with_super():
         def __init__(self, a, b):
             super().__init__()
 
-    with catch_warnings(AstropyDeprecationWarning) as w:
+    with pytest.warns(AstropyDeprecationWarning) as w:
         TB(1, 2)
-
     assert len(w) == 1
     if TB.__doc__ is not None:
         assert 'function' not in TB.__doc__
@@ -214,9 +192,8 @@ def test_deprecated_class_with_custom_metaclass():
     other than type did not restore the metaclass properly.
     """
 
-    with catch_warnings(AstropyDeprecationWarning) as w:
+    with pytest.warns(AstropyDeprecationWarning) as w:
         TB()
-
     assert len(w) == 1
     assert type(TB) is TMeta
     assert TB.metaclass_attr == 1
@@ -243,16 +220,14 @@ def test_deprecated_static_and_classmethod():
         def C(cls):
             pass
 
-    with catch_warnings(AstropyDeprecationWarning) as w:
+    with pytest.warns(AstropyDeprecationWarning) as w:
         A.B()
-
     assert len(w) == 1
     if A.__doc__ is not None:
         assert 'deprecated' in A.B.__doc__
 
-    with catch_warnings(AstropyDeprecationWarning) as w:
+    with pytest.warns(AstropyDeprecationWarning) as w:
         A.C()
-
     assert len(w) == 1
     if A.__doc__ is not None:
         assert 'deprecated' in A.C.__doc__
@@ -294,23 +269,29 @@ def test_deprecated_argument():
         assert method(overwrite=1) == 1
 
         # Using the deprecated name
-        with catch_warnings(AstropyDeprecationWarning) as w:
+        with pytest.warns(AstropyDeprecationWarning, match=r"1\.3") as w:
             assert method(clobber=1) == 1
-            assert len(w) == 1
-            assert '1.3' in str(w[0].message)
-            assert 'test_decorators.py' in str(w[0].filename)
-
-            if method.__name__ == 'test4':
-                w[0].category == NewDeprecationWarning
+        assert len(w) == 1
+        assert 'test_decorators.py' in str(w[0].filename)
+        if method.__name__ == 'test4':
+            assert issubclass(w[0].category, NewDeprecationWarning)
 
         # Using both. Both keyword
-        with pytest.raises(TypeError):
+        with pytest.raises(TypeError), pytest.warns(AstropyDeprecationWarning):
             method(clobber=2, overwrite=1)
         # One positional, one keyword
-        with pytest.raises(TypeError):
+        with pytest.raises(TypeError), pytest.warns(AstropyDeprecationWarning):
             method(1, clobber=2)
 
 
+def test_deprecated_argument_custom_message():
+    @deprecated_renamed_argument('foo', 'bar', '4.0', message='Custom msg')
+    def test(bar=0):
+        pass
+    with pytest.warns(AstropyDeprecationWarning, match='Custom msg'):
+        test(foo=0)
+
+
 def test_deprecated_argument_in_kwargs():
     # To rename an argument that is consumed by "kwargs" the "arg_in_kwargs"
     # parameter is used.
@@ -327,17 +308,16 @@ def test_deprecated_argument_in_kwargs():
     assert test(overwrite=1) == 1
 
     # Using the deprecated name
-    with catch_warnings(AstropyDeprecationWarning) as w:
+    with pytest.warns(AstropyDeprecationWarning, match=r"1\.3") as w:
         assert test(clobber=1) == 1
-        assert len(w) == 1
-        assert '1.3' in str(w[0].message)
-        assert 'test_decorators.py' in str(w[0].filename)
+    assert len(w) == 1
+    assert 'test_decorators.py' in str(w[0].filename)
 
     # Using both. Both keyword
-    with pytest.raises(TypeError):
+    with pytest.raises(TypeError), pytest.warns(AstropyDeprecationWarning):
         test(clobber=2, overwrite=1)
     # One positional, one keyword
-    with pytest.raises(TypeError):
+    with pytest.raises(TypeError), pytest.warns(AstropyDeprecationWarning):
         test(1, clobber=2)
 
 
@@ -355,20 +335,23 @@ def test_deprecated_argument_relaxed():
     assert test(overwrite=1) == 1
 
     # Using the deprecated name
-    with catch_warnings(AstropyDeprecationWarning) as w:
+    with pytest.warns(AstropyDeprecationWarning, match=r"1\.3") as w:
         assert test(clobber=1) == 1
-        assert len(w) == 1
-        assert '1.3' in str(w[0].message)
+    assert len(w) == 1
 
     # Using both. Both keyword
-    with catch_warnings(AstropyUserWarning) as w:
+    with pytest.warns(AstropyUserWarning) as w:
         assert test(clobber=2, overwrite=1) == 1
-        assert len(w) == 1
+    assert len(w) == 2
+    assert '"clobber" was deprecated' in str(w[0].message)
+    assert '"clobber" and "overwrite" keywords were set' in str(w[1].message)
 
     # One positional, one keyword
-    with catch_warnings(AstropyUserWarning) as w:
+    with pytest.warns(AstropyUserWarning) as w:
         assert test(1, clobber=2) == 1
-        assert len(w) == 1
+    assert len(w) == 2
+    assert '"clobber" was deprecated' in str(w[0].message)
+    assert '"clobber" and "overwrite" keywords were set' in str(w[1].message)
 
 
 def test_deprecated_argument_pending():
@@ -385,19 +368,13 @@ def test_deprecated_argument_pending():
     assert test(overwrite=1) == 1
 
     # Using the deprecated name
-    with catch_warnings(AstropyUserWarning, AstropyDeprecationWarning) as w:
-        assert test(clobber=1) == 1
-        assert len(w) == 0
+    assert test(clobber=1) == 1
 
     # Using both. Both keyword
-    with catch_warnings(AstropyUserWarning, AstropyDeprecationWarning) as w:
-        assert test(clobber=2, overwrite=1) == 1
-        assert len(w) == 0
+    assert test(clobber=2, overwrite=1) == 1
 
     # One positional, one keyword
-    with catch_warnings(AstropyUserWarning, AstropyDeprecationWarning) as w:
-        assert test(1, clobber=2) == 1
-        assert len(w) == 0
+    assert test(1, clobber=2) == 1
 
 
 def test_deprecated_argument_multi_deprecation():
@@ -406,22 +383,22 @@ def test_deprecated_argument_multi_deprecation():
     def test(a, b, c):
         return a, b, c
 
-    with catch_warnings(AstropyDeprecationWarning) as w:
+    with pytest.warns(AstropyDeprecationWarning) as w:
         assert test(x=1, y=2, z=3) == (1, 2, 3)
-        assert len(w) == 3
+    assert len(w) == 3
 
     # Make sure relax is valid for all arguments
-    with catch_warnings(AstropyUserWarning) as w:
+    with pytest.warns(AstropyUserWarning) as w:
         assert test(x=1, y=2, z=3, b=3) == (1, 3, 3)
-        assert len(w) == 1
+    assert len(w) == 4
 
-    with catch_warnings(AstropyUserWarning) as w:
+    with pytest.warns(AstropyUserWarning) as w:
         assert test(x=1, y=2, z=3, a=3) == (3, 2, 3)
-        assert len(w) == 1
+    assert len(w) == 4
 
-    with catch_warnings(AstropyUserWarning) as w:
+    with pytest.warns(AstropyUserWarning) as w:
         assert test(x=1, y=2, z=3, c=5) == (1, 2, 5)
-        assert len(w) == 1
+    assert len(w) == 4
 
 
 def test_deprecated_argument_multi_deprecation_2():
@@ -430,15 +407,15 @@ def test_deprecated_argument_multi_deprecation_2():
     def test(a, b, c):
         return a, b, c
 
-    with catch_warnings(AstropyUserWarning) as w:
+    with pytest.warns(AstropyUserWarning) as w:
         assert test(x=1, y=2, z=3, b=3) == (1, 3, 3)
-        assert len(w) == 1
+    assert len(w) == 4
 
-    with catch_warnings(AstropyUserWarning) as w:
+    with pytest.warns(AstropyUserWarning) as w:
         assert test(x=1, y=2, z=3, a=3) == (3, 2, 3)
-        assert len(w) == 1
+    assert len(w) == 4
 
-    with pytest.raises(TypeError):
+    with pytest.raises(TypeError), pytest.warns(AstropyUserWarning):
         assert test(x=1, y=2, z=3, c=5) == (1, 2, 5)
 
 
@@ -468,24 +445,20 @@ def test_deprecated_argument_remove():
     def test(dummy=11, x=3):
         return dummy, x
 
-    with catch_warnings(AstropyDeprecationWarning) as w:
+    with pytest.warns(AstropyDeprecationWarning, match=r"Use astropy\.y instead") as w:
         assert test(x=1) == (11, 1)
-        assert len(w) == 1
-        assert 'Use astropy.y instead' in str(w[0].message)
+    assert len(w) == 1
 
-    with catch_warnings(AstropyDeprecationWarning) as w:
+    with pytest.warns(AstropyDeprecationWarning) as w:
         assert test(x=1, dummy=10) == (10, 1)
-        assert len(w) == 1
+    assert len(w) == 1
 
-    with pytest.warns(AstropyDeprecationWarning,
-                      match=r'Use astropy.y instead'):
+    with pytest.warns(AstropyDeprecationWarning, match=r'Use astropy\.y instead'):
         test(121, 1) == (121, 1)
 
-    with catch_warnings(AstropyDeprecationWarning) as w:
-        assert test() == (11, 3)
-        assert test(121) == (121, 3)
-        assert test(dummy=121) == (121, 3)
-        assert len(w) == 0
+    assert test() == (11, 3)
+    assert test(121) == (121, 3)
+    assert test(dummy=121) == (121, 3)
 
 
 def test_sharedmethod_reuse_on_subclasses():
@@ -555,6 +528,62 @@ def test_classproperty_docstring():
     assert B.__dict__['foo'].__doc__ == "The foo."
 
 
+@pytest.mark.slow
+def test_classproperty_lazy_threadsafe(fast_thread_switching):
+    """
+    Test that a class property with lazy=True is thread-safe.
+    """
+    workers = 8
+    with concurrent.futures.ThreadPoolExecutor(max_workers=workers) as executor:
+        # This is testing for race conditions, so try many times in the
+        # hope that we'll get the timing right.
+        for p in range(10000):
+
+            class A:
+                @classproperty(lazy=True)
+                def foo(cls):
+                    nonlocal calls
+                    calls += 1
+                    return object()
+
+            # Have all worker threads query in parallel
+            calls = 0
+            futures = [executor.submit(lambda: A.foo) for i in range(workers)]
+            # Check that only one call happened and they all received it
+            values = [future.result() for future in futures]
+            assert calls == 1
+            assert values[0] is not None
+            assert values == [values[0]] * workers
+
+
+@pytest.mark.slow
+def test_lazyproperty_threadsafe(fast_thread_switching):
+    """
+    Test thread safety of lazyproperty.
+    """
+    # This test is generally similar to test_classproperty_lazy_threadsafe
+    # above. See there for comments.
+
+    class A:
+        def __init__(self):
+            self.calls = 0
+
+        @lazyproperty
+        def foo(self):
+            self.calls += 1
+            return object()
+
+    workers = 8
+    with concurrent.futures.ThreadPoolExecutor(max_workers=workers) as executor:
+        for p in range(10000):
+            a = A()
+            futures = [executor.submit(lambda: a.foo) for i in range(workers)]
+            values = [future.result() for future in futures]
+            assert a.calls == 1
+            assert a.foo is not None
+            assert values == [a.foo] * workers
+
+
 def test_format_doc_stringInput_simple():
     # Simple tests with string input
 
