@@ -12,8 +12,10 @@ from matplotlib.transforms import Affine2D, Bbox, Transform
 
 import astropy.units as u
 from astropy.coordinates import SkyCoord, BaseCoordinateFrame
+from astropy.utils import minversion
+from astropy.utils.compat.optional_deps import HAS_PIL
 from astropy.wcs import WCS
-from astropy.wcs.wcsapi import BaseHighLevelWCS
+from astropy.wcs.wcsapi import BaseHighLevelWCS, BaseLowLevelWCS
 
 from .transforms import CoordinateTransform
 from .coordinates_map import CoordinatesMap
@@ -195,16 +197,15 @@ class WCSAxes(Axes):
         elif origin == 'upper':
             raise ValueError("Cannot use images with origin='upper' in WCSAxes.")
 
-        # To check whether the image is a PIL image we can check if the data
-        # has a 'getpixel' attribute - this is what Matplotlib's AxesImage does
+        if HAS_PIL:
+            from PIL.Image import Image
+
+            if minversion('PIL', '9.1'):
+                from PIL.Image import Transpose
+                FLIP_TOP_BOTTOM = Transpose.FLIP_TOP_BOTTOM
+            else:
+                from PIL.Image import FLIP_TOP_BOTTOM
 
-        try:
-            from PIL.Image import Image, FLIP_TOP_BOTTOM
-        except ImportError:
-            # We don't need to worry since PIL is not installed, so user cannot
-            # have passed RGB image.
-            pass
-        else:
             if isinstance(X, Image) or hasattr(X, 'getpixel'):
                 X = X.transpose(FLIP_TOP_BOTTOM)
 
@@ -288,8 +289,8 @@ class WCSAxes(Axes):
 
         See Also
         --------
-
-        matplotlib.axes.Axes.plot : This method is called from this function with all arguments passed to it.
+        matplotlib.axes.Axes.plot :
+            This method is called from this function with all arguments passed to it.
 
         """
 
@@ -404,7 +405,6 @@ class WCSAxes(Axes):
         self._bboxes = []
         # This generates a structure like [coords][axis] = [...]
         ticklabels_bbox = defaultdict(partial(defaultdict, list))
-        ticks_locs = defaultdict(partial(defaultdict, list))
 
         visible_ticks = []
 
@@ -418,8 +418,7 @@ class WCSAxes(Axes):
 
             for coord in coords:
                 coord._draw_ticks(renderer, bboxes=self._bboxes,
-                                  ticklabels_bbox=ticklabels_bbox[coord],
-                                  ticks_locs=ticks_locs[coord])
+                                  ticklabels_bbox=ticklabels_bbox[coord])
                 visible_ticks.extend(coord.ticklabels.get_visible_axes())
 
         for coords in self._all_coords:
@@ -427,7 +426,6 @@ class WCSAxes(Axes):
             for coord in coords:
                 coord._draw_axislabels(renderer, bboxes=self._bboxes,
                                        ticklabels_bbox=ticklabels_bbox,
-                                       ticks_locs=ticks_locs[coord],
                                        visible_ticks=visible_ticks)
 
         self.coords.frame.draw(renderer)
@@ -435,6 +433,14 @@ class WCSAxes(Axes):
     def draw(self, renderer, **kwargs):
         """Draw the axes."""
 
+        # Before we do any drawing, we need to remove any existing grid lines
+        # drawn with contours, otherwise if we try and remove the contours
+        # part way through drawing, we end up with the issue mentioned in
+        # https://github.com/astropy/astropy/issues/12446
+        for coords in self._all_coords:
+            for coord in coords:
+                coord._clear_grid_contour()
+
         # In Axes.draw, the following code can result in the xlim and ylim
         # values changing, so we need to force call this here to make sure that
         # the limits are correct before we update the patch.
@@ -460,11 +466,7 @@ class WCSAxes(Axes):
 
         self._drawn = True
 
-    # MATPLOTLIB_LT_30: The ``kwargs.pop('label', None)`` is to ensure
-    # compatibility with Matplotlib 2.x (which has label) and 3.x (which has
-    # xlabel). While these are meant to be a single positional argument,
-    # Matplotlib internally sometimes specifies e.g. set_xlabel(xlabel=...).
-
+    # Matplotlib internally sometimes calls set_xlabel(label=...).
     def set_xlabel(self, xlabel=None, labelpad=1, loc=None, **kwargs):
         """Set x-label."""
         if xlabel is None:
@@ -574,7 +576,9 @@ class WCSAxes(Axes):
         Return a transform from data to the specified frame
         """
 
-        if isinstance(frame, WCS):
+        if isinstance(frame, (BaseLowLevelWCS, BaseHighLevelWCS)):
+            if isinstance(frame, BaseHighLevelWCS):
+                frame = frame.low_level_wcs
 
             transform, coord_meta = transform_coord_meta_from_wcs(frame, self.frame_class)
             transform_world2pixel = transform.inverted()
@@ -590,7 +594,7 @@ class WCSAxes(Axes):
                                             transform_world2pixel.frame_in) +
                         transform_world2pixel)
 
-        elif frame == 'pixel':
+        elif isinstance(frame, str) and frame == 'pixel':
 
             return Affine2D()
 
@@ -600,7 +604,7 @@ class WCSAxes(Axes):
 
         else:
 
-            if frame == 'world':
+            if isinstance(frame, str) and frame == 'world':
 
                 return self._transform_pixel2world
 
@@ -617,7 +621,7 @@ class WCSAxes(Axes):
 
         # FIXME: we should determine what to do with the extra arguments here.
         # Note that the expected signature of this method is different in
-        # Matplotlib 3.x compared to 2.x.
+        # Matplotlib 3.x compared to 2.x, but we only support 3.x now.
 
         if not self.get_visible():
             return
