@@ -1,9 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-
-import os
 import warnings
-from packaging.version import Version
 
+from packaging.version import Version
 import pytest
 import numpy as np
 import matplotlib
@@ -14,7 +12,9 @@ from astropy import units as u
 from astropy.wcs import WCS
 from astropy.io import fits
 from astropy.coordinates import SkyCoord
-from astropy.tests.helper import catch_warnings
+
+from astropy.utils.data import get_pkg_data_filename
+from astropy.wcs.wcsapi import SlicedLowLevelWCS, HighLevelWCSWrapper
 
 from astropy.visualization.wcsaxes.core import WCSAxes
 from astropy.visualization.wcsaxes.frame import (
@@ -22,15 +22,11 @@ from astropy.visualization.wcsaxes.frame import (
 from astropy.visualization.wcsaxes.utils import get_coord_meta
 from astropy.visualization.wcsaxes.transforms import CurvedTransform
 
-mpl_version = Version(matplotlib.__version__)
 ft_version = Version(matplotlib.ft2font.__freetype_version__)
-MATPLOTLIB_LT_21 = mpl_version < Version("2.1")
-MATPLOTLIB_LT_22 = mpl_version < Version("2.2")
-MATPLOTLIB_LT_30 = mpl_version < Version("3.0")
 FREETYPE_261 = ft_version == Version("2.6.1")
 TEX_UNAVAILABLE = not matplotlib.checkdep_usetex(True)
 
-DATA = os.path.abspath(os.path.join(os.path.dirname(__file__), 'data'))
+MATPLOTLIB_DEV = Version(matplotlib.__version__).is_devrelease
 
 
 def teardown_function(function):
@@ -39,7 +35,7 @@ def teardown_function(function):
 
 def test_grid_regression(ignore_matplotlibrc):
     # Regression test for a bug that meant that if the rc parameter
-    # axes.grid was set to True, WCSAxes would crash upon initalization.
+    # axes.grid was set to True, WCSAxes would crash upon initialization.
     plt.rc('axes', grid=True)
     fig = plt.figure(figsize=(3, 3))
     WCSAxes(fig, [0.1, 0.1, 0.8, 0.8])
@@ -80,22 +76,23 @@ COORDSYS= 'icrs    '
 
 @pytest.mark.parametrize('grid_type', ['lines', 'contours'])
 def test_no_numpy_warnings(ignore_matplotlibrc, tmpdir, grid_type):
-
-    # Make sure that no warnings are raised if some pixels are outside WCS
-    # (since this is normal)
-
-    ax = plt.subplot(1, 1, 1, projection=WCS(TARGET_HEADER))
+    fig = plt.figure()
+    ax = fig.add_subplot(1, 1, 1, projection=WCS(TARGET_HEADER))
     ax.imshow(np.zeros((100, 200)))
     ax.coords.grid(color='white', grid_type=grid_type)
 
-    with catch_warnings(RuntimeWarning) as ws:
-        plt.savefig(tmpdir.join('test.png').strpath)
-
-    # For debugging
-    for w in ws:
-        print(w)
-
-    assert len(ws) == 0
+    # There should be no warnings raised if some pixels are outside WCS
+    # (since this is normal).
+    # BUT our own catch_warning was ignoring some warnings before, so now we
+    # have to catch it. Otherwise, the pytest filterwarnings=error
+    # setting in setup.cfg will fail this test.
+    # There are actually multiple warnings but they are all similar.
+    with warnings.catch_warnings():
+        warnings.filterwarnings('ignore', message=r'.*converting a masked element to nan.*')
+        warnings.filterwarnings('ignore', message=r'.*No contour levels were found within the data range.*')
+        warnings.filterwarnings('ignore', message=r'.*np\.asscalar\(a\) is deprecated since NumPy v1\.16.*')
+        warnings.filterwarnings('ignore', message=r'.*PY_SSIZE_T_CLEAN will be required.*')
+        fig.savefig(tmpdir.join('test.png').strpath)
 
 
 def test_invalid_frame_overlay(ignore_matplotlibrc):
@@ -113,7 +110,7 @@ def test_invalid_frame_overlay(ignore_matplotlibrc):
 
 def test_plot_coord_transform(ignore_matplotlibrc):
 
-    twoMASS_k_header = os.path.join(DATA, '2MASS_k_header')
+    twoMASS_k_header = get_pkg_data_filename('data/2MASS_k_header')
     twoMASS_k_header = fits.Header.fromtextfile(twoMASS_k_header)
     fig = plt.figure(figsize=(6, 6))
     ax = fig.add_axes([0.15, 0.15, 0.8, 0.8],
@@ -186,34 +183,23 @@ def test_slicing_warnings(ignore_matplotlibrc, tmpdir):
     wcs3d.wcs.cdelt = [6.25, 6.25, 23]
     wcs3d.wcs.crval = [0., 0., 1.]
 
-    with warnings.catch_warnings(record=True) as warning_lines:
-        warnings.resetwarnings()
+    with warnings.catch_warnings():
+        # https://github.com/astropy/astropy/issues/9690
+        warnings.filterwarnings('ignore', message=r'.*PY_SSIZE_T_CLEAN.*')
         plt.subplot(1, 1, 1, projection=wcs3d, slices=('x', 'y', 1))
         plt.savefig(tmpdir.join('test.png').strpath)
 
-    # For easy debugging if there are indeed warnings
-    for warning in warning_lines:
-        print(warning)
-        # https://github.com/astropy/astropy/issues/9690
-        if 'PY_SSIZE_T_CLEAN' not in str(warning.message):
-            raise AssertionError(f'Unexpected warning: {warning}')
-
     # Angle case
 
     wcs3d = WCS(GAL_HEADER)
 
-    with warnings.catch_warnings(record=True) as warning_lines:
-        warnings.resetwarnings()
+    with warnings.catch_warnings():
+        # https://github.com/astropy/astropy/issues/9690
+        warnings.filterwarnings('ignore', message=r'.*PY_SSIZE_T_CLEAN.*')
+        plt.clf()
         plt.subplot(1, 1, 1, projection=wcs3d, slices=('x', 'y', 2))
         plt.savefig(tmpdir.join('test.png').strpath)
 
-    # For easy debugging if there are indeed warnings
-    for warning in warning_lines:
-        print(warning)
-        # https://github.com/astropy/astropy/issues/9690
-        if 'PY_SSIZE_T_CLEAN' not in str(warning.message):
-            raise AssertionError(f'Unexpected warning: {warning}')
-
 
 def test_plt_xlabel_ylabel(tmpdir):
 
@@ -311,7 +297,6 @@ def test_contour_return():
     assert isinstance(cset, QuadContourSet)
 
 
-@pytest.mark.skipif('MATPLOTLIB_LT_21')
 def test_contour_empty():
 
     # Regression test for a bug that caused contour to crash if no contours
@@ -364,20 +349,26 @@ def test_invalid_slices_errors(ignore_matplotlibrc):
     wcs2d = WCS(naxis=2)
     wcs2d.wcs.ctype = ['x', 'y']
 
+    plt.clf()
     ax = plt.subplot(1, 1, 1, projection=wcs2d)
     assert ax.frame_class is RectangularFrame
+    plt.clf()
     ax = plt.subplot(1, 1, 1, projection=wcs2d, slices=('x', 'y'))
     assert ax.frame_class is RectangularFrame
+    plt.clf()
     ax = plt.subplot(1, 1, 1, projection=wcs2d, slices=('y', 'x'))
     assert ax.frame_class is RectangularFrame
+    plt.clf()
     ax = plt.subplot(1, 1, 1, projection=wcs2d, slices=['x', 'y'])
     assert ax.frame_class is RectangularFrame
+    plt.clf()
     ax = plt.subplot(1, 1, 1, projection=wcs2d, slices=(1, 'x'))
     assert ax.frame_class is RectangularFrame1D
 
     wcs1d = WCS(naxis=1)
     wcs1d.wcs.ctype = ['x']
 
+    plt.clf()
     ax = plt.subplot(1, 1, 1, projection=wcs1d)
     assert ax.frame_class is RectangularFrame1D
 
@@ -424,6 +415,7 @@ def test_repr(ignore_matplotlibrc):
 
     # Now slice in a way that all world coordinates are still present:
 
+    plt.clf()
     ax = plt.subplot(1, 1, 1, projection=wcs3d, slices=('x', 'y', 1))
     assert repr(ax.coords) == EXPECTED_REPR_2
 
@@ -444,7 +436,7 @@ def test_time_wcs(time_spectral_wcs_2d):
     plt.subplot(projection=time_spectral_wcs_2d)
 
 
-@pytest.mark.skipif('MATPLOTLIB_LT_22 or TEX_UNAVAILABLE')
+@pytest.mark.skipif('TEX_UNAVAILABLE')
 def test_simplify_labels_usetex(ignore_matplotlibrc, tmpdir):
     """Regression test for https://github.com/astropy/astropy/issues/8004."""
     plt.rc('text', usetex=True)
@@ -511,14 +503,8 @@ def test_set_labels_with_coords(ignore_matplotlibrc, frame_class):
 
 @pytest.mark.parametrize('atol', [0.2, 1.0e-8])
 def test_bbox_size(atol):
-    """Test for the size of a WCSAxes bbox with 'tight' layout.
-    """
-    extents = [11.38888888888889, 3.5]
-    # Upper bounding box limits changed between Matplotlib major versions.
-    if MATPLOTLIB_LT_30:
-        extents += [579.5, 435.5]
-    else:
-        extents += [576.0, 432.0]
+    # Test for the size of a WCSAxes bbox (only have Matplotlib >= 3.0 now)
+    extents = [11.38888888888889, 3.5, 576.0, 432.0]
 
     fig = plt.figure()
     ax = WCSAxes(fig, [0.1, 0.1, 0.8, 0.8])
@@ -531,3 +517,21 @@ def test_bbox_size(atol):
     if atol < 0.1 and not FREETYPE_261:
         pytest.xfail("Exact BoundingBox dimensions are only ensured with FreeType 2.6.1")
     assert np.allclose(ax_bbox.extents, extents, atol=atol)
+
+
+def test_wcs_type_transform_regression():
+    wcs = WCS(TARGET_HEADER)
+    sliced_wcs = SlicedLowLevelWCS(wcs, np.s_[1:-1, 1:-1])
+    ax = plt.subplot(1, 1, 1, projection=wcs)
+    ax.get_transform(sliced_wcs)
+
+    high_wcs = HighLevelWCSWrapper(sliced_wcs)
+    ax.get_transform(sliced_wcs)
+
+
+def test_multiple_draws_grid_contours(tmpdir):
+    fig = plt.figure()
+    ax = fig.add_subplot(1, 1, 1, projection=WCS())
+    ax.grid(color='black', grid_type='contours')
+    fig.savefig(tmpdir / 'plot.png')
+    fig.savefig(tmpdir / 'plot.png')
