@@ -2,47 +2,41 @@
 
 import io
 import os
-import sys
-import warnings
+from contextlib import nullcontext
 from datetime import datetime
-from distutils.version import LooseVersion
 
+from packaging.version import Version
 import pytest
 import numpy as np
 from numpy.testing import (
     assert_allclose, assert_array_almost_equal, assert_array_almost_equal_nulp,
     assert_array_equal)
 
-from astropy.tests.helper import raises, catch_warnings
 from astropy import wcs
 from astropy.wcs import _wcs  # noqa
-from astropy.wcs.wcs import FITSFixedWarning
-from astropy.utils.compat.context import nullcontext
+from astropy import units as u
 from astropy.utils.data import (
     get_pkg_data_filenames, get_pkg_data_contents, get_pkg_data_filename)
 from astropy.utils.misc import NumpyRNGContext
 from astropy.utils.exceptions import (
     AstropyUserWarning, AstropyWarning, AstropyDeprecationWarning)
+from astropy.tests.helper import assert_quantity_allclose
 from astropy.io import fits
 from astropy.coordinates import SkyCoord
+from astropy.nddata import Cutout2D
+
+_WCSLIB_VER = Version(_wcs.__version__)
 
 
-_WCSLIB_VER = LooseVersion(_wcs.__version__)
-
-
-def _check_v71_dateref_warnings(w, nmax=None):
-    if _WCSLIB_VER >= '7.1' and _WCSLIB_VER < '7.3' and w:
-        if nmax is None:
-            assert w
-        else:
-            assert len(w) == nmax
-
-        for item in w:
-            if (str(item.message) == "'datfix' made the change "
-                "'Set DATE-REF to '1858-11-17' from MJD-REF'."):
-                break
-        else:
-            assert False, "No 'datfix' warning found"
+# NOTE: User can choose to use system wcslib instead of bundled.
+def ctx_for_v71_dateref_warnings():
+    if _WCSLIB_VER >= Version('7.1') and _WCSLIB_VER < Version('7.3'):
+        ctx = pytest.warns(
+            wcs.FITSFixedWarning,
+            match=r"'datfix' made the change 'Set DATE-REF to '1858-11-17' from MJD-REF'\.")
+    else:
+        ctx = nullcontext()
+    return ctx
 
 
 class TestMaps:
@@ -105,7 +99,15 @@ class TestSpectra:
             header = get_pkg_data_contents(
                 os.path.join("data", "spectra", filename), encoding='binary')
             # finally run the test.
-            all_wcs = wcs.find_all_wcs(header)
+            if _WCSLIB_VER >= Version('7.4'):
+                ctx = pytest.warns(
+                    wcs.FITSFixedWarning,
+                    match=r"'datfix' made the change 'Set MJD-OBS to 53925\.853472 from DATE-OBS'\.")  # noqa
+            else:
+                ctx = nullcontext()
+            with ctx:
+                all_wcs = wcs.find_all_wcs(header)
+
             assert len(all_wcs) == 9
 
 
@@ -113,25 +115,20 @@ def test_fixes():
     """
     From github issue #36
     """
-    def run():
-        header = get_pkg_data_contents(
-            'data/nonstandard_units.hdr', encoding='binary')
-        try:
-            wcs.WCS(header, translate_units='dhs')
-        except wcs.InvalidTransformError:
-            pass
-        else:
-            assert False, "Expected InvalidTransformError"
+    header = get_pkg_data_contents('data/nonstandard_units.hdr', encoding='binary')
 
-    with catch_warnings(wcs.FITSFixedWarning) as w:
-        run()
+    with pytest.raises(wcs.InvalidTransformError), pytest.warns(wcs.FITSFixedWarning) as w:
+        wcs.WCS(header, translate_units='dhs')
 
-    assert len(w) == 2
-    for item in w:
-        if 'unitfix' in str(item.message):
-            assert 'Hz' in str(item.message)
-            assert 'M/S' in str(item.message)
-            assert 'm/s' in str(item.message)
+    if Version('7.4') <= _WCSLIB_VER < Version('7.6'):
+        assert len(w) == 3
+        assert "'datfix' made the change 'Success'." in str(w.pop().message)
+    else:
+        assert len(w) == 2
+
+    first_wmsg = str(w[0].message)
+    assert 'unitfix' in first_wmsg and 'Hz' in first_wmsg and 'M/S' in first_wmsg
+    assert 'plane angle' in str(w[1].message) and 'm/s' in str(w[1].message)
 
 
 # Ignore "PV2_2 = 0.209028857410973 invalid keyvalue" warning seen on Windows.
@@ -155,14 +152,18 @@ def test_pix2world():
     """
     # TODO: write this to test the expected output behavior of pix2world,
     # currently this just makes sure it doesn't error out in unexpected ways
+    # (and compares `wcs.pc` and `result` values?)
     filename = get_pkg_data_filename('data/sip2.fits')
-    with catch_warnings(wcs.wcs.FITSFixedWarning) as caught_warnings:
+    with pytest.warns(wcs.FITSFixedWarning) as caught_warnings:
         # this raises a warning unimportant for this testing the pix2world
         #   FITSFixedWarning(u'The WCS transformation has more axes (2) than
         #        the image it is associated with (0)')
         ww = wcs.WCS(filename)
 
-        # might as well monitor for changing behavior
+    # might as well monitor for changing behavior
+    if Version('7.4') <= _WCSLIB_VER < Version('7.6'):
+        assert len(caught_warnings) == 2
+    else:
         assert len(caught_warnings) == 1
 
     n = 3
@@ -172,23 +173,22 @@ def test_pix2world():
     # Catch #2791
     ww.wcs_pix2world(pixels[..., 0], pixels[..., 1], 0, ra_dec_order=True)
 
-    close_enough = 1e-8
     # assuming that the data of sip2.fits doesn't change
     answer = np.array([[0.00024976, 0.00023018],
                        [0.00023043, -0.00024997]])
 
-    assert np.all(np.abs(ww.wcs.pc - answer) < close_enough)
+    assert np.allclose(ww.wcs.pc, answer, atol=1.e-8)
 
     answer = np.array([[202.39265216, 47.17756518],
                        [202.39335826, 47.17754619],
                        [202.39406436, 47.1775272]])
 
-    assert np.all(np.abs(result - answer) < close_enough)
+    assert np.allclose(result, answer, atol=1.e-8, rtol=1.e-10)
 
 
 def test_load_fits_path():
     fits_name = get_pkg_data_filename('data/sip.fits')
-    with pytest.warns(FITSFixedWarning):
+    with pytest.warns(wcs.FITSFixedWarning):
         wcs.WCS(fits_name)
 
 
@@ -198,9 +198,8 @@ def test_dict_init():
     """
 
     # Dictionary with no actual WCS, returns identity transform
-    with catch_warnings(wcs.FITSFixedWarning) as wrng:
+    with ctx_for_v71_dateref_warnings():
         w = wcs.WCS({})
-    _check_v71_dateref_warnings(wrng)
 
     xp, yp = w.wcs_world2pix(41., 2., 1)
 
@@ -220,10 +219,18 @@ def test_dict_init():
         'CDELT1': -0.1,
         'CDELT2': 0.1
     }
-    if _WCSLIB_VER >= '7.1':
+    if _WCSLIB_VER >= Version('7.1'):
         hdr['DATEREF'] = '1858-11-17'
 
-    w = wcs.WCS(hdr)
+    if _WCSLIB_VER >= Version('7.4'):
+        ctx = pytest.warns(
+            wcs.wcs.FITSFixedWarning,
+            match=r"'datfix' made the change 'Set MJDREF to 0\.000000 from DATEREF'\.")
+    else:
+        ctx = nullcontext()
+
+    with ctx:
+        w = wcs.WCS(hdr)
 
     xp, yp = w.wcs_world2pix(41., 2., 0)
 
@@ -231,7 +238,6 @@ def test_dict_init():
     assert_array_almost_equal_nulp(yp, 20., 10)
 
 
-@raises(TypeError)
 def test_extra_kwarg():
     """
     Issue #444
@@ -239,7 +245,8 @@ def test_extra_kwarg():
     w = wcs.WCS()
     with NumpyRNGContext(123456789):
         data = np.random.rand(100, 2)
-        w.wcs_pix2world(data, origin=1)
+        with pytest.raises(TypeError):
+            w.wcs_pix2world(data, origin=1)
 
 
 def test_3d_shapes():
@@ -328,41 +335,28 @@ def test_invalid_shape():
 
 
 def test_warning_about_defunct_keywords():
-    def run():
-        header = get_pkg_data_contents(
-            'data/defunct_keywords.hdr', encoding='binary')
-        wcs.WCS(header)
-
-    with catch_warnings(wcs.FITSFixedWarning) as w:
-        run()
-
-    assert len(w) == 4
-    for item in w:
-        assert 'PCi_ja' in str(item.message)
+    header = get_pkg_data_contents('data/defunct_keywords.hdr', encoding='binary')
+    if Version('7.4') <= _WCSLIB_VER < Version('7.6'):
+        n_warn = 5
+    else:
+        n_warn = 4
 
     # Make sure the warnings come out every time...
+    for _ in range(2):
+        with pytest.warns(wcs.FITSFixedWarning) as w:
+            wcs.WCS(header)
 
-    with catch_warnings(wcs.FITSFixedWarning) as w:
-        run()
-
-    assert len(w) == 4
-    for item in w:
-        assert 'PCi_ja' in str(item.message)
+        assert len(w) == n_warn
+        # 7.4 adds a fifth warning "'datfix' made the change 'Success'."
+        for item in w[:4]:
+            assert 'PCi_ja' in str(item.message)
 
 
 def test_warning_about_defunct_keywords_exception():
-    def run():
-        header = get_pkg_data_contents(
-            'data/defunct_keywords.hdr', encoding='binary')
+    header = get_pkg_data_contents('data/defunct_keywords.hdr', encoding='binary')
+    with pytest.warns(wcs.FITSFixedWarning):
         wcs.WCS(header)
 
-    with pytest.raises(wcs.FITSFixedWarning):
-        warnings.simplefilter("error", wcs.FITSFixedWarning)
-        run()
-
-    # Restore warnings filter to previous state
-    warnings.simplefilter("default")
-
 
 def test_to_header_string():
     hdrstr = (
@@ -376,12 +370,12 @@ def test_to_header_string():
         "LATPOLE =                 90.0 / [deg] Native latitude of celestial pole        ",
     )
 
-    if _WCSLIB_VER >= '7.3':
+    if _WCSLIB_VER >= Version('7.3'):
         hdrstr += (
             "MJDREF  =                  0.0 / [d] MJD of fiducial time                       ",
         )
 
-    elif _WCSLIB_VER >= '7.1':
+    elif _WCSLIB_VER >= Version('7.1'):
         hdrstr += (
             "DATEREF = '1858-11-17'         / ISO-8601 fiducial time                         ",
             "MJDREFI =                  0.0 / [d] MJD of fiducial time, integer part         ",
@@ -403,10 +397,10 @@ def test_to_header_string():
 
 
 def test_to_fits():
-    nrec = 11 if _WCSLIB_VER >= '7.1' else 8
-    if _WCSLIB_VER < '7.1':
+    nrec = 11 if _WCSLIB_VER >= Version('7.1') else 8
+    if _WCSLIB_VER < Version('7.1'):
         nrec = 8
-    elif _WCSLIB_VER < '7.3':
+    elif _WCSLIB_VER < Version('7.3'):
         nrec = 11
     else:
         nrec = 9
@@ -421,12 +415,11 @@ def test_to_fits():
 
 def test_to_header_warning():
     fits_name = get_pkg_data_filename('data/sip.fits')
-    with pytest.warns(FITSFixedWarning):
+    with pytest.warns(wcs.FITSFixedWarning):
         x = wcs.WCS(fits_name)
-    with catch_warnings() as w:
+    with pytest.warns(AstropyWarning, match='A_ORDER') as w:
         x.to_header()
     assert len(w) == 1
-    assert 'A_ORDER' in str(w[0])
 
 
 def test_no_comments_in_header():
@@ -442,7 +435,6 @@ def test_no_comments_in_header():
     assert 'COMMENT' + w.wcs.alt.strip() not in header
 
 
-@raises(wcs.InvalidTransformError)
 def test_find_all_wcs_crash():
     """
     Causes a double free without a recent fix in wcslib_wrap.C
@@ -452,28 +444,31 @@ def test_find_all_wcs_crash():
     # We have to set fix=False here, because one of the fixing tasks is to
     # remove redundant SCAMP distortion parameters when SIP distortion
     # parameters are also present.
-    with pytest.warns(FITSFixedWarning):
+    with pytest.raises(wcs.InvalidTransformError), pytest.warns(wcs.FITSFixedWarning):
         wcs.find_all_wcs(header, fix=False)
 
 
+# NOTE: Warning bubbles up from C layer during wcs.validate() and
+# is hard to catch, so we just ignore it.
+@pytest.mark.filterwarnings("ignore")
 def test_validate():
-    with catch_warnings():
-        results = wcs.validate(get_pkg_data_filename("data/validate.fits"))
-        results_txt = repr(results)
-        version = wcs._wcs.__version__
-        if version[0] in ['6', '7']:
-            filename = 'data/validate.6.txt'
-        elif version[0] == '5':
-            if version >= '5.13':
-                filename = 'data/validate.5.13.txt'
-            else:
-                filename = 'data/validate.5.0.txt'
-        else:
-            filename = 'data/validate.txt'
-        with open(get_pkg_data_filename(filename), "r") as fd:
-            lines = fd.readlines()
-            assert set([x.strip() for x in lines]) == set([
-                x.strip() for x in results_txt.splitlines()])
+    results = wcs.validate(get_pkg_data_filename("data/validate.fits"))
+    results_txt = sorted(set([x.strip() for x in repr(results).splitlines()]))
+    if _WCSLIB_VER >= Version('7.6'):
+        filename = 'data/validate.7.6.txt'
+    elif _WCSLIB_VER >= Version('7.4'):
+        filename = 'data/validate.7.4.txt'
+    elif _WCSLIB_VER >= Version('6.0'):
+        filename = 'data/validate.6.txt'
+    elif _WCSLIB_VER >= Version('5.13'):
+        filename = 'data/validate.5.13.txt'
+    elif _WCSLIB_VER >= Version('5.0'):
+        filename = 'data/validate.5.0.txt'
+    else:
+        filename = 'data/validate.txt'
+    with open(get_pkg_data_filename(filename), "r") as fd:
+        lines = fd.readlines()
+    assert sorted(set([x.strip() for x in lines])) == results_txt
 
 
 def test_validate_with_2_wcses():
@@ -535,8 +530,7 @@ def test_all_world2pix(fname=None, ext=0,
         fname = get_pkg_data_filename('data/j94f05bgq_flt.fits')
         ext = ('SCI', 1)
     if not os.path.isfile(fname):
-        raise OSError("Input file '{:s}' to 'test_all_world2pix' not found."
-                      .format(fname))
+        raise OSError(f"Input file '{fname:s}' to 'test_all_world2pix' not found.")
     h = fits.open(fname)
     w = wcs.WCS(h[ext].header, h)
     h.close()
@@ -582,10 +576,8 @@ def test_all_world2pix(fname=None, ext=0,
         if e.divergent is not None:
             ndiv = e.divergent.shape[0]
             print(f"There are {ndiv} diverging solutions.")
-            print("Indices of diverging solutions:\n{}"
-                  .format(e.divergent))
-            print("Diverging solutions:\n{}\n"
-                  .format(e.best_solution[e.divergent]))
+            print(f"Indices of diverging solutions:\n{e.divergent}")
+            print(f"Diverging solutions:\n{e.best_solution[e.divergent]}\n")
             print("Mean radius of the diverging solutions: {}"
                   .format(np.mean(
                       np.linalg.norm(e.best_solution[e.divergent], axis=1))))
@@ -598,19 +590,15 @@ def test_all_world2pix(fname=None, ext=0,
         nslow = 0
         if e.slow_conv is not None:
             nslow = e.slow_conv.shape[0]
-            print("There are {} slowly converging solutions."
-                  .format(nslow))
-            print("Indices of slowly converging solutions:\n{}"
-                  .format(e.slow_conv))
-            print("Slowly converging solutions:\n{}\n"
-                  .format(e.best_solution[e.slow_conv]))
+            print(f"There are {nslow} slowly converging solutions.")
+            print(f"Indices of slowly converging solutions:\n{e.slow_conv}")
+            print(f"Slowly converging solutions:\n{e.best_solution[e.slow_conv]}\n")
         else:
             print("There are no slowly converging solutions.\n")
 
         print("There are {} converged solutions."
               .format(e.best_solution.shape[0] - ndiv - nslow))
-        print("Best solutions (all points):\n{}"
-              .format(e.best_solution))
+        print(f"Best solutions (all points):\n{e.best_solution}")
         print(f"Accuracy:\n{e.accuracy}\n")
         print("\nFinished running 'test_all_world2pix' with errors.\n"
               "ERROR: {}\nRun time: {}\n"
@@ -637,7 +625,7 @@ def test_scamp_sip_distortion_parameters():
     parameters.
     """
     header = get_pkg_data_contents('data/validate.fits', encoding='binary')
-    with pytest.warns(FITSFixedWarning):
+    with pytest.warns(wcs.FITSFixedWarning):
         w = wcs.WCS(header)
     # Just check that this doesn't raise an exception.
     w.all_pix2world(0, 0, 0)
@@ -729,7 +717,7 @@ def test_error_message():
     with pytest.raises(wcs.InvalidTransformError):
         # Both lines are in here, because 0.4 calls .set within WCS.__init__,
         # whereas 0.3 and earlier did not.
-        with pytest.warns(FITSFixedWarning):
+        with pytest.warns(wcs.FITSFixedWarning):
             w = wcs.WCS(header, _do_set=False)
             w.all_pix2world([[536.0, 894.0]], 0)
 
@@ -752,7 +740,7 @@ def test_out_of_bounds():
 
 def test_calc_footprint_1():
     fits = get_pkg_data_filename('data/sip.fits')
-    with pytest.warns(FITSFixedWarning):
+    with pytest.warns(wcs.FITSFixedWarning):
         w = wcs.WCS(fits)
 
         axes = (1000, 1051)
@@ -767,7 +755,7 @@ def test_calc_footprint_1():
 def test_calc_footprint_2():
     """ Test calc_footprint without distortion. """
     fits = get_pkg_data_filename('data/sip.fits')
-    with pytest.warns(FITSFixedWarning):
+    with pytest.warns(wcs.FITSFixedWarning):
         w = wcs.WCS(fits)
 
         axes = (1000, 1051)
@@ -827,17 +815,16 @@ def test_printwcs(capsys):
     """
     h = get_pkg_data_contents(
         'data/spectra/orion-freq-1.hdr', encoding='binary')
-    with pytest.warns(FITSFixedWarning):
+    with pytest.warns(wcs.FITSFixedWarning):
         w = wcs.WCS(h)
         w.printwcs()
         captured = capsys.readouterr()
         assert 'WCS Keywords' in captured.out
     h = get_pkg_data_contents('data/3d_cd.hdr', encoding='binary')
-    with pytest.warns(AstropyUserWarning):
-        w = wcs.WCS(h)
-        w.printwcs()
-        captured = capsys.readouterr()
-        assert 'WCS Keywords' in captured.out
+    w = wcs.WCS(h)
+    w.printwcs()
+    captured = capsys.readouterr()
+    assert 'WCS Keywords' in captured.out
 
 
 def test_invalid_spherical():
@@ -900,7 +887,7 @@ def test_sip_tpv_agreement():
     tpv_header = get_pkg_data_contents(
         os.path.join("data", "tpvonly.hdr"), encoding='binary')
 
-    with pytest.warns(FITSFixedWarning):
+    with pytest.warns(wcs.FITSFixedWarning):
         w_sip = wcs.WCS(sip_header)
         w_tpv = wcs.WCS(tpv_header)
 
@@ -930,7 +917,7 @@ def test_tpv_copy():
     tpv_header = get_pkg_data_contents(
         os.path.join("data", "tpvonly.hdr"), encoding='binary')
 
-    with pytest.warns(FITSFixedWarning):
+    with pytest.warns(wcs.FITSFixedWarning):
         w_tpv = wcs.WCS(tpv_header)
 
         ra, dec = w_tpv.wcs_pix2world([0, 100, 200], [0, -100, 200], 0)
@@ -941,66 +928,103 @@ def test_tpv_copy():
 def test_hst_wcs():
     path = get_pkg_data_filename("data/dist_lookup.fits.gz")
 
-    hdulist = fits.open(path)
-    # wcslib will complain about the distortion parameters if they
-    # weren't correctly deleted from the header
-    w = wcs.WCS(hdulist[1].header, hdulist)
+    with fits.open(path) as hdulist:
+        # wcslib will complain about the distortion parameters if they
+        # weren't correctly deleted from the header
+        w = wcs.WCS(hdulist[1].header, hdulist)
 
-    # Exercise the main transformation functions, mainly just for
-    # coverage
-    w.p4_pix2foc([0, 100, 200], [0, -100, 200], 0)
-    w.det2im([0, 100, 200], [0, -100, 200], 0)
+        # Check pixel scale and area
+        assert_quantity_allclose(
+            w.proj_plane_pixel_scales(), [1.38484378e-05, 1.39758488e-05] * u.deg)
+        assert_quantity_allclose(
+            w.proj_plane_pixel_area(), 1.93085492e-10 * (u.deg * u.deg))
 
-    w.cpdis1 = w.cpdis1
-    w.cpdis2 = w.cpdis2
+        # Exercise the main transformation functions, mainly just for
+        # coverage
+        w.p4_pix2foc([0, 100, 200], [0, -100, 200], 0)
+        w.det2im([0, 100, 200], [0, -100, 200], 0)
 
-    w.det2im1 = w.det2im1
-    w.det2im2 = w.det2im2
+        w.cpdis1 = w.cpdis1
+        w.cpdis2 = w.cpdis2
 
-    w.sip = w.sip
+        w.det2im1 = w.det2im1
+        w.det2im2 = w.det2im2
 
-    w.cpdis1.cdelt = w.cpdis1.cdelt
-    w.cpdis1.crpix = w.cpdis1.crpix
-    w.cpdis1.crval = w.cpdis1.crval
-    w.cpdis1.data = w.cpdis1.data
+        w.sip = w.sip
 
-    assert w.sip.a_order == 4
-    assert w.sip.b_order == 4
-    assert w.sip.ap_order == 0
-    assert w.sip.bp_order == 0
-    assert_array_equal(w.sip.crpix, [2048., 1024.])
-    wcs.WCS(hdulist[1].header, hdulist)
-    hdulist.close()
+        w.cpdis1.cdelt = w.cpdis1.cdelt
+        w.cpdis1.crpix = w.cpdis1.crpix
+        w.cpdis1.crval = w.cpdis1.crval
+        w.cpdis1.data = w.cpdis1.data
+
+        assert w.sip.a_order == 4
+        assert w.sip.b_order == 4
+        assert w.sip.ap_order == 0
+        assert w.sip.bp_order == 0
+        assert_array_equal(w.sip.crpix, [2048., 1024.])
+        wcs.WCS(hdulist[1].header, hdulist)
 
 
-def test_list_naxis():
+def test_cpdis_comments():
     path = get_pkg_data_filename("data/dist_lookup.fits.gz")
 
-    hdulist = fits.open(path)
-    # wcslib will complain about the distortion parameters if they
-    # weren't correctly deleted from the header
-    with catch_warnings(wcs.FITSFixedWarning) as wrng:
-        w = wcs.WCS(hdulist[1].header, hdulist, naxis=['celestial'])
-    _check_v71_dateref_warnings(wrng)
+    f = fits.open(path)
+    w = wcs.WCS(f[1].header, f)
+    hdr = w.to_fits()[0].header
+    f.close()
 
-    assert w.naxis == 2
-    assert w.wcs.naxis == 2
+    wcscards = list(hdr['CPDIS*'].cards) + list(hdr['DP*'].cards)
+    wcsdict = {k: (v, c) for k, v, c in wcscards}
 
-    path = get_pkg_data_filename("data/maps/1904-66_SIN.hdr")
-    with open(path, 'rb') as fd:
-        content = fd.read()
-    with catch_warnings(wcs.FITSFixedWarning) as wrng:
-        w = wcs.WCS(content, naxis=['celestial'])
-    _check_v71_dateref_warnings(wrng)
-    assert w.naxis == 2
-    assert w.wcs.naxis == 2
+    refcards = [
+        ('CPDIS1', 'LOOKUP', 'Prior distortion function type'),
+        ('DP1.EXTVER', 1.0, 'Version number of WCSDVARR extension'),
+        ('DP1.NAXES', 2.0, 'Number of independent variables in CPDIS function'),
+        ('DP1.AXIS.1', 1.0, 'Axis number of the 1st variable in a CPDIS function'),
+        ('DP1.AXIS.2', 2.0, 'Axis number of the 2nd variable in a CPDIS function'),
+        ('CPDIS2', 'LOOKUP', 'Prior distortion function type'),
+        ('DP2.EXTVER', 2.0, 'Version number of WCSDVARR extension'),
+        ('DP2.NAXES', 2.0, 'Number of independent variables in CPDIS function'),
+        ('DP2.AXIS.1', 1.0, 'Axis number of the 1st variable in a CPDIS function'),
+        ('DP2.AXIS.2', 2.0, 'Axis number of the 2nd variable in a CPDIS function'),
+    ]
 
-    with catch_warnings(wcs.FITSFixedWarning) as wrng:
-        w = wcs.WCS(content, naxis=['spectral'])
-    _check_v71_dateref_warnings(wrng)
-    assert w.naxis == 0
-    assert w.wcs.naxis == 0
-    hdulist.close()
+    assert len(wcsdict) == len(refcards)
+
+    for k, v, c in refcards:
+        assert wcsdict[k] == (v, c)
+
+
+def test_d2im_comments():
+    path = get_pkg_data_filename("data/ie6d07ujq_wcs.fits")
+
+    f = fits.open(path)
+    with pytest.warns(wcs.FITSFixedWarning):
+        w = wcs.WCS(f[0].header, f)
+    f.close()
+    wcscards = list(w.to_fits()[0].header['D2IM*'].cards)
+    wcsdict = {k: (v, c) for k, v, c in wcscards}
+
+    refcards = [
+        ('D2IMDIS1', 'LOOKUP', 'Detector to image correction type'),
+        ('D2IM1.EXTVER', 1.0, 'Version number of WCSDVARR extension'),
+        ('D2IM1.NAXES', 2.0, 'Number of independent variables in D2IM function'),
+        ('D2IM1.AXIS.1', 1.0, 'Axis number of the 1st variable in a D2IM function'),
+        ('D2IM1.AXIS.2', 2.0, 'Axis number of the 2nd variable in a D2IM function'),
+        ('D2IMDIS2', 'LOOKUP', 'Detector to image correction type'),
+        ('D2IM2.EXTVER', 2.0, 'Version number of WCSDVARR extension'),
+        ('D2IM2.NAXES', 2.0, 'Number of independent variables in D2IM function'),
+        ('D2IM2.AXIS.1', 1.0, 'Axis number of the 1st variable in a D2IM function'),
+        ('D2IM2.AXIS.2', 2.0, 'Axis number of the 2nd variable in a D2IM function'),
+        # ('D2IMERR1', 0.049, 'Maximum error of D2IM correction for axis 1'),
+        # ('D2IMERR2', 0.035, 'Maximum error of D2IM correction for axis 2'),
+        # ('D2IMEXT', 'iref$y7b1516hi_d2i.fits', ''),
+    ]
+
+    assert len(wcsdict) == len(refcards)
+
+    for k, v, c in refcards:
+        assert wcsdict[k] == (v, c)
 
 
 def test_sip_broken():
@@ -1023,8 +1047,8 @@ def test_no_truncate_crval():
 
     header = w.to_header()
     for ii in range(3):
-        assert header['CRVAL{}'.format(ii + 1)] == w.wcs.crval[ii]
-        assert header['CDELT{}'.format(ii + 1)] == w.wcs.cdelt[ii]
+        assert header[f'CRVAL{ii + 1}'] == w.wcs.crval[ii]
+        assert header[f'CDELT{ii + 1}'] == w.wcs.cdelt[ii]
 
 
 def test_no_truncate_crval_try2():
@@ -1042,8 +1066,8 @@ def test_no_truncate_crval_try2():
 
     header = w.to_header()
     for ii in range(3):
-        assert header['CRVAL{}'.format(ii + 1)] == w.wcs.crval[ii]
-        assert header['CDELT{}'.format(ii + 1)] == w.wcs.cdelt[ii]
+        assert header[f'CRVAL{ii + 1}'] == w.wcs.crval[ii]
+        assert header[f'CDELT{ii + 1}'] == w.wcs.cdelt[ii]
 
 
 def test_no_truncate_crval_p17():
@@ -1086,7 +1110,7 @@ def test_passing_ImageHDU():
     """
     path = get_pkg_data_filename('data/validate.fits')
     with fits.open(path) as hdulist:
-        with pytest.warns(FITSFixedWarning):
+        with pytest.warns(wcs.FITSFixedWarning):
             wcs_hdu = wcs.WCS(hdulist[0])
             wcs_header = wcs.WCS(hdulist[0].header)
             assert wcs_hdu.wcs.compare(wcs_header.wcs)
@@ -1100,60 +1124,53 @@ def test_inconsistent_sip():
     Test for #4814
     """
     hdr = get_pkg_data_contents("data/sip-broken.hdr")
-    with catch_warnings(wcs.FITSFixedWarning) as wrng:
+    ctx = ctx_for_v71_dateref_warnings()
+    with ctx:
         w = wcs.WCS(hdr)
-    _check_v71_dateref_warnings(wrng)
     with pytest.warns(AstropyWarning):
         newhdr = w.to_header(relax=None)
     # CTYPE should not include "-SIP" if relax is None
-    with catch_warnings(wcs.FITSFixedWarning) as wrng:
+    with ctx:
         wnew = wcs.WCS(newhdr)
-    _check_v71_dateref_warnings(wrng)
     assert all(not ctyp.endswith('-SIP') for ctyp in wnew.wcs.ctype)
     newhdr = w.to_header(relax=False)
-    assert('A_0_2' not in newhdr)
+    assert 'A_0_2' not in newhdr
     # CTYPE should not include "-SIP" if relax is False
-    with catch_warnings(wcs.FITSFixedWarning) as wrng:
+    with ctx:
         wnew = wcs.WCS(newhdr)
-    _check_v71_dateref_warnings(wrng)
     assert all(not ctyp.endswith('-SIP') for ctyp in wnew.wcs.ctype)
     with pytest.warns(AstropyWarning):
         newhdr = w.to_header(key="C")
-    assert('A_0_2' not in newhdr)
+    assert 'A_0_2' not in newhdr
     # Test writing header with a different key
-    with catch_warnings(wcs.FITSFixedWarning) as wrng:
+    with ctx:
         wnew = wcs.WCS(newhdr, key='C')
-    _check_v71_dateref_warnings(wrng)
     assert all(not ctyp.endswith('-SIP') for ctyp in wnew.wcs.ctype)
     with pytest.warns(AstropyWarning):
         newhdr = w.to_header(key=" ")
     # Test writing a primary WCS to header
-    with catch_warnings(wcs.FITSFixedWarning) as wrng:
+    with ctx:
         wnew = wcs.WCS(newhdr)
-    _check_v71_dateref_warnings(wrng)
     assert all(not ctyp.endswith('-SIP') for ctyp in wnew.wcs.ctype)
     # Test that "-SIP" is kept into CTYPE if relax=True and
     # "-SIP" was in the original header
     newhdr = w.to_header(relax=True)
-    with catch_warnings(wcs.FITSFixedWarning) as wrng:
+    with ctx:
         wnew = wcs.WCS(newhdr)
-    _check_v71_dateref_warnings(wrng)
     assert all(ctyp.endswith('-SIP') for ctyp in wnew.wcs.ctype)
-    assert('A_0_2' in newhdr)
+    assert 'A_0_2' in newhdr
     # Test that SIP coefficients are also written out.
     assert wnew.sip is not None
     # ######### broken header ###########
     # Test that "-SIP" is added to CTYPE if relax=True and
     # "-SIP" was not in the original header but SIP coefficients
     # are present.
-    with catch_warnings(wcs.FITSFixedWarning) as wrng:
+    with ctx:
         w = wcs.WCS(hdr)
-    _check_v71_dateref_warnings(wrng)
     w.wcs.ctype = ['RA---TAN', 'DEC--TAN']
     newhdr = w.to_header(relax=True)
-    with catch_warnings(wcs.FITSFixedWarning) as wrng:
+    with ctx:
         wnew = wcs.WCS(newhdr)
-    _check_v71_dateref_warnings(wrng)
     assert all(ctyp.endswith('-SIP') for ctyp in wnew.wcs.ctype)
 
 
@@ -1197,7 +1214,7 @@ def test_sip_with_altkey():
     fix for #5443.
     """
     with fits.open(get_pkg_data_filename('data/sip.fits')) as f:
-        with pytest.warns(FITSFixedWarning):
+        with pytest.warns(wcs.FITSFixedWarning):
             w = wcs.WCS(f[0].header)
     # create a header with two WCSs.
     h1 = w.to_header(relax=True, key='A')
@@ -1205,9 +1222,8 @@ def test_sip_with_altkey():
     h1['CTYPE1A'] = "RA---SIN-SIP"
     h1['CTYPE2A'] = "DEC--SIN-SIP"
     h1.update(h2)
-    with catch_warnings(wcs.FITSFixedWarning) as wrng:
+    with ctx_for_v71_dateref_warnings():
         w = wcs.WCS(h1, key='A')
-    _check_v71_dateref_warnings(wrng)
     assert (w.wcs.ctype == np.array(['RA---SIN-SIP', 'DEC--SIN-SIP'])).all()
 
 
@@ -1241,7 +1257,7 @@ def test_keyedsip():
 
 def test_zero_size_input():
     with fits.open(get_pkg_data_filename('data/sip.fits')) as f:
-        with pytest.warns(FITSFixedWarning):
+        with pytest.warns(wcs.FITSFixedWarning):
             w = wcs.WCS(f[0].header)
 
     inp = np.zeros((0, 2))
@@ -1356,12 +1372,12 @@ def test_cunit():
 
 class TestWcsWithTime:
     def setup(self):
-        if _WCSLIB_VER >= '7.1':
+        if _WCSLIB_VER >= Version('7.1'):
             fname = get_pkg_data_filename('data/header_with_time_wcslib71.fits')
         else:
             fname = get_pkg_data_filename('data/header_with_time.fits')
         self.header = fits.Header.fromfile(fname)
-        with pytest.warns(FITSFixedWarning):
+        with pytest.warns(wcs.FITSFixedWarning):
             self.w = wcs.WCS(self.header, key='A')
 
     def test_keywods2wcsprm(self):
@@ -1373,7 +1389,8 @@ class TestWcsWithTime:
         cdelt = [self.header[val] for val in self.header["CDELT*"]]
         cunit = [self.header[val] for val in self.header["CUNIT*"]]
         assert list(self.w.wcs.ctype) == ctype
-        assert list(self.w.wcs.axis_types) == [2200, 2201, 3300, 0]
+        time_axis_code = 4000 if _WCSLIB_VER >= Version('7.9') else 0
+        assert list(self.w.wcs.axis_types) == [2200, 2201, 3300, time_axis_code]
         assert_allclose(self.w.wcs.crval, crval)
         assert_allclose(self.w.wcs.crpix, crpix)
         assert_allclose(self.w.wcs.cdelt, cdelt)
@@ -1440,3 +1457,124 @@ def test_invalid_coordinate_masking():
     assert_allclose(wx, [-33, -33, 6])
     assert_allclose(wy, [np.nan, 178.75, 177.5])
     assert_allclose(wz, [np.nan, -28.75, -27.5])
+
+
+def test_no_pixel_area():
+    w = wcs.WCS(naxis=3)
+
+    # Pixel area cannot be computed
+    with pytest.raises(ValueError, match='Pixel area is defined only for 2D pixels'):
+        w.proj_plane_pixel_area()
+
+    # Pixel scales still possible
+    assert_quantity_allclose(w.proj_plane_pixel_scales(), 1)
+
+
+def test_distortion_header(tmpdir):
+    """
+    Test that plate distortion model is correctly described by `wcs.to_header()`
+    and preserved when creating a Cutout2D from the image, writing it to FITS,
+    and reading it back from the file.
+    """
+    path = get_pkg_data_filename("data/dss.14.29.56-62.41.05.fits.gz")
+    cen = np.array((50, 50))
+    siz = np.array((20, 20))
+
+    with fits.open(path) as hdulist:
+        with pytest.warns(wcs.FITSFixedWarning):
+            w = wcs.WCS(hdulist[0].header)
+        cut = Cutout2D(hdulist[0].data, position=cen, size=siz, wcs=w)
+
+    # This converts the DSS plate solution model with AMD[XY]n coefficients into a
+    # Template Polynomial Distortion model (TPD.FWD.n coefficients);
+    # not testing explicitly for the header keywords here.
+
+    if _WCSLIB_VER < Version("7.4"):
+        with pytest.warns(AstropyWarning, match="WCS contains a TPD distortion model in CQDIS"):
+            w0 = wcs.WCS(w.to_header_string())
+        with pytest.warns(AstropyWarning, match="WCS contains a TPD distortion model in CQDIS"):
+            w1 = wcs.WCS(cut.wcs.to_header_string())
+        if _WCSLIB_VER >= Version("7.1"):
+            pytest.xfail("TPD coefficients incomplete with WCSLIB >= 7.1 < 7.4")
+    else:
+        w0 = wcs.WCS(w.to_header_string())
+        w1 = wcs.WCS(cut.wcs.to_header_string())
+
+    assert w.pixel_to_world(0, 0).separation(w0.pixel_to_world(0, 0)) < 1.e-3 * u.mas
+    assert w.pixel_to_world(*cen).separation(w0.pixel_to_world(*cen)) < 1.e-3 * u.mas
+
+    assert w.pixel_to_world(*cen).separation(w1.pixel_to_world(*(siz / 2))) < 1.e-3 * u.mas
+
+    cutfile = str(tmpdir.join('cutout.fits'))
+    fits.writeto(cutfile, cut.data, cut.wcs.to_header())
+
+    with fits.open(cutfile) as hdulist:
+        w2 = wcs.WCS(hdulist[0].header)
+
+    assert w.pixel_to_world(*cen).separation(w2.pixel_to_world(*(siz / 2))) < 1.e-3 * u.mas
+
+
+def test_pixlist_wcs_colsel():
+    """
+    Test selection of a specific pixel list WCS using ``colsel``. See #11412.
+    """
+    hdr_file = get_pkg_data_filename('data/chandra-pixlist-wcs.hdr')
+    hdr = fits.Header.fromtextfile(hdr_file)
+    with pytest.warns(wcs.FITSFixedWarning):
+        w = wcs.WCS(hdr, keysel=['image', 'pixel'], colsel=[11, 12])
+    assert w.naxis == 2
+    assert list(w.wcs.ctype) == ['RA---TAN', 'DEC--TAN']
+    assert np.allclose(w.wcs.crval, [229.38051931869, -58.81108068885])
+    assert np.allclose(w.wcs.pc, [[1, 0], [0, 1]])
+    assert np.allclose(w.wcs.cdelt, [-0.00013666666666666, 0.00013666666666666])
+    assert np.allclose(w.wcs.lonpole, 180.)
+
+
+@pytest.mark.skipif(
+    _WCSLIB_VER < Version('7.8'),
+    reason="TIME axis extraction only works with wcslib 7.8 or later"
+)
+def test_time_axis_selection():
+    w = wcs.WCS(naxis=3)
+    w.wcs.ctype = ['RA---TAN', 'DEC--TAN', 'TIME']
+    w.wcs.set()
+    assert list(w.sub([wcs.WCSSUB_TIME]).wcs.ctype) == ['TIME']
+    assert (w.wcs_pix2world([[1, 2, 3]], 0)[0, 2] ==
+            w.sub([wcs.WCSSUB_TIME]).wcs_pix2world([[3]], 0)[0, 0])
+
+
+@pytest.mark.skipif(
+    _WCSLIB_VER < Version('7.8'),
+    reason="TIME axis extraction only works with wcslib 7.8 or later"
+)
+def test_temporal():
+    w = wcs.WCS(naxis=3)
+    w.wcs.ctype = ['RA---TAN', 'DEC--TAN', 'TIME']
+    w.wcs.set()
+    assert w.has_temporal
+    assert w.sub([wcs.WCSSUB_TIME]).is_temporal
+    assert (w.wcs_pix2world([[1, 2, 3]], 0)[0, 2] ==
+            w.temporal.wcs_pix2world([[3]], 0)[0, 0])
+
+
+def test_swapaxes_same_val_roundtrip():
+    w = wcs.WCS(naxis=3)
+    w.wcs.ctype = ["RA---TAN", "DEC--TAN", "FREQ"]
+    w.wcs.crpix = [32.5, 16.5, 1.]
+    w.wcs.crval = [5.63, -72.05, 1.]
+    w.wcs.pc = [[5.9e-06, 1.3e-05, 0.0], [-1.2e-05, 5.0e-06, 0.0], [0.0, 0.0, 1.0]]
+    w.wcs.cdelt = [1.0, 1.0, 1.0]
+    w.wcs.set()
+    axes_order = [3, 2, 1]
+    axes_order0 = list(i - 1 for i in axes_order)
+    ws = w.sub(axes_order)
+    imcoord = np.array([3, 5, 7])
+    imcoords = imcoord[axes_order0]
+    val_ref = w.wcs_pix2world([imcoord], 0)[0]
+    val_swapped = ws.wcs_pix2world([imcoords], 0)[0]
+
+    # check original axis and swapped give same results
+    assert np.allclose(val_ref[axes_order0], val_swapped, rtol=0, atol=1e-8)
+
+    # check round-tripping:
+    assert np.allclose(w.wcs_world2pix([val_ref], 0)[0], imcoord, rtol=0, atol=1e-8)
